/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Insets;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JMenuBar;
import javax.swing.JTextField;
import javax.swing.JToolTip;
import javax.swing.LookAndFeel;
import javax.swing.ToolTipManager;
import javax.swing.UIManager;
import javax.swing.border.Border;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;
import javax.swing.filechooser.FileFilter;
import javax.swing.tree.DefaultMutableTreeNode;

import net.sf.dc.console.components.DcButton;
import net.sf.dc.console.components.DcCheckBox;
import net.sf.dc.console.components.DcComboBox;
import net.sf.dc.console.components.DcDateField;
import net.sf.dc.console.components.DcFileField;
import net.sf.dc.console.components.DcFileLauncherField;
import net.sf.dc.console.components.DcFontSelector;
import net.sf.dc.console.components.DcImageLabel;
import net.sf.dc.console.components.DcLabel;
import net.sf.dc.console.components.DcLongTextField;
import net.sf.dc.console.components.DcLookAndFeelSelector;
import net.sf.dc.console.components.DcMenu;
import net.sf.dc.console.components.DcMenuBar;
import net.sf.dc.console.components.DcMenuItem;
import net.sf.dc.console.components.DcModuleSelector;
import net.sf.dc.console.components.DcMultiLineToolTip;
import net.sf.dc.console.components.DcNumberField;
import net.sf.dc.console.components.DcPasswordField;
import net.sf.dc.console.components.DcPictureField;
import net.sf.dc.console.components.DcProgramDefinitionsField;
import net.sf.dc.console.components.DcRadioButton;
import net.sf.dc.console.components.DcRatingComboBox;
import net.sf.dc.console.components.DcReferencesField;
import net.sf.dc.console.components.DcShortTextField;
import net.sf.dc.console.components.DcTabbedPane;
import net.sf.dc.console.components.DcTextPane;
import net.sf.dc.console.components.DcTimeField;
import net.sf.dc.console.components.DcTree;
import net.sf.dc.console.components.DcUrlField;
import net.sf.dc.console.components.renderers.AvailabilityComboBoxRenderer;
import net.sf.dc.console.components.renderers.ComboBoxRenderer;
import net.sf.dc.console.components.tables.DcTable;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.modules.DcStateModule;
import net.sf.dc.core.modules.DcStorageMediaModule;
import net.sf.dc.core.objects.DcLookAndFeel;
import net.sf.dc.core.objects.Picture;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.ext.jflexsett.settings.ui.ISettingsComponent;

import com.l2fprod.gui.plaf.skin.Skin;
import com.l2fprod.gui.plaf.skin.SkinLookAndFeel;

public final class ComponentFactory {

    public static final int _SOFTWARECATEGORYCOMBO = 0;
    public static final int _YESNOCOMBO = 1;
    public static final int _MP3GENRECOMBO = 2;
    public static final int _LONGTEXTFIELD = 3;
    public static final int _CHECKBOX = 6;
    public static final int _NUMBERFIELD = 7;
    public static final int _SHORTTEXTFIELD = 8;
    public static final int _URLFIELD = 9;
    public static final int _PICTUREFIELD = 10;
    public static final int _FONTSELECTOR = 11;
    public static final int _TIMEFIELD = 12;
    public static final int _THEMEFIELD = 13;
    public static final int _RATINGCOMBOBOX = 14;
    public static final int _FILEFIELD = 15;
    public static final int _PASSWORDFIELD = 16;
    public static final int _BOOKCATEGORYCOMBO = 17;
    public static final int _PROGRAMDEFINITIONFIELD = 19;
    public static final int _AVAILABILITYCOMBO = 20;
    public static final int _CONTACTPERSONSCOMBO = 21;
    public static final int _CONTACTPERSONCATEGORYCOMBO = 22;
    public static final int _STATECOMBO = 23;
    public static final int _LOOKANDFEELSELECTOR = 24;
    public static final int _MODULESELECTOR = 25;
    public static final int _FILELAUNCHFIELD = 26;
    public static final int _PLATFORMCOMBO = 27;
    public static final int _MEDIACOMBO = 28;
    public static final int _COLORSELECTOR = 29;
    public static final int _IMAGECATEGORYCOMBO = 30;
    public static final int _DATEFIELD = 31;
    public static final int _REFERENCESFIELD = 32;

    private static Font fontUnreadable = new Font("Dialog", Font.PLAIN, 1);
    private static final Color colorDisabled = new Color(240,240,240);
    private static final Color colorRequired = new Color(120, 0, 0);

    private static javax.swing.plaf.ButtonUI checkBoxUI;
    private static LookAndFeel defaultLaf;


    public static void setLookAndFeel() {
        try {
            
            if (defaultLaf == null)
                defaultLaf = UIManager.getLookAndFeel();
            
            DcLookAndFeel laf = DcSettings.getValueAsLookAndFeel(Repository.settings.stLookAndFeel);
            // fix for the check box UI of the look and feel
            if (laf.getType() == DcLookAndFeel._SKINLF) {
                Skin theSkinToUse = SkinLookAndFeel.loadThemePack(DataCrow.baseDir + "/themes/" +  laf.getFileName());
                SkinLookAndFeel.setSkin(theSkinToUse);
                UIManager.setLookAndFeel(new SkinLookAndFeel());
            } else if (laf.getType() == DcLookAndFeel._SYSTEMLF) {
                UIManager.setLookAndFeel(laf.getClassName());
            } else {
                UIManager.setLookAndFeel(defaultLaf);
            }
            
            UIManager.getLookAndFeelDefaults().put("SplitPane.border", null);
            UIManager.put("TabbedPane.lightHighlight", UIManager.get("TabbedPane.background") );
            UIManager.put("TabbedPane.darkShadow", UIManager.get("TabbedPane.background") );
            UIManager.put("TabbedPane.shadow", UIManager.get("TabbedPane.background") );
            
            UIManager.put("Tree.leafIcon", IconLibrary._icoTreeLeaf);
            UIManager.put("Tree.openIcon", IconLibrary._icoTreeOpen);
            UIManager.put("Tree.closedIcon", IconLibrary._icoTreeClosed);
            
        } catch (Exception e) {
            Messages.add(e);
        }        
    }

    public static final JComponent getSettingsComponent(int index,
                                                        String label,
                                                        int maxTextLength) {
        return getComponent(DcModules._GENERIC, index, label, maxTextLength);
    }
    
    public static final JComponent getComponent(int module, int index, String label, int maxTextLength) {
        switch (index) {
            case _SOFTWARECATEGORYCOMBO:
                return getSoftwareCatComboBox();
            case _YESNOCOMBO:
                return getYesNoCombo();
            case _MP3GENRECOMBO:
                return getMP3GenreComboBox();
            case _LONGTEXTFIELD:
                return getLongTextField(maxTextLength);
            case _CHECKBOX:
                return getCheckBox(label);
            case _NUMBERFIELD:
                return getNumberField();
            case _URLFIELD:
                return getURLField(maxTextLength);
            case _PICTUREFIELD:
                return getPictureField(true, true, false);
            case _FONTSELECTOR:
                return getFontSelector(label);
            case _TIMEFIELD:
                return getTimeField();
            case _RATINGCOMBOBOX:
                return getRatingComboBox();
            case _FILEFIELD:
                return getFileField(false, false);
            case _PASSWORDFIELD:
                return getPasswordField();
            case _BOOKCATEGORYCOMBO:
                return getBookCatComboBox();
            case _PROGRAMDEFINITIONFIELD:
                return getProgramDefinitionField();
            case _AVAILABILITYCOMBO:
                return getAvailabilityCombo();
            case _CONTACTPERSONSCOMBO:
                return getContactPersonCombo();
            case _CONTACTPERSONCATEGORYCOMBO:
                return getContactPersonCategoryCombo();
            case _STATECOMBO:
                return getStateCombo(module);
            case _LOOKANDFEELSELECTOR:
                return getLookAndFeelSelector();
            case _MODULESELECTOR:
                return getModuleSelector();
            case _FILELAUNCHFIELD:
                return getFileLaunchField();
            case _PLATFORMCOMBO:
                return getPlatformCombo();
            case _MEDIACOMBO:
                return getMediaCombo(module);
            case _IMAGECATEGORYCOMBO:
                return getImageCatComboBox();
            case _REFERENCESFIELD:
                return getReferencesField(module);
            case _DATEFIELD:
                return getDateField();                
            default:
                return getShortTextField(maxTextLength);
        }
    }

	public static final JComponent setUneditable(JComponent component) {
		if (component instanceof JComboBox)
			 ((JComboBox)component).setEnabled(false);
		else if (component instanceof JTextField)
			((JTextField) component).setEditable(false);

        component.setBackground(ComponentFactory.getDisabledColor());
		return component;
	}

	public static final DcProgramDefinitionsField getProgramDefinitionField() {
		return new DcProgramDefinitionsField();
	}

    public static final DcFileLauncherField getFileLaunchField() {
        return new DcFileLauncherField();
    }
    
    public static final DcModuleSelector getModuleSelector() {
        return new DcModuleSelector();
    }

    public static final DcReferencesField getReferencesField(int module) {
        return new DcReferencesField(module);
    }
    
    public static final DcPasswordField getPasswordField() {
    	return new DcPasswordField();
    }

    public static final DcFontSelector getFontSelector(String title) {
        return new DcFontSelector(title);
    }

    public static final DcLookAndFeelSelector getLookAndFeelSelector() {
        return new DcLookAndFeelSelector();
    }    
    
    public static final DcTimeField getTimeField() {
    	return new DcTimeField();
    }
    
    public static final DcComboBox getMediaCombo(int module) {
        DcModule m = DcModules.get(module);
        int modIx = !(m instanceof DcStorageMediaModule) ? m.getStorageMediaModule().getIndex() : m.getIndex();

        DcComboBox combo = getComboBox();
        DataManager.registerUiComponent(combo, modIx);
        return combo;
    }    
    
    public static final JComboBox getStateCombo(int module) {
        DcModule m = DcModules.get(module);
        int modIx = !(m instanceof DcStateModule) ? m.getStateModule().getIndex() : m.getIndex();

        DcComboBox combo = getComboBox();
        DataManager.registerUiComponent(combo, modIx);
        return combo;
    }    

    public static final DcComboBox getContactPersonCombo() {
        DcComboBox comboBox = getComboBox();
        DataManager.registerUiComponent(comboBox, DcModules._CONTACTPERSON);
        return comboBox;
    }

    public static final DcComboBox getContactPersonCategoryCombo() {
        DcComboBox comboBox = getComboBox();
        DataManager.registerUiComponent(comboBox, DcModules._CONTACTPERSONCATEGORY);
        return comboBox;
    }

    public static final DcComboBox getAvailabilityCombo() {
        DcComboBox comboBox = new DcComboBox();
        comboBox.setFont(getStandardFont());
        comboBox.setBorder(getBorder());
        comboBox.setRenderer(new AvailabilityComboBoxRenderer());

        comboBox.addItem(null);
        comboBox.addItem(Boolean.TRUE);
        comboBox.addItem(Boolean.FALSE);

        return comboBox;
    }
    
    public static final DcComboBox getPlatformCombo() {
        DcComboBox combo = getComboBox();
        DataManager.registerUiComponent(combo, DcModules._SOFTWAREPLATFORM);
        return combo;
    }

    public static final DcRadioButton getRadioButton(String label, ImageIcon icon, String command) {
        DcRadioButton radioButton = new DcRadioButton(label, icon, false);
        radioButton.setActionCommand(command);
        return radioButton;
    }
    
    public static final DcRadioButton getRadioButton(String label, ImageIcon icon) {
        DcRadioButton radioButton = new DcRadioButton(label, icon, false);
        return radioButton;
    }

    public static final DcFileField getFileField(boolean save, boolean dirsOnly) {
        DcFileField ff = new DcFileField(null);
        ff.setModus(save, dirsOnly);
        return ff;
    }

    public static final DcFileField getFileField(boolean save, boolean dirsOnly, FileFilter filter) {
        DcFileField ff = new DcFileField(filter);
        ff.setModus(save, dirsOnly);
        return ff;
    }

    public static final DcDateField getDateField() {
        DcDateField dateField = new DcDateField();
        dateField.setFont(getStandardFont());
        dateField.setPreferredSize(new Dimension(dateField.getWidth(), 23));
        return dateField;
    }

    public static final DcPictureField getPictureField(boolean scaled,
                                                     boolean allowZoom,
                                                     boolean thumbnail) {
        DcPictureField pictureField = new DcPictureField(scaled, allowZoom, thumbnail) {
            public JToolTip createToolTip() {
                return new DcMultiLineToolTip();
            }
        };
        pictureField.setBorder(getBorder());
        return pictureField;
    }

    public static final DcPictureField getPictureField(boolean allowZoom) {
        DcPictureField pictureField = new DcPictureField(allowZoom) {
            public JToolTip createToolTip() {
                return new DcMultiLineToolTip();
            }
        };
        pictureField.setBorder(getBorder());
        return pictureField;
    }

    public static final DcUrlField getURLField(int maxLength) {
        DcUrlField urlField = new DcUrlField(maxLength);
        urlField.setBorder(getBorder());
        urlField.setFont(getStandardFont());
        return urlField;
    }

    public static final DcTree getTree(DefaultMutableTreeNode model) {
        DcTree tree = new DcTree(model);
        tree.setFont(getStandardFont());
        return tree;
    }

    public static final DcNumberField getNumberField() {
        DcNumberField numberField = new DcNumberField();
        numberField.setFont(getStandardFont());
        numberField.setBorder(getBorder());
        return numberField;
    }

    public static final DcRatingComboBox getRatingComboBox() {
    	DcRatingComboBox combo = new DcRatingComboBox();
        return combo;
    }

    public static final DcComboBox getMP3GenreComboBox() {
        DcComboBox genreComboBox = getComboBox();
        genreComboBox.addItem("");
        genreComboBox.setSelectedIndex(0);
        genreComboBox.setFont(getStandardFont());

        for (int i = 0; i < Repository.collections.colMusicGenres.length; i++) {
            genreComboBox.addItem(Repository.collections.colMusicGenres[i]);
        }

        return genreComboBox;
    }

	public static final DcComboBox getSoftwareCatComboBox() {
        DcComboBox comboBox = getComboBox();
        DataManager.registerUiComponent(comboBox, DcModules._SOFTWARECAT);
		return comboBox;
	}
    
    public static final DcComboBox getImageCatComboBox() {
        DcComboBox comboBox = getComboBox();
        DataManager.registerUiComponent(comboBox, DcModules._IMAGECAT);
        return comboBox;
    }    

    public static final DcComboBox getBookCatComboBox() {
    	DcComboBox comboBox = getComboBox();
        DataManager.registerUiComponent(comboBox, DcModules._BOOKCAT);
        return comboBox;
    }

    public static final DcComboBox getComboBox(Object[] items) {
        DcComboBox comboBox = new DcComboBox(items);
        comboBox.setFont(getStandardFont());
        comboBox.setBorder(getBorder());
        comboBox.setRenderer(new ComboBoxRenderer());
        return comboBox;
    }    
    
    public static final DcImageLabel getImageLabel(ImageIcon icon) {
        return new DcImageLabel(icon);
    }
    
    public static final DcComboBox getComboBox() {
        DcComboBox comboBox = new DcComboBox();
        comboBox.setFont(getStandardFont());
        comboBox.setBorder(getBorder());
        comboBox.setRenderer(new ComboBoxRenderer());
        return comboBox;
    }

    public static final DcCheckBox getCheckBox(String labelText) {
        DcCheckBox checkBox = new DcCheckBox(labelText);
        
        if (checkBoxUI != null)
            checkBox.setUI(checkBoxUI);
        
        checkBox.setFont(getSystemFont());
        return checkBox;
    }

    public static final DcMenuItem getMenuItem(String text) {
        DcMenuItem menuItem = new DcMenuItem(text);
        menuItem.setFont(getSystemFont());
        menuItem.setLayout(new FlowLayout(FlowLayout.LEFT));
        return menuItem;
    }
    
    public static final DcMenuItem getMenuItem(AbstractAction action) {
        DcMenuItem menuItem = new DcMenuItem(action);
        menuItem.setFont(getSystemFont());
        menuItem.setLayout(new FlowLayout(FlowLayout.LEFT));
        return menuItem;
    }    

    public static final DcMenuItem getMenuItem(ImageIcon icon, String text) {
        DcMenuItem menuItem = getMenuItem(text);
        menuItem.setIcon(icon);
        return menuItem;
    }

    public static final DcMenu getMenu(String text) {
        DcMenu menu = new DcMenu(text);
        menu.setFont(getSystemFont());
        return menu;
    }

    public static final DcMenu getMenu(ImageIcon icon, String text) {
        DcMenu menu = new DcMenu(text);
        menu.setFont(getSystemFont());
        menu.setIcon(icon);
        return menu;
    }

    
    public static final DcButton getTableHeader(String title) {
        DcButton button = getButton(title);
        button.setFont(ComponentFactory.getStandardFont());
        button.setPreferredSize(new Dimension(button.getWidth(), 20));
        button.setBorder(new EtchedBorder(5));
        return button;
    }

    public static final DcButton getButton(String buttonText, int length) {
        DcButton button = getButton(buttonText);
        button.setPreferredSize(new Dimension(length, 23));
        return button;
    }

    public static final DcButton getButton(ImageIcon icon) {
        DcButton button = getButton("");
        button.setIcon(icon);
        button.setFont(getSystemFont());
        return button;
    }

    public static final DcButton getButton(String buttonText) {
        DcButton button = new DcButton() {
            public JToolTip createToolTip() {
                return new DcMultiLineToolTip();
            }
        };
        
        ToolTipManager.sharedInstance().registerComponent(button);
        button.setName("bt" + buttonText);
        button.setText(buttonText);
        
        int height = 25;
        
        button.setPreferredSize(new Dimension(110, height));
        button.setMaximumSize(new Dimension(110, height));
        button.setMinimumSize(new Dimension(110, height));
        button.setFont(getSystemFont());
        return button;
    }

    public static final DcLongTextField getTextArea() {
        DcLongTextField textArea = new DcLongTextField(Integer.MAX_VALUE);
        textArea.setWrapStyleWord(true);
        textArea.setLineWrap(true);
        textArea.setEditable(true);
        textArea.setMargin(new Insets(5,5,5,5));
        textArea.setFont(getStandardFont());
        return textArea;
    }

    public static final DcTextPane getTextPane() {
        DcTextPane textpane = new DcTextPane();
        textpane.setFont(getStandardFont());
        return textpane;
    }

    public static final DcLongTextField getLongTextField(int maxTextLength) {
        DcLongTextField longText = new DcLongTextField(maxTextLength);
        longText.setWrapStyleWord(true);
        longText.setLineWrap(true);
        longText.setEditable(true);
        longText.setMargin(new Insets(5,5,5,5));
        longText.setFont(getStandardFont());
        return longText;
    }

    public static final DcLabel getLabel(ImageIcon icon) {
        DcLabel label = new DcLabel(icon);
        return label;
    }

    public static final JLabel getLabel(String labelText, ImageIcon icon) {
        JLabel label = new DcLabel(labelText);
        label.setIcon(icon);
        return label;
    }

    public static final  JLabel getLabel(String labelText, int length) {
        JLabel label = getLabel(labelText);
        label.setPreferredSize(new Dimension(length, 23));
        label.setText(labelText);
        return label;
    }

    public static final DcLabel getLabel(String labelText) {
        DcLabel label = new DcLabel() {
            public JToolTip createToolTip() {
                return new DcMultiLineToolTip();
            }
        };

        label.setText(labelText);
        label.setRequestFocusEnabled(false);
        label.setFont(getSystemFont());
        label.setToolTipText(labelText);

        return label;
    }

    public static final DcShortTextField getShortTextField(int maxTextLength) {
        DcShortTextField textField = new DcShortTextField(maxTextLength);
        textField.setFont(getStandardFont());
        textField.setPreferredSize(new Dimension(50, 21));
        textField.setBorder(getBorder());
        return textField;
    }

    public static final DcShortTextField getTextFieldDisabled() {
        DcShortTextField textField = new DcShortTextField(4000);
        textField.setPreferredSize(new Dimension(50, 21));
        textField.setBorder(getBorder());
        textField.setEnabled(false);
        textField.setEditable(false);
        textField.setFont(ComponentFactory.getSystemFont());
        textField.setForeground(ComponentFactory.getDisabledColor());

        return textField;
    }
    
    public static final DcNumberField getIdFieldDisabled() {
        DcNumberField textField = new DcNumberField();
        textField.setPreferredSize(new Dimension(50, 21));
        textField.setBorder(getBorder());
        textField.setEnabled(false);
        textField.setEditable(false);
        textField.setFont(ComponentFactory.getSystemFont());
        textField.setForeground(ComponentFactory.getDisabledColor());
        return textField;
    }    

    public static final DcTabbedPane getTabbedPane() {
        DcTabbedPane tabbedPane = new DcTabbedPane();
        tabbedPane.setFont(getSystemFont());
        return tabbedPane;
    }

    public static final DcTable getDCTable(boolean readonly, boolean caching) {
        DcTable table = new DcTable(readonly, caching) {
            public JToolTip createToolTip() {
                return new DcMultiLineToolTip();
            }
        };
        return table;
    }

    public static final DcTable getDCTable(DcModule module, boolean readonly, boolean caching) {
    	DcTable table = new DcTable(module, readonly, caching) {
            public JToolTip createToolTip() {
                return new DcMultiLineToolTip();
            }
        };
        return table;
    }

    public static final Icon getIcon(String iconLocation) {
        Icon icon = new ImageIcon(iconLocation);
        return icon;
    }

    public static final JMenuBar getMenuBar() {
        DcMenuBar menuBar = new DcMenuBar() {
            public JToolTip createToolTip() {
                return new DcMultiLineToolTip();
            }
        };
        menuBar.setFont(getStandardFont());
        return menuBar;
    }

    public static final DcComboBox getYesNoCombo() {
        DcComboBox comboBox = getComboBox();
        comboBox.addItem("");
        comboBox.addItem(DcResources.getText("lblYes"));
        comboBox.addItem(DcResources.getText("lblNo"));
        comboBox.setFont(getStandardFont());

        return comboBox;
    }

    public static final Border getBorder() {
        return BorderFactory.createLineBorder(Color.GRAY);
    }

    public static final TitledBorder getTitleBorder(String title) {
        TitledBorder border = new TitledBorder(BorderFactory.createLineBorder(Color.LIGHT_GRAY, 1), title);
        border.setTitleFont(getSystemFont());
        return border;
    }

    public static final Color getCurrentForegroundColor() {
        Color color = UIManager.getLookAndFeelDefaults().getColor("TextField.foreground");
        return color == null ? Color.BLACK : color;
    }
    
    public static final Font getStandardFont() {
        return DcSettings.getValueAsFont(Repository.settings.stSystemFontNormal);
    }

    public static final Font getSystemFont() {
        return DcSettings.getValueAsFont(Repository.settings.stSystemFontBold);
    }

    public static final Font getUnreadableFont() {
        return fontUnreadable;
    }

    public static final Color getDisabledColor() {
       return colorDisabled;
    }

    public static final Color getRequiredColor() {
        return colorRequired;
    }

    public static final Color getTableHeaderColor() {
        return new Color(220, 220, 220);
    }

    public static final void setComponentValue(int fieldType, JComponent comp, Object value) {
        switch (fieldType) {
            case (ComponentFactory._SOFTWARECATEGORYCOMBO):
            case (ComponentFactory._BOOKCATEGORYCOMBO):
            case (ComponentFactory._YESNOCOMBO):
            case (ComponentFactory._MP3GENRECOMBO):
            case (ComponentFactory._RATINGCOMBOBOX):
            case (ComponentFactory._AVAILABILITYCOMBO):
            case (ComponentFactory._CONTACTPERSONCATEGORYCOMBO):
            case (ComponentFactory._STATECOMBO):
            case (ComponentFactory._PLATFORMCOMBO):
            case (ComponentFactory._MEDIACOMBO):
            case (ComponentFactory._CONTACTPERSONSCOMBO):
            case (ComponentFactory._IMAGECATEGORYCOMBO):
                ((JComboBox) comp).setSelectedItem(value);
                break;
            case (ComponentFactory._CHECKBOX):
                if (value instanceof String)
                    ((DcCheckBox) comp).setSelected(new Boolean((String) value).booleanValue());    
                else
                    ((DcCheckBox) comp).setSelected(((Boolean) value).booleanValue());
                
                break;
            case (ComponentFactory._DATEFIELD):
                ((DcDateField) comp).setValue(value);
                break;
            case (ComponentFactory._REFERENCESFIELD):
                ((DcReferencesField) comp).setValue(value);
                break;
            case (ComponentFactory._NUMBERFIELD):
                ((DcNumberField) comp).setValue(value);
                break;
            case (ComponentFactory._LONGTEXTFIELD):
                ((DcLongTextField) comp).setValue(value);
                break;
            case (ComponentFactory._PICTUREFIELD):
                if (value instanceof Picture) {
                    ((DcPictureField) comp).setValue(((Picture) value).getValue(Picture._D_IMAGE));
                } else {
                    ((DcPictureField) comp).setValue(value);
                }
                break;
            case (ComponentFactory._TIMEFIELD):
                ((DcTimeField) comp).setValue(value);
                break;
            case (ComponentFactory._PROGRAMDEFINITIONFIELD):
                ((ISettingsComponent) comp).setValue(value);
                break;
            case (ComponentFactory._FILELAUNCHFIELD):
                ((DcFileLauncherField) comp).setValue(value);
                break;                
            default:
                ((JTextField) comp).setText(value.toString());
        }
    }

    public static Object getComponentValue(int fieldType, JComponent comp) {
        switch (fieldType) {
            case (ComponentFactory._SOFTWARECATEGORYCOMBO):
            case (ComponentFactory._BOOKCATEGORYCOMBO):
            case (ComponentFactory._YESNOCOMBO):
            case (ComponentFactory._AVAILABILITYCOMBO):
            case (ComponentFactory._CONTACTPERSONSCOMBO):
            case (ComponentFactory._CONTACTPERSONCATEGORYCOMBO):
            case (ComponentFactory._STATECOMBO):
            case (ComponentFactory._PLATFORMCOMBO):
            case (ComponentFactory._MEDIACOMBO):
            case (ComponentFactory._MP3GENRECOMBO):
            case (ComponentFactory._IMAGECATEGORYCOMBO):
                Object o = ((JComboBox) comp).getSelectedItem();
                return  o;
            case (ComponentFactory._RATINGCOMBOBOX):
                return ((DcRatingComboBox) comp).getValue();
            case (ComponentFactory._NUMBERFIELD):
                return ((DcNumberField) comp).getValue();
            case (ComponentFactory._LONGTEXTFIELD):
                return ((DcLongTextField) comp).getValue();
            case (ComponentFactory._PICTUREFIELD):
                return ((DcPictureField) comp).getValue();
            case (ComponentFactory._CHECKBOX):
                return new Boolean(((DcCheckBox) comp).isSelected());
            case (ComponentFactory._TIMEFIELD):
                return ((DcTimeField) comp).getValue();
            case (ComponentFactory._PROGRAMDEFINITIONFIELD):
                return ((ISettingsComponent) comp).getValue();
            case (ComponentFactory._DATEFIELD):
                return ((ISettingsComponent) comp).getValue();
            case (ComponentFactory._REFERENCESFIELD):
                return ((DcReferencesField) comp).getValue();
            case (ComponentFactory._FILELAUNCHFIELD):
                return ((DcFileLauncherField) comp).getFileAsString();
            default:
                return ((JTextField) comp).getText();
        }
    }
}
