/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.audiocd;

import java.awt.Font;
import java.util.Iterator;

import net.sf.dc.console.components.panels.QuickViewPanel;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.audiocd.AudioCD;
import net.sf.dc.core.objects.audiocd.AudioTrack;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;

public class AudioCDQuickViewPane extends QuickViewPanel {
    
    public AudioCDQuickViewPane() {
        super(DcModules.get(DcModules._MOVIE), true);
    }
    
    /**
     * Create the quickview pane for all fields 
     */
    protected String getDescriptionTable() {
        String table = "<table " + getBodyStyle() + ">\n";
        table = addTableRow(table, AudioCD._F_ARTIST, _DIRECTION_VERTICAL);
        table = addTableRow(table, AudioCD._A_TITLE, _DIRECTION_VERTICAL);
        table = addTableRow(table, AudioCD._B_DESCRIPTION, _DIRECTION_VERTICAL);
        table = addTableRow(table, AudioCD._C_YEAR, _DIRECTION_HORIZONTAL);
        table = addTableRow(table, AudioCD._G_GENRES, _DIRECTION_HORIZONTAL);
        table = addTableRow(table, AudioCD._D_LOCATION, _DIRECTION_HORIZONTAL);
        table = addTableRow(table, AudioCD._H_PICTUREFRONT, _DIRECTION_HORIZONTAL);
        table = addTableRow(table, AudioCD._I_PICTUREBACK, _DIRECTION_HORIZONTAL);
        table = addTableRow(table, AudioCD._J_PICTURECD, _DIRECTION_HORIZONTAL);
        table = addTableRow(table, AudioCD._E_RATING, _DIRECTION_HORIZONTAL);

        table = addTracks(table);
        table += "</table>";
        return table;
    }
    
    
    protected String addTracks(String table) {
        
        Font font = DcSettings.getValueAsFont(Repository.settings.stSystemFontNormal);

        table += "<tr><td>\n";
        table += "<b>" + DcResources.getText("lblTracks") + "</b>";
        
        table += "</td></tr>";
        table += "<tr><td>\n";
        
        int counter = 0;
        String description = "";
        
        dco.removeChildren();
        dco.loadChildren();
        for (Iterator iter = dco.getChildren().iterator(); iter.hasNext(); ) {
        	AudioTrack audioTrack = (AudioTrack) iter.next();
            description = description.trim();
        	
            if (counter > 0) description += ", ";
            
            String title = (String) audioTrack.getValue(AudioTrack._A_TITLE);
            title = title == null ? "" : title; 
            
            Integer i = (Integer) audioTrack.getValue(AudioTrack._H_PLAYLENGTH);
            String length = i == null || i.equals(new Integer(0)) ? "" : " " + audioTrack.getDisplayString(AudioTrack._H_PLAYLENGTH);
            
            description += "<b>" + audioTrack.getValue(AudioTrack._F_TRACKNUMBER) + "</b> ";
            description += title;
            description += length;
            counter++;
        }
        
        
        if (font.getStyle() == Font.BOLD) 
        	description = "<b>" + description + "</b>";

        table += description;
        table += "</td></tr>";
        table += "<tr><td></td></tr>";
        return table;
    }    
}
