/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components;

import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JPanel;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.ext.jflexsett.settings.ui.ISettingsComponent;

public class DcFieldSelectorField extends JComponent implements ISettingsComponent {

    private final Map componentMap = new HashMap();
    private final int module;

    public DcFieldSelectorField(int module) {
        this.module = module;
        buildComponent();
    }

    public Object getValue() {
        return getSelectedFields();
    }

    public void setValue(Object o) {
        if (o instanceof int[]) {
            setSelectedFields((int[]) o);
        }
    }

    public void invertSelection() {
        for (Iterator iter = componentMap.keySet().iterator(); iter.hasNext();) {
            DcField field = (DcField) iter.next();
            JCheckBox checkBox = (JCheckBox) componentMap.get(field);
            checkBox.setSelected(!checkBox.isSelected());
        }
    }
    
    public void selectAll() {
        for (Iterator iter = componentMap.keySet().iterator(); iter.hasNext();) {
            DcField field = (DcField) iter.next();
            JCheckBox checkBox = (JCheckBox) componentMap.get(field);
            checkBox.setSelected(true);
        }
    }

    public void unselectAll() {
        for (Iterator iter = componentMap.keySet().iterator(); iter.hasNext();) {
            DcField field = (DcField) iter.next();
            JCheckBox checkBox = (JCheckBox) componentMap.get(field);
            checkBox.setSelected(false);
        }
    }

    public Object getSelectedFieldIndices() {
        Collection fields = getSelectedFields();
        int[] indices = new int[fields.size()];
        int counter = 0;
        for (Iterator iter = fields.iterator(); iter.hasNext(); counter++) {
            indices[counter] = ((DcField) iter.next()).getIndex();
        }
        
        return indices;
    }    

    public Collection getSelectedFields() {
        Collection fields = new ArrayList();
        for (Iterator iter = componentMap.keySet().iterator(); iter.hasNext();) {
            DcField field = (DcField) iter.next();
            JCheckBox checkBox = (JCheckBox) componentMap.get(field);

            if (checkBox.isSelected())
                fields.add(field);
        }

        return fields;
    }

    public void setSelectedFields(int[] fields) {
        for (Iterator iter = componentMap.keySet().iterator(); iter.hasNext(); ) {
            DcField field = (DcField) iter.next();
            JCheckBox checkBox = (JCheckBox) componentMap.get(field);

            if (fields.length == 0) {
                checkBox.setSelected(!field.isHidden());
            } else {
                for (int j = 0; j < fields.length; j++) {
                    if (field.getIndex() == fields[j])
                        checkBox.setSelected(true);
                }
            }
        }
    }

    private void buildComponent() {
        setLayout(Layout.getGBL());

        JPanel panel = new JPanel();
        panel.setLayout(Layout.getGBL());

        DcObject dco = DcModules.get(module).getDcObject();

        int x = 0;
        int y = 0;

        Collection fields = dco.getFields();
        for (Iterator iter = fields.iterator(); iter.hasNext(); ) {
            DcField field = (DcField) iter.next();

            JCheckBox checkBox = ComponentFactory.getCheckBox(field.getLabel());
            componentMap.put(field, checkBox);

            panel.add(checkBox, Layout.getGBC(x++, y, 1, 1, 1.0, 1.0
                     ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                      new Insets( 0, 10, 0, 0), 0, 0));

            if (x == 4) {
                x = 0;
                y++;
            }
        }

        add(panel, Layout.getGBC(0, 0, 1, 1, 1.0, 1.0
                  ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                   new Insets( 0, 0, 0, 0), 0, 0));

    }
}
