package net.sf.dc.console.components;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.JButton;
import javax.swing.JComponent;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.dialogs.BrowserDialog;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.resources.DcResources;
import net.sf.ext.jflexsett.settings.ui.ISettingsComponent;
import net.sf.ext.util.FileLauncher;

public class DcFileLauncherField extends JComponent implements ISettingsComponent {

    private final DcShortTextField text;

    private File file = null;

    public DcFileLauncherField() {
        text = ComponentFactory.getTextFieldDisabled();
        
        setBounds(0,0,0,0);

        JButton buttonBrowse = ComponentFactory.getButton(IconLibrary._icoOpen);
        buttonBrowse.addActionListener(new ShowSelectDialogAction());
        JButton buttonLaunch = ComponentFactory.getButton(IconLibrary._icoOpenApplication);
        buttonLaunch.addActionListener(new LaunchFileAction());
        
        buttonBrowse.setPreferredSize(new Dimension(25, 22));
        buttonLaunch.setPreferredSize(new Dimension(25, 22));
        buttonBrowse.setMaximumSize(new Dimension(25, 22));
        buttonLaunch.setMaximumSize(new Dimension(25, 22));
        buttonBrowse.setMinimumSize(new Dimension(25, 22));
        buttonLaunch.setMinimumSize(new Dimension(25, 22));
        
        
        setLayout(Layout.getGBL());
        add( text,         Layout.getGBC( 0, 0, 1, 1, 80.0, 80.0
                ,GridBagConstraints.WEST, GridBagConstraints.BOTH,
                 new Insets(0, 0, 0, 0), 0, 0));
        add( buttonBrowse, Layout.getGBC( 1, 0, 1, 1, 1.0, 1.0
                ,GridBagConstraints.EAST, GridBagConstraints.NONE,
                 new Insets(0, 0, 0, 0), 0, 0));
        add( buttonLaunch, Layout.getGBC( 2, 0, 1, 1, 1.0, 1.0
                ,GridBagConstraints.EAST, GridBagConstraints.NONE,
                 new Insets(0, 0, 0, 0), 0, 0));    
    }

    public File getFile() {
        return file;
    }

    public String getFileAsString() {
        return file == null ? "" : file.toString();
    }

    public void setFile(File file) {
        this.file = file;
        text.setText(file == null ? "" : file.toString());
    }
    
    public void setValue(Object value) {
        if (value != null && value.toString().length() > 0) {
            File file = new File(value.toString());
            setFile(file);
        } else {
            setFile(null);
        }
    }

    public Object getValue() {
        return file == null ? "" : file.toString();
    }    

    private void showFileOpenDialog() {
        BrowserDialog dialog = new BrowserDialog(DcResources.getText("lblFileBrowser"), null);
        file = dialog.showOpenFileDialog(this, null);
        text.setText(getFileAsString());

        dialog.dispose();
        dialog = null;
    }

    private class LaunchFileAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            new FileLauncher(text.getText()).launchFile();
        }
    }
    
    private class ShowSelectDialogAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            showFileOpenDialog();
        }
    }
}
