/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components;

import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeListener;

import javax.swing.Action;
import javax.swing.ActionMap;
import javax.swing.ComponentInputMap;
import javax.swing.ImageIcon;
import javax.swing.InputMap;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JRootPane;
import javax.swing.KeyStroke;

import net.sf.dc.core.IconLibrary;
import net.sf.dc.helpsystem.JavaHelp;
import net.sf.ext.util.SwingUtil;
import net.sf.ext.util.Utilities;

public class DcFrame extends JFrame {

    private String helpIndex = null;

    public DcFrame(String title, ImageIcon icon, JFrame parent) {
        super(title);
        setIconImage(icon.getImage());
        
        SwingUtil.setRootFrame(this);
        setIconImage(IconLibrary._icoMain.getImage());

        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
            }
        });

        setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        
        addKeyListener(KeyStroke.getKeyStroke("F1"), new HelpAction(), "HelpAction");
    }

    protected void addKeyListener(KeyStroke keyStroke, Action action, String name) {
        rootPane.getInputMap().put(keyStroke, name);
        rootPane.getActionMap().put(name, action);

        JRootPane rootPane = getRootPane();
        ActionMap parentActionMap = rootPane.getActionMap();
        ActionMap am = new ActionMap();
        am.setParent(parentActionMap);
        am.put(name, action);
        rootPane.setActionMap(am);

        InputMap parentInputMap = rootPane.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
        ComponentInputMap im = new ComponentInputMap(getRootPane());
        try {
            im.setParent(parentInputMap);
        } catch (Exception ignore) {}
        im.put(keyStroke, name);
        rootPane.setInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT, im);
    }

    public void setVisible(boolean visible) {
        super.setVisible(visible);
        if (visible) SwingUtil.setRootFrame(this);
    }

    public void setHelpIndex(String helpID) {
        helpIndex =  helpID;
    }

    protected void setCenteredLocation() {
        setLocation(Utilities.getCenteredWindowLocation(getSize()));
    }

    protected void showHelp() {
        if (helpIndex != null) 
            JavaHelp.setHelpIndex(helpIndex);
        new JavaHelp(this);
    }

    private class HelpAction implements Action {
        public boolean isEnabled() {
            return true;
        }

        public void setEnabled(boolean arg0) {}
        public void actionPerformed(ActionEvent arg0) {
            showHelp();
        }

        public void putValue(String arg0, Object arg1) {}
        public void addPropertyChangeListener(PropertyChangeListener arg0) {}

        public Object getValue(String arg0) {
            return null;
        }

        public void removePropertyChangeListener(PropertyChangeListener arg0) {}
    }
}
