/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.Toolkit;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JTextArea;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.PlainDocument;

import net.sf.dc.console.dialogs.TextDialog;
import net.sf.dc.core.Repository;
import net.sf.dc.settings.DcSettings;

public class DcLongTextField extends JTextArea {

    private int MAX_TEXT_LENGTH = 0;
    private Toolkit toolkit;
    private final DcUndoListenerer undoListener;
    
    public DcLongTextField(int maxTextLength) {
        super();

        addMouseListener(new TextWindowAction());

        MAX_TEXT_LENGTH = maxTextLength;
        toolkit = Toolkit.getDefaultToolkit();
        
        undoListener = new DcUndoListenerer(this);
    }
    
    public DcUndoListenerer getUndoListener() {
        return undoListener;
    }
    
    protected Document createDefaultModel() {
        return new LongStringDocument();
    }

    public Object getValue() {
        return getText();
    }

    public void setValue(Object o) {
        setText((String) o);
        setCaretPosition(0);
    }

    protected class LongStringDocument extends PlainDocument {
        public void insertString(int i, String s, AttributeSet attributeset)
            throws BadLocationException {

            if (i + 1 > MAX_TEXT_LENGTH) {
                toolkit.beep();
            } else {
                super.insertString(i, s, attributeset);
            }
        }
    }

    public void openTextWindow() {
        TextDialog textView = new TextDialog(getValue().toString(), isEditable() && isEnabled());
        if (textView.isSuccess())
            setText(textView.getText());
    }

    public class TextWindowAction implements MouseListener {
        public void mouseReleased(MouseEvent e) {
            if (e.getClickCount() == 2) {
                openTextWindow();
            }
        }
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {}
    }
    
    protected void paintComponent(Graphics g) {
        Graphics2D g2d = (Graphics2D) g;
        
        if (DcSettings.getValueAsBoolean(Repository.settings.stFontAntiAliasing)) {
           g2d.setRenderingHint( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );
           g2d.setRenderingHint( RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON );
           g2d.setRenderingHint( RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY );
        }
        super.paintComponent( g2d );
    }  
}
