package net.sf.dc.console.components;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.JButton;
import javax.swing.JComponent;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.dialogs.DcReferencesDialog;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DcMappingModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcMapping;
import net.sf.dc.settings.DcSettings;
import net.sf.ext.jflexsett.settings.ui.ISettingsComponent;

public class DcReferencesField extends JComponent implements ISettingsComponent {

    private DcShortTextField fld = ComponentFactory.getTextFieldDisabled();
    private JButton btn = ComponentFactory.getButton(IconLibrary._icoOpen);

    private Collection dcos = new ArrayList();
    
    private final int mappingModIdx;
    
    public DcReferencesField(int mappingModIdx) {
        super();
        
        this.mappingModIdx = mappingModIdx;
        
        setBounds(0,0,0,0);
        setLayout(Layout.getGBL());
        
        btn.setMinimumSize(new Dimension(25, 22));
        btn.setMaximumSize(new Dimension(25, 22));
        btn.setPreferredSize(new Dimension(25, 22));
        
        add(fld, Layout.getGBC( 0, 0, 1, 1, 80.0, 80.0
                ,GridBagConstraints.WEST, GridBagConstraints.BOTH,
                 new Insets( 0, 0, 0, 0), 0, 0));        
        add(btn, Layout.getGBC( 1, 0, 1, 1, 1.0, 1.0
                ,GridBagConstraints.EAST, GridBagConstraints.NONE,
                 new Insets( 0, 0, 0, 0), 0, 0));        
        
        btn.addActionListener(new OpenDialogAction());
    }

    public Object getValue() {
        return dcos;
    }
    
    public void setValue(Collection c) {
        dcos = new ArrayList();
        
        if (c != null)
            dcos.addAll(c);
        
        setDescription();
    }

    public void setValue(Object o) {
        setValue((Collection) o);
    }
    
    private void setDescription() {
        fld.setText("");
        if (dcos == null)
            return;

        String desc = "";
        boolean first = true;
        for (Iterator iter = dcos.iterator(); iter.hasNext(); ) {
            DcMapping dco = (DcMapping) iter.next();
            if (!first) desc += ", ";
            first = false;
            desc += dco;
        }
        fld.setText(desc);
    }
    
    protected void paintComponent(Graphics g) {
        Graphics2D g2d = (Graphics2D) g;
        
        if (DcSettings.getValueAsBoolean(Repository.settings.stFontAntiAliasing)) {
           g2d.setRenderingHint( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );
           g2d.setRenderingHint( RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON );
           g2d.setRenderingHint( RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY );
        }
        super.paintComponent(g2d);
    }  
    
    private class OpenDialogAction implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            DcMappingModule mappingModule = (DcMappingModule) DcModules.get(mappingModIdx);
            DcReferencesDialog dlg = new DcReferencesDialog(dcos, mappingModule);
            
            dlg.setVisible(true);
            
            if (dlg.isSaved()) {
                dcos.clear();
                dcos = new ArrayList();
                dcos.addAll(dlg.getDcObjects());
                setDescription();
            }
            dlg.dispose();
        }
    }
}