/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components.lists.elements;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.JLabel;
import javax.swing.JTextArea;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcMediaObject;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.audiocd.AudioCD;
import net.sf.dc.core.objects.audiocd.AudioTrack;

public class DcAudioCDListHwElement extends DcObjectListHwElement {

    private JTextArea tracksField;
    
    public DcAudioCDListHwElement(DcObject dco) {
        super(dco);
    }
    
    public Collection getPictures() {
        Collection pictures = new ArrayList();
        pictures.add(dco.getValue(AudioCD._H_PICTUREFRONT));
        pictures.add(dco.getValue(AudioCD._I_PICTUREBACK));
        return pictures;
    }

    public void setBackground(Color color) {
        super.setBackground(color);
        if (tracksField != null)
            tracksField.setBackground(new Color(color.getRed() - 20, color.getGreen() - 20, color.getBlue() - 20));
    }    
    
    public void build() {
        setLayout(Layout.getGBL());
        
        tracksField = getTracksField();
        JLabel titleLabel = getLabel(AudioCD._A_TITLE, true, label1Length);
        if (DcModules.getCurrentModule().getIndex() == DcModules._GENERIC)
            titleLabel.setIcon(dco.getModule().getIconSmall());

        addComponent(titleLabel, 0, 0);
        addComponent(getLabel(AudioCD._A_TITLE, false, field1Length), 1, 0);
        addComponent(getLabel(AudioCD._G_GENRES, true, label2Length), 2, 0);
        addComponent(getLabel(AudioCD._G_GENRES, false, field2Length), 3, 0);
        
        addComponent(getLabel(AudioCD._F_ARTIST, true, label1Length), 0, 1);
        addComponent(getLabel(AudioCD._F_ARTIST, false, field1Length), 1, 1);
        addComponent(getLabel(DcMediaObject._E_RATING, true, label2Length), 2, 1);
        addComponent(getRatingValueLabel(), 3, 1);        
        
        add(tracksField, Layout.getGBC( 0, 2, 4, 1, 10.0, 10.0,
            GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH, new Insets(0, 5, 5, 0), 0, 0));
        add(getPicturePanel(getPictures()), Layout.getGBC( 4, 0, 1, 3, 10.0, 10.0,
            GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH, new Insets(1, 1, 1, 1), 0, 0));        
    }   
    
    protected JTextArea getTracksField() {
        AudioTrack audioTrack = new AudioTrack();
        String description = ""; 
        int counter = 0;
        for (Iterator iter = dco.getChildren().iterator(); iter.hasNext(); ) {
            audioTrack = (AudioTrack) iter.next();
            
            if (counter > 0) description += " / ";
            
            String title = (String) audioTrack.getValue(AudioTrack._A_TITLE);
            title = title == null ? "" : title; 
            
            Integer i = (Integer) audioTrack.getValue(AudioTrack._H_PLAYLENGTH);
            String length = i == null || i.equals(new Integer(0)) ? "" : " " + audioTrack.getDisplayString(AudioTrack._H_PLAYLENGTH);
            
            description += audioTrack.getValue(AudioTrack._F_TRACKNUMBER) + " - ";
            description += title;
            description += length;
            counter++;
        }

        JTextArea textField = ComponentFactory.getTextArea();
        textField.setPreferredSize(new Dimension(600, 120));
        textField.setText(description);
        return textField;        
    }
}
