/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components.lists.elements;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Image;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.components.DcImageLabel;
import net.sf.dc.console.components.DcLabel;
import net.sf.dc.console.components.DcPictureField;
import net.sf.dc.console.components.DcTextPane;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.DcMediaObject;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.Picture;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.settings.definitions.DcFieldDefinitions;
import net.sf.ext.util.Utilities;

public abstract class DcObjectListElement extends JPanel {
    
    private static final Color bg = new Color(255, 255, 230);
    private static final Dimension dimTxt = new Dimension(145, 45);
    private static final Dimension dimPicLbl = new Dimension(146, 140);
    
    protected final int fieldHeight = 21;
    protected DcObject dco;
    
    protected DcObjectListElement() {}
    
    public DcObjectListElement(DcObject dco) {
        this.dco = dco;
        dco.markAsUnchanged();
        setBackground(Color.WHITE);
        build();
    }

    public DcObject getDcObject() {
        return dco;
    }
    
    public abstract Collection getPictures();
    
    protected DcFieldDefinitions getFieldDefinitions() {
        return dco.getModule().getFieldDefinitions();        
    }
    
    public void update() {
        removeAll();
        build();
        revalidate();
    }    

    public void update(DcObject o, boolean overwrite, boolean allowDeletes, boolean mark) {
        int[] indices = dco.getFieldIndices();
        
        for (int i = 0; i < indices.length; i++) {
            try {
                if (o.isChanged(indices[i])) {
                    Object newValue = o.getValue(indices[i]);
                    Object existingValue = dco.getValue(indices[i]);

                    if (newValue instanceof Picture)
                        newValue = ((Picture) newValue).isDeleted() ? null : newValue;
                    if (overwrite && ((newValue != null && !newValue.equals("")) || allowDeletes)) {
                        dco.setValue(indices[i], newValue);
                    } else if (existingValue == null || existingValue.equals("")) {
                        dco.setValue(indices[i], newValue);
                    }
                }
            } catch (Exception ignore) {}
        }        

        dco.removeChildren();
        dco.loadChildren();
        
        if (!mark)
            dco.markAsUnchanged();
        
        update();
        revalidate();
    }    

    public void build() {
        addPicture(getPictures());
        
        DcTextPane title = ComponentFactory.getTextPane();
        JScrollPane scroller = new JScrollPane(title);
        scroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_NEVER);
        scroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        title.setText((String) dco.getValue(DcMediaObject._A_TITLE));
        title.setBackground(bg);
        
        title.setPreferredSize(dimTxt);
        title.setMinimumSize(dimTxt);
        title.setMaximumSize(dimTxt);
        add(title);
    }
    
    private void addPicture(Collection pictures) {
        DcImageLabel lbl = null;
        for (Iterator iter = pictures.iterator(); iter.hasNext(); ) {
            Picture p = (Picture) iter.next();
            if (p != null) {
                p.loadImage();
                ImageIcon scaledImage = p.getScaledPicture();
                byte[] bytes = (byte[]) p.getValue(Picture._D_IMAGE);
                DcImageLabel label = new DcImageLabel();
                if (scaledImage != null) { 
                    label = ComponentFactory.getImageLabel(scaledImage);
                } else if (bytes != null && bytes.length > 10) {
                    Image img = Utilities.getScaledImage(bytes);
                    label = ComponentFactory.getImageLabel(new ImageIcon(img));
                }                
                
                lbl = label;
                break;
            }
        }
        
        if (lbl == null) lbl = ComponentFactory.getImageLabel(new ImageIcon(IconLibrary._icoEmptyPictureSmall));
        
        lbl.setPreferredSize(dimPicLbl);
        lbl.setMinimumSize(dimPicLbl);
        lbl.setMaximumSize(dimPicLbl);
        add(lbl);
    }
    
    public JPanel getPanel() {
        JPanel panel = new JPanel();
        panel.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 0));
        return panel;
    }
    
    protected DcLabel getLabel(int field, boolean label, int width) {
        DcLabel lbl = new DcLabel();
        if (label) {
            Font font = DcSettings.getValueAsFont(Repository.settings.stSystemFontBold);
            lbl.setFont(new Font(font.getName(), Font.BOLD, font.getSize()));
            lbl.setText(dco.getLabel(field));
        } else {
            lbl.setFont(DcSettings.getValueAsFont(Repository.settings.stSystemFontNormal));
            lbl.setText(dco.getDisplayString(field));
        }
        
        lbl.setPreferredSize(new Dimension(width, fieldHeight));
        
        return lbl;
    }       
    
    public void setBackground(Color color) {
        super.setBackground(color);
        for (int i = 0; i < getComponents().length; i++) {
            getComponents()[i].setBackground(color);
        }
    }    

    public void setFont(Font font) {
        Font fontNormal = DcSettings.getValueAsFont(Repository.settings.stSystemFontNormal);
        Font fontSystem = DcSettings.getValueAsFont(Repository.settings.stSystemFontNormal);        
        
        super.setFont(fontNormal);
        for (int i = 0; i < getComponents().length; i++) {
            Component component = getComponents()[i];
            if (component instanceof JPanel) {
                
                for (int j = 0; j < ((JPanel) component).getComponents().length; j++) {
                    Component subComponent = ((JPanel) component).getComponents()[j];
                    if (subComponent instanceof JLabel) 
                        subComponent.setFont(fontSystem);
                    else
                        subComponent.setFont(fontNormal);
                }
            }
        }
    }     
    
    public void clear() {
        int count = getComponentCount();
        for (int i = 0; i < count; i++) {
            Component c = getComponent(i);
            if (c instanceof DcImageLabel) {
                ((DcImageLabel) c).flush();
            } else if (c instanceof DcPictureField) {
                ((DcPictureField) c).flush();
            } else if (c instanceof JPanel) {
                int count2 = getComponentCount();
                for (int i2 = 0; i2 < count2; i2++) {
                    Component c2 = getComponent(i2);
                    if (c2 instanceof DcImageLabel)
                        ((DcImageLabel) c).flush();
                    else if (c2 instanceof DcPictureField)
                        ((DcPictureField) c).flush();
                }
            }
        }
        
        getGraphics().dispose();
        dco = null;
    }
}
