/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components.panels;

import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.DefaultCellEditor;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JViewport;
import javax.swing.table.TableColumn;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.renderers.CheckBoxTableCellRenderer;
import net.sf.dc.console.components.tables.DcTable;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcMediaObject;
import net.sf.dc.core.objects.enhancement.AutoIncrementer;
import net.sf.dc.core.objects.enhancement.ValueEnhancers;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.settings.definitions.DcFieldDefinition;
import net.sf.dc.settings.definitions.DcFieldDefinitions;

public class AutoIncrementSettingsPanel extends JPanel {

    private DcModule module;
    private final DcTable table;

    public AutoIncrementSettingsPanel() {
        super();

        table = ComponentFactory.getDCTable(false, false);
        setModule(DcSettings.getValueAsInt(Repository.settings.stModule));
        buildComponent();
    }

    public void save() {
        Collection enhancers = getEnhancers();
        module.removeEnhancers();
        for (Iterator iter = enhancers.iterator(); iter.hasNext(); ) {
            AutoIncrementer incrementer = (AutoIncrementer) iter.next();
            DcFieldDefinitions definitions = module.getFieldDefinitions();
            
            if (incrementer.isEnabled()) {
                // enable the fields and make it visible!
                for (Iterator iterDefs = definitions.iterator(); iterDefs.hasNext(); ) {
                    DcFieldDefinition def = (DcFieldDefinition) iterDefs.next();
                    if (def.getIndex() == incrementer.getField()) {
                        def.isEnabled(true);
                        def.isVisible(true);
                    }
                }
            }
            
            DcSettings.setValue(module.getFieldDefinitionsSettingsKey(), definitions);
            
            if (module.getSearchView() != null)
                module.getSearchView().applySettings();

            if (module.getInsertView() != null)
                module.getInsertView().applySettings();
            
            ValueEnhancers.registerEnhancer(module.getField(incrementer.getField()), incrementer);
        }
        ValueEnhancers.save();
    }
    
    public void setModule(int index) {
        module = DcModules.get(index);
    }

    public void setEnhancers(Collection enhancers) {
        table.clearTable();
        
        if (enhancers.size() == 0) {
            enhancers.add(new AutoIncrementer(DcMediaObject._U4_USER_NUMERIC1));
            enhancers.add(new AutoIncrementer(DcMediaObject._U5_USER_NUMERIC2));
        }
        
        for (Iterator iter = enhancers.iterator(); iter.hasNext(); ) {
            AutoIncrementer incrementer = (AutoIncrementer) iter.next();
            
            int field = incrementer.getField();
            Object[] row = new Object[] {  module.getField(field),  
                                           new Boolean(incrementer.isEnabled()), 
                                           new Boolean(incrementer.isFillGaps()), 
                                           new Integer(incrementer.getStep())};
            table.addRow(row);
        }
        table.applyHeaders();
    }

    public Collection getEnhancers() {
        Collection enhancers = new ArrayList(); 
        
        for (int i = 0; i < table.getRowCount(); i++) {
            DcField field = (DcField) table.getValueAt(i, 0, true);
            if (field != null) {
                boolean enabled = ((Boolean) table.getValueAt(i, 1, true)).booleanValue();
                boolean fillGaps = ((Boolean) table.getValueAt(i, 2, true)).booleanValue();
                int step = ((Integer) table.getValueAt(i, 3, true)).intValue();
                
                enhancers.add(new AutoIncrementer(field.getIndex(), enabled, fillGaps, step));
            }
        }
        return enhancers;
    }

    private void buildComponent() {
        setLayout(Layout.getGBL());

        JPanel panel = new JPanel();
        panel.setLayout(Layout.getGBL());

        table.setColumnCount(4);

        TableColumn cField = table.getColumnModel().getColumn(0);
        JTextField textField = new JTextField();
        textField.setEditable(false);
        textField.setEnabled(false);
        textField.setFont(ComponentFactory.getSystemFont());
        textField.setForeground(ComponentFactory.getDisabledColor());
        cField.setCellEditor(new DefaultCellEditor(textField));

        TableColumn columnEnabled = table.getColumnModel().getColumn(1);
        columnEnabled.setCellEditor(new DefaultCellEditor(new JCheckBox()));
        columnEnabled.setCellRenderer(new CheckBoxTableCellRenderer());

        TableColumn cFillGaps = table.getColumnModel().getColumn(2);
        cFillGaps.setCellEditor(new DefaultCellEditor(new JCheckBox()));
        cFillGaps.setCellRenderer(new CheckBoxTableCellRenderer());

        TableColumn cStep = table.getColumnModel().getColumn(3);
        JComboBox comboWeight = ComponentFactory.getComboBox();
        for (int i = 1; i < 101; i++) {
            comboWeight.addItem(new Integer(i));
        }
        cStep.setCellEditor(new DefaultCellEditor(comboWeight));

        cField.setHeaderValue(DcResources.getText("lblField"));
        columnEnabled.setHeaderValue(DcResources.getText("lblEnabled"));
        cFillGaps.setHeaderValue(DcResources.getText("lblFillGaps"));
        cStep.setHeaderValue(DcResources.getText("lblStep"));

        Collection enhancers = ValueEnhancers.getEnhancers(module.getIndex(), ValueEnhancers._AUTOINCREMENT);
        setEnhancers(enhancers);

        JScrollPane scroller = new JScrollPane(table);
        scroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        scroller.getViewport().setScrollMode(JViewport.SIMPLE_SCROLL_MODE);

        add(scroller,  Layout.getGBC( 0, 0, 1, 4, 50.0, 50.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                 new Insets( 5, 5, 5, 5), 0, 0));
    }
}
