/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components.panels;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.JButton;
import javax.swing.JComponent;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.console.views.table.TableInsertView;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.resources.DcResources;

public class ChildInsertTablePanel extends TableInsertView {

    private ArrayList children = new ArrayList();

    public void add(Collection objects) {
        children.addAll(objects);
    }
    
    public ChildInsertTablePanel(int module) {
        super(DcModules.get(module),
              DcResources.getText("lblNewItem", DcModules.get(module).getObjectName()),
              IconLibrary._icoItemsNew);
    }

    public void setParentID(String sID) {
        cacheChildren();
        if (sID != null && !sID.equals(parentID)) {
            parentID = sID;
            loadChildren();
        }
    }

    public void clear() {
        clearTable(true);
        children.clear();
        parentID = null;
    }

    public void open() {
    }

    public void removeRows(String[] sIDs) {
        super.removeRows(sIDs);
        
        DcObject dco = null;
        for (Iterator iter = children.iterator(); iter.hasNext(); ) {
            DcObject o = (DcObject) iter.next();
            for (int i = 0; i < sIDs.length; i++) {
                if (o.getID().equals(sIDs[i])) {
                    dco = o;
                }
            }
        }

        children.remove(dco);
    }

    public void clearTable(boolean cache) {
        table.clearTable();
        if (cache) {
            children.clear();
        }
    }

    public void cacheChildren() {
        for (int i = 0; i < table.getRowCount(); i++) {
            DcObject dco = getObjectForSaving(i);
            
            if (dco != null) { 
                dco.setValue(dco.getParentReferenceFieldIndex(), getParentID());
    
                boolean exists = false;
                DcObject[] c = new DcObject[children.size()];
                c = (DcObject[]) children.toArray(c);
                for (int j = 0; j < c.length; j++) {
                    DcObject dcoCached = c[j];
                    if (dcoCached.getID().equals(dco.getID())) {
                        dcoCached.copy(dco);
                        exists = true;
                        break;
                    }
                }
    
                if (!exists)
                    children.add(dco);
            }
        }
    }

    public void removeChildren(String parentID) {
        Collection children = getChildren(parentID);
        children.removeAll(children);
        clearTable(false);
    }

    public void loadChildren() {
        clearTable(false);
        Collection children = getChildren(parentID);
        for (Iterator iter = children.iterator(); iter.hasNext(); ) {
            DcObject child = ((DcObject) iter.next());
            child.setValue(child.getParentReferenceFieldIndex(), parentID);
        }
        
        super.add(children);
    }

    public Collection getChildren(String parentID) {
        ArrayList al = new ArrayList();
        for (int i = 0; i < children.size(); i++) {
            DcObject dco = (DcObject) children.get(i);
            if (dco.getValue(dco.getParentReferenceFieldIndex()).equals(parentID) ) {
                al.add(dco);
            }
        }
        return al;
    }

    public JComponent[] getExtraComponents() {
        JButton buttonAddTrack = ComponentFactory.getButton(DcResources.getText("lblAddTrack"));
        buttonAddTrack.addActionListener(new AddTrackAcionListener());
        buttonAddTrack.setMnemonic('T');
        return new JComponent[] {buttonAddTrack};
    }

    private class AddTrackAcionListener implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            if (getParentID() != null && !getParentID().equals("")) {
                DcObject dco = getModule().getDcObject();
                dco.setValue(dco.getParentReferenceFieldIndex(), getParentID());
                add(dco);
                children.add(dco);
            } else {
                new MessageBox(DcResources.getText("msgAddSelectAlbum"), MessageBox._ERROR);
            }
        }
    }
}