/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components.panels;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComponent;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.console.views.table.TableSearchView;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.resources.DcResources;
import net.sf.dc.wf.requests.RefreshChildView;

public class ChildSearchTablePanel extends TableSearchView {

    private int parentModIdx;
    
    public ChildSearchTablePanel(int module, int parentModIdx) {
        super(DcModules.get(module), DcModules.get(module).getObjectName());
        
        this.parentModIdx = parentModIdx;
        
        table.getSelectionModel().removeListSelectionListener(rowSelectionListener);
        table.removeKeyListener(changeRowByKey);
        treePanel = null;
    }
    
    public JComponent[] getExtraComponents() {
        JButton buttonAddTrack = ComponentFactory.getButton(DcResources.getText("lblAddTrack"));
        buttonAddTrack.addActionListener(new AddTrackAcionListener());
        buttonAddTrack.setMnemonic('T');
        return new JComponent[] {buttonAddTrack};
    }
    
    /**
     * Indicates whether there are unsaved changes
     */
    public boolean changesSaved() {
        return table.noChanges();
    }    
    
    protected void addAfterDeleteRequest(DcObject dco) {
        dco.addRequest(new RefreshChildView(null, parentModIdx, null, null, getParentID(), false));
    }    
    
    /**
     * Clears the table and its cache
     */
    public void clear() {
        table.clearTable();
        table.clearCache();
    }
    
    public void loadChildValues() {}
    
    private class AddTrackAcionListener implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            if (getParentID() != null && !getParentID().equals("")) {
                DcObject dco = getModule().getDcObject();
                dco.setIDs();
                dco.setValue(dco.getParentReferenceFieldIndex(), getParentID());
                open(dco, false, true);
            } else {
                new MessageBox(DcResources.getText("msgAddSelectAlbum"), MessageBox._ERROR);
            }
        }
    }  
}
