/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components.panels;

import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JPanel;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.core.services.DcRegion;
import net.sf.dc.core.services.DcServer;
import net.sf.dc.resources.DcResources;

public class OnlineServicePanel extends JPanel {
    
    private JComboBox comboServers = ComponentFactory.getComboBox();
    private JComboBox comboRegions = ComponentFactory.getComboBox();
    
    private JCheckBox checkUseOnlineService = ComponentFactory.getCheckBox(DcResources.getText("lblUseOnlineService"));

    public OnlineServicePanel(Collection servers, boolean toggle) {
        build(servers, toggle);
    }
    
    public boolean useOnlineService() {
        return checkUseOnlineService.isSelected();
    }
    
    public DcServer getServer() {
        return checkUseOnlineService.isSelected() ? (DcServer) comboServers.getSelectedItem() : null;
    }
    
    public DcRegion getRegion() {
        return checkUseOnlineService.isSelected() ? (DcRegion) comboRegions.getSelectedItem() : null;
    }
    
    private void applyServer() {
        DcServer server = getServer();
        comboRegions.removeAllItems();

        for (Iterator iter = server.getRegions().iterator(); iter.hasNext(); )
            comboRegions.addItem(iter.next());
    
        repaint();
    }    
    
    private void toggleServer() {
        boolean b = checkUseOnlineService.isSelected();
        comboRegions.setEnabled(b);
        comboServers.setEnabled(b);
    } 
    
    private void build(Collection servers, boolean toggle) {
        setLayout(Layout.getGBL());

        for (Iterator iter = servers.iterator(); iter.hasNext(); )
            comboServers.addItem(iter.next());

        comboServers.addActionListener(new ServerChangedListener());
        checkUseOnlineService.addActionListener(new UseOnlineServiceListener());
        
        if (toggle)
            add(checkUseOnlineService, Layout.getGBC(0, 0, 2, 1, 1.0, 1.0
               ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                new Insets(5, 5, 5, 5), 0, 0));
        
        add(comboServers, Layout.getGBC(0, 1, 1, 1, 1.0, 1.0
           ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
            new Insets(5, 5, 5, 5), 0, 0));
        add(comboRegions, Layout.getGBC(1, 1, 1, 1, 1.0, 1.0
           ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
            new Insets(5, 5, 5, 5), 0, 0));
        
        setBorder(ComponentFactory.getTitleBorder(DcResources.getText("lblOnlineServiceConfig")));
        
        checkUseOnlineService.setSelected(true);
        comboServers.setSelectedIndex(0);
    }
    
    private class UseOnlineServiceListener implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            toggleServer();
        }
    }      
    
    private class ServerChangedListener implements ActionListener {

        public void actionPerformed(ActionEvent arg0) {
            applyServer();
        }
    }     
}   