/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components.panels;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JPanel;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.dialogs.OnlineUpdateOverwriteSettingsDialog;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;

public class OnlineServiceSettingsPanel extends JPanel {
    
    private JButton buttonSettings = ComponentFactory.getButton(IconLibrary._icoSettings);
    private JCheckBox checkOverwrite;
    private JCheckBox checkAutoCreateSubItems;
    private JCheckBox checkAutoAdd;
    
    private JFrame parent;
    
    private int module;

    public OnlineServiceSettingsPanel(int module, boolean showOverwriteSettings) {
        this(null, showOverwriteSettings, module);
    }

    public OnlineServiceSettingsPanel(JFrame parent, boolean showOverwriteSettings, int module) {
        this.module = module;
        this.parent = parent;
        build(showOverwriteSettings);
        
        setOverwrite(DcSettings.getValueAsBoolean(Repository.settings.stODBOverwriteValues));
        setAutoCreateSubItems(DcSettings.getValueAsBoolean(Repository.settings.stAutoCreateSubItems));
        setAutoAdd(DcSettings.getValueAsBoolean(Repository.settings.stAutoAddPerfectMatch));
        
        buttonSettings.setEnabled(isOverwriteAllowed());
    }

    public void save() {
        DcSettings.setValue(Repository.settings.stODBOverwriteValues, new Boolean(checkOverwrite.isSelected()));        
        DcSettings.setValue(Repository.settings.stAutoCreateSubItems, new Boolean(checkAutoCreateSubItems.isSelected()));
        DcSettings.setValue(Repository.settings.stAutoAddPerfectMatch, new Boolean(checkAutoAdd.isSelected()));
    }

    private void setAutoCreateSubItems(boolean b) {
        checkAutoCreateSubItems.setSelected(b);
    }
    
    private void setOverwrite(boolean b) {
        checkOverwrite.setSelected(b);
    }
    
    public boolean isOverwriteAllowed() {
        return checkOverwrite.isSelected();
    }
    
    private void setAutoAdd(boolean b) {
        checkAutoAdd.setSelected(b);
    }
    
    public boolean isAutoAddAllowed() {
        return checkAutoAdd.isSelected();
    }    
    
    public void clear() {
        buttonSettings = null;
        checkOverwrite = null;
        parent = null;
        removeAll();
    }
    
    private void build(boolean showOverwriteSettings) {
        JPanel panel = new JPanel();
        panel.setLayout(new FlowLayout(FlowLayout.LEFT));

        setLayout(Layout.getGBL());

        checkAutoCreateSubItems = ComponentFactory.getCheckBox(DcResources.getText("lblAutoCreateMissingItems"));
        checkAutoCreateSubItems.setToolTipText(DcResources.getText("tpAutoCreateMissingItems"));

        checkAutoAdd = ComponentFactory.getCheckBox(DcResources.getText("lblAutoAddPerfectMatch"));
        checkAutoAdd.setToolTipText(DcResources.getText("tpAutoAddPerfectMatch"));
        
        checkOverwrite = ComponentFactory.getCheckBox(DcResources.getText("lblOverwriteExistingValues"));
        checkOverwrite.setToolTipText(DcResources.getText("tpOverwriteExistingValues"));
        checkOverwrite.addActionListener(new OverwriteListener());
        
        buttonSettings.setMnemonic('S');
        buttonSettings.setMaximumSize(new Dimension(25, 22));
        buttonSettings.setMinimumSize(new Dimension(25, 22));
        buttonSettings.setPreferredSize(new Dimension(25, 22));
        buttonSettings.addActionListener(new OpenSettingsDialogAction());
        buttonSettings.setEnabled(checkOverwrite.isSelected());
        
        panel.add(checkOverwrite);
        panel.add(buttonSettings);
        
        if (showOverwriteSettings) {
            add(panel, Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
               ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                new Insets(0, 0, 0, 0), 0, 0));
        }

        add(checkAutoAdd, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets(0, 5, 0, 0), 0, 0));            
        
        add(checkAutoCreateSubItems, Layout.getGBC( 0, 2, 1, 1, 1.0, 1.0
           ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
            new Insets(0, 5, 0, 0), 0, 0));

        setBorder(ComponentFactory.getTitleBorder(DcResources.getText("lblSettings")));        
    }
    
    private void openOverwriteSettingsDlg() {
        OnlineUpdateOverwriteSettingsDialog dlg = new OnlineUpdateOverwriteSettingsDialog(parent, module);
        dlg.setVisible(true);
    }    
    
    public class OpenSettingsDialogAction implements ActionListener{
        public void actionPerformed(ActionEvent e) {
            openOverwriteSettingsDlg();
        }
    }
    
    private class OverwriteListener implements ActionListener {
        public void actionPerformed(ActionEvent ae) {
            buttonSettings.setEnabled(checkOverwrite.isSelected());
        }
    }       

}
