/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2007 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components.panels;

import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.settings.definitions.DcFieldDefinition;
import net.sf.dc.settings.definitions.Definitions;

public class SortingPanel extends JPanel {
    
    private JComboBox comboOrderBy1 = ComponentFactory.getComboBox();
    private JComboBox comboOrderBy2 = ComponentFactory.getComboBox();
    
    private JLabel labelOrderBy = ComponentFactory.getLabel(DcResources.getText("lblSortOn"));
    private JLabel labelFirst = ComponentFactory.getLabel(DcResources.getText("lblFirst"));
    private JLabel labelSecond = ComponentFactory.getLabel(DcResources.getText("lblSecond"));
    
    private final int module;
    
    public SortingPanel(int module) {
        this.module = module;
        build();
    }
    
    public DcField[] getOrderBy() {
        Collection c = new ArrayList();
        if (comboOrderBy1.getSelectedIndex() > 0)
            c.add(comboOrderBy1.getSelectedItem());
        if (comboOrderBy2.getSelectedIndex() > 0)
            c.add(comboOrderBy2.getSelectedItem());
        
        DcField[] fields = new DcField[c.size()];
        return (DcField[]) c.toArray(fields);
    }
    
    public void save() {
        if (DcModules.get(module).getSearchOrderSettingsKey() != null) {
            DcField[] fields = getOrderBy();
            String[] labels = new String[fields.length];
            for (int i = 0; i < fields.length; i++) 
                labels[i] = fields[i].getDatabaseFieldName();
            
            DcSettings.setValue(DcModules.get(module).getSearchOrderSettingsKey(), labels);
        }
    }    
    
    public void clear() {
        comboOrderBy1 = null;
        comboOrderBy2 = null;
        labelOrderBy = null;
        labelFirst = null;
        labelSecond = null;
    }

    private void refill(JComboBox combo) {
        combo.removeAllItems();
        combo.addItem("");
        
        DcModule mod = DcModules.get(module);
        Definitions definitions = mod.getFieldDefinitions();

        for (Iterator iter = definitions.iterator(); iter.hasNext(); ) {
            DcFieldDefinition definition = (DcFieldDefinition) iter.next();
            DcField field = mod.getField(definition.getIndex());
            if (field.isSearchable() && !field.isUiOnly())
                combo.addItem(field);
        }
    }
    
    private void build() {
        refill(comboOrderBy1);
        refill(comboOrderBy2);
        
        comboOrderBy1.addActionListener(new OrderBySelectionListener(0));
        comboOrderBy2.addActionListener(new OrderBySelectionListener(1));
        
        String key = DcModules.get(module).getSearchOrderSettingsKey();
        if (key != null) {
            String[] ordering = DcSettings.getValueAsStringArray(key);
            applyOrderSettings(ordering);
        }
        
        setLayout(Layout.getGBL());
        add( labelOrderBy,   Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets( 5, 0, 0, 5), 0, 0));
        add( labelFirst,     Layout.getGBC( 1, 0, 1, 1, 5.0, 5.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 5, 0, 0, 5), 0, 0));
        add( comboOrderBy1,  Layout.getGBC( 2, 0, 1, 1, 5.0, 5.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets( 5, 0, 0, 5), 0, 0));
        add( labelSecond,     Layout.getGBC( 1, 1, 1, 1, 5.0, 5.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 0, 0, 0, 5), 0, 0));
        add( comboOrderBy2,  Layout.getGBC( 2, 1, 1, 1, 5.0, 5.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets( 0, 0, 0, 5), 0, 0));
    }    

    public void setFont(Font font) {
        if (labelOrderBy != null) { 
            labelOrderBy.setFont(font);
            comboOrderBy1.setFont(font);
            comboOrderBy2.setFont(font);
            labelFirst.setFont(font);
            labelSecond.setFont(font);
        }
    }

    public void applyOrderSettings(DcField[] ordering) {
        String[] s = new String[ordering.length];
        for (int i = 0; i < ordering.length; i++)
            s[i] = ordering[i].getDatabaseFieldName();
        
        applyOrderSettings(comboOrderBy1, s, 0);
        applyOrderSettings(comboOrderBy2, s, 1);
    }    
    
    public void applyOrderSettings(String[] ordering) {
        applyOrderSettings(comboOrderBy1, ordering, 0);
        applyOrderSettings(comboOrderBy2, ordering, 1);
    }
    
    private void applyOrderSettings(JComboBox combo, String[] ordering, int index) {
        if (ordering.length < (index + 1))
            return;
        
        for (int i = 1; i < combo.getModel().getSize(); i++) {
            DcField field = (DcField) combo.getItemAt(i);
            if (field != null && field.getDatabaseFieldName().equals(ordering[index])) {
                combo.setSelectedItem(field);
                break;
            }
        }
    }   
    
    private class OrderBySelectionListener implements ActionListener {
        
        private final int idx;
        
        public OrderBySelectionListener(int idx) {
            this.idx = idx;
        }
        
        public void actionPerformed(ActionEvent ae) {
            if (idx == 0) {
                if (comboOrderBy1.getSelectedIndex() < 1) {
                    comboOrderBy2.setEnabled(false);
                    refill(comboOrderBy2);
                    comboOrderBy2.setSelectedIndex(0);
                } else {
                    comboOrderBy2.setEnabled(true);
                    Object o = comboOrderBy1.getSelectedItem();
                    comboOrderBy2.removeItem(o);
                }
            } else {
                if (comboOrderBy2.getSelectedIndex() > 0) {
                    Object o = comboOrderBy2.getSelectedItem();
                    comboOrderBy1.removeItem(o);
                } else {
                    Object o = comboOrderBy1.getSelectedItem();
                    refill(comboOrderBy1);
                    comboOrderBy1.setSelectedItem(o);
                }
            }
        }
    }    
}
