/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components.renderers;

import java.awt.Color;
import java.awt.Component;
import java.awt.Rectangle;

import javax.swing.BorderFactory;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JList;
import javax.swing.border.Border;

import net.sf.dc.console.components.lists.elements.DcObjectListElement;
import net.sf.dc.core.Repository;
import net.sf.dc.settings.DcSettings;

public class DcObjectListRenderer extends DefaultListCellRenderer  {

    private final boolean evenOddColors;
    private static final Border defaultBorder = BorderFactory.createLineBorder(Color.WHITE);
    private static final Border selectedBorder = BorderFactory.createLineBorder(Color.BLACK);
    
    public DcObjectListRenderer(boolean evenOddColors) {
        this.evenOddColors = evenOddColors;
    }
    
    public Component getListCellRendererComponent(JList list, Object value,
                                                  int index, boolean isSelected, boolean cellHasFocus) {

        Component component = (Component) value;
        
        component.setFont(DcSettings.getValueAsFont(Repository.settings.stSystemFontNormal));

        if (evenOddColors) {
            component.setBackground(Color.white);
            
            Color colorOddRow = DcSettings.getValueAsColor(Repository.settings.stOddRowColor);
            Color colorEvenRow = DcSettings.getValueAsColor(Repository.settings.stEvenRowColor);
            Color colorRowSelection = DcSettings.getValueAsColor(Repository.settings.stRowSelectionColor);
            
            if (!isSelected) {
                if ((index % 2) == 0)
                    component.setBackground(colorEvenRow);
                else
                    component.setBackground(colorOddRow);
            } else {
                component.setBackground(colorRowSelection);
            }
        } else {
            if (isSelected) {
                if (component instanceof DcObjectListElement) 
                    ((DcObjectListElement) component).setBorder(selectedBorder);
            } else if (component instanceof DcObjectListElement) { 
                ((DcObjectListElement) component).setBorder(defaultBorder);
            }
        }
        
        return component;
    }
    
    @Override
    public void repaint(final long tm, final int x, final int y, final int width, final int height) {
    }
    
    /**
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     */
    @Override
    public void repaint(final Rectangle r) {
    }
    
    /**
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     */
    @Override
    protected void firePropertyChange(final String propertyName, final Object oldValue, final Object newValue) {
    }
    
    /**
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     */
    @Override
    public void firePropertyChange(final String propertyName, final byte oldValue, final byte newValue) {
    }
    
    /**
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     */
    @Override
    public void firePropertyChange(final String propertyName, final char oldValue, final char newValue) {
    }
    
    /**
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     */
    @Override
    public void firePropertyChange(final String propertyName, final short oldValue, final short newValue) {
    }
    
    /**
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     */
    @Override
    public void firePropertyChange(final String propertyName, final int oldValue, final int newValue) {
    }
    
    /**
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     */
    @Override
    public void firePropertyChange(final String propertyName, final long oldValue, final long newValue) {
    }
    
    /**
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     */
    @Override
    public void firePropertyChange(final String propertyName, final float oldValue, final float newValue) {
    }
    
    /**
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     */
    @Override
    public void firePropertyChange(final String propertyName, final double oldValue, final double newValue) {
    }
    
    /**
     * Overridden for performance reasons.
     * See the <a href="#override">Implementation Note</a>
     * for more information.
     */
    @Override
    public void firePropertyChange(final String propertyName, final boolean oldValue, final boolean newValue) {
    }    
}


