/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components.renderers;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;

import javax.swing.JComponent;
import javax.swing.JTable;
import javax.swing.ToolTipManager;
import javax.swing.border.Border;
import javax.swing.table.DefaultTableCellRenderer;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.core.Repository;
import net.sf.dc.settings.DcSettings;

public class DcTableCellRenderer extends DefaultTableCellRenderer {

    private static boolean disabled = false;
    private static final Border focusBorder = ComponentFactory.getBorder();

    protected boolean allowTooltips() {
    	return true;
    }
    
    public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, 
                                                   boolean hasFocus, int row, int column) {

        JComponent c = (JComponent) super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
        
        if (DcSettings.getValueAsBoolean(Repository.settings.stShowTableTooltip) && allowTooltips()) {
        	if (disabled) {
        		ToolTipManager.sharedInstance().registerComponent(table);
                ToolTipManager.sharedInstance().registerComponent(table.getTableHeader());
                disabled = false;
        	} 
            
            ToolTipManager.sharedInstance().setDismissDelay(2000);
            setToolTipText(getTipText());

        } else {
            if (!disabled) { 
                disabled = true;
            	ToolTipManager.sharedInstance().unregisterComponent(table);
            	ToolTipManager.sharedInstance().unregisterComponent(table.getTableHeader());
            }
        }
        
        Color colorOddRow = DcSettings.getValueAsColor(Repository.settings.stOddRowColor);
        Color colorEvenRow = DcSettings.getValueAsColor(Repository.settings.stEvenRowColor);
        Color colorRowSelection = DcSettings.getValueAsColor(Repository.settings.stRowSelectionColor);
        Font font = DcSettings.getValueAsFont(Repository.settings.stSystemFontNormal);
        setFont(font);
        
        setForeground(ComponentFactory.getCurrentForegroundColor());

        if (!isSelected) {
            if ((row % 2) == 0) {
        		setBackground(colorEvenRow);
            } else {
                setBackground(colorOddRow);
            }
        } else {
        	if ((row % 2) == 0) {
        		if (hasFocus) {
        			setBackground(colorEvenRow);
        		} else {
        			setBackground(colorRowSelection);
        		}
        	} else {
        		if (hasFocus) {
        			setBackground(colorOddRow);
        		} else {
        			setBackground(colorRowSelection);
        		}
        	}
        }
        
    	if (hasFocus) {
    		setBorder(focusBorder);
    	} else {
    		setBorder(noFocusBorder);
    	}
        
        return c;
    }
    
    protected String getTipText() {
        return super.getText();
    }
}