/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components.renderers;

import java.util.Calendar;

public class TimeFieldTableCellRenderer extends DcTableCellRenderer {
    
    private static final Calendar cal;
    
    static {
        cal = Calendar.getInstance();
    }
    
    public TimeFieldTableCellRenderer() {
        setOpaque(true);

        setHorizontalAlignment(LEFT);
        setVerticalAlignment(CENTER);
    }
    
    public void setText(String str) {
        int minutes = 0;
        int seconds = 0;
        int hours = 0;
        
        cal.clear();
        
        if (str != null && str.length() > 0) {
        	int value = Integer.parseInt(str);
        	cal.set(Calendar.SECOND, value);
        	minutes = cal.get(Calendar.MINUTE);
        	seconds = cal.get(Calendar.SECOND);
            hours = cal.get(Calendar.HOUR);
        }
        
        StringBuffer sb = new StringBuffer();
        sb.append(hours);
        sb.append(":");
        sb.append(getDoubleDigitString(minutes));
        sb.append(":");
        sb.append(getDoubleDigitString(seconds));
        
        super.setText(sb.toString());
    } 
    
    private String getDoubleDigitString(int value) {
        if (value == 0) {
            return "00";
        } else if (value > 0 && value < 10) {
            StringBuffer sb = new StringBuffer();
            sb.append(0);
            sb.append(value);
            return sb.toString();
        } else {
        	return String.valueOf(value);
        }
    }
}
