/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components.tables;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;

import javax.swing.DefaultCellEditor;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableColumn;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.components.DcNumberField;
import net.sf.dc.console.components.DcRatingComboBox;
import net.sf.dc.console.components.DcShortTextField;
import net.sf.dc.console.components.renderers.AvailabilityCellRenderer;
import net.sf.dc.console.components.renderers.ComboBoxTableCellRenderer;
import net.sf.dc.console.components.renderers.ContactPersonTableCellRenderer;
import net.sf.dc.console.components.renderers.DcTableCellRenderer;
import net.sf.dc.console.components.renderers.DcTableHeaderRenderer;
import net.sf.dc.console.components.renderers.DcTableHeaderRendererRequired;
import net.sf.dc.console.components.renderers.DcTableRowIndexCellRenderer;
import net.sf.dc.console.components.renderers.ModuleTableCellRenderer;
import net.sf.dc.console.components.renderers.NumberTableCellRenderer;
import net.sf.dc.console.components.renderers.PictureTableCellRenderer;
import net.sf.dc.console.components.renderers.RatingTableCellRenderer;
import net.sf.dc.console.components.renderers.ReferencesTableCellRenderer;
import net.sf.dc.console.components.renderers.TimeFieldTableCellRenderer;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.Repository;
import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.Loan;
import net.sf.dc.core.objects.Picture;
import net.sf.dc.core.objects.generic.GenericObject;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.settings.definitions.DcFieldDefinition;
import net.sf.dc.settings.definitions.Definitions;

public class DcTable extends JTable  {

    private final DcModule module;
    private final Hashtable cache = new Hashtable();

    private final TableValueChangedAction tableChangeListener = new TableValueChangedAction();

    private final boolean caching;
    private final boolean readonly;
    
    private boolean ignoreEdit = false;

    private ArrayList columnsHidden = new ArrayList();
    private Map columns = new HashMap();
    private TableColumn rowIndexColumn;

    public DcTable(boolean readonly, boolean caching) {
        super(new DcTableModel());

        setDoubleBuffered(true);
        
        this.readonly = readonly;
        createRowIndexColumn();
        module = null;

        addSystemColumns();
        setProperties();

        this.caching = caching;
        readonly = false;
    }
    
    public DcTableModel getDcModel() {
        if (!(getModel() instanceof DcTableModel))
            setModel(new DcTableModel());
            
        DcTableModel model = (DcTableModel) getModel();
        return model;
    }    
    
    public DcTable(DcModule module, boolean readonly, boolean caching) {
        super(new DcTableModel());

        this.caching = caching;
        this.module = module;
        this.readonly = readonly;

        buildTable();
        setListeningForChanges(true);
    }

    private int getFieldForColumnIndex(int columnIndex) {
        int field = -1;
        for (Iterator iter = columns.values().iterator(); iter.hasNext(); ) {
            TableColumn column = (TableColumn) iter.next();
            if (column.getModelIndex() == columnIndex) {
                Integer identifier = (Integer) column.getIdentifier();

                if (identifier != null)
                    field = identifier.intValue();
            }
        }
        return field;
    }

    public int getColumnIndexForField(int field) {
        int columnIndex = -1;
        for (Iterator iter = columns.values().iterator(); iter.hasNext(); ) {
            TableColumn column = (TableColumn) iter.next();
            Integer identifier = (Integer) column.getIdentifier();
            if (identifier != null && identifier.intValue() == field) {
                columnIndex = column.getModelIndex();
            }
        }
        return columnIndex;
    }

    public Collection getSelectedObjects() {
        Collection c = new ArrayList();
        int[] rows = getSelectedRows();
        for (int i = 0; i < rows.length; i++)
            c.add(getObjectForRow(rows[i]));

        return c;
    }
    
    private int addRow() {
        Object[] row = new Object[module.getFieldCount()];

        DcObject dco = module.getDcObject();
        int[] fields = dco.getFieldIndices();

        for (int i = 0; i < fields.length; i++) {
            int field = fields[i];
            int col = getColumnIndexForField(field);
            row[col] = dco.getValue(field);
        }
        getDcModel().addRow(row);

        return (getRowCount() - 1);
    }

    public void addObject(DcObject dco) {
        addObject(dco, false);
    }

    public void addObject(DcObject dco, boolean setSelected) {
        setListeningForChanges(false);
        int[] fields = dco.getFieldIndices();

        int row = addRow();
        for (int i = 0; i < fields.length; i++) {
            int field = fields[i];
            int col = getColumnIndexForField(field);
            Object value = dco.getValue(fields[i]);
            getDcModel().setValueAt(value, row, col);
        }

        if (module.getIndex() == DcModules._GENERIC) {
            int col = getColumnIndexForField(GenericObject._SYS_MODULE);
            Object value = dco.getModule();
            getDcModel().setValueAt(value, row, col);
        }

        if (setSelected) {
        	setSelectedRow(getRowCount() - 1);
        }
        setListeningForChanges(true);
    }

    public void addObjects(Collection objects) {
        for (Iterator iter = objects.iterator(); iter.hasNext();) {
            DcObject dco = (DcObject) iter.next();
            addObject(dco, false);
        }
        setSelectedRow(getRowCount() - 1);
    }

    public boolean isReadOnly() {
    	return readonly;
    }

    public boolean noChanges() {
        return cache.size() == 0;
    }

    public void ignoreEdit(boolean b) {
    	ignoreEdit = b;
    }

    public void applyHeaders() {
    	removeSystemColumns();
        for (Enumeration enumCols = getColumnModel().getColumns(); enumCols.hasMoreElements(); ) {
            TableColumn column = (TableColumn) enumCols.nextElement();
            column.setHeaderRenderer(getTableHeaderRenderer());
            columns.put(column.getIdentifier(), column);
        }
        addSystemColumns();
    }

    public void moveRowDown() {
        int row = getSelectedRow();

        if (row > -1) {
            int total = getRowCount();
            if (row < total -1) {
                int destination = row + 1;
                getDcModel().moveRow(row, row, destination);
                setSelectedRow(destination);
            }
        } else {
            new MessageBox("No row selected. First select a row to move", MessageBox._WARNING);
        }
    }

    public void moveRowUp() {
        int row = getSelectedRow();

        if (row > -1) {
            if (row != 0) {
                int destination = row - 1;
                getDcModel().moveRow(row, row, destination);
                setSelectedRow(destination);
            }
        } else {
            new MessageBox("No row selected. First select a row to move", MessageBox._WARNING);
        }
    }

    public void removeFields(int[] fields) {
        cancelEdit();
        removeSystemColumns();

        for (int i = 0; i < fields.length; i++) {
        	int columnIndex = getColumnIndexForField(fields[i]);

            int tableIndex = convertColumnIndexToView(columnIndex);
            try {
                TableColumn column = getColumnModel().getColumn(tableIndex);
                columnsHidden.add(column);
                removeColumn(column);
            } catch (Exception ignore) {}
        }

        addSystemColumns();
    }

    public void clearCache() {
    	cache.clear();
    }

    public void removeFromCache(String sID) {
    	cache.remove(sID);
    }

    public Collection getObjects() {
        Collection objects = new ArrayList();
        for (int i = 0; i < getRowCount(); i++) {
            objects.add(getDCObject(i));
        }
        return objects;
    }

    private boolean isCached(String id) {
        return id != null && cache.containsKey(id);
    }

    public DcObject getDCObject(int row) {
        int col = getColumnIndexForField(DcObject._ID);
        String id = (String) getValueAt(row, col, true);

        if (id != null) {
            boolean isCached = isCached(id);
    
            DcObject dco;
            if (isCached) {
            	dco = (DcObject) cache.get(id);
            } else {
            	dco = getObjectForRow(row);
                if (dco != null)
                    dco.markAsUnchanged();
            }
            return dco;
        }
        
        return null;
    }

    private DcObject getObjectForRow(int row) {
        cancelEdit();
        DcObject dco = null;

        if (row == -1) {
        	new MessageBox(DcResources.getText("msgNoRowsSelected"), MessageBox._WARNING);
        } else {
            dco = getModuleForRow(row).getDcObject();
            for (Iterator iter = columns.values().iterator(); iter.hasNext(); ) {
                TableColumn column = (TableColumn) iter.next();

                int col = column.getModelIndex();
                int field = getFieldForColumnIndex(col);
                Object value = getValueAt(row, col, true);
                dco.setValue(field, value);
            }
        }
        return dco;
    }

    public DcModule getModuleForRow(int row) {
        DcModule result = module;

        if (module.getIndex() == DcModules._GENERIC) {
            int col = getColumnIndexForField(GenericObject._SYS_MODULE);
            Object value = getValueAt(row, col, true);
            if (value instanceof DcModule) {
                result = (DcModule) value;
            } else {
                String s = value.toString();
                for (Iterator iter = DcModules.getModules(); iter.hasNext(); ) {
                    DcModule mod = (DcModule) iter.next();
                    if (mod.getName().equals(s))
                        result = mod;
                }
            }
        }
        return result;
    }

    public String getObjectID(int row) {
        int col = getColumnIndexForField(DcObject._ID);
        return (String) getValueAt(row, col, true);
    }

    public Collection getChangedObjects() {
        Collection objects = new ArrayList();
        for (Enumeration enumKeys = cache.keys(); enumKeys.hasMoreElements(); ) {
            String objectId = enumKeys.nextElement().toString();
            objects.add(cache.get(objectId));
        }
        return objects;
    }

    public int[] getChangedObjectsIndices() {
        cancelEdit();
        int[] rows = new int[cache.size()];
        int counter = 0;
    	for (Enumeration enumKeys = cache.keys(); enumKeys.hasMoreElements(); counter++) {
            String sID = enumKeys.nextElement().toString();
            int row = getRowNumberWithID(sID);
            rows[counter] = row;
        }
        return rows;
	}

    public Object getValueAt(int row, int col, boolean hidden) {
        Object value = null;
        try {
            if (row > -1 && col > -1)
                value = hidden ? getDcModel().getValueAt(row, col) : super.getValueAt(row, col);
        } catch (Exception ignore) {}
                
        return value;
    }

    public void clearTable() {
        cache.clear();
        cancelEdit();

        setListeningForChanges(false);
        getDcModel().setRowCount(0);
        setListeningForChanges(true);
    }

    public int getRowNumberWithID(String ID) {
        cancelEdit();
        for (int i = 0; i < getDcModel().getRowCount(); i++) {
            if (ID.equals(getObjectID(i)))
                return i;
        }
        return -1;
    }

    public void setColumnCount(int count) {
    	getDcModel().setColumnCount(count);
    }

    public void removeRows(int[] rows) {
        cancelEdit();
        for (int i = rows.length -1; i > -1; i--) {
        	if (caching) {
                int row = rows[i];
                int col = getColumnIndexForField(DcObject._ID);
    			removeFromCache((String) getValueAt(row, col, true));
        	}
        	getDcModel().removeRow(rows[i]);
        }
    }

    public void removeRow(int row) {
    	getDcModel().removeRow(row);
    }

    public void setRowCount(int count) {
    	getDcModel().setRowCount(count);
    }

    public void setSelectedRow(int row) {
        try {
            if (getSelectedRow() > -1) {
                removeColumnSelectionInterval(0, getColumnCount() -1);
                removeRowSelectionInterval(getSelectedRow(), getSelectedRow());
            }

            getSelectionModel().setValueIsAdjusting(true);
            
            addRowSelectionInterval(row, row);
            addColumnSelectionInterval(0, getColumnCount() -1);
            
            if (row <= getRowCount()) {
                Rectangle rect = getCellRect(row, 0, true);
                scrollRectToVisible(rect);
            }
            
        } catch (Exception ignore) {}
    }

    public void updateRowAt(int row,
                            DcObject dco,
                            boolean overwrite,
                            boolean allowDeletes,
                            boolean mark) {
        cancelEdit();

        if (!mark) {
            setListeningForChanges(false);
            removeFromCache(dco.getID());
        }

        int[] indices = dco.getFieldIndices();

        try {
            setSelectedRow(row);
        } catch (Exception ignore) {}

        for (int i = 0; i < indices.length; i++) {
            try {
                TableColumn column = (TableColumn) columns.get(new Integer(indices[i]));
                int col = column.getModelIndex();

                Object newValue = dco.getValue(indices[i]);
                Object existingValue = getDcModel().getValueAt(row, col);

                if (newValue instanceof Picture)
                    newValue = ((Picture) newValue).isDeleted() ? null : newValue;                
                if (overwrite && ((newValue != null && !newValue.equals("")) || allowDeletes)) {
                	getDcModel().setValueAt(newValue, row, col);
                } else if (existingValue == null || existingValue.equals("")) {
                	getDcModel().setValueAt(newValue, row, col);
                }

            } catch (Exception ignore) {}
        }

        if (!mark)
            setListeningForChanges(true);
    }

    public void addRow(Object[] row) {
    	getDcModel().addRow(row);
    }

    public void addRowToCache(int row, int column) {
        setListeningForChanges(false);

        try {
            if (row != -1 && column != -1) {
                int col = getColumnIndexForField(DcObject._ID);
                String id = (String) getValueAt(row, col, true);
    
                if (id != null && !id.equals("")) {
                    DcObject dco;
                    if (!cache.containsKey(id)) {
                        dco = getObjectForRow(row);
                        dco.markAsUnchanged();
                        
                        DcObject o = DataManager.getObject(module.getIndex(), id);
                        if (o != null) {
                        	int field = getFieldForColumnIndex(column);
                            Object valueOld = o.getValue(field);
                            Object valueNew = getDcModel().getValueAt(row, column);
                            
                            valueOld = valueOld == null ? "" : valueOld;
                            valueNew = valueNew == null ? "" : valueNew;
                        	
                            if (valueOld.equals(valueNew)) 
                            	return;
                        }
                    } else {
                        dco = (DcObject) cache.get(id);
                        
                        int field = getFieldForColumnIndex(column);
                        Object valueOld = dco.getValue(field) == null ? "" : dco.getValue(field);
                        Object valueNew = getDcModel().getValueAt(row, column);
                        
                        if (valueOld.equals(valueNew))
                        	return;
                    }
    
                	dco.setValue(getFieldForColumnIndex(column), getDcModel().getValueAt(row, column));
                	cache.put(id, dco);
                }
            }
        } finally {
            setListeningForChanges(true);
        }
    }

    public void cancelEdit() {
        try {
            ignoreEdit = true;
            int selectedRow = getSelectedRow();
            if (selectedRow != -1) {
                for (int i = 0; i < getColumnCount(); i++) {
                    boolean success = false;
                    try {
                        getCellEditor(selectedRow, i).stopCellEditing();
                        success = true;
                    } catch (Exception ignore) {}
    
                    if (!success)
                        getCellEditor(selectedRow, i).cancelCellEditing();
                }
            }
        } finally {
            ignoreEdit = false;
        }
    }

    //*************************************************************************
    //Private methods and classes
    //*************************************************************************
    private void buildTable() {
        DcObject dco = module.getDcObject();
        Collection fields = dco.getFields();
        getDcModel().setColumnCount(fields.size());

        int counter = 0;
        for (Iterator iter = fields.iterator(); iter.hasNext(); counter++) {
            DcField field = (DcField) iter.next();
            TableColumn columnNew = getColumnModel().getColumn(counter);
            columnNew.setIdentifier(new Integer(field.getIndex()));
            columnNew.setHeaderValue(field.getLabel());

            JComponent comp = ComponentFactory.getComponent(module.getIndex(),
                                                            field.getFieldType(),
                                                            field.getLabel(),
                                                            field.getMaximumLength());
            comp.setAutoscrolls(false);
            comp.setBorder(null);
            comp.setIgnoreRepaint(true);
            comp.setVerifyInputWhenFocusTarget(false);

            if (field.getFieldType() == ComponentFactory._LONGTEXTFIELD) {
                comp = ComponentFactory.getComponent(module.getIndex(), 
                                                     ComponentFactory._SHORTTEXTFIELD,
                                                     field.getLabel(),
                                                     field.getMaximumLength());
            }

            if (    field.getIndex() == DcObject._ID ||
                    field.getIndex() == DcObject._SYS_LOANEDBY ||
                    field.getIndex() == DcObject._SYS_LOANDURATION ||
                    field.getIndex() == DcObject._SYS_CREATED ||
                    field.getIndex() == DcObject._SYS_MODIFIED){

                DcShortTextField text = ComponentFactory.getTextFieldDisabled();
                columnNew.setCellEditor(new DefaultCellEditor(text));
                DcTableCellRenderer renderer = new DcTableCellRenderer();
                renderer.setFont(ComponentFactory.getSystemFont());
                columnNew.setCellRenderer(renderer);
            } else if (field.getFieldType() == ComponentFactory._REFERENCESFIELD) {
                columnNew.setCellEditor(new DefaultCellEditor(ComponentFactory.getTextFieldDisabled()));
                columnNew.setMaxWidth(100);
                columnNew.setCellRenderer(new ReferencesTableCellRenderer());
            } else if (field.getIndex() == DcObject._SYS_MODULE) {
                DcShortTextField text = ComponentFactory.getTextFieldDisabled();
                columnNew.setCellEditor(new DefaultCellEditor(text));
                columnNew.setCellRenderer(new ModuleTableCellRenderer());
            } else if (field.getIndex() == DcObject._SYS_AVAILABLE) {
                DcShortTextField text = ComponentFactory.getTextFieldDisabled();
                columnNew.setCellEditor(new DefaultCellEditor(text));
                columnNew.setCellRenderer(new AvailabilityCellRenderer());
            } else if (dco instanceof Loan && field.getIndex() == Loan._C_CONTACTPERSONID) {
                DcShortTextField text = ComponentFactory.getTextFieldDisabled();
                columnNew.setCellEditor(new DefaultCellEditor(text));
                columnNew.setCellRenderer(new ContactPersonTableCellRenderer());
            } else {
                switch (field.getFieldType()) {
                    case ComponentFactory._NUMBERFIELD:
                        columnNew.setCellEditor(new DefaultCellEditor((JTextField) comp));
                        columnNew.setMaxWidth(100);
                        columnNew.setCellRenderer(new NumberTableCellRenderer());
                        break;
                    case ComponentFactory._LONGTEXTFIELD :
                    case ComponentFactory._SHORTTEXTFIELD :
                        columnNew.setCellEditor(new DefaultCellEditor((JTextField) comp));
                        break;
                    case ComponentFactory._TIMEFIELD :
                        TimeFieldTableCellRenderer timeRenderer = new TimeFieldTableCellRenderer();
                        DcNumberField numberField = ComponentFactory.getNumberField();
                        columnNew.setCellEditor(new DefaultCellEditor(numberField));
                        columnNew.setCellRenderer(timeRenderer);
                        break;
                    case ComponentFactory._URLFIELD :
                        columnNew.setCellEditor(new DefaultCellEditor((JTextField) comp));
                        DcTableCellRenderer renderer = new DcTableCellRenderer();
                        renderer.setForeground(new Color(0, 0, 255));
                        columnNew.setCellRenderer(renderer);
                        break;
                    case ComponentFactory._PICTUREFIELD :
                        DcShortTextField text = ComponentFactory.getTextFieldDisabled();
                        text.setEditable(false);
                        text.setFont(ComponentFactory.getUnreadableFont());
                        columnNew.setCellEditor(new DefaultCellEditor(text));
                        columnNew.setCellRenderer(new PictureTableCellRenderer());
                        break;
                    case ComponentFactory._BOOKCATEGORYCOMBO:
                    case ComponentFactory._SOFTWARECATEGORYCOMBO:
                    case ComponentFactory._CONTACTPERSONCATEGORYCOMBO:
                    case ComponentFactory._PLATFORMCOMBO:
                    case ComponentFactory._MEDIACOMBO:
                    case ComponentFactory._STATECOMBO:
                        columnNew.setCellRenderer(new ComboBoxTableCellRenderer(field));
                        columnNew.setCellEditor(new DefaultCellEditor((JComboBox) comp));
                        break;
                    case ComponentFactory._RATINGCOMBOBOX:
                        columnNew.setMinWidth(70);
                        columnNew.setCellRenderer(new RatingTableCellRenderer());
                        columnNew.setCellEditor(new DefaultCellEditor((DcRatingComboBox) comp));
                        break;
                    case ComponentFactory._MP3GENRECOMBO:
                    case ComponentFactory._YESNOCOMBO:
                        columnNew.setCellEditor(new DefaultCellEditor((JComboBox) comp));
                        break;
                }
                comp.setEnabled(!readonly);
            }
        }

        createRowIndexColumn();
        addSystemColumns();

        setProperties();

        applyDefinitions();
    }

    public void applyDefinitions() {
    	
    	try {
	        Definitions definitions = module.getFieldDefinitions();
	        removeColumns();
	        removeSystemColumns();
	        for (Iterator iter = definitions.iterator(); iter.hasNext(); ) {
	            DcFieldDefinition definition = (DcFieldDefinition) iter.next();
	            TableColumn column = (TableColumn) columns.get(new Integer(definition.getIndex()));

	            if (definition.isRequired())
	                column.setHeaderRenderer(getRequiredTableHeaderRenderer());
	            else
	                column.setHeaderRenderer(getTableHeaderRenderer());
	
	            String label = definition.getLabel();
                
	            if (label != null && label.length() > 0) {
	                column.setHeaderValue(label);
	            } else {
	                column.setHeaderValue(module.getField(definition.getIndex()).getSystemName());
	            }
	
	            if (definition.isVisible() && definition.isEnabled())
	                addColumn(column);
	
	        }
    	} catch (Exception exp) {
    		Messages.add(exp);
    	}
        
        addSystemColumns();
    }

    private void removeColumns() {
        for (Iterator iter = columns.values().iterator(); iter.hasNext(); ) {
            TableColumn column = (TableColumn) iter.next();
            try {
                removeColumn(column);
            } catch (Exception ignore) {}
        }
    }

    public void resetTable() {

        Definitions definitions = module.getFieldDefinitions();

        removeColumns();
        removeSystemColumns();
        for (Iterator iter = definitions.iterator(); iter.hasNext(); ) {
            DcFieldDefinition definition = (DcFieldDefinition) iter.next();
            TableColumn column = (TableColumn) columns.get(new Integer(definition.getIndex()));
            addColumn(column);
        }
    }

    private void setProperties() {
        setAutoscrolls(true);

        setDefaultRenderer(Object.class, new DcTableCellRenderer());
        setFont(ComponentFactory.getStandardFont());

        setRowSelectionAllowed(true);
        setColumnSelectionAllowed(false);
        setRequestFocusEnabled(true);
        setAutoResizeMode(JTable.AUTO_RESIZE_SUBSEQUENT_COLUMNS);
        setAlignmentY(JTable.TOP_ALIGNMENT);
        getTableHeader().setReorderingAllowed(false);
        setShowHorizontalLines(true);

        setBackground(new Color(255, 255, 255));
        setGridColor(new Color(220,220,200));

        applyHeaders();

        setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
    }

    public boolean isListeningForChanges() {
        TableModelListener[] listeners = getDcModel().getTableModelListeners();
        for (int i = 0; i < listeners.length; i++) {
            if (listeners[i] instanceof TableValueChangedAction)
                return true;
        }
        return false;
    }
    
    public void setListeningForChanges(boolean b) {
        boolean enable = b && caching;

        TableModelListener[] listeners = getDcModel().getTableModelListeners();
        for (int i = 0; i < listeners.length; i++) {
            if (listeners[i] instanceof TableValueChangedAction)
            	getDcModel().removeTableModelListener(listeners[i]);
        }
        
        if (enable && getListeners(tableChangeListener.getClass()).length == 0)
        	getDcModel().addTableModelListener(tableChangeListener);
    }

    private DefaultTableCellRenderer getRequiredTableHeaderRenderer() {
        DcTableHeaderRendererRequired hdrRenderer = new DcTableHeaderRendererRequired();
        return hdrRenderer;
    }

    private DefaultTableCellRenderer getTableHeaderRenderer() {
        DcTableHeaderRenderer hdrRenderer = new DcTableHeaderRenderer();
        return hdrRenderer;
    }

    private void createRowIndexColumn() {
        rowIndexColumn = new TableColumn();
        rowIndexColumn.setHeaderValue("#");
        rowIndexColumn.setMaxWidth(40);
        rowIndexColumn.setPreferredWidth(40);
        rowIndexColumn.setWidth(40);
        JTextField text = ComponentFactory.getShortTextField(40);
        text.setEditable(false);
        text.setEnabled(false);
        text.setBackground(DcSettings.getValueAsColor(Repository.settings.stTableHeaderColor));
        rowIndexColumn.setCellEditor(new DefaultCellEditor(text));
        DcTableRowIndexCellRenderer renderer = new DcTableRowIndexCellRenderer();
        renderer.setFont(ComponentFactory.getSystemFont());
        rowIndexColumn.setCellRenderer(renderer);
        rowIndexColumn.setHeaderRenderer(getTableHeaderRenderer());
    }

    private void addSystemColumns() {
        getColumnModel().addColumn(rowIndexColumn);
        moveColumn(getColumnCount() - 1, 0);
    }

    private void removeSystemColumns() {
    	getColumnModel().removeColumn(rowIndexColumn);
    }

    private class TableValueChangedAction implements TableModelListener {
        public void tableChanged(TableModelEvent e) {
            if (    !ignoreEdit &&
            		(e.getType() == TableModelEvent.INSERT ||
                     e.getType() == TableModelEvent.UPDATE)) {

                int row = getSelectedRow();
                
                Component component = null;
                
                try {
                    component = getEditorComponent();
                } catch (Exception exp) {}
                
                if (component == null) {
                    addRowToCache(row, e.getColumn());
                } else {
                    int field = getFieldForColumnIndex(e.getColumn());    
                    try {
                        if (component != null && component.isEnabled() && !module.getField(field).isUiOnly())
                            addRowToCache(row, e.getColumn());
                    } catch (Exception whatever) {
                        addRowToCache(row, e.getColumn());
                    }
                }
            }
        }
    }
    
    protected void paintComponent ( Graphics g ) {
        try {
            Graphics2D g2d = (Graphics2D) g;
            if (DcSettings.getValueAsBoolean(Repository.settings.stFontAntiAliasing)) {
               g2d.setRenderingHint( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );
               g2d.setRenderingHint( RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON );
               g2d.setRenderingHint( RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY );
            }
            super.paintComponent(g2d);
        } catch (Exception exp) {
            super.paintComponent(g);
        }
    }     
}
