/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.sql.ResultSet;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcDialog;
import net.sf.dc.console.components.DcLongTextField;
import net.sf.dc.console.components.panels.AutoIncrementSettingsPanel;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.enhancement.AutoIncrementer;
import net.sf.dc.core.objects.enhancement.IValueEnhancer;
import net.sf.dc.db.DatabaseManager;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.definitions.DcFieldDefinition;
import net.sf.dc.settings.definitions.DcFieldDefinitions;

public class AutoIncrementDialog extends DcDialog {

    private AutoIncrementSettingsPanel pSettings = new AutoIncrementSettingsPanel();
    
    private JProgressBar progressBar = new JProgressBar();
    
    private JComboBox comboModus = ComponentFactory.getComboBox();
    private JComboBox comboOrderBy1 = ComponentFactory.getComboBox();
    private JComboBox comboOrderBy2 = ComponentFactory.getComboBox();
    private JComboBox comboOrderBy3 = ComponentFactory.getComboBox();
    
    JButton buttonSave = ComponentFactory.getButton(DcResources.getText("lblSave"));
    JButton buttonCancel = ComponentFactory.getButton(DcResources.getText("lblCancel"));
    JButton buttonRun = ComponentFactory.getButton(DcResources.getText("lblRun"));
    

    public AutoIncrementDialog() {
        super(DataCrow.mainFrame);

        buildDialog();
        
        setHelpIndex("dc.Tools.AutoNumbering");
        setTitle(DcResources.getText("lblAutoNumbering"));

        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                close();
            }
        });

        setSize(new Dimension(600, 600));
        setModal(true);
    }

    private void close() {
        pSettings = null;
        comboOrderBy1 = null;
        comboOrderBy2 = null;
        comboOrderBy3 = null;
        comboModus = null;
        progressBar = null;
        dispose();
    }

    private void save() {
        pSettings.save();
    }
    
    public void initProgressBar(int maxValue) {
        progressBar.setValue(0);
        progressBar.setMaximum(maxValue);
    }

    public void updateProgressBar() {
        int current = progressBar.getValue();
        progressBar.setValue(current + 1);
    }    

    private void buildDialog() {
        getContentPane().setLayout(Layout.getGBL());

        /***********************************************************************
         * Renumber
         **********************************************************************/
        JPanel panelRenumber = new JPanel(false);
        panelRenumber.setLayout(Layout.getGBL());
        
        buttonRun.addActionListener(new RenumberAction());
        
        DcLongTextField explanation = ComponentFactory.getLongTextField(2000);
        ComponentFactory.setUneditable(explanation);
        
        DcModule module = DcModules.getCurrentModule();
        
        explanation.setText(DcResources.getText("lblRenumberExplanation", module.getObjectNamePlural()));
        
        JLabel labelOrderBy = ComponentFactory.getLabel(DcResources.getText("lblOrderBy"));
        comboOrderBy1.addItem("");
        comboOrderBy2.addItem("");
        comboOrderBy3.addItem("");
        
        DcFieldDefinitions definitions = module.getFieldDefinitions();
        for (Iterator iter = definitions.iterator(); iter.hasNext(); ) {
            DcFieldDefinition definition = (DcFieldDefinition) iter.next();
            DcField field = module.getField(definition.getIndex());
            
            if (field.isSearchable() && !field.isUiOnly()) {
                comboOrderBy1.addItem(field);
                comboOrderBy2.addItem(field);
                comboOrderBy3.addItem(field);
            }
        }
        
        JLabel labelMode = ComponentFactory.getLabel(DcResources.getText("lblRenumberMode"));
        comboModus.addItem(DcResources.getText("lblRestartNumbering"));
        comboModus.addItem(DcResources.getText("lblFillGapsOnly"));
        
        panelRenumber.add(explanation,   Layout.getGBC(0, 0, 2, 1, 1.0, 1.0,
                GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                new Insets(5, 5, 5, 5), 0, 0));
        panelRenumber.add(labelMode,  Layout.getGBC(0, 1, 1, 1, 1.0, 1.0,
                GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                new Insets(5, 5, 5, 5), 0, 0));        
        panelRenumber.add(comboModus, Layout.getGBC(1, 1, 1, 1, 1.0, 1.0,
                GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                new Insets(5, 5, 5, 5), 0, 0));        
        
        panelRenumber.add(labelOrderBy,  Layout.getGBC(0, 2, 1, 1, 1.0, 1.0,
                GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                new Insets(5, 5, 5, 5), 0, 0));        
        panelRenumber.add(comboOrderBy1, Layout.getGBC(1, 2, 1, 1, 1.0, 1.0,
                GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                new Insets(5, 5, 5, 5), 0, 0));        
        panelRenumber.add(comboOrderBy2, Layout.getGBC(1, 3, 1, 1, 1.0, 1.0,
                GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                new Insets(5, 5, 5, 5), 0, 0));        
        panelRenumber.add(comboOrderBy3, Layout.getGBC(1, 4, 1, 1, 1.0, 1.0,
                GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                new Insets(5, 5, 5, 5), 0, 0));     
        panelRenumber.add(buttonRun,     Layout.getGBC(1, 5, 1, 1, 1.0, 1.0,
                GridBagConstraints.NORTHEAST, GridBagConstraints.NONE,
                new Insets(5, 5, 5, 5), 0, 0));        
        panelRenumber.add(progressBar,   Layout.getGBC(0, 6, 2, 1, 1.0, 1.0,
                GridBagConstraints.NORTHEAST, GridBagConstraints.HORIZONTAL,
                new Insets(5, 5, 5, 5), 0, 0));        
        
        /***********************************************************************
         * MAIN PANEL
         **********************************************************************/
        pSettings.setBorder(ComponentFactory.getTitleBorder(DcResources.getText("lblSettings")));
        panelRenumber.setBorder(ComponentFactory.getTitleBorder(DcResources.getText("lblRenumberAll")));
        
        buttonSave.setMnemonic('S');
        buttonCancel.setMnemonic('C');
        
        buttonCancel.addActionListener(new CloseAction());
        buttonSave.addActionListener(new SaveAction());          
        
        getContentPane().add(pSettings,     Layout.getGBC(0, 0, 1, 1, 5.0, 5.0,
                GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                new Insets(5, 5, 5, 5), 0, 0));
        getContentPane().add(buttonSave,    Layout.getGBC(0, 1, 1, 1, 1.0, 1.0,
                GridBagConstraints.NORTHEAST, GridBagConstraints.NONE,
                new Insets(5, 5, 5, 5), 0, 0));
        getContentPane().add(panelRenumber, Layout.getGBC(0, 2, 1, 1, 1.0, 1.0,
                GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                new Insets(5, 5, 5, 5), 0, 0));
        getContentPane().add(buttonCancel,  Layout.getGBC(0, 3, 1, 1, 1.0, 1.0,
                GridBagConstraints.SOUTHEAST, GridBagConstraints.NONE,
                new Insets(5, 5, 5, 5), 0, 0));        

        pack();
        setCenteredLocation();
    }
    
    private void renumber() {
        save();
        
        DcField field1 = comboOrderBy1.getSelectedIndex() > 0 ? (DcField) comboOrderBy1.getSelectedItem() : null;
        DcField field2 = comboOrderBy2.getSelectedIndex() > 0 ? (DcField) comboOrderBy2.getSelectedItem() : null;
        DcField field3 = comboOrderBy3.getSelectedIndex() > 0 ? (DcField) comboOrderBy3.getSelectedItem() : null;
        
        int total = 0;
        if (field1 != null)
            total++;
        if (field2 != null)
            total++;
        if (field3 != null)
            total++;      
        
        DcField[] fields = new DcField[total];
        
        if (fields.length >= 1)
            fields[0] = field1;
        if (fields.length >= 2)
            fields[1] = field2;
        if (fields.length >= 3)
            fields[2] = field3;       
        
        Renumberer renumberer = new Renumberer(fields, comboModus.getSelectedIndex());
        renumberer.start();
    }

    private class Renumberer extends Thread {
        
        private final DcField[] ordering;
        private final int modus;
        private DcModule module = DcModules.getCurrentModule();
        
        public Renumberer(DcField[] ordering, int modus) {
            this.ordering = ordering;
            this.modus = modus;
        }
        
        public void run() {
            boolean active = false;

            
            initiliazeUI(module);

            
            for (Iterator iter = module.getFields().iterator(); iter.hasNext(); ) {
                DcField field = (DcField) iter.next();

                IValueEnhancer[] enhancers = field.getValueEnhancers();
                
                for (int i = 0; i < enhancers.length; i++) {
                    if (enhancers[i].isEnabled() && enhancers[i] instanceof AutoIncrementer) {
                        active = true;
                        renumber((AutoIncrementer) enhancers[i], field);
                    }
                }
            }
            
            if (!active)
                new MessageBox(DcResources.getText("msgNoRenumbersFound"), MessageBox._ERROR);
            
        }
        
        private void initiliazeUI(DcModule module) {
            int totalEnhancers = 0;
            for (Iterator iter = module.getFields().iterator(); iter.hasNext(); ) {
                IValueEnhancer[] enhancers = ((DcField) iter.next()).getValueEnhancers();
                for (int i = 0; i < enhancers.length; i++) {
                    totalEnhancers = enhancers[i].isEnabled() && enhancers[i] instanceof AutoIncrementer ? 
                            totalEnhancers + 1 : totalEnhancers;
                }
            }            
            
            try {
                ResultSet rs = DatabaseManager.runQueryDirectUnclosed("SELECT COUNT(ID) AS TOTAL FROM " + module.getTableName(), true);
                int total = 0;
                while (rs.next()) {
                    total = rs.getInt("TOTAL");
                }
                rs.close();
                
                initProgressBar(total * totalEnhancers);
            } catch (Exception exp) {
                Messages.add(exp);
            }
        }
        
        private void renumber(AutoIncrementer incrementer, DcField field) {
            buttonRun.setEnabled(false);
            buttonSave.setEnabled(false);
            
            String order = "";
            for (int i = 0; i < ordering.length; i++) {
                if (i > 0)
                    order += ", ";
                
                order += ordering[i].getDatabaseFieldName();
            }
            
            String qry = "SELECT ID, " + field.getDatabaseFieldName() + " FROM " + module.getTableName();
            
            if (order.length() > 0)
                qry += " ORDER BY " + order; 
            
            
            String qryCurrent = "SELECT " + field.getDatabaseFieldName() + " FROM " + module.getTableName() + 
                                " WHERE " + field.getDatabaseFieldName() + " IS NOT NULL AND " +
                                field.getDatabaseFieldName() + " > 0 " +
                                "ORDER BY 1";

            Collection currentValues = new ArrayList();
            try {
                ResultSet rs = DatabaseManager.runQueryDirectUnclosed(qryCurrent, true);
                while (rs.next()) {
                    currentValues.add(new Integer(rs.getInt(field.getDatabaseFieldName())));
                }
                rs.close();

                rs = DatabaseManager.runQueryDirectUnclosed(qry, true);
                int counter = 0;
                while (rs.next()) {
                    String ID = rs.getString("ID");
                    int current = rs.getInt(field.getDatabaseFieldName());
                    
                    boolean allow = false;
                    if (modus == 1 && current == 0) {

                        counter = counter + incrementer.getStep();
                        
                        if (currentValues.contains(new Integer(counter))) {
                            boolean currentfound = false;
                            for (Iterator iter = currentValues.iterator(); iter.hasNext(); ) {
                                int x = ((Integer) iter.next()).intValue();
                                
                                while (!currentfound && x == counter) {
                                    counter += incrementer.getStep();
                                }
                            }
                        }
                        
                        allow = true;
                        
                    } else if (modus == 0) {
                        counter = counter + incrementer.getStep();
                        allow = true;
                    }
                    
                    if (allow) {
                        String updateQuery = "UPDATE " + module.getTableName() + 
                                             " SET " + field.getDatabaseFieldName() + " = " + counter  + 
                                             " WHERE ID = " + ID;
                        
                        DatabaseManager.runQueryDirectUnclosed(updateQuery, true);

                        DcObject dco = DataManager.getObject(module.getIndex(), ID);
                        dco.setValue(field.getIndex(), new Integer(counter));
                        dco.markAsUnchanged();

                        updateUI(module, field, ID, counter);
                    }
                    
                    updateProgressBar();
                }
                
                rs.close();
                
            } catch (Exception exp) {
                Messages.add(exp);
            }
            
            buttonRun.setEnabled(true);
            buttonSave.setEnabled(true);
        }
    }
    
    private void updateUI(DcModule module, DcField field, String ID, int number) {
        DcObject dco = module.getDcObject();
        dco.setValue(field.getIndex(), new Integer(number));
        module.getSearchView().updateRow(ID, dco, true, false, false);     
    }
    
    private class RenumberAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            renumber();
        }
    }    
    
    private class CloseAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            close();
        }
    }

    private class SaveAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            save();
        }
    }
}
