/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.dialogs;

import java.awt.Component;
import java.awt.Font;
import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.filechooser.FileFilter;

import net.sf.dc.core.Repository;
import net.sf.dc.settings.DcSettings;
import net.sf.ext.util.SwingUtil;

public class BrowserDialog extends JFrame {

    private final JFileChooser browser = new JFileChooser();

    public BrowserDialog(String sTitle) {
        browser.setMultiSelectionEnabled(false);
        this.setTitle(sTitle);
    }
    
    public BrowserDialog(String sTitle, FileFilter filter) {
        this(sTitle);
        
        if (filter != null) 
            browser.setFileFilter(filter);
    }

    public void setFont(Font font) {
        if (browser != null)
            browser.setFont(font);
    }

    public File showSelectDirectoryDialog(Component c, File file) {
        setCurrentDirectory(file);
        browser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        
        if (c != null) { 
        	browser.showOpenDialog(c);
        } else {
        	browser.showOpenDialog(SwingUtil.getRootFrame());
        }        
        
        File f = browser.getSelectedFile(); 
        rememberUsedDirectory(f);
        return f;
    }
    
    public File showCreateFileDialog(Component c, File file) {
        setCurrentDirectory(file);
        browser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        
        if (c != null) { 
        	browser.showSaveDialog(c);
        } else {
        	browser.showSaveDialog(SwingUtil.getRootFrame());
        }

        File f = browser.getSelectedFile(); 
        rememberUsedDirectory(f);
        return f;
    }
    
    public File showOpenFileDialog(Component c, File file) {
        setCurrentDirectory(file);
        browser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
        
        if (c != null) { 
        	browser.showOpenDialog(c);
        } else {
        	browser.showOpenDialog(SwingUtil.getRootFrame());
        }

        File f = browser.getSelectedFile(); 
        rememberUsedDirectory(f);
        return f;
    }
    
    private void setCurrentDirectory(File file) {
        if (file != null) {
            browser.setCurrentDirectory(file);
        } else {
            String s = DcSettings.getValueAsString(Repository.settings.stLastDirectoryUsed);
            file = new File(s);
            browser.setCurrentDirectory(file);
        }
    }
    
    private void rememberUsedDirectory(File file) {
        if (file != null) {
            String s = file.toString();

            try {
            
            	if (file.isFile() || !file.isDirectory())
            		s = s.substring(0, s.lastIndexOf(File.separator));
            
            } catch (Exception ignore) {}
            DcSettings.setStringAsValue(Repository.settings.stLastDirectoryUsed, s);
        }        
    }
}
