/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.dialogs;

import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcDialog;
import net.sf.dc.console.components.lists.DcObjectList;
import net.sf.dc.core.Repository;
import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.modules.DcMappingModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcMapping;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;

public class DcReferencesDialog extends DcDialog {
    
    private final Collection dcos;
    private DcObjectList list;
    private DcMappingModule mappingModule;
    
    private boolean saved = false;
    
    public DcReferencesDialog(Collection current, DcMappingModule mappingModule) {
        dcos = new ArrayList();
        this.mappingModule = mappingModule;
        
        current = current == null ? new ArrayList() : current;
        dcos.addAll(current);
        
        setTitle(DcResources.getText(Repository.systemlabels.sysGenrePlural));
        buildDialog();
        
        DcObject[] dcos = DataManager.get(mappingModule.getSimpleModuleIdx(), null);
        list.add(dcos);
        
        
        Collection selected = new ArrayList();
        for (Iterator iter = current.iterator(); iter.hasNext(); ) {
            DcMapping mapping = (DcMapping) iter.next();
            selected.add(mapping.getSimpleObject());
        }
        list.setSelected(selected);
        
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                close();
            }
        });
        
        pack();
        
        setSize(DcSettings.getValueAsDimension(Repository.settings.stReferencesDialogSize));
        setCenteredLocation();
        setModal(true);
    }
    
    public Collection getDcObjects() {
        return dcos;
    }
    
    public boolean isSaved() {
        return saved;
    }
    
    private void save() {
        dcos.clear();

        Collection c = list.getSelectedObjects();
        for (Iterator iter = c.iterator(); iter.hasNext(); ) {
            DcObject dco = (DcObject) iter.next();
            DcObject mapping = mappingModule.getDcObject();
            mapping.setValue(DcMapping._B_SIMPLE_OBJECT_ID, dco.getID());
            dcos.add(mapping);
        }
        
        saved = true;
        close();
    }
    
    private void close() {
        setVisible(false);
        DcSettings.setValue(Repository.settings.stReferencesDialogSize, getSize());
    }
    
    public void dispose() {
        list.clear();
        list = null;
        dcos.clear();
        mappingModule = null;
        super.dispose();
    }    
    
    private void buildDialog() {
        setLayout(Layout.getGBL());
        
        list = new DcObjectList(DcModules.getCurrentModule(), false, false, true);
        list.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        list.addMouseListener(new ListMouseListener());
        
        JScrollPane scroller = new JScrollPane(list);
        scroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
        scroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        
        JButton buttonSave = ComponentFactory.getButton(DcResources.getText("lblSave"));
        JButton buttonCancel = ComponentFactory.getButton(DcResources.getText("lblCancel"));
        
        JPanel panelActions = new JPanel();
        panelActions.add(buttonSave);
        panelActions.add(buttonCancel);
        
        buttonSave.addActionListener(new SaveAction());
        buttonCancel.addActionListener(new CancelAction());
        
        add(scroller,   Layout.getGBC( 0, 0, 1, 1, 20.0, 10.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                        new Insets( 0, 0, 0, 0), 0, 0));
        add(panelActions,   
                        Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.SOUTHEAST, GridBagConstraints.NONE,
                        new Insets( 5, 0, 5, 0), 0, 0));
    }
    
    private class SaveAction implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            save();
        }
    }     

    private class CancelAction implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            close();
        }
    }     

    private class ListMouseListener implements MouseListener {
        
        public void mouseReleased(MouseEvent e) {
            if (SwingUtilities.isRightMouseButton(e)) {
                if (list.getSelectedIndex() == -1) {
                    int index = list.locationToIndex(e.getPoint());
                    list.setSelectedIndex(index);
                }
            }
        }
        
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {}
    }    
}
