/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Iterator;

import javax.swing.ButtonGroup;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcDialog;
import net.sf.dc.console.components.DcLongTextField;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;

public class ItemTypeDialog extends DcDialog {

    private int selectedModule = -1;
    
    public ItemTypeDialog() {
        this(null);
    }
    
    public ItemTypeDialog(JFrame frame) {
        super(frame);
        setTitle(DcResources.getText("lblSelectModule"));
        
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                close();
            }
        });

        setModal(true);
        buildDialog();
    }
    
    public int getSelectedModule() {
        return selectedModule;
    }
    
    private void close() {
        DcSettings.setValue(Repository.settings.stModuleSelectDialogSize, getSize());
        dispose();
    }
    
    
    private void buildDialog() {
        getContentPane().setLayout(Layout.getGBL());
        
        //**********************************************************
        //Create Backup Panel
        //**********************************************************
        JPanel panelModules = new JPanel();
        panelModules.setLayout(Layout.getGBL());
        
        final ButtonGroup buttonGroup = new ButtonGroup();
        class SelectModuleAction implements ItemListener {
            public void itemStateChanged(ItemEvent ev) {
                String command = buttonGroup.getSelection().getActionCommand();
                selectedModule = Integer.parseInt(command);
            }
        }         
        
        DcLongTextField textHelp = ComponentFactory.getLongTextField(8000);
        textHelp.setBorder(null);
        JScrollPane scroller = new JScrollPane(textHelp);
        scroller.setBorder(null);
        scroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        scroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_NEVER);
        textHelp.setEditable(false);
        textHelp.setText(DcResources.getText("lblSelectModuleHelp"));
        panelModules.add(textHelp, Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets( 0, 5, 5, 5), 0, 0));        
        
        int y = 1;
        for (Iterator iter = DcModules.getModules(); iter.hasNext(); ) {
            DcModule module = (DcModule) iter.next();

            if (module.isTopModule() && module.getIndex() != DcModules._GENERIC) {
                JRadioButton radioButton = ComponentFactory.getRadioButton(module.getName(), module.getIcon(), "" + module.getIndex());
                
                radioButton.addActionListener(new OkAction());
                
                radioButton.addItemListener(new SelectModuleAction());
                buttonGroup.add(radioButton);
                panelModules.add(radioButton, Layout.getGBC( 0, y++, 1, 1, 1.0, 1.0
                        ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                         new Insets( 0, 5, 5, 5), 0, 0));
            }
        }

        //**********************************************************
        //Main panel
        //**********************************************************
        this.getContentPane().setLayout(Layout.getGBL());
        this.getContentPane().add(panelModules,  Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                                 ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                  new Insets( 5, 5, 5, 5), 0, 0));

        pack();
        Dimension size = DcSettings.getValueAsDimension(Repository.settings.stModuleSelectDialogSize);
        setSize(size);
        setCenteredLocation();
    }

    private class OkAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            Object source = e.getSource();
            if (source instanceof JRadioButton) {
                if (((JRadioButton) source).isSelected())
                    close();    
            }
        }
    }      
}
