/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcDialog;
import net.sf.dc.console.components.DcLookAndFeelSelector;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.Repository;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;

public class LookAndFeelDialog extends DcDialog {
    
    public LookAndFeelDialog() {
        super(DataCrow.mainFrame);
        
        setTitle(DcResources.getText("lblLookAndFeel"));
        setHelpIndex("dc.Settings.Laf");
        
        getContentPane().setLayout(Layout.getGBL());
        
        DcLookAndFeelSelector lafSelector = ComponentFactory.getLookAndFeelSelector();
        lafSelector.setParent(this);
        lafSelector.setValue(DcSettings.getValueAsLookAndFeel(Repository.settings.stLookAndFeel));
        
        JButton buttonCancel = ComponentFactory.getButton(DcResources.getText("lblCancel"));
        buttonCancel.addActionListener(new CloseAction());
        buttonCancel.setMnemonic('C');
        
        getContentPane().add(lafSelector,  Layout.getGBC( 0, 0, 1, 1, 10.0, 10.0
           ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
            new Insets( 5, 5, 5, 5), 0, 0));
        getContentPane().add(buttonCancel,  Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
           ,GridBagConstraints.NORTHEAST, GridBagConstraints.NONE,
            new Insets( 5, 5, 5, 5), 0, 0));

        pack();
        
        setSize(new Dimension(400, 300));
        
        setCenteredLocation();
    }

    private void exit() {
        dispose();
        setVisible(false);
    }
    
    private class CloseAction implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            exit();
        }
    }
}
