/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.dialogs;

import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.KeyStroke;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcDialog;
import net.sf.dc.console.components.DcFieldSelectorField;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;

public class OnlineUpdateOverwriteSettingsDialog extends DcDialog { 
    
    private final int module;
    
    private String settingsKey;
    private DcFieldSelectorField fs;
    
    public OnlineUpdateOverwriteSettingsDialog(JFrame parent, int module) {
        
        super(parent);
        this.module = module;
        
        setTitle(DcResources.getText("lblOnlineUpdateFieldSettings"));
        
        try {
            setModal(true);
            
            addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    close();
                }
            });
            
            settingsKey = DcModules.get(module).getOnlineUpdateFieldsSettingsKey();
            
            build();
            
            fs.setSelectedFields(DcSettings.getValueAsIntegerArray(settingsKey));
        } catch (Exception e) {
            Messages.add(e);
        }
        
        setSize(DcSettings.getValueAsDimension(Repository.settings.stOnlineUpdateOverwriteSettingsDialogSize));
        setCenteredLocation();
    }
    
    private void close() {
        DcSettings.setValue(Repository.settings.stOnlineUpdateOverwriteSettingsDialogSize, getSize());
        settingsKey = null;
        fs = null;
        dispose();
    }

    private void apply() {
        DcSettings.setValue(settingsKey, fs.getSelectedFieldIndices());
        close();
    }    
    
    private void build() {
        setLayout(Layout.getGBL());
        
        fs = new DcFieldSelectorField(module);
        
        getContentPane().add(fs, Layout.getGBC(0, 0, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                             new Insets(5, 5, 5, 5), 0, 0));        
        
        JPanel panelActions = new JPanel();
        panelActions.setLayout(new FlowLayout(FlowLayout.RIGHT));

        JButton buttonApply = ComponentFactory.getButton(DcResources.getText("lblApply"));
        JButton buttonCancel = ComponentFactory.getButton(DcResources.getText("lblCancel"));
        
        buttonApply.addActionListener(new ApplyAction());
        buttonCancel.addActionListener(new CancelAction());
        
        panelActions.add(buttonApply);  
        panelActions.add(buttonCancel);
        
        getContentPane().add(panelActions, Layout.getGBC(0, 1, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.SOUTHEAST, GridBagConstraints.NONE,
                             new Insets(0, 0, 5, 5), 0, 0));
        
        JMenu editMenu = createMenu();
        JMenuBar mb = ComponentFactory.getMenuBar();
        mb.add(editMenu);
        setJMenuBar(mb);
        
        pack();
    }
    
    private JMenu createMenu() {
        JMenu menu = ComponentFactory.getMenu("Edit");
        
        JMenuItem menuSelectAll = ComponentFactory.getMenuItem(DcResources.getText("lblSelectAll"));
        JMenuItem menuUnselectAll = ComponentFactory.getMenuItem(DcResources.getText("lblUnselectAll"));
        JMenuItem menuInvertSelection = ComponentFactory.getMenuItem(DcResources.getText("lblInvertSelection"));

        menuSelectAll.addActionListener(new SelectAllAction());
        menuUnselectAll.addActionListener(new UnselectAllAction());
        menuInvertSelection.addActionListener(new InvertSelectionAction());
        
        menuSelectAll.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_MASK));
        menuUnselectAll.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_U, InputEvent.CTRL_MASK));
        menuInvertSelection.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_I, InputEvent.CTRL_MASK));
        
        menu.add(menuSelectAll);
        menu.add(menuUnselectAll);
        menu.addSeparator();
        menu.add(menuInvertSelection);
        
        return menu;
    }    

    private class CancelAction implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            close();
        }
    }    
    
    private class ApplyAction implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            apply();
        }
    }
    
    private class SelectAllAction implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            fs.selectAll();
        }
    }

    private class InvertSelectionAction implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            fs.invertSelection();
        }
    }
    
    private class UnselectAllAction implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            fs.unselectAll();
        }
    }
}

