/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ScrollPaneConstants;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.exceptions.ValidationException;
import net.sf.dc.resources.DcResources;
import net.sf.ext.util.SwingUtil;

public class SaveQuestionBox extends JDialog {

    public static final int _CONTINUE = 0;
    public static final int _IGNORE = 1;
    public static final int _CANCEL = 2;

    private int result = -1;

    private JTextArea textMessage;
    private JButton buttonContinue;
    private JButton buttonIgnore;
    private JButton buttonCancel;

    private JPanel panel = new JPanel();

    private final JLabel labelIcon = ComponentFactory.getLabel("");

    public SaveQuestionBox(ValidationException vExp) {
        super(SwingUtil.getRootFrame());
        init(vExp);
    }

    public SaveQuestionBox(ValidationException vExp, JFrame parent) {
        super(parent);
        init(vExp);
    }

    public void close() {
        this.dispose();
    }

    public int getResult() {
        return result;
    }

    private void init(ValidationException vExp) {
        buildDialog();
        textMessage.setText(vExp.getMessage());

        setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);

        this.pack();
        this.setModal(true);

        if (DataCrow.mainFrame != null) {
            Dimension size = DataCrow.mainFrame.getSize();
            Point location = DataCrow.mainFrame.getLocation();
            int x = ((location.x + size.width - getSize().width) / 2);
            int y = ((location.y + size.height  - getSize().height) / 2);
            setLocation(x, y);
        } else {
        	setLocation(150, 150);
        }

        buttonContinue.requestFocus();
        this.setVisible(true);
    }

    private void buildDialog() {
        this.setResizable(false);
        this.getContentPane().setLayout(new GridLayout());

        // Input panel
        panel.setLayout(Layout.getGBL());

        textMessage = ComponentFactory.getTextArea();
        textMessage.setBackground(getBackground());
        textMessage.setEditable(false);
        JScrollPane scrollIn = new JScrollPane(textMessage);
        scrollIn.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        scrollIn.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
        scrollIn.setPreferredSize(new Dimension(350,50));
        scrollIn.setBorder(null);

        buttonContinue = ComponentFactory.getButton(DcResources.getText("lblContinue"), 100);
        buttonIgnore = ComponentFactory.getButton(DcResources.getText("lblIgnoreErrors"), 100);
        buttonCancel = ComponentFactory.getButton(DcResources.getText("lblCancel"), 100);

        buttonContinue.setToolTipText(DcResources.getText("tpContinue"));
        buttonIgnore.setToolTipText(DcResources.getText("tpIgnoreErrors"));
        buttonCancel.setToolTipText(DcResources.getText("tpCancel"));

        buttonContinue.addActionListener(new ContinueButtonAction());
        buttonIgnore.addActionListener(new IgnoreButtonAction());
        buttonCancel.addActionListener(new CancelButtonAction());

        labelIcon.setIcon(IconLibrary._icoQuestion);

        JLabel labelMessage = ComponentFactory.getLabel(DcResources.getText("lblItemCannotBeSaved"));

        panel.setLayout(Layout.getGBL());
        panel.add(labelMessage,     Layout.getGBC( 0, 0, 7, 1, 0.0, 0.0
                                   ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
								    new Insets(5, 5, 5, 5), 0, 0));
        panel.add(labelIcon,        Layout.getGBC( 0, 1, 1, 1, 0.0, 0.0
                                   ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                    new Insets(5, 5, 5, 5), 0, 0));
        panel.add(scrollIn,         Layout.getGBC( 1, 1, 6, 3, 90.0, 90.0
                                   ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
								    new Insets(5, 5, 5, 5), 0, 0));
        panel.add(buttonContinue,   Layout.getGBC( 0, 4, 2, 1, 0.0, 0.0
                                   ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
								    new Insets(5, 5, 5, 5), 0, 0));
        panel.add(buttonIgnore,     Layout.getGBC(  2, 4, 1, 1, 0.0, 0.0
                                   ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
								    new Insets(5, 5, 5, 5), 0, 0));
        panel.add(buttonCancel,     Layout.getGBC(  4, 4, 1, 1, 0.0, 0.0
        		                   ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
								    new Insets(5, 5, 5, 5), 0, 0));
        this.pack();
        this.setModal(true);
        this.getContentPane().add(panel);
    }

    private class ContinueButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            result = SaveQuestionBox._CONTINUE;
            close();
        }
    }

    private class IgnoreButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            result = SaveQuestionBox._IGNORE;
            close();
        }
    }

    private class CancelButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            result = SaveQuestionBox._CANCEL;
            close();
        }
    }
}