/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2007 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.dialogs;

import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JButton;
import javax.swing.JPanel;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcDialog;
import net.sf.dc.console.components.panels.SortingPanel;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;

public class SortingDialog extends DcDialog {

    private SortingPanel panelSorting;
    private int module;
    
    public SortingDialog(int module) {
        super(DataCrow.mainFrame);
        
        setTitle(DcResources.getText("lblSort"));
        
        this.module = module;
        this.panelSorting = new SortingPanel(module);
        
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                close();
            }
        });
        
        build();
        
        setSize(DcSettings.getValueAsDimension(Repository.settings.stSortDialogSize));
        setCenteredLocation();
    }
    
    private void close() {
        panelSorting.clear();
        panelSorting = null;
        DcSettings.setValue(Repository.settings.stSortDialogSize, getSize());
        dispose();
    }
    
    private void sort() {
        panelSorting.save();
        DcModules.get(module).getSearchView().sort();
    }
    
    private void build() {
        
        getContentPane().setLayout(Layout.getGBL());
        
        //**********************************************************
        //Action Panel
        //**********************************************************
        JPanel panelActions = new JPanel();
        JButton buttonApply = ComponentFactory.getButton(DcResources.getText("lblApply"));
        JButton buttonCancel = ComponentFactory.getButton(DcResources.getText("lblCancel"));
        
        buttonApply.setMnemonic('A');
        buttonCancel.setMnemonic('C');
        
        buttonApply.addActionListener(new SortAction());
        buttonCancel.addActionListener(new CancelAction());
        
        panelActions.add(buttonApply);
        panelActions.add(buttonCancel);
        
        //**********************************************************
        //Main Panel
        //**********************************************************
        getContentPane().add(panelSorting,  Layout.getGBC( 1, 1, 1, 1, 5.0, 5.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets( 0, 5, 5, 5), 0, 0));
        getContentPane().add(panelActions,  Layout.getGBC( 1, 1, 1, 1, 5.0, 5.0
                ,GridBagConstraints.SOUTHEAST, GridBagConstraints.NONE,
                 new Insets( 5, 5, 5, 5), 0, 0));
        
        pack();
    }
    
    private class CancelAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            close();
        }
    }
    
    private class SortAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            sort();
        }
    }    
}

