/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.dialogs;

import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.HashMap;

import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.KeyStroke;
import javax.swing.ScrollPaneConstants;
import javax.swing.text.DefaultEditorKit;
import javax.swing.text.JTextComponent;
import javax.swing.undo.UndoManager;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcDialog;
import net.sf.dc.console.components.DcLongTextField;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.ext.util.SwingUtil;
import net.sf.ext.util.Utilities;

public class TextDialog extends DcDialog {

    private DcLongTextField textArea = ComponentFactory.getTextArea();
    private JPanel panelText = new JPanel();
    private JPanel panelAction = new JPanel();

    private boolean success = true;

    private HashMap actions = new HashMap();

    protected UndoManager undo = new UndoManager();

    public TextDialog(String s, boolean edit) {
        super(SwingUtil.getRootFrame());
        buildDialog(edit);

        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                DcSettings.setValue(Repository.settings.stTextViewerSize, getSize());
                dispose();
            }
        });

        createActionTable(textArea);

        textArea.setEditable(edit);
        textArea.setEnabled(edit);
        textArea.setText(s);
        setSize(DcSettings.getValueAsDimension(Repository.settings.stTextViewerSize));
        setLocation(Utilities.getCenteredWindowLocation(getSize()));

        if (edit) {
            JMenu editMenu = createEditMenu();
            JMenuBar mb = ComponentFactory.getMenuBar();
            mb.add(editMenu);
            setJMenuBar(mb);
        }

        setModal(true);
        setVisible(true);
    }

    private JMenu createEditMenu() {
        JMenu menu = ComponentFactory.getMenu("Edit");

        JMenuItem menuUndo = ComponentFactory.getMenuItem(textArea.getUndoListener().getUndoAction());
        JMenuItem menuRedo = ComponentFactory.getMenuItem(textArea.getUndoListener().getRedoAction());
        JMenuItem menuCut = ComponentFactory.getMenuItem(IconLibrary._icoCut, DcResources.getText("lblCut"));
        JMenuItem menuCopy =ComponentFactory.getMenuItem(IconLibrary._icoCopy, DcResources.getText("lblCopy"));
        JMenuItem menuPaste = ComponentFactory.getMenuItem(IconLibrary._icoPaste, DcResources.getText("lblPaste"));
        JMenuItem menuSelectAll = ComponentFactory.getMenuItem(DcResources.getText("lblSelectAll"));

        menuCut.addActionListener(getActionByName(DefaultEditorKit.cutAction));
        menuCopy.addActionListener(getActionByName(DefaultEditorKit.copyAction));
        menuPaste.addActionListener(getActionByName(DefaultEditorKit.pasteAction));
        menuSelectAll.addActionListener(getActionByName(DefaultEditorKit.selectAllAction));
        
        menuUndo.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, InputEvent.CTRL_MASK));
        menuRedo.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Y, InputEvent.CTRL_MASK));
        menuCut.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, InputEvent.CTRL_MASK));
        menuCopy.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, InputEvent.CTRL_MASK));
        menuPaste.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P, InputEvent.CTRL_MASK));
        menuSelectAll.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A, InputEvent.CTRL_MASK));
        
        
        menu.add(menuCut);
        menu.add(menuCopy);
        menu.add(menuPaste);
        menu.addSeparator();
        menu.add(menuUndo);
        menu.add(menuRedo);
        menu.addSeparator();
        menu.add(menuSelectAll);

        return menu;
    }

    private void createActionTable(JTextComponent textComponent) {
        Action[] actionsArray = textComponent.getActions();
        for (int i = 0; i < actionsArray.length; i++) {
            Action a = actionsArray[i];
            actions.put(a.getValue(Action.NAME), a);
        }
    }

    private Action getActionByName(String name) {
        return (Action) actions.get(name);
    }

    public boolean isSuccess() {
        return success;
    }

    public String getText() {
        return textArea.getText();
    }

    private void buildDialog(boolean edit) {

        //**********************************************************
        //Text panel
        //**********************************************************
        panelText.setLayout(Layout.getGBL());

        JScrollPane pane = new JScrollPane(textArea);
        pane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        pane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);

        panelText.add(pane, Layout.getGBC( 0, 0, 1, 1, 40.0, 40.0
                     ,GridBagConstraints.NORTH, GridBagConstraints.BOTH,
                      new Insets( 5, 5, 5, 5), 0, 0));

        //**********************************************************
        //Action panel
        //**********************************************************
        panelAction.setLayout(new FlowLayout(FlowLayout.LEFT));
        JButton buttonCancel = ComponentFactory.getButton(DcResources.getText("lblCancel"));
        JButton buttonSave = ComponentFactory.getButton(DcResources.getText("lblSave"));

        buttonCancel.setMnemonic('C');
        buttonSave.setMnemonic('S');
        
        buttonCancel.addActionListener(new CloseAction());
        buttonSave.addActionListener(new ApplyAction());

        if (edit)
            panelAction.add(buttonSave);
        
        panelAction.add(buttonCancel);

        //**********************************************************
        //Main panel
        //**********************************************************
        this.getContentPane().setLayout(Layout.getGBL());
        setTitle(DcResources.getText("lblTextViewer"));

        this.getContentPane().add(panelText,  Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                                 ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                  new Insets( 0, 0, 0, 0), 0, 0));
        this.getContentPane().add(panelAction,Layout.getGBC( 0, 1, 1, 1, 0.0, 0.0
                                 ,GridBagConstraints.NORTHEAST, GridBagConstraints.NONE,
                                  new Insets( 0, 0, 0, 0), 0, 0));
    }

    private class CloseAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            success = false;
            dispose();
        }
    }

    private class ApplyAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            success = true;
            dispose();
        }
    }
}
