/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.dialogs;

import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Properties;
import java.util.Random;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLEditorKit;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcButton;
import net.sf.dc.console.components.DcCheckBox;
import net.sf.dc.console.components.DcDialog;
import net.sf.dc.console.components.DcTextPane;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.Repository;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.ext.util.Utilities;

public class TipOfTheDayDialog extends DcDialog {

    private DcTextPane tipPane = ComponentFactory.getTextPane();
    private DcCheckBox checkShowTips = ComponentFactory.getCheckBox(DcResources.getText("lblShowTipsOnStartup"));
    
    private HTMLEditorKit kit = new HTMLEditorKit();
    private HTMLDocument document = new HTMLDocument();

    private LinkedList tips = new LinkedList();
    
    private int currentTip = 0;
    
    public TipOfTheDayDialog() {
        super(DataCrow.mainFrame);
        setTitle(DcResources.getText("lblTipOfTheDay"));
        
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                close();
            }
        });
        
        build();
        setSize(400, 300);
        setCenteredLocation();
    }
    
    private void showNextTip() {
        currentTip = currentTip < tips.size() -1 ? currentTip + 1 : 0; 
        showNextTip(currentTip);
    }    

    private void showNextTip(int idx) {
        try {
            String tip = (String) tips.get(idx);
            Font font = DcSettings.getValueAsFont(Repository.settings.stSystemFontNormal);
            String hexColor = Utilities.getHexColor(ComponentFactory.getCurrentForegroundColor());
            
            String fontstyle = "";
            if (font.isItalic())
                fontstyle = ";font-style:italic";
            else if (font.isBold())
                fontstyle = ";font-weigth:bolder";
            
            String style = "style=" + '"' + 
                           ";font-family:" + font.getFamily() + 
                           ";font-size:" + font.getSize() + 
                           fontstyle + 
                           ";color:" + hexColor + ";" + '"';            
            
            String html = "<html><body " + style + ">" + tip + "</body></html>";
            tipPane.setText("");
            kit.insertHTML(document, 0, html, 0, 0, null);
        } catch (Exception exp) {
            Messages.add(exp);
        }
    }    

    
    private void build() {
        tipPane.setEditorKit(kit);
        tipPane.setDocument(document);
        tipPane.setEditable(false);
        tipPane.setBounds(1,1,1,1);
        
        getContentPane().setLayout(Layout.getGBL());
        
        DcButton buttonNext = ComponentFactory.getButton(DcResources.getText("lblNext"));
        DcButton buttonCancel = ComponentFactory.getButton(DcResources.getText("lblCancel"));
        
        buttonNext.addActionListener(new ShowNextTipAction());
        buttonCancel.addActionListener(new CloseAction());
        checkShowTips.addActionListener(new ToggleShowTipsAction());
        checkShowTips.setSelected(DcSettings.getValueAsBoolean(Repository.settings.stShowTipsOnStartup));
        
        JScrollPane scroller = new JScrollPane(tipPane);
        scroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_NEVER);
        scroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        
        JPanel panelActions = new JPanel();
        panelActions.setLayout(new FlowLayout(FlowLayout.RIGHT));
        panelActions.add(buttonNext);
        panelActions.add(buttonCancel);
        
        buttonNext.setMnemonic('N');
        buttonCancel.setMnemonic('C');
        
        getContentPane().add(scroller,      Layout.getGBC( 0, 0, 4, 1, 10.0, 10.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                 new Insets( 5, 5, 5, 5), 0, 0));
        getContentPane().add(checkShowTips, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                 new Insets( 5, 5, 5, 5), 0, 0));
        getContentPane().add(panelActions,  Layout.getGBC( 3, 1, 1, 1, 1.0, 1.0
                ,GridBagConstraints.SOUTHEAST, GridBagConstraints.NONE,
                 new Insets( 5, 5, 5, 0), 0, 0));
        
        setResizable(false);
        pack();

        initializeTips();
        currentTip = new Random().nextInt(tips.size());
        showNextTip(currentTip);
    }
    
    
    private void initializeTips() {
        Properties properties = DcResources.getResources();
        for (Iterator iter = properties.keySet().iterator(); iter.hasNext(); ) {
            String key = (String) iter.next();
            if (key.startsWith("tip")) 
                tips.add(properties.get(key));
        }
    }
    
    private void close() {
        tipPane = null;
        checkShowTips = null;
        tips = null;
        dispose();
    }    
    
    private class ToggleShowTipsAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            DcSettings.setValue(Repository.settings.stShowTipsOnStartup, new Boolean(checkShowTips.isSelected()));
        }
    }      

    private class ShowNextTipAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            showNextTip();
        }
    }    
    
    private class CloseAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            close();
        }
    }    
}
