/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcDialog;
import net.sf.dc.console.views.View;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.settings.definitions.DcFieldDefinition;
import net.sf.dc.settings.definitions.DcFieldDefinitions;

public class UpdateAllDialog extends DcDialog {

    private JButton buttonApply;
    private JButton buttonCancel;

    private Map labels = new HashMap();
    private Map fields = new HashMap();
    
    private boolean keepOnRunning = true;

    private JProgressBar progressBar = new JProgressBar();
    
    private View view;

    private final DcModule module;

    private JCheckBox checkSelectedItemsOnly;

    public UpdateAllDialog(View view) {
        super(DataCrow.mainFrame);

        this.view = view;
        this.module = view.getModule();

        setHelpIndex("dc.Tools.UpdateAll");

        setTitle(DcResources.getText("lblUpdateAll"));
        buildDialog(module);

        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                finish();
            }
        });

        setSize(DcSettings.getValueAsDimension(
                Repository.settings.stUpdateAllDialogSize));

        checkSelectedItemsOnly.setSelected(
                DcSettings.getValueAsBoolean(Repository.settings.stUpdateAllSelectedItemsOnly));

        setCenteredLocation();
    }

    private void updateAll() {
        Updater updater = new Updater();
        updater.start();
    }

    public void initProgressBar(int maxValue) {
        progressBar.setValue(0);
        progressBar.setMaximum(maxValue);
    }

    public void updateProgressBar(int value) {
        progressBar.setValue(value);
    }    
    
    private class Updater extends Thread {
        
        public void run() {
            DcObject dco = getValues();
            int[] rows;
    
            if (isUpdateSelectedItemsOnly()) {
                rows = view.getSelectedRows();
            } else {
                rows = new int[view.getRows()];
                for (int i = 0; i < rows.length; i++) {
                    rows[i] = i;
                }
            }

            initProgressBar(rows.length);
            for (int i = 0; i < rows.length && keepOnRunning; i++) {
                view.updateRow(rows[i], dco, true, false, true);
                updateProgressBar(i + 1);

                try {
                    sleep(20);
                } catch (Exception ignore) {}
            }
            
            finish();
        }
    }

    public boolean isUpdateSelectedItemsOnly() {
    	return checkSelectedItemsOnly.isSelected();
    }

    public DcObject getValues() {
        DcObject dco = module.getDcObject();
        int[] indices = module.getFieldIndices();
        for (int i = 0; i < indices.length; i++) {
            DcField field = module.getField(indices[i]);
            int fieldType = field.getFieldType() == ComponentFactory._LONGTEXTFIELD ?
                                ComponentFactory._SHORTTEXTFIELD : field.getFieldType();

            JComponent component = (JComponent) fields.get(field);
            if (component != null) {
                Object value = ComponentFactory.getComponentValue(fieldType, component);
    
                if (value != null && !value.toString().equals(""))
                    dco.setValue(field.getIndex(), value);
            }
        }

        return dco;
    }

    private void finish() {
        DcSettings.setValue(Repository.settings.stUpdateAllDialogSize, getSize());
        
        if (checkSelectedItemsOnly != null) {
            DcSettings.setValue(Repository.settings.stUpdateAllSelectedItemsOnly,
                                new Boolean(checkSelectedItemsOnly.isSelected()));
            labels.clear();
            fields.clear();
        }

        keepOnRunning = false;
        
        buttonApply = null;
        buttonCancel = null;
        
        labels = null;
        
        fields = null;
        view = null;
        checkSelectedItemsOnly = null;

        dispose();
    }

    private void buildDialog(DcModule module) {
        //**********************************************************
        //Input panel
        //**********************************************************
        JPanel panelInput = new JPanel();
        panelInput.setLayout(Layout.getGBL());

        DcFieldDefinitions definitions = (DcFieldDefinitions) 
            DcSettings.getValue(module.getFieldDefinitionsSettingsKey());
        
        int i = 0;
        for (Iterator iter = definitions.iterator(); iter.hasNext(); i++) {
            DcFieldDefinition definition = (DcFieldDefinition) iter.next();
            DcField field = module.getField(definition.getIndex());
            
            JLabel label = ComponentFactory.getLabel(field.getLabel());

            int fieldType = field.getFieldType() == ComponentFactory._LONGTEXTFIELD ?
                                ComponentFactory._SHORTTEXTFIELD : field.getFieldType();

            JComponent component = ComponentFactory.getComponent(
                    field.getModRef(), fieldType, field.getLabel(), field.getMaximumLength());

            fields.put(field, component);
            labels.put(field, label);

            component.setPreferredSize(new Dimension(100, 20));

            boolean visible = (!field.isUiOnly() || field.getValueType() == Repository.datatypes._DCOBJECTCOLLECTION) && 
                               field.isEnabled() && field.getIndex() != DcObject._ID;
            
            label.setVisible(visible);
            component.setVisible(visible);

            panelInput.add(label       , Layout.getGBC(0, i, 1, 1, 1.0, 1.0
                    , GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                      new Insets(0, 5, 0, 5), 0, 0));
            panelInput.add(component   , Layout.getGBC(1, i, 1, 1, 10.0, 10.0
                    , GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                      new Insets(0, 5, 0, 5), 0, 0));
        }

        //**********************************************************
        //Options panel
        //**********************************************************
        checkSelectedItemsOnly = ComponentFactory.getCheckBox(DcResources.getText("lblSelectedItemsOnly"));
        JPanel panelOptions = new JPanel();
        panelOptions.setLayout(Layout.getGBL());

        panelOptions.add(checkSelectedItemsOnly    , Layout.getGBC(0, 0, 1, 1, 0.0, 0.0
                       , GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                         new Insets(5, 5, 5, 5), 0, 0));

        //**********************************************************
        //Action panel
        //**********************************************************
        JPanel panelActions = new JPanel();
        panelActions.setLayout(Layout.getGBL());

        buttonApply = ComponentFactory.getButton(DcResources.getText("lblApply"));
        buttonCancel = ComponentFactory.getButton(DcResources.getText("lblCancel"));

        buttonApply.addActionListener(new ApplyAction());
        buttonCancel.addActionListener(new CancelAction());

        buttonApply.setMnemonic(KeyEvent.VK_A);
        buttonCancel.setMnemonic(KeyEvent.VK_C);

        panelActions.add(buttonApply    , Layout.getGBC(0, 0, 1, 1, 0.0, 0.0
                                        , GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                          new Insets(5, 5, 5, 5), 0, 0));
        panelActions.add(buttonCancel   , Layout.getGBC(1, 0, 1, 1, 0.0, 0.0
                                        , GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                          new Insets(5, 5, 5, 5), 0, 0));

        
        //**********************************************************
        //Progress panel
        //**********************************************************        
        JPanel panelProgress = new JPanel();
        panelProgress.setLayout(Layout.getGBL());
        panelProgress.add(progressBar, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                          new Insets(5, 5, 5, 5), 0, 0));        
        
        //**********************************************************
        //Main panel
        //**********************************************************
        this.getContentPane().setLayout(Layout.getGBL());
        this.getContentPane().add(panelInput  ,Layout.getGBC(0, 0, 1, 1, 10.0, 10.0
                                              ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                               new Insets(5, 5, 5, 5), 0, 0));
        this.getContentPane().add(panelOptions,Layout.getGBC(0, 1, 1, 1, 1.0, 1.0
                                              ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                                               new Insets(10, 5, 10, 5), 0, 0));
        this.getContentPane().add(panelActions,Layout.getGBC(0, 2, 1, 1, 1.0, 1.0
                                              ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                                               new Insets(5, 5, 5, 5), 0, 0));
        this.getContentPane().add(panelProgress, Layout.getGBC( 0, 3, 1, 1, 1.0, 1.0
                                              ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                              new Insets( 0, 0, 0, 0), 0, 0));        

        pack();

        Collection removeFields = new ArrayList();
        removeFields.addAll(module.getDcObject().getHiddenFields());
        removeFields.addAll(module.getDcObject().getNotSearchableFields());
    }

    private class CancelAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            finish();
        }
    }

    private class ApplyAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            updateAll();
        }
    }
}
