/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.expert;

import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcFrame;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;

public class ExpertForm extends DcFrame {

    public ExpertForm(JFrame parent) {
        super(DcResources.getText("lblExpertUser"), IconLibrary._icoSettings, parent);
        
        buildForm();
        
        setHelpIndex("dc.Tools.ExpertUser");
        
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                finish();
            }
        });
    }
    
    private void finish() {
        DcSettings.setValue(Repository.settings.stExpertFormSize, getSize());
        dispose();
    }
    
    private void buildForm() {
        JTabbedPane tabbedPane = ComponentFactory.getTabbedPane();
        
    	SystemInfoPanel systemInfoPanel = new SystemInfoPanel();
        MaintenancePanel maintenancePanel = new MaintenancePanel();
        PerformancePanel performancePanel = new PerformancePanel();
        QueryPanel queryPanel = new QueryPanel();
        
        JPanel systemPanel = new JPanel();
        systemPanel.setLayout(Layout.getGBL());
        
        systemInfoPanel.setBorder(ComponentFactory.getTitleBorder(DcResources.getText("lblSystemInformation")));
        maintenancePanel.setBorder(ComponentFactory.getTitleBorder(DcResources.getText("lblMaintenance")));
        performancePanel.setBorder(ComponentFactory.getTitleBorder(DcResources.getText("lblPerformanceTweaking")));
        
        systemPanel.add(systemInfoPanel,  Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                 ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                  new Insets(5, 5, 5, 5), 0, 0));
        systemPanel.add(maintenancePanel, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                 ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                  new Insets(5, 5, 5, 5), 0, 0));
        systemPanel.add(performancePanel, Layout.getGBC( 0, 2, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets(5, 5, 5, 5), 0, 0));
        
        tabbedPane.addTab(DcResources.getText("lblSystem"), IconLibrary._icoSettings, systemPanel);
        tabbedPane.addTab(DcResources.getText("lblSQLTool"), IconLibrary._icoSQLTool, queryPanel);

        getContentPane().setLayout(Layout.getGBL());
        getContentPane().add(tabbedPane, Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
        		            ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
							 new Insets(0, 0, 0, 0), 0, 0));
        
        pack();
        setSize(DcSettings.getValueAsDimension(Repository.settings.stExpertFormSize));
        setCenteredLocation();
    }
}
