/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.expert;

import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.RandomAccessFile;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Collection;
import java.util.Enumeration;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JEditorPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ScrollPaneConstants;
import javax.swing.ToolTipManager;
import javax.swing.table.TableColumn;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.console.menu.DcPopupMenu;
import net.sf.dc.console.views.table.TableBaseView;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DcResources;
import net.sf.dc.wf.WorkFlow;
import net.sf.ext.util.Converter;

public class QueryPanel extends TableBaseView {

    private JEditorPane textInput;
	private JButton buttonRunSql;
	private JButton buttonClear;

	private JComboBox comboSQLCommands;

    public QueryPanel() {
        super(null, null, null);
        setHelpIndex("dc.Tools.QueryTool");
        buildPanel();
    }

    private void fillTable(ResultSet result) throws Exception {
        clearTable();

        int columns = result.getMetaData().getColumnCount();


        table.setColumnCount(columns);
        int counter = 1;
        for (Enumeration enumerator = table.getColumnModel().getColumns(); enumerator.hasMoreElements(); counter++) {
            TableColumn column = (TableColumn) enumerator.nextElement();
            column.setHeaderValue(result.getMetaData().getColumnName(counter).toLowerCase());
        }

        Object[] values = new String[columns];
        while (result.next()) {
            for (int i = 0; i < columns; i++) {
                values[i] = result.getString((result.getMetaData().getColumnName(i + 1)));
            }
            table.addRow(values);
        }

        // close the result set
        try {
            result.close();
        } catch (SQLException sqlExp) {
            Messages.add(sqlExp);
        }

        // apply the correct headers (colors and such)
        table.applyHeaders();
        setStatus(DcResources.getText("msgSQLTableHasBeenFilled"));
    }

    public DcObject getObjectForRow(int row) {
    	return null;
    }

    public DcPopupMenu getPopupMenu() {
        return null;
    }

    public void showQuickView(boolean b) {}
    public void open() {}
    public void delete() {}
    public void createReport() {}
    public void add(Collection c) {}
    public Collection getSelectedObjects() {
        return null;
    }

    public DcObject getSelectedObject() {
        return null;
    }

    public DcObject getDcObject(String ID) {
        return null;
    }
    
    public void save() {}
    public void saveSelected() {}

    protected void runQuery() {
        String sQuery   = textInput.getText().trim();

        if (sQuery.trim().equals("")){
            new MessageBox( DcResources.getText("msgNoInput"), MessageBox._WARNING);
            return;
        }

        try {
        	ResultSet result = WorkFlow.runQueryDirectUnclosed(sQuery);

            boolean empty = false;
            if (result != null) {
                try {
                    result.isLast();
                } catch (Exception exp) {
                	empty = true;
                }
            }

            if (empty) {
                new MessageBox(DcResources.getText("msgQueryWasSuccessFull"), MessageBox._INFORMATION);
            } else {
                fillTable(result);
            }

            addQueryToComboBox(sQuery);

            saveDataToFile();
        } catch (Exception exp) {
            Messages.add(exp);
            new MessageBox(exp.toString(), MessageBox._ERROR);
        }
    }

    private void addQueryToComboBox(String sNewQuery) {
        sNewQuery = Converter.deleteEntersFromString(sNewQuery.trim());

        boolean found = false;
        for (int i = 0; i < comboSQLCommands.getItemCount(); i++) {
            Object o = comboSQLCommands.getItemAt(i);
            if (o != null && o instanceof QueryObject) {
            	String s = ((QueryObject) o).getQryString();
            	if (s.toLowerCase().equals(sNewQuery.toLowerCase())) {
            		found = true;
            		break;
                }
            }
        }
        if (!found) {
            QueryObject o = new QueryObject(sNewQuery);
            comboSQLCommands.addItem(o);
        }
    }

    private void fillQueryComboBox() {
        File queryFile = new File(DataCrow.baseDir + "data/data_crow_queries.txt");

        try {
            if (queryFile.exists()) {
                RandomAccessFile fileAccess  = new RandomAccessFile(queryFile, "rw");
                long filePointer = 0;
                long fileLength  = queryFile.length();
                comboSQLCommands.addItem("");
                while (filePointer < fileLength) {
                    String query = fileAccess.readLine().trim();
                    addQueryToComboBox(query);
                    filePointer = fileAccess.getFilePointer();
                }
                fileAccess.close();
            } else if (!queryFile.exists()) {
                Messages.add(DcResources.getText("msgFileNotFound", queryFile.toString()), Messages._STANDARD);
                RandomAccessFile fileAccess  = new RandomAccessFile(queryFile, "rw");
                fileAccess.write(Repository.database._PREDEFINEDQRY.getBytes());
                fileAccess.close();
                fillQueryComboBox();
            }
        } catch (Exception exp) {
            comboSQLCommands.addItem("");
        	Messages.add(exp);
        }
        saveDataToFile();
    }

    protected void saveDataToFile() {
        File queryFile = new File(DataCrow.baseDir + "data/data_crow_queries.txt");
        if (queryFile.exists()) {
            queryFile.delete();
            queryFile = new File(DataCrow.baseDir + "data/data_crow_queries.txt");
        }

        try {
            queryFile.createNewFile();
            RandomAccessFile access  = new RandomAccessFile(queryFile, "rw");

            for (int i = 0; i < comboSQLCommands.getItemCount(); i++) {
                Object o = comboSQLCommands.getItemAt(i);
                if (o != null && o instanceof QueryObject) {
                    String query = ((QueryObject) o).getQryString();
                    query = Converter.deleteEntersFromString(query);
                    access.writeBytes(query + '\n');
                }
            }
        } catch (Exception exp) {
            Messages.add(DcResources.getText("msgFileSaveError", queryFile.toString()), Messages._ERROR);
            Messages.add(exp);
        }
        Messages.add(DcResources.getText("msgFileSaved", queryFile.toString()), Messages._STANDARD);
    }

    protected void setQuery() {
        Object o = comboSQLCommands.getSelectedItem();
        if (o != null) {
            if (o instanceof QueryObject) {
            	QueryObject qry = (QueryObject) o ;
            	textInput.setText(qry.getQryString());
            }
        }
    }

    protected void clearPanel() {
        textInput.setText("");
        clearTable();
    }

    public void clearTable() {
        table.clearTable();
    }

    private void buildPanel() {
        //**********************************************************
        //SQL panel
        //**********************************************************
        JPanel panelSQL = new JPanel();
        panelSQL.setLayout(Layout.getGBL());

        textInput = ComponentFactory.getTextPane();

        JScrollPane scrollIn = new JScrollPane(textInput);
        scrollIn.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        scrollIn.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);

        panelSQL.add(scrollIn, Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                     new Insets(0, 5, 5, 5), 0, 0));

        //**********************************************************
        //SQL panel
        //**********************************************************
        JPanel panelQueries = new JPanel();
        panelQueries.setLayout(Layout.getGBL());

        comboSQLCommands = ComponentFactory.getComboBox();
        comboSQLCommands.addItemListener(new PredefineQueryAction());
        fillQueryComboBox();

        panelQueries.add(comboSQLCommands, Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                        ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                         new Insets(5, 5, 5, 5), 0, 0));

        //**********************************************************
        //Result panel
        //**********************************************************
        JPanel panelResult = new JPanel();
        panelResult.setLayout(Layout.getGBL());
        table = ComponentFactory.getDCTable(true, false);
        table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);

        JScrollPane scrollOut = new JScrollPane(table);
        scrollOut.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        scrollOut.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);

        panelResult.add(scrollOut, Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                        new Insets(5, 5, 5, 5), 0, 0));

        //**********************************************************
        //Action panel
        //**********************************************************
        JPanel panelActions = new JPanel();
        panelActions.setLayout(Layout.getGBL());

        buttonRunSql = ComponentFactory.getButton(DcResources.getText("lblRun"));
        buttonClear = ComponentFactory.getButton(DcResources.getText("lblClear"));

        buttonRunSql.setMnemonic(KeyEvent.VK_R);
        buttonClear.setMnemonic(KeyEvent.VK_C);

        buttonRunSql.addActionListener(new QueryButtonAction());
        buttonClear.addActionListener(new ClearButtonAction());
        buttonRunSql.setToolTipText(DcResources.getText("tpRunSQL"));
        buttonClear.setToolTipText(DcResources.getText("tpClear"));

        panelActions.add(buttonRunSql, Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                        ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                         new Insets(0, 5, 0, 5), 0, 0));
        panelActions.add(buttonClear, Layout.getGBC( 1, 0, 1, 1, 1.0, 1.0
                        ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                         new Insets(0, 0, 0, 5), 0, 0));

        //**********************************************************
        //Main panel
        //**********************************************************
        setLayout(Layout.getGBL());

        panelSQL.setBorder(ComponentFactory.getTitleBorder(DcResources.getText("lblQueryIn")));
        panelResult.setBorder(ComponentFactory.getTitleBorder(DcResources.getText("lblQueryOut")));
        panelQueries.setBorder(ComponentFactory.getTitleBorder(DcResources.getText("lblStoredSQL")));

        add(  panelSQL,     Layout.getGBC( 0, 0, 10, 1, 10.0, 10.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                            new Insets(5, 5, 5, 5), 0, 0));
        add(  panelQueries, Layout.getGBC( 0, 1, 10, 1, 1.0, 1.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                            new Insets(5, 5, 5, 5), 0, 0));
        add(  panelResult,  Layout.getGBC( 0, 2, 10, 1, 20.0, 20.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                            new Insets(5, 5, 5, 5), 0, 0));
        add(  panelActions, Layout.getGBC( 0, 3, 1, 1,  1.0, 1.0
                           ,GridBagConstraints.NORTHEAST, GridBagConstraints.NONE,
                            new Insets(5, 5, 5, 5), 0, 0));
        add(  panelStatus,  Layout.getGBC( 0, 4, 10, 1, 1.0, 1.0
                           ,GridBagConstraints.SOUTH, GridBagConstraints.HORIZONTAL,
                            new Insets(0, 5, 0, 5), 0, 0));

        ToolTipManager.sharedInstance().registerComponent(table);
    }



    // ****************************************************************************
    // INNER CLASSES
    // ****************************************************************************
    private class QueryObject {

        private final String query;

        public QueryObject(String query) {
            this.query = query;
        }

        public String getQryString() {
            return query;
        }

        public String toString() {
        	return query.length() > 50 ? query.substring(0, 50) + "..." : query;
        }
    }

    private class QueryButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            runQuery();
        }
    }

    private class ClearButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            clearPanel();
        }
    }

    private class PredefineQueryAction implements ItemListener {
        public void itemStateChanged(ItemEvent e) {
            setQuery();
        }
    }

    public void removeFromCache(String id) {}

    public boolean isChangesSaved() {
        return true;
    }

    public void setParentID(String ID) {
    }

    public boolean changesSaved() {
        return true;
    }

    public void applyGrouping(int[] fields) {}

    public void undoGrouping() {}
}