/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.forms;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcDialog;
import net.sf.dc.console.components.DcEditableComboBox;
import net.sf.dc.console.components.DcRatingComboBox;
import net.sf.dc.console.components.DcShortTextField;
import net.sf.dc.console.components.panels.SortingPanel;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.console.dialogs.QuestionBox;
import net.sf.dc.console.views.MasterView;
import net.sf.dc.console.views.View;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.data.DataFilter;
import net.sf.dc.core.data.DataFilterOptions;
import net.sf.dc.core.data.DataFilters;
import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.modules.DcMediaModule;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcMediaObject;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.settings.definitions.DcFieldDefinition;
import net.sf.dc.settings.definitions.DcFieldDefinitions;

public class FilterForm extends DcDialog {

    private final int modIdx;
    
    private final Map fields = new HashMap();
    private final Map labels = new HashMap();
    
    private final MasterView parent;
    
    private final JPanel panelTech = new JPanel();
    private final JPanel panelInfo = new JPanel();
    private final JPanel panelLoan = new JPanel();
    private final JPanel panelActions = new JPanel();
    
    private final JPanel dummy1 = new JPanel();
    private final JPanel dummy2 = new JPanel();

    private final JButton buttonDeleteFilter = ComponentFactory.getButton(DcResources.getText("lblDelete"));
    private final JButton buttonSaveFilter = ComponentFactory.getButton(DcResources.getText("lblSave"));
    private final JButton buttonSaveNewFilter = ComponentFactory.getButton(DcResources.getText("lblSaveNew"));
    
    private final JButton buttonApply = ComponentFactory.getButton(DcResources.getText("lblApply"));
    private final JButton buttonCancel = ComponentFactory.getButton(DcResources.getText("lblCancel"));
    private final JButton buttonReset = ComponentFactory.getButton(DcResources.getText("lblReset"));

    private final JComboBox comboFilters = ComponentFactory.getComboBox(); 
    
    private final JTabbedPane tabbedPane = ComponentFactory.getTabbedPane();
    
    private final SearchConditionPanel panelConditions;
    private final SearchOptionsPanel panelOptions;
    
    public FilterForm(DcModule module, MasterView parent) {
        super(DataCrow.mainFrame);
        
        setTitle(DcResources.getText("lblFilter"));
        setHelpIndex("dc.Searching");
        
        this.parent = parent;
        this.modIdx = module.getIndex();
        
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                close();
            }
        });        
        
        panelConditions = new SearchConditionPanel();
        panelOptions = new SearchOptionsPanel(module);
        
        buildForm(); 
        
        setCenteredLocation();
        setModal(false);
    }

    public void setFont(Font font) {
        Font fontNormal = DcSettings.getValueAsFont(Repository.settings.stSystemFontNormal);
        Font fontSystem = DcSettings.getValueAsFont(Repository.settings.stSystemFontNormal);

        super.setFont(fontNormal);
        
        if (tabbedPane != null) {
            tabbedPane.setFont(fontSystem);
    
            panelOptions.setFont(font);
            buttonCancel.setFont(fontSystem);
            buttonReset.setFont(fontSystem);
            buttonApply.setFont(fontSystem);

            for (Iterator iter = fields.values().iterator(); iter.hasNext(); ) {
                JComponent component = (JComponent) iter.next();
                component.setFont(fontNormal);
            }
    
            for (Iterator iter = labels.values().iterator(); iter.hasNext(); ) {
                JLabel component = (JLabel) iter.next();
                component.setFont(fontSystem);
            }
        }
    }    
    
    public void setVisible(boolean b) {
        if (b) { 
            panelConditions.applySettings();
            DcModule module = DcModules.get(modIdx);
            if (module instanceof DcMediaModule) {
                JComponent comp = (JComponent) fields.get(module.getDcObject().getField(DcMediaObject._A_TITLE));
                comp.requestFocusInWindow();
            }
            
            panelOptions.applyOrderSettings(DcSettings.getValueAsStringArray(
                    DcModules.get(modIdx).getSearchOrderSettingsKey()));
        }
        
        super.setVisible(b);
    }    
    
    public void searchValues() {
        SearchTread thread = new SearchTread();
        thread.start();
    }   
    
    public DataFilter toDataFilter() {
        return new DataFilter(modIdx, panelConditions.getSearchValues(), panelOptions.getOptions());
    }
    
    private class SearchTread extends Thread {
        
        public void run() {
            View view = parent.getCurrent();
            
            boolean saved = view.isChangesSaved();
            
            if (!saved && view.getCurrentTask() != null) {
                
                new QuestionBox(DcResources.getText("msgNotSaved"));
                if (QuestionBox.result.equals(Boolean.TRUE))
                    view.save();
                
                try {
                    view.getCurrentTask().join();
                } catch (Exception exp) {
                    Messages.add(exp);
                }
            }
            
            view.markAsUnchanged();
            
            DataFilter df = toDataFilter(); 
            
            DataFilters.setCurrent(modIdx, df);
                
            DataManager.bindData(parent, modIdx, df, true);

            parent.setStatus(DcResources.getText("msgSearchHasBeenExecuted"));

            addToHistory();
            
            DataCrow.mainFrame.setSelectedTab(0);        
            
            close();
        }
    }
    
    private void deleteFilter() {
        DataFilter df = getCurrent();
        DataFilters.delete(df);
        comboFilters.removeItem(df);
    }    
    
    private void saveFilter(boolean asNew) {
        DataFilter df = toDataFilter();
        if (asNew) {
            
            final DcDialog dialog = new DcDialog(DataCrow.mainFrame);
            DcShortTextField text = ComponentFactory.getShortTextField(50);
            
            text.setPreferredSize(new Dimension(200, 24));
            
            dialog.getContentPane().setLayout(Layout.getGBL());
            dialog.getContentPane().add(ComponentFactory.getLabel(DcResources.getText("lblName")),
                    Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                   ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                    new Insets(5, 5, 5, 5), 0, 0));
            dialog.getContentPane().add(text, Layout.getGBC( 1, 0, 1, 1, 1.0, 1.0,
                    GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                    new Insets(5, 5, 5, 5), 0, 0));

            
            JButton buttonOk = ComponentFactory.getButton(DcResources.getText("lblOK"));
            
            buttonOk.addActionListener(new ActionListener() 
                {   public void actionPerformed(ActionEvent ae) {
                        dialog.dispose();
                    }
                });
            
            dialog.getContentPane().add(buttonOk, Layout.getGBC( 2, 0, 1, 1, 1.0, 1.0,
                    GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                    new Insets(5, 5, 5, 5), 0, 0));
            
            dialog.setModal(true);
            dialog.setTitle(DcResources.getText("msgEnterFilterName"));
            dialog.pack();
            dialog.setPreferredSize(new Dimension(300, 50));
            dialog.setCenteredLocation();
            
            dialog.setVisible(true);
            
            String name = text.getText();
            if (name.trim().length() == 0) {
                new MessageBox("Could not save, name not filled", MessageBox._WARNING);
            } else {
                df.setName(name);
                DataFilters.add(df);
                comboFilters.addItem(df);
                comboFilters.setSelectedItem(df);
            }
        } else {
            df.setName(getCurrent().getName());
            deleteFilter();
            DataFilters.add(df);
            comboFilters.addItem(df);
            comboFilters.setSelectedItem(df);
        }
        DataCrow.mainFrame.updateQuickFilterBar();
    }    
    
    private void resetConditions() {
        for (Iterator iter = fields.values().iterator(); iter.hasNext(); ) {
            JComboBox comp = (JComboBox) iter.next();
            comp.setSelectedIndex(0);
            comp.getEditor().setItem("");
            if (comp instanceof DcEditableComboBox) {
                DcEditableComboBox combo = (DcEditableComboBox) comp;
                combo.storeValue();
            }
        }         
    }
    
    private DataFilter getCurrent() {
        DataFilter df = comboFilters.getSelectedIndex() != 0 && comboFilters.getSelectedIndex() != -1 ?
                            (DataFilter) comboFilters.getSelectedItem() : null;
        return df;
    }

    private void applySelectedFilter() {
        DataFilter df = getCurrent();
        resetConditions();
        if (df != null) {
            panelOptions.checkComplyAll.setSelected(df.complyAll());
            panelOptions.checkPrecise.setSelected(df.exactMatch());
            
            if (df.getConditions() != null)
                panelConditions.setSearchValues(df.getConditions());
            
            panelOptions.applyOrderSettings(df.getOrdering());
        }
    }    
    
    private void addToHistory() {
        for (Iterator iter = fields.values().iterator(); iter.hasNext(); ) {
            JComboBox comp = (JComboBox) iter.next();
            if (comp instanceof DcEditableComboBox) {
                DcEditableComboBox combo = (DcEditableComboBox) comp;
                combo.storeValue();
            }
        }
    }
    
    private void buildForm() {
        getContentPane().setLayout(Layout.getGBL());
        
        getContentPane().add( getFilterPanel(), Layout.getGBC( 0, 0, 2, 1, 2.0, 2.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                  new Insets(5, 5, 5,  5), 0, 0));        
        getContentPane().add( panelConditions,  Layout.getGBC( 0, 1, 2, 1, 2.0, 2.0
                 ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
				  new Insets(5, 5, 5,  5), 0, 0));
        getContentPane().add( panelOptions,     Layout.getGBC( 0, 2, 2, 1, 2.0, 2.0
	             ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
  				  new Insets(5, 5, 5,  5), 0, 0));
        
        buttonApply.setToolTipText(DcResources.getText("tpSearch"));
        buttonApply.addActionListener(new SearchButtonAction());
        buttonApply.requestFocus();
        buttonCancel.addActionListener(new CloseButtonAction());
        buttonReset.addActionListener(new SaveButtonAction());
        
        buttonApply.setMnemonic('A');
        buttonReset.setMnemonic('R');
        buttonCancel.setMnemonic('C');        
        
        panelActions.add(buttonApply);
        panelActions.add(buttonReset);
        panelActions.add(buttonCancel);
        
        getContentPane().add( panelActions, Layout.getGBC( 0, 3, 2, 1, 1.0, 1.0
                             ,GridBagConstraints.SOUTHEAST, GridBagConstraints.NONE,
                              new Insets( 5, 5, 5, 5), 0, 0));
        pack();     
    }
    
    private JPanel getFilterPanel() {
        JPanel panelFilter = new JPanel();
        panelFilter.setLayout(Layout.getGBL());
        
        JPanel panelFilterActions = new JPanel();
        panelFilterActions.setLayout(new FlowLayout(FlowLayout.RIGHT));
        panelFilterActions.add(buttonDeleteFilter);
        panelFilterActions.add(buttonSaveFilter);
        panelFilterActions.add(buttonSaveNewFilter);
        
        buttonSaveFilter.addActionListener(new SaveFilterAction(false));
        buttonSaveNewFilter.addActionListener(new SaveFilterAction(true));
        buttonDeleteFilter.addActionListener(new DeleteFilterAction());

        panelFilter.add( comboFilters,         Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                 ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                  new Insets(5, 5, 5, 5), 0, 0));
        panelFilter.add( panelFilterActions,   Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                 ,GridBagConstraints.NORTHEAST, GridBagConstraints.NONE,
                  new Insets(5, 5, 5, 0), 0, 0));
        
        panelFilter.setBorder(ComponentFactory.getTitleBorder(DcResources.getText("lblFilters")));  
        
        Collection filters = DataFilters.get(modIdx);
        comboFilters.addItem(DcResources.getText("lblSystemDefault"));
        for (Iterator iter = filters.iterator(); iter.hasNext(); ) {
            comboFilters.addItem(iter.next());
        }
        
        comboFilters.addItemListener(new FilterSelectionListener());
        comboFilters.setSelectedIndex(0);
        
        return panelFilter;
    }    
    
    private class SearchConditionPanel extends JPanel {
        
        private boolean technicalInfoPresent = false;
        
        public SearchConditionPanel() {
            buildDialog();         
        }
        
        public void setSearchValues(DcObject baseDco) {
            Map objects = new HashMap();
            
            objects.put(new Integer(baseDco.getModule().getIndex()), baseDco);
            for (Iterator iter = baseDco.getChildren().iterator(); iter.hasNext(); ) {
                DcObject child = (DcObject) iter.next();
                objects.put(new Integer(child.getModule().getIndex()), child);
            }

            for (Iterator iter = objects.keySet().iterator(); iter.hasNext(); ) {
                Integer key = (Integer) iter.next();
                DcObject dco = (DcObject) objects.get(key);
                int currentModule = key.intValue();
                
                for (Iterator iterFields = fields.keySet().iterator(); iterFields.hasNext(); ) {
                    DcField field = (DcField) iterFields.next();
                    JComboBox component = (JComboBox) fields.get(field);
                    
                    Object value = dco.getValue(field.getIndex());
                    if (field.getModule() == currentModule && !isEmpty(value)) {
                        if (component instanceof DcEditableComboBox) 
                            ((DcEditableComboBox) component).setValue(value);
                        else 
                            component.setSelectedItem(value);
                    }
                }
            }
        }
        
        private boolean isEmpty(Object o) {
            return o == null || o.toString().length() == 0 || 
                   o.toString().equals("0") || o.toString().equals("-1");
        }        
        
        public DcObject getSearchValues() {
            Map objects = new HashMap();
            
            DcModule m = DcModules.get(modIdx);
            objects.put(new Integer(modIdx), m.getDcObject());
            
            if (m.getChildModule() != null) {
                DcModule childModule = m.getChildModule();
                objects.put(new Integer(childModule.getIndex()), childModule.getDcObject());
            }

            for (Iterator iter = objects.keySet().iterator(); iter.hasNext(); ) {
                Integer key = (Integer) iter.next();
                
                DcObject dco = (DcObject) objects.get(key);
                int currentModule = key.intValue();
                
                for (Iterator iterFields = fields.keySet().iterator(); iterFields.hasNext(); ) {
                    DcField field = (DcField) iterFields.next();
                    JComboBox component = (JComboBox) fields.get(field);
                    
                    Object value = null;
                    if (component instanceof DcRatingComboBox) {
                        value = component.getSelectedItem();
                        if (value != null && value.equals(new Integer(-1)))
                            value = null;
                        
                    } else if ( component instanceof DcEditableComboBox && 
                                !((DcEditableComboBox) component).getEditor().getItem().equals("")) {

                        value = component.getSelectedItem();
                    } else if (!(component instanceof DcEditableComboBox)) {
                        value = component.getSelectedItem();
                    }
                    
                    if (field.getModule() == currentModule) {
                        String s = value == null ? "" : value.toString();
                        if (s.length() > 0) {
                            dco.setValue(field.getIndex(), value);
                            dco.setChanged(field.getIndex(), true);
                        }
                    }
                }
            }
            
            DcObject dco = (DcObject) objects.get(new Integer(modIdx));
            for (Iterator iter = objects.values().iterator(); iter.hasNext(); ) {
                DcObject o = (DcObject) iter.next();
                if (o.getModule().getIndex() != dco.getModule().getIndex()) 
                    dco.addChild(o);
            }
           
            return dco;
        }
        
        private Collection getModules() {
            Collection modules = new ArrayList();
            
            DcModule module = DcModules.get(modIdx);
            modules.add(module);
            
            if (module.getChildModule() != null) 
                modules.add(module.getChildModule());
            
            return modules;
        }
        
        protected void createFields() {
            Collection modules = getModules();
            
            for (Iterator iterModules = modules.iterator(); iterModules.hasNext(); ) {
                DcModule module = (DcModule) iterModules.next();
                
                DcFieldDefinitions definitions = module.getFieldDefinitions();
                DcObject dco = module.getDcObject();
                
                for (Iterator iter = definitions.iterator(); iter.hasNext(); ) {
                    DcFieldDefinition definition = (DcFieldDefinition) iter.next();
                    int field = definition.getIndex();
                    JLabel label = ComponentFactory.getLabel(dco.getLabel(field));
                    JComponent input = ComponentFactory.getComponent(modIdx,
                                                            dco.getFieldType(field), 
                                                            dco.getLabel(field),
                                                            dco.getMaxFieldLength(field));
                    
                    if (!(input instanceof JComboBox)) {
                        DcEditableComboBox combo = new DcEditableComboBox();
                        combo.getEditor().getEditorComponent().addKeyListener(new SearchOnEnterAdapter());
                        input = combo;
                    } else if (!dco.getField(field).isUiOnly()){
                        JComboBox combo = (JComboBox) input;
                        combo.addKeyListener(new SearchOnEnterAdapter());
                        combo.addItem(DcResources.getText("lblEmptySearchValue"));
                    }
                    
                    input.setPreferredSize(new Dimension(200,21));
                    label.setPreferredSize(new Dimension(150,21));                    

                    labels.put(dco.getField(definition.getIndex()), label);
                    fields.put(dco.getField(definition.getIndex()), input);
                }
            }
        }
            
        protected void applySettings() {
            panelInfo.removeAll();
            panelTech.removeAll();
            panelLoan.removeAll();
            
            int yTech = 0;
            int yInfo = 0;
            int yLoan = 0;
            
            Collection modules = getModules();
            for (Iterator iterModules = modules.iterator(); iterModules.hasNext(); ) {
                DcModule module = (DcModule) iterModules.next();
                DcFieldDefinitions definitions = module.getFieldDefinitions();
                DcObject dco = module.getDcObject();
                for (Iterator iter = definitions.iterator(); iter.hasNext(); ) {
                    DcFieldDefinition definition = (DcFieldDefinition) iter.next();
                    
                    int index = definition.getIndex();
                    JComboBox input = (JComboBox) fields.get(module.getField(definition.getIndex()));
                    
                    
                    JLabel label = (JLabel) labels.get(module.getField(definition.getIndex()));
                    
                    if (modules.size() > 1) {
                        label.setText(module.getName() + " - " + dco.getLabel(definition.getIndex()));
                    } else {
                        label.setText(dco.getLabel(definition.getIndex()));
                    }
                    
                    if (definition.isEnabled() && dco.isSearchable(index)) {
                        if (    index == DcObject._SYS_AVAILABLE || index == DcObject._SYS_LOANEDBY || index == DcObject._SYS_LOANDURATION) {
                            panelLoan.add(label, Layout.getGBC(0,  yLoan, 1, 1, 1.0, 1.0
                                     ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                      new Insets(0, 0, 0, 5), 0, 0));
                            panelLoan.add(input,  Layout.getGBC(1, yLoan, 1, 1, 1.0, 1.0
                                     ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                      new Insets(0,  5, 0, 5), 0, 0));
                            yLoan++;
                        } else if (!dco.getField(index).isTechnicalInfo()) {
                            panelInfo.add(label, Layout.getGBC(0, yInfo, 1, 1, 1.0, 1.0
                                     ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                      new Insets(0, 0, 0, 5), 0, 0));
                            panelInfo.add(input,  Layout.getGBC(1, yInfo, 1, 1, 1.0, 1.0
                                     ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                      new Insets(0,  5, 0, 5), 0, 0));
                            yInfo++;
                        } else {
                            technicalInfoPresent = true;
                            panelTech.add(label, Layout.getGBC(0, yTech, 1, 1, 1.0, 1.0
                                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                     new Insets(0, 0, 0, 5), 0, 0));
                            panelTech.add(input,  Layout.getGBC(1, yTech, 1, 1, 1.0, 1.0
                                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                     new Insets(0,  5, 0, 5), 0, 0));
                            yTech++;
                        }
                    }
                }
            }
        }
        
        /**
         * Creates the panel
         */
        private void buildDialog() {
            setLayout(Layout.getGBL());

            panelInfo.setLayout(Layout.getGBL());
            panelTech.setLayout(Layout.getGBL());
            panelLoan.setLayout(Layout.getGBL());
            
            createFields();
            applySettings();
            
            tabbedPane.addTab(DcResources.getText("lblDescription"), IconLibrary._icoInformation, panelInfo);
            if (technicalInfoPresent) {
                dummy1.setLayout(Layout.getGBL());
                dummy1.add(panelTech,  Layout.getGBC(0, 0, 1, 1, 1.0, 1.0
                          ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                          new Insets(2, 0, 0, 0), 0, 0));
                tabbedPane.addTab(DcResources.getText("lblTechnicalInfo"), IconLibrary._icoInformationTechnical, dummy1);
            }  
            
            if (DcModules.get(modIdx).canBeLended()) {
                dummy2.setLayout(Layout.getGBL());
                dummy2.add(panelLoan,  Layout.getGBC(0, 0, 1, 1, 1.0, 1.0
                          ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                          new Insets(2, 0, 0, 0), 0, 0));
                tabbedPane.addTab(DcResources.getText(Repository.systemlabels.sysLoan), IconLibrary._icoLoan, dummy2);
            }
            
            add(tabbedPane, Layout.getGBC(0, 0, 1, 1, 1.0, 1.0
                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                    new Insets(0, 0, 0, 0), 0, 0));
        } 
    }
    
    public class SearchOptionsPanel extends JPanel {
        
        private JCheckBox checkComplyAll = ComponentFactory.getCheckBox(DcResources.getText("lblAndOrSelect"));
        private JCheckBox checkPrecise = ComponentFactory.getCheckBox(DcResources.getText("lblPreciseSelect"));
        private SortingPanel sortingPanel = new SortingPanel(modIdx);
        
        public SearchOptionsPanel(DcModule module) {
            setLayout(Layout.getGBL());

            checkComplyAll.setSelected(true);
            
            add( checkComplyAll, Layout.getGBC( 0, 0, 2, 1, 1.0, 1.0
                                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                 new Insets( 0, 5, 3, 5), 0, 0));
            add( checkPrecise,   Layout.getGBC( 0, 1, 2, 1, 5.0, 5.0
                                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                 new Insets( 0, 5, 3, 5), 0, 0));
            add( sortingPanel,   Layout.getGBC( 0, 2, 1, 1, 1.0, 1.0
                                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                 new Insets( 0, 5, 0, 5), 0, 0));
            
            setBorder(ComponentFactory.getTitleBorder(DcResources.getText("lblSearchOpts")));
            
            applySelectedFilter();
        }
        
        public void save() {
            sortingPanel.save();
        }
        
        public void applyOrderSettings(DcField[] fields) {
            sortingPanel.applyOrderSettings(fields);
        }
        
        public void applyOrderSettings(String[] fields) {
            sortingPanel.applyOrderSettings(fields);
        }

        public DataFilterOptions getOptions() {
            boolean complyAll = checkComplyAll.isSelected();
            boolean exactMatch = checkPrecise.isSelected();
            DcField[] fields = sortingPanel.getOrderBy();
            return new DataFilterOptions(fields, complyAll, exactMatch);
        }

        public void setBackground(Color color) {
            super.setBackground(color);
            
            if (checkComplyAll != null) {
                checkComplyAll.setBackground(color);
                checkPrecise.setBackground(color);
            }
        }
        
        public void setFont(Font font) {
            super.setFont(font);
            Font normalFont = DcSettings.getValueAsFont(Repository.settings.stSystemFontNormal);
            
            if (checkComplyAll != null) {
                checkComplyAll.setFont(normalFont);
                checkPrecise.setFont(normalFont);
                sortingPanel.setFont(normalFont);
            }
        }
    }  

    private void close() {
        new DataFilters().save();
        panelOptions.save();
        setVisible(false);
    }
    
    public class SaveButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            resetConditions();
        }
    } 
    
    public class CloseButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            close();
        }
    } 
    
    public class DeleteFilterAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            deleteFilter();
        }
    }     

    public class SaveFilterAction implements ActionListener {
        
        private final boolean asNew;
        
        public SaveFilterAction(boolean asNew) {
            this.asNew = asNew;
        }
        
        public void actionPerformed(ActionEvent e) {
            saveFilter(asNew);
        }
    }    
    
    public class FilterSelectionListener implements ItemListener {

        public void itemStateChanged(ItemEvent ie) {
            if (ie.getStateChange() == ItemEvent.SELECTED) {
                applySelectedFilter();
                DataFilter df = getCurrent();
                buttonDeleteFilter.setEnabled(df != null);
                buttonSaveFilter.setEnabled(df != null);
            }
        }
    }    
    
    public class SearchButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            searchValues();
        }
    }
    
    private class SearchOnEnterAdapter extends KeyAdapter {
        public void keyReleased(KeyEvent e) {
            char ch = e.getKeyChar();
            if (ch == KeyEvent.VK_ENTER) 
                searchValues();
        }
    }    
}
