/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.forms;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.ScrollPaneConstants;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.book.AmazonBookSearchForm;
import net.sf.dc.console.components.DcCheckBox;
import net.sf.dc.console.components.DcFrame;
import net.sf.dc.console.components.DcLongTextField;
import net.sf.dc.console.components.DcPictureField;
import net.sf.dc.console.components.panels.LoanPanel;
import net.sf.dc.console.dialogs.ItemTypeDialog;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.console.dialogs.QuestionBox;
import net.sf.dc.console.movie.OnlineMovieSearchForm;
import net.sf.dc.console.software.OnlineSoftwareSearchForm;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.modules.DcSimpleModule;
import net.sf.dc.core.modules.IChildModule;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcMapping;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.Loan;
import net.sf.dc.core.objects.Picture;
import net.sf.dc.core.objects.template.ITemplate;
import net.sf.dc.exceptions.ValidationException;
import net.sf.dc.helpsystem.JavaHelp;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.settings.definitions.DcFieldDefinition;
import net.sf.dc.settings.definitions.Definitions;
import net.sf.dc.wf.requests.CloseWindowRequest;
import net.sf.ext.util.Base64;
import net.sf.ext.util.Utilities;

public class ItemForm extends DcFrame {

    protected DcObject dco;
    private int moduleIndex;

    protected Map labels = new HashMap();
    protected Map fields = new HashMap();

    protected JTabbedPane tabbedPane = ComponentFactory.getTabbedPane();

    private boolean hideFields = false;
    protected final boolean update;
    protected final boolean readonly;
    private boolean applyTemplate = true;
    
    private DcSimpleItemView childView;
    
    private ITemplate template;
    
    public ItemForm(    JFrame parent,
                        boolean readonly,
                        boolean update,
                        DcObject o,
                        boolean bImdb,
                        boolean bAmazon,
                        boolean applyTemplate) {
        this(null, parent, readonly, update, o, bImdb, bAmazon, applyTemplate);
    }
    

    public ItemForm(    ITemplate template,
                        JFrame parent,
                        boolean readonly,
                        boolean update,
                        DcObject o,
                        boolean bImdb,
                        boolean bAmazon,
                        boolean applyTemplate) {

        super(  !update && !readonly ?
                    DcResources.getText("lblNewItem", o.getModule().getObjectName()) :
                    DcResources.getText("lblItem",    new String[] {o.getModule().getObjectName(),
                                                                    o.getDisplayString(1)}),
                IconLibrary._icoMain,
                parent);

        this.applyTemplate =  applyTemplate && !update;
        
        JavaHelp.setHelpIndex("dc.ItemForm");

        this.template = template;
        this.readonly = readonly;
        this.update = update;

        if (!update && !readonly &&  o.getModule().getIndex() == DcModules._GENERIC) {
            ItemTypeDialog dialog = new ItemTypeDialog(this);
            dialog.setVisible(true);
            this.moduleIndex = dialog.getSelectedModule();
            this.dco = DcModules.get(moduleIndex).getDcObject();
        } else {
            this.dco = (DcObject) o.clone();
            this.moduleIndex = dco.getModule().getIndex();
        }

        hideFields = !DcSettings.getValueAsBoolean(Repository.settings.stShowHiddenFieldsInForm);
        setResizable(true);
        setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                close(false);
            }
        });

        this.getContentPane().setLayout(Layout.getGBL());

        initializeComponents();

        DcModule module = DcModules.get(moduleIndex);
        JPanel panelActions = getActionPanel(readonly, module.isAmazonUpdateAllowed(), module.isImdbUpdateAllowed());

        addInputPanel();
        addChildrenPanel();
        addPictureTabs();
        if (module.canBeLended() && update && !readonly)
            addLoanTab();

        getContentPane().add(tabbedPane,  Layout.getGBC(0, 0, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                             new Insets(0,0,15,0), 0, 0));
        getContentPane().add(panelActions,  Layout.getGBC(0, 1, 1, 1, 0.0, 0.0
                            ,GridBagConstraints.SOUTHEAST, GridBagConstraints.NONE,
                             new Insets(5, 5, 5, 5), 0, 0));

        setRequiredFields();
        setReadonly(readonly);
        setData(dco, true);

        pack();

        setSize(DcSettings.getValueAsDimension(getItemSizeSetting()));

        setCenteredLocation();
    }

    public JTabbedPane getTabbedPane() {
        return tabbedPane;
    }

    /**
     * Closes this form by setting the visibility to false
     */
    public void close(boolean aftersave) {
        try {
            if (!aftersave && update && dco != null && (!readonly && isChanged())) {
            	new QuestionBox(DcResources.getText("msgNotSaved"));
            	if (QuestionBox.result != null && QuestionBox.result.booleanValue()) {
            		saveValues();
            	}
            }
        } catch (Exception ignore) {}
        

    	DcSettings.setValue(getItemSizeSetting(), getSize());
    	
        for (Iterator iter = fields.values().iterator(); iter.hasNext(); ) {
            JComponent c = (JComponent) iter.next();
            if (c instanceof DcPictureField) 
                ((DcPictureField) c).flush();
        }
        
        dco.unload();
        labels.clear();
        fields.clear();
        tabbedPane.removeAll();
        childView = null;
        template = null;        

        dco = null;
        fields = null;
        labels = null;
        tabbedPane = null;
        dispose();
    }

    public void setData(DcObject object, boolean overwrite) {
        
        try {
            dco.applyEnhancers(update);
            
            if (applyTemplate && !(dco instanceof ITemplate))
                dco.applyTemplate(template);  
            
            if (childView != null) {
                if (update)
                    childView.addObjects();
                else
                    childView.setObjects(object.getChildren());
            }
    
            int[] indices = object.getFieldIndices();
            for (int i = 0; i < indices.length; i++) {
                int index = indices[i];
                DcField field = dco.getField(index);
                JComponent component = (JComponent) fields.get(field);
    
                Object oldValue = ComponentFactory.getComponentValue(field.getFieldType(), component);
                Object newValue = object.getValue(index);
    
                if (newValue instanceof Picture)
                    ((Picture) newValue).loadImage();
    
                boolean empty = oldValue == null || oldValue.toString().trim().length() == 0;
                if ((empty || overwrite) && (newValue != null && !newValue.equals(""))) {
                    ComponentFactory.setComponentValue(field.getFieldType(), component, newValue);
                }
            }
        } catch (Exception exp) {
            Messages.add(exp);
        }
    }

    private void setReadonly(boolean readonly) {
        if (readonly) {
            for (Iterator iter = fields.values().iterator(); iter.hasNext(); ) {
                JComponent component = (JComponent) iter.next();
                ComponentFactory.setUneditable(component);
            }
        }
    }

    private void setRequiredFields() {
        int[] fields = dco.getFieldIndices();
        for (int i = 0; i < fields.length; i++) {
            DcField field = dco.getField(fields[i]);
            if (field.isRequired()) {
                JLabel label = (JLabel) labels.get(field);
                label.setForeground(ComponentFactory.getRequiredColor());
            }
        }
    }

    private String getItemSizeSetting() {
        String setting = Repository.settings.stItemFormViewSize;

        if (moduleIndex == DcModules._MOVIE)
            setting = Repository.settings.stMovieItemFormViewSize;
        else if (moduleIndex == DcModules._SOFTWARE)
            setting = Repository.settings.stSoftwareItemFormViewSize;
        else if (moduleIndex == DcModules._MUSICALBUM)
            setting = Repository.settings.stMusicAlbumFormViewSize;
        else if (moduleIndex == DcModules._IMAGE)
            setting = Repository.settings.stImageItemFormViewSize;
        else if (moduleIndex == DcModules._BOOK)
            setting = Repository.settings.stBookItemFormViewSize;
        else if (moduleIndex == DcModules._AUDIOCD)
            setting = Repository.settings.stAudioCDItemFormViewSize;
        else if (moduleIndex == DcModules._CONTACTPERSON)
            setting = Repository.settings.stPersonItemFormViewSize;
        else if (DcModules.get(moduleIndex) instanceof DcSimpleModule)
            setting = Repository.settings.stSimpleItemFormSize;

        return setting;
    }

    private void startAmazonUpdate() {
        OnlineSearchForm amazonForm = null;
        if (moduleIndex == DcModules._SOFTWARE)
            amazonForm = new OnlineSoftwareSearchForm(getObject(), this, true);
        else
            amazonForm = new AmazonBookSearchForm(getObject(), this, true);

        amazonForm.setVisible(true);
    }

    private void startIMDBUpdate() {
        OnlineMovieSearchForm imdbForm = new OnlineMovieSearchForm(getObject(), this, true);
        imdbForm.setVisible(true);
    }

    protected void deleteValues() {
        new QuestionBox(DcResources.getText("msgDeleteQuestion"), this);

        if (QuestionBox.result.booleanValue()) {
            String id = dco.getID();
            dco.clearValues();
            dco.setValue(DcObject._ID, id);
            dco.setSilence(true);

            dco.addRequest(new CloseWindowRequest(this));
            dco.delete();
        }
    }

    protected boolean isChanged() {
        boolean changed = dco.isChanged();

        int[] indices = dco.getFieldIndices();
        for (int i = 0; i < indices.length; i++) {
            int index = indices[i];

            DcField field = dco.getField(index);
            JComponent component = (JComponent) fields.get(field);
            Object o = ComponentFactory.getComponentValue(field.getFieldType(), component);

            if (field.getValueType() == Repository.datatypes._ICON) {
                byte[] newValue = o == null ? new byte[0] : (byte[]) o;
                
                Object oOld = dco.getValue(index);
                byte[] oldValue;
                if (oOld instanceof String)
                    oldValue = Base64.decode(((String) oOld).toCharArray());
                else 
                    oldValue = (byte[]) oOld;

                oldValue = oldValue == null ? new byte[0] : oldValue;
                if (!Utilities.sameImage(newValue, oldValue)) {
                    dco.setChanged(DcObject._ID, true);
                    changed = true;
                    break;
                }
            } else if (field.getValueType() == Repository.datatypes._DCOBJECTCOLLECTION) {
                List oldList = (List) o;
                List newList = dco.getValue(index) instanceof List ? (List) dco.getValue(index) : null;
                
                oldList = oldList == null ? new ArrayList() : oldList;
                newList = newList == null ? new ArrayList() : newList;
                
                if (oldList.size() == newList.size()) {
                    for (Iterator iterNew = newList.iterator(); iterNew.hasNext(); ) {
                        DcMapping newMapping = (DcMapping) iterNew.next();
                        boolean found = false;
                        for (Iterator iterOld = oldList.iterator(); iterOld.hasNext(); ) {
                            DcMapping oldMapping = (DcMapping) iterOld.next();
                            if (oldMapping.getSimpleObjectId().equals(newMapping.getSimpleObjectId()))
                                found = true;
                        }
                        
                        if (!found) {
                            changed = true;
                            break;
                        }
                    }
                } else {
                    changed = true;
                }
                
            } else if (field.getValueType() == Repository.datatypes._DCOBJECTCOLLECTION || (!field.isUiOnly() && field.getValueType() != Repository.datatypes._PICTURE)) {
                if (field.getValueType() == Repository.datatypes._DATE) {
                    Date dateOld = (Date) dco.getValue(index);
                    Date dateNew = (Date) o;
                    
                    if (   (dateOld == null && dateNew != null) || (dateNew == null && dateOld != null) ||
                           (dateOld != null && dateNew != null && dateOld.compareTo(dateNew) != 0)) {
                        changed = true;
                        break;
                    }
                } else if (field.getFieldType() == ComponentFactory._FILELAUNCHFIELD) {
                    String newValue = o == null ? "" : o.toString();
                    String oldValue = dco.getValue(index) != null ? new File(dco.getValue(index).toString()).toString() : "";
                    if (!oldValue.toString().equals(newValue)) {
                        changed = true;
                        break;
                    }
                } else {
                    String newValue = o == null ? "" : o.toString();
                    String oldValue = dco.getValue(index) != null ? dco.getValue(index).toString() : "";
                    if (!oldValue.toString().equals(newValue)) {
                        changed = true;
                        break;
                    }
                } 
            } else if (field.getValueType() == Repository.datatypes._PICTURE) {
                Picture picture = (Picture) dco.getValue(index);
                byte[] newValue = o != null ? (byte[]) o : new byte[0];
                byte[] oldValue = picture != null && picture.getValue(Picture._D_IMAGE) != null ?
                                    (byte[]) picture.getValue(Picture._D_IMAGE) : new byte[0];
                if (!Utilities.sameImage(newValue, oldValue)) {
                    dco.setChanged(DcObject._ID, true);
                    changed = true;
                    break;
                }
            }
        }
        return changed;
    }
    
    public DcObject getObject() {
        return getObjectForSaving();
    }

    public DcObject getObjectForSaving() {
        DcObject clone = (DcObject) dco.clone();
        
        clone.removeChildren();
        if (DcModules.get(moduleIndex).getChildModule() != null) {
            for (Iterator iter = childView.getObjects().iterator(); iter.hasNext(); ) {
                clone.addChild((DcObject) iter.next());
            }
        }

        if (update) dco.markAsUnchanged();

        for (Iterator iter = fields.keySet().iterator(); iter.hasNext(); ) {
            DcField field = (DcField) iter.next();
            JComponent component = (JComponent) fields.get(field);

            Object value = ComponentFactory.getComponentValue(field.getFieldType(), component);
            value = value == null ? "" : value;

            if (update) {
                Object oldValue = clone.getValue(field.getIndex()) == null ? "" : clone.getValue(field.getIndex());
                if (oldValue == null || !oldValue.equals(value)) {
                    clone.setValue(field.getIndex(), value);
                    if (field.getValueType() == Repository.datatypes._PICTURE)
                        clone.setChanged(DcObject._ID, true);
                }
            } else if (isChanged()) {
                if (!value.equals(""))
                    clone.setValue(field.getIndex(), value);
            }
        }

        if (update && clone.getModule().canBeLended()) {
            Loan loan = DataManager.getCurrentLoan(clone.getID());
            clone.setValue(DcObject._SYS_AVAILABLE, new Boolean(loan.isAvailable(clone.getID())));
            clone.setValue(DcObject._SYS_LOANEDBY, loan.getPersonDescription());
            clone.setValue(DcObject._SYS_LOANDURATION, loan.getDaysLoaned());
        }
        
        clone.setSilence(true);
        return clone;
    }

    protected void saveValues() {
        DcObject clone = getObjectForSaving();

        if (!update)  {
            try {
                clone.addRequest(new CloseWindowRequest(this));
                clone.setSilence(true);
                clone.saveNew(true);
            } catch (ValidationException vExp) {
                new MessageBox(vExp.getMessage(), MessageBox._WARNING);
                return;
            }
        } else if (isChanged()) {
            try {
                clone.addRequest(new CloseWindowRequest(this));
                clone.setSilence(true);
                clone.saveUpdate(true);
            } catch (ValidationException vExp) {
                new MessageBox(vExp.getMessage(), MessageBox._WARNING);
                return;
            }
        } else if (! isChanged()) {
        	new MessageBox(DcResources.getText("msgNoChangesToSave"), MessageBox._WARNING);
        }
    }

    private void initializeComponents() {
        int[] indices = dco.getFieldIndices();
        for (int i = 0; i < indices.length; i ++) {
            int index = indices[i];
            DcField field = dco.getField(index);

            labels.put(field, ComponentFactory.getLabel(dco.getLabel(index)));
            if (index == DcObject._ID) {
                fields.put(field, ComponentFactory.getIdFieldDisabled());
            } else {
                JComponent c = ComponentFactory.getComponent(field.getModRef(),
                                                             field.getFieldType(),
                                                             field.getLabel(),
                                                             field.getMaximumLength());
                fields.put(field, c);
            }
        }
    }

    protected void addInputPanel() {
        JPanel panelInfo = new JPanel();
        JPanel panelTech = new JPanel();
        panelInfo.setLayout(Layout.getGBL());
        panelTech.setLayout(Layout.getGBL());

        DcModule module = DcModules.get(moduleIndex);
        Definitions definitions = module.getFieldDefinitions();

        int yTech = 0;
        int yInfo = 0;
        boolean technicalInfoPresent = false;
        for (Iterator iter = definitions.iterator(); iter.hasNext(); ) {
            DcFieldDefinition definition = (DcFieldDefinition) iter.next();
            int index = definition.getIndex();

            DcField field = dco.getField(index);
            JLabel label = (JLabel) labels.get(field);
            JComponent component = (JComponent) fields.get(field);
        	if ((!field.isUiOnly() || field.getValueType() == Repository.datatypes._DCOBJECTCOLLECTION) && definition.isEnabled() && // enabled
                ((hideFields && definition.isVisible()) || !hideFields) && // check this visibility
                field.getValueType() != Repository.datatypes._PICTURE && // check the field type
                index != dco.getParentReferenceFieldIndex()) { // not a reference field

                int stretch = GridBagConstraints.HORIZONTAL;
                int factor = 1;

                if (field.getFieldType() == ComponentFactory._LONGTEXTFIELD) {
                    stretch = GridBagConstraints.BOTH;
                    factor = 50;

                    DcLongTextField longText = (DcLongTextField) component;
                    longText.setMargin(new Insets(1, 1, 1, 5));

                    JScrollPane pane = new JScrollPane(longText);
                    pane.setBorder(ComponentFactory.getBorder());
                    pane.setPreferredSize(new Dimension(100,100));
                    pane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
                    pane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
                    component = pane;
                } else {
                	component.setPreferredSize(new Dimension(300, 21));
                }
                
                if (component instanceof DcCheckBox)
                    ((DcCheckBox) component).setText("");

                label.setPreferredSize(new Dimension(100, 20));
                if (!field.isTechnicalInfo()) {
                    panelInfo.add(label, Layout.getGBC(0, yInfo, 1, 1, 1.0, 1.0
                             ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                              new Insets(0, 0, 0, 5), 0, 0));
                    panelInfo.add(component,  Layout.getGBC(1, yInfo, 1, 1, factor, factor
                             ,GridBagConstraints.NORTHWEST, stretch,
                              new Insets(0, 0, 0, 0), 0, 0));
                    yInfo++;
                } else {
                    technicalInfoPresent = true;
                    panelTech.add(label, Layout.getGBC(0, yTech, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                             new Insets(0, 0, 0, 5), 0, 0));
                    panelTech.add(component,  Layout.getGBC(1, yTech, 1, 1, 50, 50
                            ,GridBagConstraints.NORTHWEST, stretch,
                             new Insets(0, 0, 0, 0), 0, 0));
                    yTech++;
                }
                
                if (field.isReadOnly())
                    ComponentFactory.setUneditable(component);

            }
        }
        tabbedPane.addTab(DcResources.getText("lblDescription"), IconLibrary._icoInformation, panelInfo);
        if (technicalInfoPresent) {
            JPanel dummy = new JPanel();
            dummy.setLayout(Layout.getGBL());
            dummy.add(panelTech,  Layout.getGBC(0, 0, 1, 1, 1.0, 1.0
                      ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                      new Insets(2, 0, 0, 0), 0, 0));
            tabbedPane.addTab(DcResources.getText("lblTechnicalInfo"), IconLibrary._icoInformationTechnical, dummy);
        }
    }

    protected void addChildrenPanel() {
        DcModule module = DcModules.get(moduleIndex);
        DcModule childModule = module.getChildModule();
        
        if (childModule != null) {
            IChildModule m = (IChildModule) childModule;
            childView = m.getItemView(this, dco, childModule.getIndex(), !update);
            tabbedPane.addTab(childModule.getObjectNamePlural(), childView.getContentPane());
        }
    }

    protected void addLoanTab() {
        tabbedPane.addTab(DcResources.getText("lblLoan"), IconLibrary._icoLoan, new LoanPanel(dco, null));
    }

    protected void addPictureTabs() {
        DcModule module = DcModules.get(moduleIndex);

        Definitions definitions = module.getFieldDefinitions();
        for (Iterator iter = definitions.iterator(); iter.hasNext(); ) {
            DcFieldDefinition definition = (DcFieldDefinition) iter.next();

            int index = definition.getIndex();
            DcField field = dco.getField(index);
            JComponent component = (JComponent) fields.get(field);

            if (definition.isEnabled() &&
               ((hideFields && definition.isVisible()) || !hideFields) &&
               field.getValueType() == Repository.datatypes._PICTURE) {

                JPanel panel = new JPanel();
                panel.setLayout(Layout.getGBL());

                component.setPreferredSize(component.getMinimumSize());
                panel.add(component, Layout.getGBC(0, 0, 1, 1, 1.0, 1.0
                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                          new Insets(5, 5, 5, 5), 0, 0));

                tabbedPane.addTab(field.getLabel(), IconLibrary._icoPicture, panel);
            }
        }
    }

    private JPanel getActionPanel(boolean readonly, boolean amazon, boolean imdb) {
        JPanel panel = new JPanel();
        panel.setLayout(Layout.getGBL());

        JButton buttonCancel = ComponentFactory.getButton(DcResources.getText("lblCancel"));
        JButton buttonDelete = ComponentFactory.getButton(DcResources.getText("lblDelete"));
        JButton buttonInternet = null;

        if (imdb) {
            buttonInternet = ComponentFactory.getButton(DcResources.getText("lblOnlineUpdate"));
            buttonInternet.setIcon(IconLibrary._icoSearchOnline);
            buttonInternet.addActionListener(new IMDBAction());
            buttonInternet.setMnemonic(KeyEvent.VK_U);
        } else if (amazon) {
            buttonInternet = ComponentFactory.getButton(DcResources.getText("lblOnlineUpdate"));
            buttonInternet.setIcon(IconLibrary._icoSearchOnline);
            buttonInternet.addActionListener(new AmazonAction());
            buttonInternet.setMnemonic(KeyEvent.VK_U);
        }

        JButton buttonSave = ComponentFactory.getButton(DcResources.getText("lblSave"));
        buttonSave.setMnemonic(KeyEvent.VK_S);
        buttonSave.addActionListener(new SaveAction());

        buttonCancel.setMnemonic(KeyEvent.VK_C);
        buttonCancel.addActionListener(new ExitAction());

        buttonDelete.setMnemonic(KeyEvent.VK_D);
        buttonDelete.addActionListener(new DeleteAction());

        if (!readonly && (amazon || imdb)) {
            panel.add(buttonInternet, Layout.getGBC(0, 0, 1, 1, 1.0, 1.0
                     ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                      new Insets(5, 5, 5, 5), 0, 0));
        }
        
        if (!readonly) {
        panel.add(buttonSave, Layout.getGBC(1, 0, 1, 1, 1.0, 1.0
                 ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                  new Insets(5, 5, 5, 5), 0, 0));
        }

        if (update && !readonly) {
            panel.add(buttonDelete, Layout.getGBC(2, 0, 1, 1, 1.0, 1.0
                     ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                      new Insets(5, 5, 5, 5), 0, 0));
        }

        panel.add(buttonCancel , Layout.getGBC(3, 0, 1, 1, 1.0, 1.0
                 ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                  new Insets(5, 5, 5, 5), 0, 0));

        return panel;
    }

    private class IMDBAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startIMDBUpdate();
        }
    }

    private class AmazonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startAmazonUpdate();
        }
    }

    private class SaveAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            saveValues();
        }
    }

    private class DeleteAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            deleteValues();
        }
    }

    private class ExitAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            close(false);
        }
    }
}
