/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.forms;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.net.URL;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcFrame;
import net.sf.dc.console.components.DcPictureField;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.resources.DcResources;

public class LogForm extends DcFrame {

    private JTextArea logger;
    private JPanel panelLogging = new JPanel();
    private JPanel panelPicture = new JPanel();
    private JPanel panelProductInfo = new JPanel();

    private JScrollPane scroller;
    
    private JLabel labelVersion = ComponentFactory.getLabel(Repository.system.version);
    private JLabel labelLastUpd = ComponentFactory.getLabel(Repository.system.releasedate);

    public LogForm() {
        super(DcResources.getText("lblLog"), IconLibrary._icoEventLog, DataCrow.mainFrame);
        buildPanel();
    }
    
    public void logToFront(boolean show) {
        this.setVisible(show);

        if (show) {
            this.toFront();
        }
    }

    public void setFont(Font font) {
        if (logger != null) {
            labelVersion.setFont(ComponentFactory.getSystemFont());
            labelLastUpd.setFont(ComponentFactory.getSystemFont());
            logger.setFont(ComponentFactory.getStandardFont());
        }
    }
    
    public void writeToScreen(String message) {
        try {
            logger.setText(logger.getText().substring(0, 30000));
        } catch (Exception ignore) {}
        logger.insert(message, 0);
    }    

    private void buildPanel() {
        //**********************************************************
        //Logo panel
        //**********************************************************
        panelPicture.setLayout(Layout.getGBL());
        DcPictureField logo = ComponentFactory.getPictureField(false, false, false);
        panelPicture.setBorder(ComponentFactory.getBorder());

        try {
            logo.setValue(new URL("file://" + DataCrow.baseDir + "icons_system/logo.jpg"));
        } catch (Exception exp) {
            new MessageBox(exp.toString(), MessageBox._ERROR);
        }

        panelPicture.setPreferredSize(new Dimension(575,200));
        panelPicture.setMinimumSize(new Dimension(575,200));
        panelPicture.add(logo,  Layout.getGBC( 0, 0, 1, 1, 10.0, 10.0
                               ,GridBagConstraints.NORTH, GridBagConstraints.BOTH,
                                new Insets(0, 0, 0, 0), 0, 0));

        //**********************************************************
        //Logging panel
        //**********************************************************
        panelLogging.setLayout(Layout.getGBL());

        logger = ComponentFactory.getTextArea();
        logger.setMargin(new Insets(5,5,5,5));
        logger.setEditable(false);
        logger.setWrapStyleWord(true);
        logger.setLineWrap(true);
        
        scroller = new JScrollPane(logger);
        scroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);

        panelLogging.add(scroller,  Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                                   ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                    new Insets( 0, 5, 0, 5), 0, 0));

        //**********************************************************
        //Product information panel
        //**********************************************************
        panelProductInfo.setLayout(Layout.getGBL());

        panelProductInfo.add(labelVersion,     Layout.getGBC( 0, 0, 1, 1, 0.0, 0.0
                                              ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                               new Insets( 0, 5, 0, 0), 0, 0));
        panelProductInfo.add(labelLastUpd,     Layout.getGBC( 0, 1, 1, 1, 0.0, 0.0
                                              ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                               new Insets( 0, 5, 0, 0), 0, 0));


        //**********************************************************
        //Main panel
        //**********************************************************
        getContentPane().setLayout(Layout.getGBL());
        
        // Build the panel
        this.getContentPane().add(   panelPicture,  Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                                                   ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                                    new Insets( 5, 5, 0, 5), 0, 0));
        this.getContentPane().add(   panelLogging,  Layout.getGBC( 0, 1, 1, 1, 20.0, 20.0
                                                   ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                                    new Insets( 0, 0, 0, 0), 0, 0));
        this.getContentPane().add(panelProductInfo, Layout.getGBC( 0, 2, 1, 1, 0.0, 0.0
                                                   ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                                    new Insets( 5, 0, 5, 0), 0, 0));
        pack();

        setSize(550, 600);
        setResizable(false);
        
        setCenteredLocation();
    }
}
