/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.forms;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.KeyStroke;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.MainFrame;
import net.sf.dc.console.components.DcFrame;
import net.sf.dc.console.components.lists.DcObjectList;
import net.sf.dc.console.components.panels.OnlineServiceSettingsPanel;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcMediaObject;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.services.DcRegion;
import net.sf.dc.core.services.DcSearchMode;
import net.sf.dc.core.services.DcServer;
import net.sf.dc.helpsystem.JavaHelp;
import net.sf.dc.messages.Messages;
import net.sf.dc.processes.http.IOnlineUpdateListener;
import net.sf.dc.processes.http.InternetUpdateProcess;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.ext.util.StringUtils;

public abstract class OnlineSearchForm extends DcFrame implements IOnlineUpdateListener {

    private ItemForm itemForm;
    private String ID;

    private final int module;
    private final ArrayList items = new ArrayList();

    private JTextField textSearch;
    private JButton buttonSearch;
    private JButton buttonStop;

    protected JComboBox comboServers;
    protected JComboBox comboModes;
    protected JComboBox comboRegions;

    private JLabel labelSearch = ComponentFactory.getLabel(DcResources.getText("lblSearchFor"));
    private JPanel panelSearch = new JPanel();
    private OnlineServiceSettingsPanel panelSettings;
    private JPanel panel;

    private JTextArea textLog = ComponentFactory.getTextArea();
    private JProgressBar progressBar = new JProgressBar();

    private MouseListener mouseListener;
    
    private DcObjectList list;

    protected InternetUpdateProcess process;
    private boolean startSearchOnOpen = false;
    private boolean listenForServerChanges = true;

    public OnlineSearchForm(JFrame parent, DcObject dco, ItemForm itemForm, boolean advanced) {
        this(parent, dco, itemForm, -1, advanced);
    }

    public OnlineSearchForm(JFrame parent, DcObject dco, int tab, boolean advanced) {
        this(parent, dco, null, tab, advanced);
    }

    private OnlineSearchForm(JFrame parent, DcObject dco, ItemForm itemForm, int tab, boolean advanced) {
        super(DcResources.getText("lblOnlineXSearch", DcModules.getCurrentModule().getObjectName()),
              IconLibrary._icoSearchOnline, parent);

        textSearch = ComponentFactory.getShortTextField(100);
        buttonSearch =  ComponentFactory.getButton(DcResources.getText("lblFind"));
        buttonStop =  ComponentFactory.getButton(DcResources.getText("lblStop"));

        startSearchOnOpen = dco != null;

        this.ID = dco != null ? dco.getID() : null;
        this.itemForm = itemForm;
        this.module = DcSettings.getValueAsInt(Repository.settings.stModule);
        this.mouseListener = new TableMouseListener();
        
        buildDialog(advanced);

        JavaHelp.setHelpIndex("dc.Searching.Odb");
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                close(true);
            }
        });

        setSearchString(dco != null ? (String) dco.getValue(DcMediaObject._A_TITLE) : "");

        setSize(DcSettings.getValueAsDimension(Repository.settings.stOnlineSearchFormSize));
        setCenteredLocation();
        setSearchMode();

        addKeyListener(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0, false), new SearchOnEnterAction(), "searchaction");

        buttonSearch.requestFocus();

        if (startSearchOnOpen && textSearch.getText().trim().length() > 0) {
            perfectMatchFound = false;
            startSearch();
        }
    }

    public void addDoubleClickListener(MouseListener ml) {
        try {
            list.removeMouseListener(mouseListener);
        } catch (Exception ignore) {}
        mouseListener = ml;
        list.addMouseListener(ml);
    }
    
    public DcModule getModule() {
        return DcModules.get(module);
    }

    public String getServerAddress() {
        DcServer server = getServer();
        String url = server.getUrl();
        if (server.getRegions() != null && server.getRegions().size() > 0) {
            DcRegion region = (DcRegion) comboRegions.getSelectedItem();
            url = region.getUrl();
        }
        return url;
    }

    public String getSearchString() {
        return textSearch.getText();
    }

    public DcSearchMode getMode() {
        DcServer server = getServer();
        DcSearchMode mode = null;
        if (server.getModes() != null && server.getModes().size() > 0) {
            mode = (DcSearchMode) comboModes.getSelectedItem();
        }
        return mode;
    }
    
    public DcServer getServer() {
        return (DcServer) comboServers.getSelectedItem();
    }

    public DcRegion getRegion() {
        DcServer server = getServer();
        DcRegion region = null;
        if (server.getRegions() != null && server.getRegions().size() > 0) {
            region = (DcRegion) comboRegions.getSelectedItem();
        }
        return region;
    }


    public void addObject(DcObject dco) {
        if (process != null && !process.isCancelled()) {
            dco.applyTemplate();
            dco.setValue(DcObject._ID, ID);
            list.add(dco);
            items.add(dco);
            checkPerfectMatch(dco);
        }
    }

    public void addMessage(String message) {
        textLog.insert(message + '\n', 0);
    }

    public Collection getObjects() {
        return list.getObjects();
    }
    
    public DcObject getSelectedObject() {
        int row = list.getSelectedIndex();

        DcObject dco = null;
        
        if (row > -1) {
            dco = (DcObject) items.get(row);
            dco.setValue(DcObject._ID, ID);
        }

        return dco;
    }

    public Collection getSelectedData() {
        int row = list.getSelectedIndex();

        if (row < 0) {
            new MessageBox(DcResources.getText("msgSelectRowForTransfer"), MessageBox._WARNING);
            return null;
        }

        ArrayList result = new ArrayList();
        if (ID == null) {
            int[] rows = list.getSelectedIndices();
            for (int i = 0; i < rows.length; i++) {
                result.add(items.get(rows[i]));
            }
        } else {
            DcObject o = (DcObject) items.get(row);
        	o.setValue(DcObject._ID, ID);
            result.add(o);
        }
        return result;
    }

    public int getResultCount() {
        return list.getModel().getSize();
    }

    private void applyServer() {
        listenForServerChanges = false;
        
        try {
            DcServer server = getServer();
            Dimension size = getSize();
    
            comboRegions.removeAllItems();
            comboModes.removeAllItems();
    
            try {
                panelSearch.remove(textSearch);
                panelSearch.remove(comboModes);
                panelSearch.remove(buttonSearch);
                panelSearch.remove(labelSearch);
            } catch (Exception exp) {}
            
            if (server.isKeywordSearchAllowed()) {
                int stretchX = server.hasModes() ? 1 : 2;
                panelSearch.add(labelSearch,  Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                      ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
                      new Insets(5, 5, 5, 5), 0, 0));
                panelSearch.add(textSearch,   Layout.getGBC( 1, 0, stretchX, 1, 10.0, 10.0
                      ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
                      new Insets(5, 5, 5, 5), 0, 0));                
            }
    
            if (server.hasModes()) {
                for (Iterator iter = server.getModes().iterator(); iter.hasNext(); ) {
                    comboModes.addItem(iter.next());
                }

                panelSearch.add(comboModes,   Layout.getGBC( 2, 0, 1, 1, 10.0, 10.0
                               ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
                                new Insets(5, 5, 5, 5), 0, 0));
            }
            
            buttonSearch = getSearchButton();
            panelSearch.add(buttonSearch, Layout.getGBC( 3, 1, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL,
                            new Insets(5, 5, 5, 5), 0, 0));
    
            for (Iterator iter = server.getRegions().iterator(); iter.hasNext(); ) {
                comboRegions.addItem(iter.next());
            }
    
            applyServerSettings(getServer());
    
            buttonSearch.setMnemonic(KeyEvent.VK_F);
            buttonSearch.requestFocusInWindow();
    
            setSize(size);
        } finally {
            listenForServerChanges = true;
        }
        
        repaint();
        panel.revalidate();
    }
    
    private void setDefaultServer() {
        String s = getDefaultServer();
        if (s.indexOf(",") < 0)
            return;
            
        String serverName = s.substring(0, s.indexOf(","));
        String serverAddress = s.substring(s.indexOf(",") + 1, s.length());
        Collection servers = getServerDefinitions();
        
        listenForServerChanges = false;
        comboServers.setSelectedIndex(0);
        for (Iterator iter = servers.iterator(); iter.hasNext(); ) {
            DcServer server = (DcServer) iter.next();

            if (!server.getName().equals(serverName))
                continue;
                
            if (server.getUrl().equals(serverAddress)) {
                comboServers.setSelectedItem(server);
            } else {
                for (Iterator iterRegions = server.getRegions().iterator(); iterRegions.hasNext(); ) {
                    DcRegion region = (DcRegion) iterRegions.next();
                    if (region.getUrl().equals(serverAddress))
                        comboServers.setSelectedItem(server);
                }                
            }
        }

        listenForServerChanges = true;
        applyServer();
    }

    private void applyServerSettings(DcServer server) {
        String s = getDefaultServer();
        String serverAddress = s.substring(s.indexOf(",") + 1, s.length());

        for (Iterator iterRegions = server.getRegions().iterator(); iterRegions.hasNext(); ) {
            DcRegion region = (DcRegion) iterRegions.next();
            if (region.equals(serverAddress))
                comboRegions.setSelectedItem(region);
        }

        if (server.hasModes()) {
            for (Iterator iterModes = server.getModes().iterator(); iterModes.hasNext(); ) {
                DcSearchMode searchMode = (DcSearchMode) iterModes.next();
                if (searchMode.equals(getDefaultMode()))
                    comboModes.setSelectedItem(searchMode);
            }
        }
    }

    private void open() {
        int selectedRow = list.getSelectedIndex();
        if (selectedRow == -1) {
            new MessageBox(DcResources.getText("msgSelectRowToOpen"), MessageBox._WARNING);
            return;
        }

        DcObject o = getSelectedObject();
        if (o != null) {
        	ItemForm itemForm = new ItemForm(this, true, false, o, false, false, true);
            itemForm.setVisible(true);
        }
    }

    public void setCancelMode() {
        buttonSearch.setEnabled(false);
        buttonStop.setEnabled(true);
        addKeyListener(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0, false), new CancelOnEnterAction(), "searchaction");
        buttonStop.requestFocus();
    }
    
    private void checkPerfectMatch(DcObject dco) {
        if (panelSettings.isAutoAddAllowed()) {
            DcSearchMode mode = getMode();
            if (mode != null && mode.singleIsPerfect()) {
                perfectMatch(dco);
            } else {
                String string = StringUtils.normalize(textSearch.getText()).toLowerCase();
                String item = StringUtils.normalize(dco.toString()).toLowerCase();
                if (string.equals(item))
                    perfectMatch(dco);    
            }
        }
    }
    
    public void perfectMatch(DcObject dco) {
        perfectMatchFound = true;
        list.setSelected(dco, DcMediaObject._A_TITLE);
        if (ID != null)
            update(true);
        else 
            addNew(false);
        
        process.cancelSearch();
        clear();
        setSearchMode();
        toFront();
    }
    
    public void setSearchMode() {
        if (buttonSearch != null) {
            initProgressBar(0);
            buttonSearch.setEnabled(true);
            buttonStop.setEnabled(false);
            addKeyListener(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0, false), new SearchOnEnterAction(), "searchaction");
            
            if (perfectMatchFound) {
                textSearch.setText("");
                textSearch.requestFocusInWindow();
            } else {
                textSearch.requestFocusInWindow();
            }
        }
    }

    public void setSearchString(String s) {
        textSearch.setText(s);
    }

    protected void saveSettings() {
        String server = getServer().getName() + "," + getServerAddress();
        String mode = getMode() != null ? getMode().toString() : null;
        DcSettings.setValue(getModule().getOnlineServiceSettingsKey(), new String[] {server, mode});
        DcSettings.setValue(Repository.settings.stOnlineSearchFormSize, getSize());
        panelSettings.save();
    }

    public void update(boolean closeForm) {
        DcObject o = getSelectedObject();
        if (o != null) {
            
            if (itemForm.isVisible()) {
                DcModule mod = DcModules.get(module);
                DcObject dco = mod.getDcObject();

                int[] fields = 
                    DcSettings.getValueAsBoolean(Repository.settings.stODBOverwriteValues) ?
                    DcSettings.getValueAsIntegerArray(DcModules.get(module).getOnlineUpdateFieldsSettingsKey()) :
                    dco.getModule().getFieldIndices();

                for (int i = 0; i < fields.length; i++) {
                    int field = fields[i];
                    
                    if (dco.isFilled(field)) {
                        if (DcSettings.getValueAsBoolean(Repository.settings.stODBOverwriteValues) && 
                            (o.getValue(fields[i]) != null && !o.getValue(fields[i]).equals("") && !o.getValue(fields[i]).equals("-1"))) {
                            dco.setValue(field, o.getValue(fields[i]));
                        }
                    } else {
                        dco.setValue(field, o.getValue(fields[i]));
                    }
                }                
                
                itemForm.setData(dco, panelSettings.isOverwriteAllowed());
            }
            
            if (closeForm)
                close(true);
        }
    }

    public void addNew(boolean closeForm) {
        Collection objects = getSelectedData();
        if (objects != null) {
            getModule().getCurrentInsertView().add(objects);
            DataCrow.mainFrame.setSelectedTab(MainFrame._INSERTTAB);
            DataCrow.mainFrame.toFront();
            if (closeForm)
                close(true);
        }
    }

    public void setSelectionMode(int selectionMode) {
        list.setSelectionMode(selectionMode);        
    }
    
    private void clear() {
    	list.clear();
        updateProgressBar(0);
        items.clear();
        textLog.setText("");
    }

    private void buildDialog(boolean advanced) {
        getContentPane().setLayout(Layout.getGBL());
        
        panel = getPanel(advanced);
        
        getContentPane().add(panel, Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                             new Insets(0, 0, 0, 0), 0, 0));
        pack();
    }

    private JPanel getPanel(boolean advanced) {
        setResizable(true);
        getContentPane().setLayout(Layout.getGBL());

        //**********************************************************
        //Settings
        //**********************************************************
        panelSettings = new OnlineServiceSettingsPanel(this, ID != null, module);

        //**********************************************************
        //Search panel
        //**********************************************************
        panelSearch.setLayout(Layout.getGBL());
        JLabel labelServer = ComponentFactory.getLabel(DcResources.getText("lblInternetServer"));

        textSearch.setMinimumSize(new Dimension(100,19));

        comboServers = ComponentFactory.getComboBox();
        Collection servers = getServerDefinitions();
        for (Iterator iter = servers.iterator(); iter.hasNext(); ) {
            comboServers.addItem(iter.next());
        }

        comboModes = ComponentFactory.getComboBox();
        comboRegions = ComponentFactory.getComboBox();

        comboServers.addActionListener(new ServerChangedListener());
        buttonStop.addActionListener(new StopSearchAction());
        
        panelSearch.add(labelServer,  Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
                 new Insets(5, 5, 5, 5), 0, 0));
        panelSearch.add(comboServers, Layout.getGBC( 1, 1, 1, 1, 10.0, 10.0
                ,GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL,
                 new Insets(5, 5, 5, 5), 0, 0));
        panelSearch.add(comboRegions, Layout.getGBC( 2, 1, 1, 1, 10.0, 10.0
                ,GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL,
                 new Insets(5, 5, 5, 5), 0, 0));

        
        buttonSearch = getSearchButton();
        panelSearch.add(buttonSearch, Layout.getGBC( 3, 1, 1, 1, 1.0, 1.0
                ,GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL,
                 new Insets(5, 5, 5, 5), 0, 0));
        panelSearch.add(buttonStop,
                 Layout.getGBC(4, 1, 1, 1, 1.0, 1.0
                ,GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL,
                 new Insets(5, 5, 5, 5), 0, 0));

        //**********************************************************
        //Progress panel
        //**********************************************************
        JPanel panelProgress = new JPanel();
        panelProgress.setLayout(Layout.getGBL());
        panelProgress.add(progressBar, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                          new Insets(5, 5, 5, 5), 0, 0));


        //**********************************************************
        //Log Panel
        //**********************************************************
        JPanel panelLog = new JPanel();
        panelLog.setLayout(Layout.getGBL());

        JScrollPane scroller = new JScrollPane(textLog);
        scroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        panelLog.add(scroller, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                     new Insets(5, 5, 5, 5), 0, 0));

        panelLog.setBorder(ComponentFactory.getTitleBorder(DcResources.getText("lblLog")));

        //**********************************************************
        //Result panel
        //**********************************************************
        JPanel panelResults = new JPanel();
        panelResults.setLayout(Layout.getGBL());

        list = new DcObjectList(getModule(), true, false, true);

        JScrollPane scrollTable = new JScrollPane(list);
        list.addMouseListener(mouseListener);
        scrollTable.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);

        panelResults.add(scrollTable,  Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                        ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                         new Insets(5, 5, 5, 5), 0, 0));

        //**********************************************************
        //Actions panel
        //**********************************************************
        JPanel panelActions = new JPanel();
        panelActions.setLayout(Layout.getGBL());

        JButton buttonDetails = ComponentFactory.getButton(DcResources.getText("lblDetails"));
        
        JButton buttonUpdate = ComponentFactory.getButton(DcResources.getText("lblUpdate"));
        JButton buttonAddNew = ComponentFactory.getButton(DcResources.getText("lblAddNew"));

        JButton buttonClear = ComponentFactory.getButton(DcResources.getText("lblClear"));
        JButton buttonCancel = ComponentFactory.getButton(DcResources.getText("lblCancel"));

        buttonCancel.setToolTipText(DcResources.getText("tpClose"));
        buttonUpdate.setToolTipText(DcResources.getText("tpUpdate"));
        buttonAddNew.setToolTipText(DcResources.getText("tpAddNew"));
        
        buttonDetails.setMnemonic('E');
        buttonCancel.setMnemonic('C');        
        buttonUpdate.setMnemonic('U');
        buttonAddNew.setMnemonic('A');
        buttonClear.setMnemonic('L');
        buttonStop.setMnemonic('T');

        buttonDetails.addActionListener(new OpenAction());
        buttonUpdate.addActionListener(new UpdateAction());
        buttonCancel.addActionListener(new CancelAction());
        buttonClear.addActionListener(new ClearAction());
        buttonAddNew.addActionListener(new AddNewAction());

        panelActions.add(buttonDetails,    Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets(5, 5, 5, 5), 0, 0));
        
        if (ID != null && advanced) {
        	panelActions.add(buttonUpdate,    Layout.getGBC( 1, 0, 1, 1, 1.0, 1.0
        			        ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
							 new Insets(5, 5, 5, 5), 0, 0));
        }

        if (advanced)
            panelActions.add(buttonAddNew,    Layout.getGBC( 2, 0, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                             new Insets(5, 5, 5, 5), 0, 0));

        panelActions.add(buttonClear,        Layout.getGBC( 3, 0, 1, 1, 1.0, 1.0
                        ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
						new Insets(5, 5, 5, 5), 0, 0));

        if (advanced)
            panelActions.add(buttonCancel,     Layout.getGBC( 4, 0, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
    						 new Insets(5, 5, 5, 5), 0, 0));

        //**********************************************************
        //Main Panel
        //**********************************************************
        panel = new JPanel();
        panel.setLayout(Layout.getGBL());

        panel.add(panelSearch,   Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                             new Insets(5, 5, 5, 5), 0, 0));

        if (advanced)
            panel.add(panelSettings, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                             new Insets(5, 5, 5, 5), 0, 0));

        panel.add(panelResults,  Layout.getGBC( 0, 2, 1, 1, 50.0, 50.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                             new Insets(5, 5, 5, 5), 0, 0));
        panel.add(panelActions,  Layout.getGBC( 0, 3, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHEAST, GridBagConstraints.NONE,
							 new Insets(5, 5, 5, 5), 0, 0));
        panel.add(panelProgress, Layout.getGBC( 0, 4, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                             new Insets(5, 5, 5, 5), 0, 0));

        if (advanced)
            panel.add(panelLog,  Layout.getGBC( 0, 5, 1, 1, 10.0, 10.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                             new Insets(5, 5, 5, 5), 0, 0));

        textSearch.addKeyListener(new SearchOnEnterListener());

        setDefaultServer();
        
        return panel;
    }

    public void initProgressBar(int maxValue) {
        progressBar.setValue(0);
        progressBar.setMaximum(maxValue);
    }

    public void updateProgressBar(int value) {
        progressBar.setValue(value);
    }
    
    private void stopSearch() {
        if (process != null)
            process.cancelSearch();

        addMessage(DcResources.getText("msgStoppedSearch"));
        setSearchMode();
    }

    public void close(boolean saveSettings) {
        if (saveSettings)
            saveSettings();
        
        stopSearch();

        ClearThread clrThread  = new ClearThread();
        clrThread.start();
        
        dispose();
    }
    
    private class ClearThread extends Thread {
        
        public void run() {
            
            while(process != null && process.isAlive()) {
                try {
                    sleep(100);
                } catch (Exception ignore) {}
            }
            
            list.removeMouseListener(mouseListener);
            list.clear();
                    
            itemForm  = null;
            ID = null;
            items.clear();

            textSearch = null;
            buttonSearch = null;
            buttonStop = null;
            comboServers = null;
            comboModes = null;
            comboRegions = null;
            labelSearch = null;
            panelSearch = null;
            panel = null;
            textLog = null;
            progressBar = null;
            panelSettings.clear();
            panelSettings = null;
            mouseListener = null;
            process = null;
            
            dispose();
        }
    }

    public void setFocus() {
        textSearch.requestFocusInWindow();
    }

    public void setVisible(boolean b) {
        if (b)
            setFocus();
        super.setVisible(b);
    }

    protected Collection getServerDefinitions() {
        return getModule().getOnlineServices();
    }

    
    public abstract void startSearch();
    protected abstract JButton getSearchButton();
    
    
    protected String getDefaultServer() {
        String[] settings = DcSettings.getValueAsStringArray(getModule().getOnlineServiceSettingsKey());
        return settings[0];
    }
    
    protected String getDefaultMode() {
        String[] settings = DcSettings.getValueAsStringArray(getModule().getOnlineServiceSettingsKey());
        return settings[1];
    }

    private class SearchOnEnterListener implements KeyListener {
        public void keyPressed(KeyEvent arg0) {
            if (arg0.getKeyCode() == KeyEvent.VK_ENTER) {
                perfectMatchFound = false;
                startSearch();
                arg0.consume();
            }
        }

        public void keyReleased(KeyEvent arg0) {}

        public void keyTyped(KeyEvent arg0) {}
    }

    private class SearchOnEnterAction implements Action {
        public boolean isEnabled() {
            return true;
        }

        public void setEnabled(boolean arg0) {}
        public void putValue(String arg0, Object arg1) {}
        public void addPropertyChangeListener(PropertyChangeListener arg0) {}
        public void actionPerformed(ActionEvent arg0) {
            perfectMatchFound = false;
            startSearch();
        }

        public Object getValue(String arg0) {
            return null;
        }

        public void removePropertyChangeListener(PropertyChangeListener arg0) {}
    }

    private class CancelOnEnterAction implements Action {
        public boolean isEnabled() {
            return true;
        }

        public void setEnabled(boolean arg0) {}
        public void putValue(String arg0, Object arg1) {}
        public void addPropertyChangeListener(PropertyChangeListener arg0) {}
        public void actionPerformed(ActionEvent arg0) {
            stopSearch();
        }

        public Object getValue(String arg0) {
            return null;
        }

        public void removePropertyChangeListener(PropertyChangeListener arg0) {}
    }

    private class CancelAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            close(true);
        }
    }

    public class StopSearchAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            stopSearch();
        }
    }

    private class ClearAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            clear();
        }
    }

    private class OpenAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            open();
        }
    }
    
    private class UpdateAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            update(true);
        }
    }

    private class AddNewAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            addNew(true);
        }
    }
    
    public class SearchAction implements ActionListener{
        public void actionPerformed(ActionEvent e) {
            perfectMatchFound = false;
            startSearch();
        }
    }

    private class TableMouseListener implements MouseListener {
        public void mouseReleased(MouseEvent e) {
            if (e.getClickCount() == 2) {
                if (ID != null)
                    update(true);
                else 
                    addNew(true);
            }
        }
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {}
    }

    private class ServerChangedListener implements ActionListener {

        public void actionPerformed(ActionEvent arg0) {
            if (listenForServerChanges) {
                saveSettings();
                applyServer();
            }
        }
    }
    
    public void addError(Throwable t) {
        new MessageBox(t.getMessage(), MessageBox._ERROR);
        Messages.add(t);
    }

    public void addError(String message) {
        new MessageBox(message, MessageBox._ERROR);
        addMessage(message);
    }

    public void addWarning(String warning) {
        if (!perfectMatchFound)
            new MessageBox(warning, MessageBox._WARNING);
    }

    private boolean perfectMatchFound = false;
    
    public void processed(int i) {
        updateProgressBar(i);
    }

    public void processing() {
        setCancelMode();
    }

    public void processingTotal(int i) {
        initProgressBar(i);
    }

    public int resultCount() {
        return getObjects().size();
    }

    public void stopped() {
        setSearchMode();
    }
}