/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.forms;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.filechooser.FileFilter;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcDialog;
import net.sf.dc.console.components.DcFileField;
import net.sf.dc.console.components.DcPictureField;
import net.sf.dc.core.Repository;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.ext.util.SwingUtil;
import net.sf.ext.util.Utilities;

public class PictureView extends DcDialog {

    private byte[] bytes;
    private DcFileField ffOpenPicture;
    private DcPictureField picField;

    private JPanel panelPicture = new JPanel();

    private boolean closed = false;

    public PictureView(byte[] bytes) {
        super(SwingUtil.getRootFrame());

        this.bytes = bytes;

        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                close();
            }
        });

        buildPanel();
        correctSize();

        setCenteredLocation();

        setModal(true);
        setVisible(true);
    }

    public void close() {
        picField.flush();
        ffOpenPicture = null;
        picField = null;
        bytes = null;
        dispose();
    }

    public Object getValue() {
        return bytes;
    }

    public boolean wasClosed() {
    	return closed;
    }

    private void correctSize() {
        Dimension dim = picField.getMaximumSize();
        if (dim.width == -1 || dim.height == -1) {
            ImageIcon icon = new ImageIcon(bytes);
            int height = icon.getIconHeight();
            int width = icon.getIconWidth();
            dim = new Dimension(width, height);
        }

        if (picField.isEmpty()) {
            panelPicture.setPreferredSize(new Dimension(dim.width,
                    dim.width + 100));
            pack();
        } else if (dim.height > 480 && dim.width > 640) {
            panelPicture.setPreferredSize(new Dimension(640, 480));
            pack();
        } else {
            panelPicture.setPreferredSize(new Dimension(dim.width + 20, dim.height + 20));
            pack();
        }
    }

    private void savePicture() {
        try {
            File file = ffOpenPicture.getFile();
            if (file != null) {
                bytes = Utilities.readFile(file);
                picField.setValue(bytes);
                correctSize();

                boolean delete = DcSettings.getValueAsBoolean(Repository.settings.stDeleteImageFileAfterImport);
                if (delete) file.delete();
            } else {
                bytes = null;
                picField.setValue(null);
            }
        } catch (Exception exp) {
            Messages.add(exp);
        }

        setCenteredLocation();
    }

    private void buildPanel() {
        JPanel panelFile = new JPanel();
        JPanel panelActions = new JPanel();

        // ************************************************************************
        // PICTURE PANEL
        // Set layout style
        panelPicture.setLayout(Layout.getGBL());
        panelPicture.setMaximumSize(new Dimension(640, 480));

        // Initialize components
        picField = new DcPictureField(true, false, false);
        picField.setValue(bytes);

        // Build the panel
        panelPicture.add(picField,  Layout.getGBC( 0, 0, 1, 1, 40.0, 40.0
                                   ,GridBagConstraints.NORTH, GridBagConstraints.BOTH,
                                    new Insets( 5, 5, 5, 5), 0, 0));

        //**********************************************************
        //File Action panel
        //**********************************************************
        panelFile.setLayout(Layout.getGBL());
        ffOpenPicture = new DcFileField(new JpegFileFilter());
        ffOpenPicture.setModus(false, false);
        ffOpenPicture.addListener(new ActionImportImage());

        panelFile.add(ffOpenPicture,  Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                                     ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                      new Insets( 0, 0, 0, 0), 0, 0));

        //**********************************************************
        //Action panel
        //**********************************************************
        panelActions.setLayout(Layout.getGBL());

        JButton buttonSave = ComponentFactory.getButton(DcResources.getText("lblSave"));
        JButton buttonDelete = ComponentFactory.getButton(DcResources.getText("lblDelete"));
        JButton buttonCancel = ComponentFactory.getButton(DcResources.getText("lblCancel"));

        buttonSave.setMnemonic('S');
        buttonDelete.setMnemonic('D');
        buttonCancel.setMnemonic('C');
        
        buttonSave.addActionListener(new ActionSave());
        buttonCancel.addActionListener(new ActionClose());
        buttonDelete.addActionListener(new ActionDelete());

        panelActions.add(buttonSave,    Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                                       ,GridBagConstraints.NORTH, GridBagConstraints.NONE,
                                        new Insets( 5, 5, 5, 5), 0, 0));
        panelActions.add(buttonDelete,  Layout.getGBC( 1, 0, 1, 1, 1.0, 1.0
                                       ,GridBagConstraints.NORTH, GridBagConstraints.NONE,
                                        new Insets( 5, 5, 5, 5), 0, 0));
        panelActions.add(buttonCancel,   Layout.getGBC( 2, 0, 1, 1, 1.0, 1.0
                                       ,GridBagConstraints.NORTH, GridBagConstraints.NONE,
                                        new Insets( 5, 5, 5, 5), 0, 0));

        //**********************************************************
        //Main panel
        //**********************************************************
        getContentPane().setLayout(Layout.getGBL());
        setTitle(DcResources.getText("lblPictureViewer"));

        getContentPane().add(   panelPicture,  Layout.getGBC( 0, 0, 1, 1, 40.0, 40.0
                                              ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                               new Insets( 0, 0, 0, 0), 0, 0));
        getContentPane().add(   panelFile,     Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                                              ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                               new Insets( 0, 0, 0, 0), 0, 0));
        getContentPane().add(   panelActions,  Layout.getGBC( 0, 2, 1, 1, 1.0, 1.0
                                              ,GridBagConstraints.NORTHEAST, GridBagConstraints.NONE,
                                               new Insets( 0, 0, 0, 0), 0, 0));
    }

    private class ActionDelete implements ActionListener {
       public void actionPerformed(ActionEvent arg0) {
           bytes = null;
           picField.setValue(null);
           picField.updateUI();
           correctSize();

           setCenteredLocation();
       }
    }

    private class ActionClose implements ActionListener {
       public void actionPerformed(ActionEvent arg0) {
       	   closed = true;
           dispose();
       }
    }

    private class ActionSave implements ActionListener {
       public void actionPerformed(ActionEvent arg0) {
           dispose();
       }
    }

    private class ActionImportImage implements ActionListener {
       public void actionPerformed(ActionEvent arg0) {
            savePicture();
       }
    }

    private class JpegFileFilter extends FileFilter {
        public boolean accept(File file) {
            if (file.isDirectory()) {
                return true;
            } else if (Utilities.getExtension(file).equals("jpg") ||
                       Utilities.getExtension(file).equals("jpeg")||
                       Utilities.getExtension(file).equals("png") ||
                       Utilities.getExtension(file).equals("gif") ||
                       Utilities.getExtension(file).equals("bmp")) {
                return true;
            } else {
                return false;
            }
        }

        public String getDescription() {
            return DcResources.getText("lblPicFileFilter");
        }
    }
}
