/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.forms;

import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.console.dialogs.QuestionBox;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.template.ITemplate;
import net.sf.dc.exceptions.ValidationException;
import net.sf.dc.resources.DcResources;
import net.sf.dc.wf.requests.CloseWindowRequest;
import net.sf.dc.wf.requests.RefreshSimpleItemViewRequest;
import net.sf.dc.wf.requests.UpdateDefaultTemplate;

public class TemplateItemForm extends ItemForm {
    
    private DcSimpleItemView parent;
    
    public TemplateItemForm(boolean update, DcObject o, DcSimpleItemView parent) {
        super(DataCrow.mainFrame, false, update, o, false, false, true);
        this.parent = parent;
    }
    
    protected void saveValues() {
        DcObject clone = getObjectForSaving();

        if (((ITemplate) clone).isDefault()) {
            clone.addRequest(new UpdateDefaultTemplate((String) clone.getValue(50), dco.getModule().getIndex()));
        }

        clone.addRequest(new RefreshSimpleItemViewRequest(parent));
        clone.addRequest(new CloseWindowRequest(this));
        
        if (!update)  {
            try {
                clone.setSilence(true);
                clone.saveNew(true);
            } catch (ValidationException vExp) {
                new MessageBox(vExp.getMessage(), MessageBox._WARNING);
            }
        } else if (isChanged()) {
            try {
                clone.setSilence(true);
                clone.saveUpdate(true);
            } catch (ValidationException vExp) {
                new MessageBox(vExp.getMessage(), MessageBox._WARNING);
            }
        } else if (! isChanged()) {
            new MessageBox(DcResources.getText("msgNoChangesToSave"), MessageBox._WARNING);
        }
    }  
    
    /**
     * Deletes this item from the database
     */
    protected void deleteValues() {
        new QuestionBox(DcResources.getText("msgDeleteQuestion"), this);
        
        if (QuestionBox.result.booleanValue()) {
            String id = dco.getID();
            dco.clearValues();
            dco.setValue(DcObject._ID, id);
            dco.setSilence(true);

            dco.addRequest(new CloseWindowRequest(this));
            dco.addRequest(new RefreshSimpleItemViewRequest(parent));
            dco.delete();
        }
    }
}
