/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.importcd;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcFileField;
import net.sf.dc.console.components.DcFrame;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcStorageMedium;
import net.sf.dc.processes.cdimport.ImportCDProcess;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;

public class ImportCDDialog extends DcFrame {
    
    protected JCheckBox checkVerify;
    protected JCheckBox checkRecursive;
    protected DcFileField ffSource;
    protected JTextField textCDName;
    protected JComboBox comboMedium;

    protected final JCheckBox checkFilesOnly =
        ComponentFactory.getCheckBox(DcResources.getText("lblSelectFilesOnly"));
    protected final JButton buttonRun =
        ComponentFactory.getButton(DcResources.getText("lblRun"));
    protected final JButton buttonStop =
        ComponentFactory.getButton(DcResources.getText("lblStop"));
    protected final JButton buttonCancel =
        ComponentFactory.getButton(DcResources.getText("lblCancel"));

    protected JTextArea textLog = ComponentFactory.getTextArea();
    protected JProgressBar progressBar = new JProgressBar();

    protected ImportCDProcess process;

    protected final int moduleIndex;

    public ImportCDDialog(int moduleIndex, JFrame parent, String title) {
        super(title, IconLibrary._icoImport, parent);

        this.moduleIndex = moduleIndex;

        setHelpIndex("dc.Tools.DataImport");

        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
            	close();
            }
        });

        buildDialog();

        Dimension dim = DcSettings.getValueAsDimension(Repository.settings.stImportCDDialogSize);
        setSize(dim);
        setCenteredLocation();
    }

    protected DcStorageMedium getStorageMedium() {
        DcModule module = DcModules.get(moduleIndex);
        
        DcStorageMedium medium = null;
        if (module.getStorageMediaModule() != null) {
            try {
                medium = (DcStorageMedium) comboMedium.getSelectedItem();
            } catch (Exception exp) {}
        }
            
        return medium;
    }

    public void initProgressBar(int maxValue) {
        progressBar.setValue(0);
        progressBar.setMaximum(maxValue);
    }

    public void updateProgressBar(int value) {
        progressBar.setValue(value);
    }

    public void addMessage(String message) {
        textLog.insert(message + '\n', 0);
    }

    protected void startImport() {
        boolean confirmDirs = checkVerify.isSelected();
        boolean recursiveDirs = checkRecursive.isSelected();

        String sourceDir = "";
        if (ffSource.getFile() != null) {
            sourceDir = ffSource.getFileAsString();
        }

        if (sourceDir == null || sourceDir.equals("")) {
            new MessageBox(this, DcResources.getText("msgSelectDirectory"), MessageBox._WARNING);
            return;
        }

        String cdName = textCDName.getText().trim();
        if (cdName.equals("")) {
            new MessageBox(this, DcResources.getText("msgEnterCDName"),
                           MessageBox._WARNING);
            return;
        }

        denyActions();

        DcModule module = DcModules.get(moduleIndex);
        process = module.getCDImportProcess(this, getStorageMedium(), sourceDir, confirmDirs, recursiveDirs, cdName);
        process.setPriority(5);
        process.start();
    }

    public void denyActions() {
        progressBar.setValue(0);
        textLog.setText("");

    	buttonRun.setEnabled(false);
    }

    public void allowActions() {
        buttonRun.setEnabled(true);
    }

    protected void close() {
    	cancel();
        DcSettings.setValue(Repository.settings.stImportCDDialogSize, getSize());
        dispose();
        setVisible(false);
    }

    protected void cancel() {
        allowActions();
    	if (process != null) {
    		process.cancel();
        }
    }

    protected void buildDialog() {
        //**********************************************************
        //Input panel
        //**********************************************************
        JPanel panelInput = new JPanel();
        panelInput.setLayout(Layout.getGBL());
        panelInput.setBorder(ComponentFactory.getTitleBorder(""));

        JLabel labelLocation = ComponentFactory.getLabel(DcResources.getText("lblLocation"));
        ffSource = new DcFileField(null);
        checkVerify = ComponentFactory.getCheckBox(DcResources.getText("lblAskPerDir"));

        checkRecursive  = ComponentFactory.getCheckBox(DcResources.getText("lblRecursiveDir"));
        JLabel labelCDName = ComponentFactory.getLabel(DcResources.getText("lblLocationName"));
        textCDName = ComponentFactory.getShortTextField(100);

        textCDName.setPreferredSize(new Dimension(25, 22));
        textCDName.setMinimumSize(new Dimension(25, 22));
        
        JLabel labelMedium = ComponentFactory.getLabel(DcResources.getText("lblStorageMedium"));
        comboMedium = ComponentFactory.getMediaCombo(DcModules.get(moduleIndex).getStorageMediaModule().getIndex());

        checkRecursive.setSelected(true);
        checkFilesOnly.addChangeListener(new ToggleSearchMode());

        ffSource.setModus(false, true);

        panelInput.add(  labelLocation,  Layout.getGBC( 0, 0, 2, 1, 1.0, 1.0
                                        ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                         new Insets(0, 5, 0, 5), 0, 0));
        panelInput.add(  ffSource,       Layout.getGBC( 1, 0, 3, 1, 1.0, 1.0
                                        ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                         new Insets(0, 5, 0, 5), 0, 0));
        panelInput.add(  labelCDName,    Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                                        ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                         new Insets(0, 5, 0, 5), 0, 0));
        panelInput.add(  textCDName,     Layout.getGBC( 1, 1, 2, 1, 20.0, 20.0
                                        ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                         new Insets(0, 5, 0, 5), 0, 0));

        DcModule module = DcModules.get(moduleIndex);
        if (module.getStorageMediaModule() != null) {
            panelInput.add(  labelMedium,    Layout.getGBC( 0, 2, 1, 1, 1.0, 1.0
                    ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                     new Insets(0, 5, 0, 5), 0, 0));
            panelInput.add(  comboMedium,    Layout.getGBC( 1, 2, 2, 1, 20.0, 20.0
                    ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                     new Insets(0, 5, 0, 5), 0, 0));
        }

        //**********************************************************
        //Settings Panel
        //**********************************************************
        JPanel panelSettings = new JPanel();
        panelSettings.setLayout(Layout.getGBL());
        panelSettings.setBorder(ComponentFactory.getTitleBorder(DcResources.getText("lblSettings")));

        panelSettings.add(checkFilesOnly, Layout.getGBC( 0, 0, 3, 1, 1.0, 1.0
                                         ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                          new Insets(5, 5, 0, 5), 0, 0));
        panelSettings.add(checkVerify,    Layout.getGBC( 0, 1, 3, 1, 1.0, 1.0
                                         ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                          new Insets(5, 5, 0, 5), 0, 0));
        panelSettings.add(checkRecursive, Layout.getGBC( 0, 2, 3, 1, 1.0, 1.0
                                         ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                          new Insets(5, 5, 5, 5), 0, 0));

        //**********************************************************
        //Actions Panel
        //**********************************************************
        JPanel panelActions = new JPanel();
        panelActions.setLayout(Layout.getGBL());

        buttonRun.addActionListener(new ExportCDButtonAction());
        buttonStop.addActionListener(new CancelButtonAction());
        buttonCancel.addActionListener(new CloseButtonAction());

        buttonRun.setMnemonic(KeyEvent.VK_R);
        buttonStop.setMnemonic(KeyEvent.VK_T);
        buttonCancel.setMnemonic(KeyEvent.VK_C);

        panelActions.add(buttonRun,     Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                        new Insets( 5, 5, 5, 5), 0, 0));
        panelActions.add(buttonStop,    Layout.getGBC( 1, 0, 1, 1, 1.0, 1.0
                                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                        new Insets( 5, 5, 5, 5), 0, 0));
        panelActions.add(buttonCancel,  Layout.getGBC( 2, 0, 1, 1, 1.0, 1.0
                                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                        new Insets( 5, 5, 5, 5), 0, 0));

        //**********************************************************
        //Progress panel
        //**********************************************************
        JPanel panelProgress = new JPanel();
        panelProgress.setLayout(Layout.getGBL());
        panelProgress.add(progressBar, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                          new Insets(5, 5, 5, 5), 0, 0));

        //**********************************************************
        //Log Panel
        //**********************************************************
        JPanel panelLog = new JPanel();
        panelLog.setLayout(Layout.getGBL());

        JScrollPane scroller = new JScrollPane(textLog);
        scroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        panelLog.add(scroller, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                     new Insets(5, 5, 5, 5), 0, 0));

        panelLog.setBorder(ComponentFactory.getTitleBorder(DcResources.getText("lblLog")));


        //**********************************************************
        //Main Panel
        //**********************************************************
        this.getContentPane().setLayout(Layout.getGBL());
        this.getContentPane().add(
                 panelInput,      Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets( 5, 5, 5, 5), 0, 0));
        this.getContentPane().add(
                panelSettings,    Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
               ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                new Insets( 5, 5, 5, 5), 0, 0));
        this.getContentPane().add(
                 panelActions,    Layout.getGBC( 0, 2, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHEAST, GridBagConstraints.NONE,
                 new Insets( 0, 0, 0, 0), 0, 0));
        this.getContentPane().add(
                 panelProgress,   Layout.getGBC( 0, 3, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets( 0, 0, 0, 0), 0, 0));
        this.getContentPane().add(
                 panelLog,        Layout.getGBC( 0, 4, 1, 1, 40.0, 40.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                 new Insets( 0, 0, 0, 0), 0, 0));

        pack();
        setSize(DcSettings.getValueAsDimension(Repository.settings.stImportCDDialogSize));
        setResizable(true);
        setCenteredLocation();
    }

    protected class ToggleSearchMode implements ChangeListener {
        public void stateChanged(ChangeEvent e) {
            ffSource.setModus(false, !checkFilesOnly.isSelected());
        }
    }

    protected class CloseButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            close();
        }
    }

    protected class CancelButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            cancel();
        }
    }

    protected class ExportCDButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startImport();
        }
    }
}
