/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.importcd;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javax.swing.ButtonGroup;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcFileField;
import net.sf.dc.console.components.DcRadioButton;
import net.sf.dc.console.components.panels.OnlineServicePanel;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.services.DcRegion;
import net.sf.dc.core.services.DcServer;
import net.sf.dc.processes.cdimport.ImportMusicFilesCD;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;

public class ImportMusicFilesDialog extends ImportCDDialog {

    private ButtonGroup group;
    
    private DcRadioButton radioDoNotUseDir;
    private DcRadioButton radio1stDirAlbum;
    private DcRadioButton radio1stDirArtist;
    private DcRadioButton radio1stDirAlbum2ndDirArtist;
    
    private OnlineServicePanel panelServer;
    
    private JCheckBox checkAutoItemCreation;
    
    
    public ImportMusicFilesDialog(int moduleIndex, JFrame parent, String title) {
        super(moduleIndex, parent, title);
        setHelpIndex("dc.Tools.MusicFileImport");
        Dimension dim = DcSettings.getValueAsDimension(Repository.settings.stImportMusicFileDialogSize);
        setSize(dim);
        setCenteredLocation();
    }
    
    public DcServer getServer() {
        return panelServer.getServer();
    }

    public DcRegion getRegion() {
        return panelServer.getRegion();
    }
    
    public boolean useOnlineService() {
        return panelServer.useOnlineService();
    }
    
    protected void close() {
        cancel();
        DcSettings.setValue(Repository.settings.stImportMusicFileDialogSize, getSize());
        dispose();
        setVisible(false);
    }
    
    protected void startImport() {
        boolean confirmDirs = checkVerify.isSelected();
        boolean recursiveDirs = checkRecursive.isSelected();

        String sourceDir = "";
        if (ffSource.getFile() != null) {
            sourceDir = ffSource.getFileAsString();
        }

        if (sourceDir == null || sourceDir.equals("")) {
            new MessageBox(this, DcResources.getText("msgSelectDirectory"), MessageBox._WARNING);
            return;
        }

        String cdName = textCDName.getText().trim();
        if (cdName.equals("")) {
            new MessageBox(this, DcResources.getText("msgEnterCDName"),
                           MessageBox._WARNING);
            return;
        }

        denyActions();

        int dirUsage = radioDoNotUseDir.isSelected() ? 0 : radio1stDirAlbum.isSelected() ? 1 :
                       radio1stDirArtist.isSelected() ? 2 : 3;
        
        process = new ImportMusicFilesCD(this, getStorageMedium(), sourceDir, confirmDirs, recursiveDirs, cdName, dirUsage);
        process.setPriority(5);
        process.start();
    }    
    
    protected void buildDialog() {

        //**********************************************************
        //Settings panel
        //**********************************************************
        JPanel panelSettings = new JPanel();
        panelSettings.setLayout(Layout.getGBL());
        
        checkVerify = ComponentFactory.getCheckBox(DcResources.getText("lblAskPerDir"));
        checkRecursive = ComponentFactory.getCheckBox(DcResources.getText("lblRecursiveDir"));
        checkRecursive.setSelected(true);
        
        checkAutoItemCreation = ComponentFactory.getCheckBox(DcResources.getText("lblAutoCreateMissingItems"));
        checkAutoItemCreation.addActionListener(new AutoCreationListener());
        checkAutoItemCreation.setSelected(DcSettings.getValueAsBoolean(Repository.settings.stAutoCreateSubItems));
        
        panelSettings.add(checkRecursive,           Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets(5, 5, 5, 5), 0, 0));
        panelSettings.add(checkAutoItemCreation,    Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets(5, 5, 5, 5), 0, 0));
        
        panelSettings.setBorder(ComponentFactory.getTitleBorder(DcResources.getText("lblSettings")));
        
        //**********************************************************
        //Input panel
        //**********************************************************
        JPanel panelInput = new JPanel();
        panelInput.setLayout(Layout.getGBL());

        JLabel labelLocation = ComponentFactory.getLabel(DcResources.getText("lblLocation"));
        ffSource = new DcFileField(null);
        JLabel labelCDName = ComponentFactory.getLabel(DcResources.getText("lblLocationName"));
        textCDName = ComponentFactory.getShortTextField(100);

        textCDName.setPreferredSize(new Dimension(25, 22));
        textCDName.setMinimumSize(new Dimension(25, 22));
        
        JLabel labelMedium = ComponentFactory.getLabel(DcResources.getText("lblStorageMedium"));
        comboMedium = ComponentFactory.getMediaCombo(DcModules.get(moduleIndex).getStorageMediaModule().getIndex());
        checkFilesOnly.addChangeListener(new ToggleSearchMode());

        ffSource.setModus(false, true);

        panelInput.add(  labelLocation,  Layout.getGBC( 0, 0, 2, 1, 1.0, 1.0
                                        ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                         new Insets(0, 5, 0, 5), 0, 0));
        panelInput.add(  ffSource,       Layout.getGBC( 1, 0, 3, 1, 1.0, 1.0
                                        ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                         new Insets(0, 5, 0, 5), 0, 0));
        panelInput.add(  labelCDName,    Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                                        ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                         new Insets(0, 5, 0, 5), 0, 0));
        panelInput.add(  textCDName,     Layout.getGBC( 1, 1, 2, 1, 1.0, 1.0
                                        ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                         new Insets(0, 5, 0, 5), 0, 0));

        DcModule module = DcModules.get(moduleIndex);
        if (module.getStorageMediaModule() != null) {
            panelInput.add(  labelMedium,    Layout.getGBC( 0, 2, 1, 1, 1.0, 1.0
                    ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                     new Insets(0, 5, 0, 5), 0, 0));
            panelInput.add(  comboMedium,    Layout.getGBC( 1, 2, 2, 1, 20.0, 20.0
                    ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                     new Insets(0, 5, 0, 5), 0, 0));
        }
        
        //**********************************************************
        //Server panel
        //**********************************************************
        panelServer = new OnlineServicePanel(DcModules.get(DcModules._MUSICALBUM).getOnlineServices(), true);

        //**********************************************************
        //Directory Usage panel
        //**********************************************************
        JPanel panelDirs = new JPanel();
        panelDirs.setLayout(Layout.getGBL());
        panelDirs.setBorder(ComponentFactory.getTitleBorder(DcResources.getText("lblDirectoryUsage")));
        
        radioDoNotUseDir = ComponentFactory.getRadioButton(DcResources.getText("lblDoNotUseDirInfo"), null);
        radio1stDirAlbum = ComponentFactory.getRadioButton(DcResources.getText("lblUseDirAsAlbumName"), null);
        radio1stDirArtist = ComponentFactory.getRadioButton(DcResources.getText("lblUseDirAsArtistName"), null);
        radio1stDirAlbum2ndDirArtist = ComponentFactory.getRadioButton(DcResources.getText("lblUseDirAsArtistSubDirAsAlbum"), null);

        group = new ButtonGroup();
        group.add(radioDoNotUseDir);
        group.add(radio1stDirAlbum);
        group.add(radio1stDirArtist);
        group.add(radio1stDirAlbum2ndDirArtist);
        
        panelDirs.add(radioDoNotUseDir, Layout.getGBC( 0, 3, 3, 1, 1.0, 1.0
                ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets(0, 5, 0, 5), 0, 0));
        panelDirs.add(radio1stDirAlbum, Layout.getGBC( 0, 4, 3, 1, 1.0, 1.0
                ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets(0, 5, 0, 5), 0, 0));
        panelDirs.add(radio1stDirArtist, Layout.getGBC( 0, 5, 3, 1, 1.0, 1.0
                ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets(0, 5, 0, 5), 0, 0));
        panelDirs.add(radio1stDirAlbum2ndDirArtist, Layout.getGBC( 0, 6, 3, 1, 1.0, 1.0
                ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets(0, 5, 0, 5), 0, 0));

        panelInput.add(panelServer,     Layout.getGBC( 0, 3, 3, 1, 1.0, 1.0
                ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets(15, 5, 0, 5), 0, 0));
        panelInput.add(panelDirs,       Layout.getGBC( 0, 4, 3, 1, 1.0, 1.0
                ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets(15, 5, 0, 5), 0, 0));
        panelInput.add(panelSettings,   Layout.getGBC( 0, 5, 3, 1, 1.0, 1.0
                ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets(15, 5, 0, 5), 0, 0));

        radioDoNotUseDir.setSelected(true);
        
        //**********************************************************
        //Actions Panel
        //**********************************************************
        JPanel panelActions = new JPanel();
        panelActions.setLayout(Layout.getGBL());

        buttonRun.addActionListener(new ExportCDButtonAction());
        buttonStop.addActionListener(new CancelButtonAction());
        buttonCancel.addActionListener(new CloseButtonAction());

        buttonRun.setMnemonic(KeyEvent.VK_R);
        buttonStop.setMnemonic(KeyEvent.VK_T);
        buttonCancel.setMnemonic(KeyEvent.VK_C);

        panelActions.add(buttonRun,  Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                          new Insets( 5, 5, 5, 5), 0, 0));
        panelActions.add(buttonStop,    Layout.getGBC( 1, 0, 1, 1, 1.0, 1.0
                                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                          new Insets( 5, 5, 5, 5), 0, 0));
        panelActions.add(buttonCancel,     Layout.getGBC( 2, 0, 1, 1, 1.0, 1.0
                                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                          new Insets( 5, 5, 5, 5), 0, 0));

        //**********************************************************
        //Progress panel
        //**********************************************************
        JPanel panelProgress = new JPanel();
        panelProgress.setLayout(Layout.getGBL());
        panelProgress.add(progressBar, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                          new Insets(5, 5, 5, 5), 0, 0));

        //**********************************************************
        //Log Panel
        //**********************************************************
        JPanel panelLog = new JPanel();
        panelLog.setLayout(Layout.getGBL());

        JScrollPane scroller = new JScrollPane(textLog);
        scroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        panelLog.add(scroller, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                     new Insets(5, 5, 5, 5), 0, 0));

        panelLog.setBorder(ComponentFactory.getTitleBorder(DcResources.getText("lblLog")));


        //**********************************************************
        //Main Panel
        //**********************************************************
        this.getContentPane().setLayout(Layout.getGBL());

        this.getContentPane().add(
                 panelInput,      Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets( 5, 5, 5, 5), 0, 0));
        this.getContentPane().add(
                 panelActions, Layout.getGBC( 0, 2, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHEAST, GridBagConstraints.NONE,
                 new Insets( 0, 0, 0, 0), 0, 0));
        this.getContentPane().add(
                 panelProgress, Layout.getGBC( 0, 3, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets( 0, 0, 0, 0), 0, 0));
        this.getContentPane().add(
                 panelLog, Layout.getGBC( 0, 4, 2, 1, 40.0, 40.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                 new Insets( 0, 0, 0, 0), 0, 0));

        // default for music files, not selectable
        checkFilesOnly.setSelected(false);
        checkVerify.setSelected(false);
        
        pack();
        setSize(DcSettings.getValueAsDimension(Repository.settings.stImportCDDialogSize));
        setResizable(true);
        setCenteredLocation();
    }    
    
    private class AutoCreationListener implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            DcSettings.setValue(Repository.settings.stAutoCreateSubItems, 
                                new Boolean(checkAutoItemCreation.isSelected()));
        }
    }
}
