/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.menu;

import java.awt.Toolkit;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.KeyStroke;
import javax.swing.text.JTextComponent;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.components.DcLongTextField;
import net.sf.dc.console.components.DcMenuItem;
import net.sf.dc.console.components.DcShortTextField;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.resources.DcResources;

/**
 * @author Golan
 */
public class DcEditPopupMenu extends JPopupMenu{

    private static final Toolkit tk = Toolkit.getDefaultToolkit();
    
    public DcEditPopupMenu(JTextComponent component) {
        super();

        if (component != null) {
            JMenuItem menuCut = ComponentFactory.getMenuItem(IconLibrary._icoCut, DcResources.getText("lblCut"));
            JMenuItem menuCopy =ComponentFactory.getMenuItem(IconLibrary._icoCopy, DcResources.getText("lblCopy"));
            JMenuItem menuPaste = ComponentFactory.getMenuItem(IconLibrary._icoPaste, DcResources.getText("lblPaste"));
            JMenuItem menuSelectAll = ComponentFactory.getMenuItem(DcResources.getText("lblSelectAll"));
             
            DcEditPopupAction action = new DcEditPopupAction(component);
            boolean isEditable = component.isEditable() && component.isEnabled();
            boolean isTextSelected = component.getSelectedText() != null;

            menuCut.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, InputEvent.CTRL_MASK));
            menuCopy.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, InputEvent.CTRL_MASK));
            menuPaste.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P, InputEvent.CTRL_MASK));
            menuSelectAll.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A, InputEvent.CTRL_MASK));
            
            if (isEditable && isTextSelected)
                menuCut.addActionListener(action.new PopupCutAction());
            else
                menuCut.setEnabled(false);
            
            if (isTextSelected)
                menuCopy.addActionListener(action.new PopupCopyAction());
            else
                menuCopy.setEnabled(false);
            
            Transferable content = tk.getSystemClipboard().getContents(null);
            if (isEditable && content.isDataFlavorSupported(DataFlavor.stringFlavor))
                menuPaste.addActionListener(action.new PopupPasteAction());
            else
                menuPaste.setEnabled(false);
            
            if(component != null && component.isEnabled() && component.getText().length() > 0 
                    && (component.getSelectedText() == null || 
                            component.getSelectedText().length() < component.getText().length()))
                menuSelectAll.addActionListener(action.new PopupSelectAllAction());
            else
                menuSelectAll.setEnabled(false);
    
            this.add(menuCut);
            this.add(menuCopy);
            this.add(menuPaste);
            
            if (component instanceof DcShortTextField || component instanceof DcLongTextField) {
                this.addSeparator();

                DcMenuItem menuUndo;
                DcMenuItem menuRedo;

                if (component instanceof DcShortTextField) {
                    menuUndo = ComponentFactory.getMenuItem(((DcShortTextField) component).getUndoListener().getUndoAction());
                    menuRedo = ComponentFactory.getMenuItem(((DcShortTextField) component).getUndoListener().getRedoAction());
                } else {
                    menuUndo = ComponentFactory.getMenuItem(((DcLongTextField) component).getUndoListener().getUndoAction());
                    menuRedo = ComponentFactory.getMenuItem(((DcLongTextField) component).getUndoListener().getRedoAction());
                }

                menuUndo.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, InputEvent.CTRL_MASK));
                menuRedo.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Y, InputEvent.CTRL_MASK));

                this.add(menuUndo);
                this.add(menuRedo);
                
                if (!isEditable) {
                    menuUndo.setEnabled(false);
                    menuRedo.setEnabled(false);
                }
            }
            
            this.addSeparator();
            this.add(menuSelectAll);
        }
    }
}
