/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.menu;

import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.components.DcMenu;
import net.sf.dc.console.components.DcMenuItem;
import net.sf.dc.console.views.MasterView;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.actions.AboutAction;
import net.sf.dc.core.actions.ChangeLookAndFeelAction;
import net.sf.dc.core.actions.ChangeViewAction;
import net.sf.dc.core.actions.CreateNewAction;
import net.sf.dc.core.actions.CreateReportAction;
import net.sf.dc.core.actions.DeleteAction;
import net.sf.dc.core.actions.ExpertUserAction;
import net.sf.dc.core.actions.FileImportAction;
import net.sf.dc.core.actions.HelpAction;
import net.sf.dc.core.actions.InternetUpdateAction;
import net.sf.dc.core.actions.LogAction;
import net.sf.dc.core.actions.NewItemWizardAction;
import net.sf.dc.core.actions.OpenAction;
import net.sf.dc.core.actions.OpenAutoIncrementSettingsDialog;
import net.sf.dc.core.actions.OpenBackupDialogAction;
import net.sf.dc.core.actions.OpenFieldsSettingsDialog;
import net.sf.dc.core.actions.OpenModuleAction;
import net.sf.dc.core.actions.OpenResourceEditorAction;
import net.sf.dc.core.actions.OpenSynchronizerDialog;
import net.sf.dc.core.actions.OpenTitleRewriterSettingsDialog;
import net.sf.dc.core.actions.QuitApplicationAction;
import net.sf.dc.core.actions.SaveAllAction;
import net.sf.dc.core.actions.SearchAction;
import net.sf.dc.core.actions.SearchAgainAction;
import net.sf.dc.core.actions.SettingsAction;
import net.sf.dc.core.actions.ShowTipOfTheDayAction;
import net.sf.dc.core.actions.StartProgramAction;
import net.sf.dc.core.actions.ToggleGroupingPaneAction;
import net.sf.dc.core.actions.ToggleModuleListAction;
import net.sf.dc.core.actions.ToggleQuickFilterBar;
import net.sf.dc.core.actions.ToggleQuickViewAction;
import net.sf.dc.core.actions.ToggleToolbarLabelsAction;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.template.ITemplate;
import net.sf.dc.core.objects.template.Templates;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;

public class DcMenuBar extends JMenuBar {

	public static final int _SEARCHPANEL = 0;
    public static final int _INSERTPANEL = 1;
    public static final int _NOTEPANEL = 2;
    
    private final DcModule module;

    public DcMenuBar(DcModule module) {
        this.module = module;
    	build();
    }
    
    public void setPanel(int index) {
        switch (index) {
            case _SEARCHPANEL:
                break;
            case _INSERTPANEL:
                break;
            case _NOTEPANEL:
                break;
        }
    }
    
    public void setFont(Font font) {
        for (int i = 0; i < getMenuCount(); i++) {
            JMenu menu = getMenu(i);
            menu.setFont(ComponentFactory.getSystemFont());
            for (int j = 0; j < menu.getMenuComponents().length; j++) {
                menu.getMenuComponents()[j].setFont(ComponentFactory.getStandardFont());
            }
        }
    }
    
    private void build() {
        String dcoName = module.getObjectName();
        
        JMenu menuTools = module.getToolsMenu();
        if (menuTools == null) {
        	menuTools = ComponentFactory.getMenu(DcResources.getText("lblTools"));
        }
        
        DcMenu menuSettings = ComponentFactory.getMenu(DcResources.getText("lblSettings"));
        DcMenu menuFile = ComponentFactory.getMenu(DcResources.getText("lblFile"));
        DcMenu menuFilter = ComponentFactory.getMenu(DcResources.getText("lblFilter"));
        DcMenu menuInformation = ComponentFactory.getMenu(DcResources.getText("lblHelp"));
        
        // view menu
        DcMenu menuView = ComponentFactory.getMenu(DcResources.getText("lblView"));
        DcMenuItem menuItemTableView = ComponentFactory.getMenuItem(IconLibrary._icoTableView,
                DcResources.getText("lblTableView"));
        DcMenuItem menuItemListView = ComponentFactory.getMenuItem(IconLibrary._icoCardView,
                DcResources.getText("lblCardView"));
        DcMenuItem menuItemLF = ComponentFactory.getMenuItem(
                IconLibrary._icoLAF, DcResources.getText("lblLookAndFeel"));
        DcMenuItem menuItemShowQuickFilterBar = ComponentFactory.getMenuItem(
                DcResources.getText("lblToggleQuickFilterBar"));
        DcMenuItem menuItemShowModList = ComponentFactory.getMenuItem(
                DcResources.getText("lblToggleModuleList"));
        DcMenuItem menuToggleToolBarLabels = ComponentFactory.getMenuItem(
                DcResources.getText("lblToggleToolbarLabelsAction"));
        DcMenuItem menuQuickView = ComponentFactory.getMenuItem(
                DcResources.getText("lblToggleQuickView"));
        DcMenuItem menuGroupingPane= ComponentFactory.getMenuItem(
                DcResources.getText("lblToggleGroupingPane"));
        
        DcMenu subMenuModule = ComponentFactory.getMenu(DcResources.getText("lblActiveModule"));
        
        for (Iterator iter = DcModules.getModules(); iter.hasNext(); ) {
            DcModule module = (DcModule) iter.next();
            if (module.isTopModule()) {
                DcMenuItem menuItem = ComponentFactory.getMenuItem(module.getIconSmall(), module.getObjectNamePlural());
                menuItem.addActionListener(new OpenModuleAction(module.getIndex()));
                subMenuModule.add(menuItem);
                
                if (module.getKeyStroke() != null)
                    menuItem.setAccelerator(module.getKeyStroke());
            }
        }        
        
        menuItemShowModList.addActionListener(new ToggleModuleListAction());
        menuItemShowQuickFilterBar.addActionListener(new ToggleQuickFilterBar());
        menuToggleToolBarLabels.addActionListener(new ToggleToolbarLabelsAction());
        menuQuickView.addActionListener(new ToggleQuickViewAction());
        menuGroupingPane.addActionListener(new ToggleGroupingPaneAction());
        
        menuItemTableView.addActionListener(new ChangeViewAction(MasterView._TABLE_VIEW));
        menuItemListView.addActionListener(new ChangeViewAction(MasterView._LIST_VIEW));
        menuItemLF.addActionListener(new ChangeLookAndFeelAction());
        
        menuView.add(menuItemTableView);
        menuView.add(menuItemListView);
        menuView.addSeparator();
        menuView.add(menuItemLF);
        menuView.addSeparator();
        menuView.add(menuItemShowQuickFilterBar);
        menuView.add(menuItemShowModList);
        menuView.add(menuQuickView);
        menuView.add(menuGroupingPane);
        menuView.add(menuToggleToolBarLabels);
        menuView.addSeparator();
        menuView.add(subMenuModule);
        
        
        // information menu
        DcMenuItem menuItemHelp = ComponentFactory.getMenuItem(IconLibrary._icoHelp, DcResources.getText("lblHelp"));
        DcMenuItem menuItemTip = ComponentFactory.getMenuItem(IconLibrary._icoHelp, DcResources.getText("lblTipOfTheDay"));
        DcMenuItem menuItemAbout = ComponentFactory.getMenuItem(IconLibrary._icoAbout, DcResources.getText("lblAbout"));

        menuItemHelp.addActionListener(new HelpAction());
        menuItemAbout.addActionListener(new AboutAction());
        menuItemTip.addActionListener(new ShowTipOfTheDayAction());

        menuInformation.add(menuItemHelp);
        menuInformation.add(menuItemTip);
        menuInformation.addSeparator();
        menuInformation.add(menuItemAbout);
        
        // search menu
        DcMenuItem menuItemFilter = ComponentFactory.getMenuItem(IconLibrary._icoSearch, DcResources.getText("lblFilter"));
        DcMenuItem menuItemApplyFilter = ComponentFactory.getMenuItem(IconLibrary._icoSearchAgain, DcResources.getText("lblApplyFilter"));
        menuItemFilter.addActionListener(new SearchAction());
        menuItemApplyFilter.addActionListener(new SearchAgainAction());

        menuFilter.add(menuItemFilter);
        menuFilter.add(menuItemApplyFilter);
        
        if (    module.isAmazonUpdateAllowed() || 
                module.isImdbUpdateAllowed() || 
                module.getIndex() == DcModules._AUDIOCD) {

            DcMenuItem menuItemOnlineSearch = ComponentFactory.getMenuItem(IconLibrary._icoSearchOnline, DcResources.getText("lblOnlineSearch"));
            
            menuItemOnlineSearch.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_U, InputEvent.CTRL_MASK));
            menuItemOnlineSearch.addActionListener(new InternetUpdateAction());
            menuTools.add(menuItemOnlineSearch);
            
            if (module.getSynchronizer() == null)
                menuTools.addSeparator();
        }
        
        if (module.getSynchronizer() != null) {
            JMenuItem menuMusicSynchronizer = ComponentFactory.getMenuItem(IconLibrary._icoMassUpdate, module.getSynchronizer().getTitle());  
            menuMusicSynchronizer.addActionListener(new OpenSynchronizerDialog());
            menuTools.add(menuMusicSynchronizer);
            menuTools.addSeparator();
        }        
        
        // file menu
        DcMenuItem menuNewItemWizard = ComponentFactory.getMenuItem(     
                IconLibrary._icoWizard, DcResources.getText("lblWizard"));
        
        DcMenuItem menuItemLog = ComponentFactory.getMenuItem(IconLibrary._icoEventLog, DcResources.getText("lblShowLog"));
        DcMenuItem menuItemNew = ComponentFactory.getMenuItem(IconLibrary._icoAdd, DcResources.getText("lblNewItem", dcoName));

        DcMenu menuCreateNew = ComponentFactory.getMenu(IconLibrary._icoAdd, DcResources.getText("lblNewItem", dcoName));
        boolean templatesPresent = false;
        if (module.getTemplateModule() != null) {
            Collection templates = Templates.getTemplates(module.getTemplateModule().getIndex());
            for (Iterator iter = templates.iterator(); iter.hasNext(); ) {
                templatesPresent = true;
                ITemplate template = (ITemplate) iter.next();
                DcMenuItem menuItem = ComponentFactory.getMenuItem(IconLibrary._icoTemplate, DcResources.getText("lblTemplateX", template.toString()));
                menuItem.addActionListener(new CreateNewAction(template));
                menuCreateNew.add(menuItem);
            }
        }        
        
        DcMenuItem menuItemOpen = ComponentFactory.getMenuItem(IconLibrary._icoOpen, DcResources.getText("lblOpenItem", dcoName));
        DcMenuItem menuItemSaveAll = ComponentFactory.getMenuItem(IconLibrary._icoSave, DcResources.getText("lblSave"));
        DcMenuItem menuItemDelete = ComponentFactory.getMenuItem(IconLibrary._icoDelete, DcResources.getText("lblDeleteItem", dcoName));
        DcMenuItem menuItemQuit = ComponentFactory.getMenuItem(IconLibrary._icoExit, DcResources.getText("lblQuit"));
        DcMenuItem menuItemLaunch = ComponentFactory.getMenuItem(IconLibrary._icoOpenApplication, DcResources.getText("lblStartProgram"));

        menuItemSaveAll.addActionListener(new SaveAllAction());
        menuItemNew.addActionListener(new CreateNewAction());
        menuItemOpen.addActionListener(new OpenAction());
        menuItemQuit.addActionListener(new QuitApplicationAction());
        menuItemDelete.addActionListener(new DeleteAction());
        menuItemLog.addActionListener(new LogAction());
        
    	menuItemLaunch.addActionListener(new StartProgramAction());
        
        menuNewItemWizard.addActionListener(new NewItemWizardAction());
        
        menuFile.add(menuNewItemWizard);
        menuFile.addSeparator();
        menuFile.add(menuItemLaunch);
        menuFile.addSeparator();

        if (templatesPresent)
            menuFile.add(menuCreateNew);
        
        menuFile.add(menuItemNew);
        menuFile.add(menuItemOpen);
        menuFile.add(menuItemSaveAll);
        menuFile.add(menuItemDelete);
        menuFile.addSeparator();
        menuFile.add(menuItemLog);
        menuFile.add(menuItemQuit);

        // settings menu
        DcMenuItem menuFieldSettings = ComponentFactory.getMenuItem(
                IconLibrary._icoSettings, DcResources.getText("lblFieldSettings"));
        DcMenuItem menuItemSettings = ComponentFactory.getMenuItem(
                IconLibrary._icoSettings, DcResources.getText("lblSettings"));
        DcMenuItem menuItemResources = ComponentFactory.getMenuItem(
                IconLibrary._icoSettings, DcResources.getText("lblResourceEditor"));
        
        menuFieldSettings.addActionListener(new OpenFieldsSettingsDialog());
        menuItemSettings.addActionListener(new SettingsAction());
        menuItemResources.addActionListener(new OpenResourceEditorAction());
                
        menuSettings.add(menuItemSettings);
        menuSettings.add(menuFieldSettings);
        menuSettings.add(menuItemResources);

        // tools menu
        DcMenuItem menuReport = ComponentFactory.getMenuItem(IconLibrary._icoReport, DcResources.getText("lblCreateReport"));  
        menuReport.addActionListener(new CreateReportAction());
        
        DcMenuItem menuFileImport = ComponentFactory.getMenuItem(IconLibrary._icoFileImport, DcResources.getText("lblFileImport"));  
        menuFileImport.addActionListener(new FileImportAction());
        
        DcMenuItem menuBackup = ComponentFactory.getMenuItem(IconLibrary._icoBackup, DcResources.getText("lblBackupAndRestore"));  
        menuBackup.addActionListener(new OpenBackupDialogAction());

        DcMenuItem menuAutoIncrement = ComponentFactory.getMenuItem(IconLibrary._icoRenumber, DcResources.getText("lblAutoNumbering"));
        menuAutoIncrement.addActionListener(new OpenAutoIncrementSettingsDialog());

        DcMenuItem menuTitleRewriter = ComponentFactory.getMenuItem(IconLibrary._icoTitleRewriter, DcResources.getText("lblTitleRewriter"));
        menuTitleRewriter.addActionListener(new OpenTitleRewriterSettingsDialog());
        
        menuTools.add(menuReport);
        menuTools.add(menuFileImport);
        
        if (module.getIndex() != DcModules._GENERIC) {
            menuTools.addSeparator();
            menuTools.add(menuAutoIncrement);
            menuTools.add(menuTitleRewriter);
        }
        
        menuTools.addSeparator();
        menuTools.add(menuBackup);
        menuTools.addSeparator();        

        DcMenuItem menuExpertUser = ComponentFactory.getMenuItem(
                IconLibrary._icoSettings, DcResources.getText("lblExpertUser"));
        menuExpertUser.addActionListener(new ExpertUserAction());
        menuTools.add(menuExpertUser);
        
        this.add(menuFile);
        this.add(menuView);
        
        this.add(menuFilter);
        
        this.add(menuSettings);

        this.add(menuTools);
        this.add(menuInformation);
        
        // accelerators
        menuItemHelp.setAccelerator(     KeyStroke.getKeyStroke("F1"));
        menuItemDelete.setAccelerator(   KeyStroke.getKeyStroke(KeyEvent.VK_D, InputEvent.CTRL_MASK));
        menuReport.setAccelerator(       KeyStroke.getKeyStroke(KeyEvent.VK_E, InputEvent.CTRL_MASK));
        menuItemFilter.setAccelerator(     KeyStroke.getKeyStroke(KeyEvent.VK_F, InputEvent.CTRL_MASK));
        menuItemApplyFilter.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_G, InputEvent.CTRL_MASK));
        menuItemLog.setAccelerator(      KeyStroke.getKeyStroke(KeyEvent.VK_L, InputEvent.CTRL_MASK));
        menuItemSettings.setAccelerator( KeyStroke.getKeyStroke(KeyEvent.VK_M, InputEvent.CTRL_MASK));
        menuItemNew.setAccelerator(      KeyStroke.getKeyStroke(KeyEvent.VK_N, InputEvent.CTRL_MASK));
        menuItemOpen.setAccelerator(     KeyStroke.getKeyStroke(KeyEvent.VK_O, InputEvent.CTRL_MASK));
        menuItemLaunch.setAccelerator(   KeyStroke.getKeyStroke(KeyEvent.VK_P, InputEvent.CTRL_MASK));
        menuItemResources.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R, InputEvent.CTRL_MASK));
        menuItemSaveAll.setAccelerator(  KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_MASK));
        menuFieldSettings.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_T, InputEvent.CTRL_MASK));
        menuNewItemWizard.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_W, InputEvent.CTRL_MASK));

        menuItemTableView.setAccelerator(KeyStroke.getKeyStroke("F2"));
        menuItemListView.setAccelerator( KeyStroke.getKeyStroke("F3"));

        menuItemShowQuickFilterBar.setAccelerator(KeyStroke.getKeyStroke("F7"));
        menuItemShowModList.setAccelerator(KeyStroke.getKeyStroke("F8"));
        menuQuickView.setAccelerator(KeyStroke.getKeyStroke("F9"));
        menuGroupingPane.setAccelerator(KeyStroke.getKeyStroke("F10"));
        menuToggleToolBarLabels.setAccelerator(KeyStroke.getKeyStroke("F11"));
        
    }
    
    protected void paintComponent(Graphics g) {
        Graphics2D g2d = (Graphics2D) g;
        
        if (DcSettings.getValueAsBoolean(Repository.settings.stFontAntiAliasing)) {
           g2d.setRenderingHint( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );
           g2d.setRenderingHint( RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON );
           g2d.setRenderingHint( RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY );
        }
        super.paintComponent( g2d );
    }     
}
