/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.menu;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.DefaultListCellRenderer;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JToolBar;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.components.DcComboBox;
import net.sf.dc.console.components.DcEditableComboBox;
import net.sf.dc.console.menu.DcToolBar.ToolBarButton;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.data.DataFilter;
import net.sf.dc.core.data.DataFilters;
import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.modules.DcMappingModule;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.settings.definitions.DcFieldDefinition;
import net.sf.dc.settings.definitions.DcFieldDefinitions;

public class DcQuickFilterToolBar extends JToolBar {

    private final DcEditableComboBox comboCriteria = new DcEditableComboBox();
    private final DcComboBox comboFields = ComponentFactory.getComboBox();
    private final DcComboBox comboFilters = ComponentFactory.getComboBox();
    
    private final SearchOnEnterListener searchOnEnterListener = new SearchOnEnterListener();
    private final SearchOnDoubleClickListener searchOnDblClickListener = new SearchOnDoubleClickListener();
    private final FilterSelectionListener filterSelListener = new FilterSelectionListener();
    private final FieldSelectionListener fieldSelListener = new FieldSelectionListener();
    
    private final DcModule module;
    
    public DcQuickFilterToolBar(DcModule module) {
        this.module = module;
        build();
        comboFields.setSelectedIndex(0);
    }
    
    public void setFont(Font font) {
        Component c;
        int i = 0;
        while ((c = getComponentAtIndex(i++)) != null) {
            if (c instanceof ToolBarButton) {
                ToolBarButton button = (ToolBarButton) c;
                button.setFont(ComponentFactory.getSystemFont());
            }
        }
    }    
    
    private void search() {
        DataManager.bindData(module.getSearchView(), module.getIndex(), getDataFilter(), true);
    }
    
    private void search(DataFilter df) {
        DataManager.bindData(module.getSearchView(), module.getIndex(), df, true);
    }
    
    private DataFilter getDataFilter() {
        DcField field = (DcField) comboFields.getSelectedItem();
        
        DcObject baseDco = module.getDcObject();
        
        Object value;
        if (field.getFieldType() ==  ComponentFactory._RATINGCOMBOBOX) {
            value = comboCriteria.getSelectedItem();
            if (value != null && value.equals(new Integer(-1)))
                value = null;
            
        } else {
            value = comboCriteria.getSelectedItem();
        }
        
        baseDco.markAsUnchanged();
        
        String s = value == null ? "" : value.toString().trim();
        if (s.length() > 0) {
            value = value instanceof String ? ((String) value).trim() : value;
            baseDco.setValue(field.getIndex(), value);
            baseDco.setChanged(field.getIndex(), true);
        }
        
        DataFilter df = DataFilters.getDefaultDataFilter(module.getIndex());
        df.setBaseDco(baseDco);
        return df;
    }

    private void setSearchField(DcField field) {
        JComponent c = ComponentFactory.getComponent(field.getModule(), field.getFieldType(), field.getLabel(), 255);
        comboCriteria.removeAllItems();
        comboCriteria.setRenderer(new DefaultListCellRenderer());
        
        if (field.getValueType() == Repository.datatypes._DCOBJECTCOLLECTION) {
            DcObject[] objects = DataManager.get(
                    ((DcMappingModule) DcModules.get(field.getModRef())).getSimpleModuleIdx(), null);
            
            comboCriteria.addItem(" ");
            for (int i = 0; objects != null && i < objects.length; i++)
                comboCriteria.addItem(objects[i]);
            
        } else if (c instanceof JComboBox) {
            JComboBox combo = (JComboBox) c;
            for (int i = 0; i < combo.getItemCount(); i++)
                comboCriteria.addItem(combo.getItemAt(i));
            
            comboCriteria.setRenderer(combo.getRenderer());
            
        } else {
        	comboCriteria.addItem(" ");
        }
        comboCriteria.addItem(DcResources.getText("lblEmptySearchValue"));

        revalidate();
    }
    
    private void build() {
        DcFieldDefinitions definitions = module.getFieldDefinitions();
        for (Iterator iter = definitions.iterator(); iter.hasNext(); ) {
            DcFieldDefinition definition = (DcFieldDefinition) iter.next();
            
            DcField field = module.getField(definition.getIndex());
            
            if (    field.isEnabled() && field.isSearchable() && 
                   (!field.isUiOnly() || field.getValueType() == Repository.datatypes._DCOBJECTCOLLECTION))
                comboFields.addItem(field);
        }
        
        Collection filters = DataFilters.get(module.getIndex());
        comboFilters.addItem(" ");
        for (Iterator iter = filters.iterator(); iter.hasNext(); ) {
            comboFilters.addItem(iter.next());
        }
        
        comboCriteria.getEditor().getEditorComponent().addKeyListener(searchOnEnterListener);
        comboCriteria.getEditor().getEditorComponent().addMouseListener(searchOnDblClickListener);
        
        comboFilters.addActionListener(filterSelListener);
        comboFields.addActionListener(fieldSelListener);
        
        comboFilters.setPreferredSize(new Dimension(200, 21));
        comboFields.setPreferredSize(new Dimension(200, 21));
        comboCriteria.setPreferredSize(new Dimension(200, 21));
        comboFilters.setMaximumSize(new Dimension(200, 21));
        comboFields.setMaximumSize(new Dimension(200, 21));
        comboCriteria.setMaximumSize(new Dimension(200, 21));
        
        
        add(ComponentFactory.getLabel(DcResources.getText("lblQuickFilter") + " "));
        add(comboFields);
        add(comboCriteria);
        
        JButton button1 = ComponentFactory.getButton(IconLibrary._icoAccept);
        button1.setMaximumSize(new Dimension(25, 23));
        button1.setMinimumSize(new Dimension(25, 23));
        button1.setPreferredSize(new Dimension(25, 23));
        button1.addActionListener(new SearchOnClickListener());

        JButton button2 = ComponentFactory.getButton(IconLibrary._icoAccept);
        button2.setMaximumSize(new Dimension(25, 23));
        button2.setMinimumSize(new Dimension(25, 23));
        button2.setPreferredSize(new Dimension(25, 23));
        button2.addActionListener(new SearchOnClickListener());

        add(button1);
        
        if (filters.size() > 0) {
            addSeparator();
            
            add(ComponentFactory.getLabel(DcResources.getText("lblFilters") + " "));
            add(comboFilters);
            add(button2);
        }
    }    
    
    private class SearchOnClickListener implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            search();
        }
    }    
    
    private class SearchOnEnterListener implements KeyListener {

        public void keyReleased(KeyEvent e) {
            if (e.getKeyCode() == KeyEvent.VK_ENTER) {
                search();
            }
        }

        public void keyPressed(KeyEvent e) {}
        public void keyTyped(KeyEvent e) {}
    }

    private class SearchOnDoubleClickListener implements MouseListener {

        public void mouseClicked(MouseEvent e) {}
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseReleased(MouseEvent e) {
            if (e.getClickCount() == 2) {
                search();
            }            
        }
    }
    
    private class FieldSelectionListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            comboCriteria.removeAllItems();
            
            DcComboBox comboFields = (DcComboBox) e.getSource();
            DcField field = (DcField) comboFields.getSelectedItem();
            setSearchField(field);
            
        }
    }
    
    private class FilterSelectionListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            DataFilter df = comboFilters.getSelectedIndex() > 0 ? (DataFilter) comboFilters.getSelectedItem() : null;
            search(df);
        }
    } 
    
    protected void paintComponent(Graphics g) {
        Graphics2D g2d = (Graphics2D) g;
        
        if (DcSettings.getValueAsBoolean(Repository.settings.stFontAntiAliasing)) {
           g2d.setRenderingHint( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );
           g2d.setRenderingHint( RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON );
           g2d.setRenderingHint( RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY );
        }
        
        super.paintComponent( g2d );
    }
}
