/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.menu;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;

import javax.swing.JMenu;
import javax.swing.JMenuBar;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.components.DcMenu;
import net.sf.dc.console.components.DcMenuItem;
import net.sf.dc.console.components.panels.TreePanel;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.definitions.DcFieldDefinition;
import net.sf.dc.settings.definitions.DcFieldDefinitions;

public class DcTreePanelMenuBar extends JMenuBar {
    
    private final int modIdx;
    private final TreePanel treePanel;

    public DcTreePanelMenuBar(int modIdx, TreePanel treePanel) {
        this.modIdx = modIdx;
        this.treePanel = treePanel;
        
        DcMenu menuEdit = ComponentFactory.getMenu(DcResources.getText("lblEdit"));
        DcMenuItem menuExpandAll = ComponentFactory.getMenuItem(DcResources.getText("lblExpandAll"));
        DcMenuItem menuCollapseAll = ComponentFactory.getMenuItem(DcResources.getText("lblCollapseAll"));
        DcMenuItem menuClear = ComponentFactory.getMenuItem(DcResources.getText("lblClear"));
        menuCollapseAll.addActionListener(new CollapseAllAction());
        
        menuExpandAll.addActionListener(new ExpandAllAction());
        menuClear.addActionListener(new ClearAction());
        
        menuEdit.add(menuCollapseAll);
        menuEdit.add(menuExpandAll);
        menuEdit.addSeparator();
        menuEdit.add(menuClear);
        
        DcModule module = DcModules.get(modIdx);
        DcMenu menuFields = ComponentFactory.getMenu(DcResources.getText("lblGroupBy"));

        DcFieldDefinitions definitions = module.getFieldDefinitions();
        
        for (Iterator iter = definitions.iterator(); iter.hasNext(); ){
            DcFieldDefinition def = (DcFieldDefinition) iter.next();
            DcField fld = module.getField(def.getIndex());
            
            if (def.isEnabled() && fld.isSearchable() && !fld.isTechnicalInfo()) {
                String label = fld.getLabel();
                DcMenu subMenu = ComponentFactory.getMenu(label);

                DcMenuItem mi = ComponentFactory.getMenuItem(label);
                mi.addActionListener(new FieldSelectionListener(new int[] {def.getIndex()}));
                
                menuFields.add(mi);
                menuFields.add(subMenu);
                
                for (Iterator iterSub = definitions.iterator(); iterSub.hasNext(); ){
                    DcFieldDefinition subDef = (DcFieldDefinition) iterSub.next();
                    DcField subFld = module.getField(subDef.getIndex());
                    if (subDef.isEnabled() && subFld.isSearchable() && subFld.getIndex() != fld.getIndex()) {
                        DcMenuItem subItem = ComponentFactory.getMenuItem(subFld.getLabel());
                        subItem.addActionListener(new FieldSelectionListener(new int[] {def.getIndex(), subDef.getIndex()}));
                        subMenu.add(subItem);
                    }
                }
            }
        }
        
        setPreferredSize(new Dimension(100, 22));
        setMaximumSize(new Dimension(100, 22));
        setMinimumSize(new Dimension(50, 22));
        
        add(menuFields);
        add(menuEdit);
    }
    
    public void setFont(Font font) {
        for (int i = 0; i < getMenuCount(); i++) {
            JMenu menu = getMenu(i);
            menu.setFont(ComponentFactory.getSystemFont());
            for (int j = 0; j < menu.getMenuComponents().length; j++) {
                menu.getMenuComponents()[j].setFont(ComponentFactory.getStandardFont());
            }
        }
    }    
    
    private class CollapseAllAction implements ActionListener {
        
        public void actionPerformed(ActionEvent arg0) {
            treePanel.collapseAll();
        }
    }    

    private class ClearAction implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            treePanel.reset();
        }
    } 
    
    private class ExpandAllAction implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            treePanel.expandAll();
        }
    }    
    
    private class FieldSelectionListener implements ActionListener {

        private final int[] fields;
        
        public FieldSelectionListener(int[] fields) {
            this.fields = fields;
        }

        public void actionPerformed(ActionEvent arg0) {
            DcModules.get(modIdx).getCurrentSearchView().applyGrouping(fields);
        }
    }
}

