/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.musicfile;

import java.awt.Font;
import java.util.Iterator;

import net.sf.dc.console.components.panels.QuickViewPanel;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.musicalbum.MusicAlbum;
import net.sf.dc.core.objects.musicalbum.MusicTrack;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;

public class MusicAlbumQuickViewPane extends QuickViewPanel {
    
    public MusicAlbumQuickViewPane() {
        super(DcModules.get(DcModules._MUSICALBUM), false);
    }

    protected String getDescriptionTable() {
        String table = "<table " + getBodyStyle() + ">\n";
        table = addTableRow(table, MusicAlbum._A_TITLE, QuickViewPanel._DIRECTION_VERTICAL);
        table = addTableRow(table, MusicAlbum._F_ARTIST, QuickViewPanel._DIRECTION_VERTICAL);
        table = addTableRow(table, MusicAlbum._G_GENRES, QuickViewPanel._DIRECTION_VERTICAL);
        table = addTableRow(table, MusicAlbum._H_STATE, QuickViewPanel._DIRECTION_HORIZONTAL);
        table = addTableRow(table, MusicAlbum._I_STORAGEMEDIUM, QuickViewPanel._DIRECTION_HORIZONTAL);
        table = addTableRow(table, MusicAlbum._C_YEAR, QuickViewPanel._DIRECTION_HORIZONTAL);
        table = addTableRow(table, MusicAlbum._D_LOCATION, QuickViewPanel._DIRECTION_VERTICAL);
        table = addTableRow(table, MusicAlbum._E_RATING, QuickViewPanel._DIRECTION_HORIZONTAL);
        
        table = addTracks(table);
        table += "</table>";

        return table;
    }
    
    protected String addTracks(String table) {
        
        Font font = DcSettings.getValueAsFont(Repository.settings.stSystemFontNormal);

        table += "<tr><td>\n";
        table += "<b>" + DcResources.getText("lblTracks") + "</b>";
        
        table += "</td></tr>";
        table += "<tr><td>\n";
        
        int counter = 0;
        String description = "";
        
        dco.removeChildren();
        dco.loadChildren();
        for (Iterator iter = dco.getChildren().iterator(); iter.hasNext(); ) {
            MusicTrack mt = (MusicTrack) iter.next();
            description = description.trim();
            
            if (counter > 0) description += ", ";
            
            String title = (String) mt.getValue(MusicTrack._A_TITLE);
            title = title == null ? "" : title; 
            
            Integer i = (Integer) mt.getValue(MusicTrack._J_PLAYLENGTH);
            String length = i == null || i.equals(new Integer(0)) ? "" : " " + 
                                            mt.getDisplayString(MusicTrack._J_PLAYLENGTH);
            
            Integer nr = (Integer) mt.getValue(MusicTrack._F_TRACKNUMBER);
            if (nr != null)
                description += "<b>" + mt.getValue(MusicTrack._F_TRACKNUMBER) + "</b> ";
                
            description += title;
            description += length;
            counter++;
        }
        
        if (font.getStyle() == Font.BOLD) 
            description = "<b>" + description + "</b>";

        table += description;
        table += "</td></tr>";
        table += "<tr><td></td></tr>";
        return table;
    }       
}
