/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.musicfile;

import java.util.Collection;

import net.sf.dc.console.forms.DcSimpleItemView;
import net.sf.dc.console.forms.ItemForm;
import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.musicalbum.MusicAlbum;
import net.sf.dc.wf.requests.RefreshChildView;
import net.sf.dc.wf.requests.RequestCollection;

public class MusicTrackForm extends DcSimpleItemView {

    private final String parentID;
    
    public MusicTrackForm(ItemForm form, MusicAlbum musicAlbum, int module, boolean readonly) {
        super(form, module, readonly);
        this.parentID = musicAlbum.getID();
        list.setEnabled(!readonly);
    }
   
    public void addObjects() {
        list.clear();
        list.add(DataManager.getChildren(parentID, getModuleIdx()));
    }
    
    public void setObjects(Collection objects) {
        list.clear();
        list.add(objects);
    }
    
    public void open() {
        DcObject dco = list.getSelectedObject();
        dco.initializeReferences();
        if (dco != null) {
            dco.markAsUnchanged();
            MusicTrackItemForm itemForm = new MusicTrackItemForm(true, dco, this);
            itemForm.setVisible(true);            
        }
    }
    
    public void createNew() {
        DcObject dco = getModule().getDcObject();
        dco.setValue(dco.getParentReferenceFieldIndex(), parentID);
        
        MusicTrackItemForm itemForm = new MusicTrackItemForm(false, dco, this);
        itemForm.setVisible(true);
    }

    public RequestCollection getAfterDeleteRequests() {
        RequestCollection requests = new RequestCollection();
        requests.addRequest(new RefreshChildView(this, DcModules._MUSICALBUM, null, null, parentID, false));
        return requests;
    }   
}
