/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.note;

import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.RandomAccessFile;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ScrollPaneConstants;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcPanel;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.Repository;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;

public class NotePanel extends DcPanel {

    private JButton  buttonSave;
    private JButton buttonClear;
    private JTextArea textNote;

    private JPanel panelActions = new JPanel();
    private JScrollPane scroller;
    private JPanel panelStatus = getStatusPanel();

    private File file = new File(DataCrow.baseDir + "data/data_crow_comments.txt");
    private RandomAccessFile access;

    public NotePanel() {
        super(null, null);
        buildPanel();
        loadNote();
    }

    private void loadNote() {
        setHelpIndex("dc.BlockNote");
        try {
            access = new RandomAccessFile(file, "rw");
            long filePointer = 0;
            long fileLength = file.length();

            StringBuffer sb = new StringBuffer("");
            while (filePointer < fileLength) {
                sb.append(access.readLine().trim() + '\n');
                filePointer = access.getFilePointer();
            }

            textNote.setText(sb.toString());
        } catch (Exception exp) {}
    }

    public void clear() {
        textNote.setText("");
    }

    public void setFont(Font font) {
        super.setFont(font);

        Font normal = DcSettings.getValueAsFont(Repository.settings.stSystemFontNormal);
        Font system = DcSettings.getValueAsFont(Repository.settings.stSystemFontNormal);

        if (textNote != null) {
            textNote.setFont(normal);
            panelStatus.setFont(system);
            buttonSave.setFont(system);
            buttonClear.setFont(system);
        }
    }

    public void saveNote() {
        boolean success = false;
        try {
            access = new RandomAccessFile(file, "rw");
            textNote.getText();
            access.setLength(0l);
            access.writeBytes(textNote.getText());
            access.close();
            success = true;
        } catch (Exception exp) {
            Messages.add(exp);
        } finally {
            if (success) {
                setStatus(DcResources.getText("msgSaveHasBeenExecuted"));
                new MessageBox(DcResources.getText("msgDataSaved"), MessageBox._INFORMATION);
            }
        }
    }

    private void buildPanel() {
        //**********************************************************
        //Note panel
        //**********************************************************
        textNote = ComponentFactory.getTextArea();

        scroller = new JScrollPane(textNote);
        scroller.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        scroller.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);

        //**********************************************************
        //Action panel
        //**********************************************************
        buttonSave  = ComponentFactory.getButton(DcResources.getText("lblSave"));
        buttonClear = ComponentFactory.getButton(DcResources.getText("lblClear"));

        buttonClear.setMnemonic(KeyEvent.VK_L);
        buttonSave.setMnemonic(KeyEvent.VK_S);

        buttonSave.addActionListener(new SaveButtonAction());
        buttonClear.addActionListener(new ClearButtonAction());

        panelActions.add(   buttonSave, Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                            new Insets(0,  5, 0, 0), 0, 0));
        panelActions.add(   buttonClear,Layout.getGBC( 1, 0, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                            new Insets(0,  5, 0, 0), 0, 0));

        //**********************************************************
        //Main panel
        //**********************************************************
        setLayout(Layout.getGBL());

        add( scroller,     Layout.getGBC( 0, 0, 1, 1, 100.0, 100.0
                          ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                           new Insets(10, 10, 5, 10), 0, 0));
        add( panelActions, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                          ,GridBagConstraints.NORTHEAST, GridBagConstraints.NONE,
                           new Insets(0, 5, 0, 5), 0, 0));
        add( panelStatus,  Layout.getGBC( 0, 2, 1, 1, 1.0, 1.0
                          ,GridBagConstraints.SOUTH, GridBagConstraints.HORIZONTAL,
                           new Insets(0, 5, 0, 5), 0, 0));
    }

    private class SaveButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            saveNote();
        }
    }

    private class ClearButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            clear();
        }
    }
}
