/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.reporting;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JCheckBox;
import javax.swing.JPanel;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcNumberField;
import net.sf.dc.reporting.templates.ReportTemplateProperties;
import net.sf.dc.resources.DcResources;

public class ReportSettingsPanel extends JPanel {
    
    private JCheckBox cbResizeImages = ComponentFactory.getCheckBox(DcResources.getText("lblScaleImages"));
    private JCheckBox cbCopyImages = ComponentFactory.getCheckBox(DcResources.getText("lblCopyImage"));
    private DcNumberField nfWidth = ComponentFactory.getNumberField();
    private DcNumberField nfHeight = ComponentFactory.getNumberField();
    
    private DcNumberField nfMaxTextLength = ComponentFactory.getNumberField();
    
    public ReportSettingsPanel() {
        super();
        build();
    }
    
    private void applySelection() {
        cbResizeImages.setEnabled(cbCopyImages.isSelected());
        cbResizeImages.setSelected(!cbCopyImages.isSelected() ? false : cbResizeImages.isSelected());
        
        nfWidth.setEnabled(cbResizeImages.isSelected());
        nfHeight.setEnabled(cbResizeImages.isSelected());
    }
    
    public void saveSettings(ReportTemplateProperties properties) {
        properties.set(ReportTemplateProperties._COPY_IMAGES, new Boolean(cbCopyImages.isSelected()));
        properties.set(ReportTemplateProperties._SCALE_IMAGES, new Boolean(cbResizeImages.isSelected()));
        properties.set(ReportTemplateProperties._MAX_TEXT_LENGTH, nfMaxTextLength.getValue());
        properties.set(ReportTemplateProperties._IMAGE_WIDTH, nfWidth.getValue());
        properties.set(ReportTemplateProperties._IMAGE_HEIGHT,nfHeight.getValue());
        properties.save();
    }
    
    public void applySettings(ReportTemplateProperties properties) {
        cbCopyImages.setSelected(properties.getBoolean(ReportTemplateProperties._COPY_IMAGES));
        cbResizeImages.setSelected(properties.getBoolean(ReportTemplateProperties._SCALE_IMAGES));
        nfMaxTextLength.setValue(properties.getInt(ReportTemplateProperties._MAX_TEXT_LENGTH));
        
        nfWidth.setValue(properties.getInt(ReportTemplateProperties._IMAGE_WIDTH));
        nfHeight.setValue(properties.getInt(ReportTemplateProperties._IMAGE_HEIGHT));
        
        applySelection();
    }
    
    public void setEnabled(boolean b) {
        cbResizeImages.setEnabled(b);
        cbCopyImages.setEnabled(b);
        nfWidth.setEnabled(b);
        nfHeight.setEnabled(b);
        nfMaxTextLength.setEnabled(b);
    }
    
    private void build() {
        setLayout(Layout.getGBL());
        
        ResizeListener rl = new ResizeListener();
        cbResizeImages.addActionListener(rl);
        cbCopyImages.addActionListener(rl);

        Dimension size = new Dimension(100, 21);
        nfHeight.setMinimumSize(size);
        nfHeight.setPreferredSize(size);
        nfWidth.setMinimumSize(size);
        nfWidth.setPreferredSize(size);
        
        JPanel panelImages = new JPanel();
        panelImages.setLayout(Layout.getGBL());

        panelImages.add(cbCopyImages,  Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets( 5, 5, 5, 5), 0, 0));
        panelImages.add(cbResizeImages,  
                        Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets( 5, 5, 5, 5), 0, 0));
        panelImages.add(nfWidth,        Layout.getGBC( 0, 2, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets( 5, 5, 5, 5), 0, 0));
        panelImages.add(ComponentFactory.getLabel("Width"), 
                        Layout.getGBC( 1, 2, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets( 5, 5, 5, 5), 0, 0));
        panelImages.add(nfHeight,       Layout.getGBC( 0, 3, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets( 5, 5, 5, 5), 0, 0));
        panelImages.add(ComponentFactory.getLabel("Height"), 
                        Layout.getGBC( 1, 3, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets( 5, 5, 5, 5), 0, 0));
        panelImages.setBorder(ComponentFactory.getTitleBorder(DcResources.getText("lblImages")));


        nfMaxTextLength.setMinimumSize(new Dimension(40, 21));
        nfMaxTextLength.setPreferredSize(new Dimension(40, 21));
        
        JPanel panelText = new JPanel();
        panelText.setLayout(Layout.getGBL());
        panelText.add(ComponentFactory.getLabel(DcResources.getText("lblMaxTextLength")), 
                        Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets( 5, 5, 5, 5), 0, 0));
        panelText.add(nfMaxTextLength, 
                        Layout.getGBC( 1, 0, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets( 5, 5, 5, 5), 0, 0));
        
        panelText.setBorder(ComponentFactory.getTitleBorder(DcResources.getText("lblText")));
        
        add(panelImages, Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                        ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                         new Insets( 5, 5, 5, 5), 0, 0));
        add(panelText,   Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                        ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                         new Insets( 5, 5, 5, 5), 0, 0));
    }
    
    private class ResizeListener implements ActionListener {

        public void actionPerformed(ActionEvent arg0) {
            applySelection();
        }
    }
}
