/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.views.list;

import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.MainFrame;
import net.sf.dc.console.book.AmazonBookSearchForm;
import net.sf.dc.console.components.lists.DcObjectList;
import net.sf.dc.console.components.lists.DcObjectListModel;
import net.sf.dc.console.components.lists.elements.DcObjectListElement;
import net.sf.dc.console.components.panels.QuickViewPanel;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.console.dialogs.SaveQuestionBox;
import net.sf.dc.console.forms.OnlineSearchForm;
import net.sf.dc.console.menu.DcPopupMenu;
import net.sf.dc.console.movie.OnlineMovieSearchForm;
import net.sf.dc.console.reporting.ReportingDialog;
import net.sf.dc.console.software.OnlineSoftwareSearchForm;
import net.sf.dc.console.views.MasterView;
import net.sf.dc.console.views.View;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.Repository;
import net.sf.dc.core.actions.SaveAllAction;
import net.sf.dc.core.modules.DcMediaModule;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.modules.contactperson.ContactPersonModule;
import net.sf.dc.core.modules.generic.GenericModule;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.db.DatabaseManager;
import net.sf.dc.exceptions.ValidationException;
import net.sf.dc.messages.Messages;
import net.sf.dc.processes.DataTask;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.wf.requests.RemoveFromCacheRequest;
import net.sf.dc.wf.requests.StatusUpdateRequest;
import net.sf.dc.wf.requests.UpdateStatusProgressBarRequest;

public abstract class ListBaseView extends View implements ComponentListener {

    protected boolean updateQuickView = true;
    protected QuickViewPanel quickView;
    
    protected final DcObjectList list;
    protected JScrollPane scrollResults;

    protected JPanel panelResult = new JPanel();
    protected JPanel panelStatus = getStatusPanel();

    protected final ListMouseListener mouseListener = new ListMouseListener();

    protected JPanel panelActions = new JPanel();
    protected JPanel panelActionsLeft = new JPanel();
    protected JPanel panelActionsRight = new JPanel();

    private JButton buttonSave;
    private JButton buttonClear;
    private JButton buttonCancel;
    private JButton buttonRemove;

    public ListBaseView(DcModule module, String title, ImageIcon icon) {
        super(module, title, icon, MasterView._LIST_VIEW);
        addComponentListener(this);
        list = new DcObjectList(module, false, true, false);
    }

    private void setListRowCount(int objectCount) {
        if (getModule() instanceof DcMediaModule || getModule() instanceof ContactPersonModule || getModule() instanceof GenericModule) {
            try {
                DcObjectListElement elem = (DcObjectListElement) list.getDcModel().getElementAt(0);
                Dimension elemSize = elem.getPreferredSize();
                
                list.setFixedCellHeight(elemSize.height);
                list.setFixedCellWidth(elemSize.width);
                
                int width = scrollResults.getViewport().getWidth();
                list.setColumnsPerRow(new Double(Math.floor(width / elemSize.width)).intValue());
            } catch (Exception exp) {}
        }
    }    
    
    public void add(DcObject dco, boolean select) {
        int rowCount = list.getModel().getSize() + 1;
    	setListRowCount(rowCount);
    	list.add(dco);
        
        if (treePanel != null)
            treePanel.add(dco);
        
        if (select)
            setSelectedIndex();
    }
    
    public void add(DcObject dco) {
        add(dco, true);
    }

    public void add(Collection c) {
        add(c, true);
    }

    public void add(Collection c, boolean updateTree) {
        DcObject[] objects = new DcObject[c.size()];
        objects = (DcObject[]) c.toArray(objects);
        add(objects, updateTree);
    }
    
    public void add(DcObject[] objects) {
        add(objects, true);
    }      
    
    public void add(DcObject[] objects, boolean updateTree) {
        if (task != null && task instanceof FillerThread && task.isAlive())
            task.stopRunning();

        if (updateTree && treePanel != null) {
            treePanel.add(objects);
        } else {
            task = new FillerThread(objects, this instanceof ListSearchView);
            task.setPriority(Thread.MAX_PRIORITY);
            task.start();
        }
    }    
    
    public DcObject getObject(int row) {
    	DcObjectListElement element = (DcObjectListElement) list.getDcModel().getElementAt(row);
    	return element.getDcObject();
    }
    
    protected void setSelectedIndex() {
        try {
            list.setSelectedIndex(getRows() - 1);
        } catch (Exception exp) {}        
    }    
    
    public void clear(boolean safe) {
        list.clear();
        setStatus(DcResources.getText("msgTableCleared"));
        updateProgressBar(0);      
        
        if (quickView != null)
            quickView.clear();
    }

    public void clear() {
        boolean saved = isChangesSaved();
        if (!saved && getCurrentTask() != null) {
            try {
                getCurrentTask().join();
            } catch (Exception exp) {
                Messages.add(exp);
            }
        }

        if (quickView != null)
            quickView.clear();
        
        list.clear();
        setStatus(DcResources.getText("msgTableCleared"));
        updateProgressBar(0);
    }

    public void applySettings() {
        setFont(DcSettings.getValueAsFont(Repository.settings.stSystemFontNormal));
        
        if (quickView != null) {
            quickView.setFont(DcSettings.getValueAsFont(Repository.settings.stSystemFontBold));
            quickView.setVisible(DcSettings.getValueAsBoolean(Repository.settings.stShowQuickView));
        }
        
        if (treePanel != null) {
            boolean treeVisibible = treePanel.isVisible();
            boolean treeVisibibleSett = DcSettings.getValueAsBoolean(Repository.settings.stShowGroupingPanel);
            
            treePanel.setVisible(treeVisibibleSett);
            if (!treeVisibible && treeVisibibleSett)
                treePanel.groupBy(DcSettings.getValueAsIntegerArray(getModule().getGroupedBySettingsKey()));
            else if (treeVisibible && !treeVisibibleSett)
                treePanel.reset();
            
            treePanel.setFont(DcSettings.getValueAsFont(Repository.settings.stSystemFontBold));
        }
        
        panelActionsLeft.setFont(DcSettings.getValueAsFont(Repository.settings.stSystemFontNormal));
        panelActionsRight.setFont(DcSettings.getValueAsFont(Repository.settings.stSystemFontNormal));
        buttonSave.setFont(DcSettings.getValueAsFont(Repository.settings.stSystemFontNormal));
        buttonClear.setFont(DcSettings.getValueAsFont(Repository.settings.stSystemFontNormal));
        buttonCancel.setFont(DcSettings.getValueAsFont(Repository.settings.stSystemFontNormal));
        buttonRemove.setFont(DcSettings.getValueAsFont(Repository.settings.stSystemFontNormal));
        panelStatus.setFont(DcSettings.getValueAsFont(Repository.settings.stSystemFontBold));
    }

    public int getRows() {
        return list.getDcModel().getSize();
    }

    public JComponent[] getExtraComponents() {
        return null;
    }

    protected abstract DcPopupMenu getPopupMenu();

    public void denyActions() {
        removeListeners();
        buttonClear.setEnabled(false);
        buttonRemove.setEnabled(false);
        buttonSave.setEnabled(false);
    }
    
    private void removeListeners() {
        MouseListener[] listeners = list.getMouseListeners();
        for (int i = 0; i < listeners.length; i++) { 
            if (listeners[i] instanceof ListMouseListener)
                list.removeMouseListener(listeners[i]);
            
        }
    }

    public void allowActions() {
        removeListeners();
        list.addMouseListener(mouseListener);
        buttonClear.setEnabled(true);
        buttonRemove.setEnabled(true);
        buttonSave.setEnabled(true);
    }

    public DcObject getObjectForIndex(int index) {
        DcObjectListElement element =  (DcObjectListElement) list.getDcModel().getElementAt(index);
        return element.getDcObject();
    }

    public DcObject getObjectForID(String ID) {
        return list.getObjectForID(ID);
    }
    
    public Collection getSelectedObjects() {
        return list.getSelectedObjects();
    }

    public DcObject getSelectedObject() {
        DcObject dco = list.getSelectedObject();
        if (dco == null)
            new MessageBox(DcResources.getText("msgSelectRowToOpen"), MessageBox._WARNING);
        return dco;
    }

    public void startAmazonUpdate() {
        OnlineSearchForm searchForm;
        
        if (getModule().getIndex() == DcModules._SOFTWARE)
            searchForm = new OnlineSoftwareSearchForm(null, DataCrow.mainFrame.getSelectedTab(), true);
        else
            searchForm = new AmazonBookSearchForm(null, DataCrow.mainFrame.getSelectedTab(), true);

        searchForm.setVisible(true);
    }

    public void startIMDBUpdate() {
        OnlineMovieSearchForm searchForm = new OnlineMovieSearchForm(null, DataCrow.mainFrame.getSelectedTab(), true);
        searchForm.setVisible(true);
    }

    public void applyDefinitions() {
        list.update();
    }

    public void createReport() {
        Collection objects = new ArrayList();
        if (treePanel != null && treePanel.isActive())
            objects.addAll(treePanel.getObjects());
        else
            objects.addAll(list.getObjects());
        
        if (objects.size() > 0) {
            ReportingDialog dialog = new ReportingDialog(objects);
            dialog.setVisible(true);
        } else {
            new MessageBox(DcResources.getText("msgReportNoItems"), MessageBox._WARNING);
        }
    }


    public void removeRows(String[] ids) {
        list.remove(ids);
        
        if (treePanel != null)
            treePanel.remove(ids);
        
        setSelectedIndex();
        
        if (getRows() == 0 && quickView != null)
            quickView.clear();
    }

    public void removeRows(int[] rows) {
        list.remove(rows);
        
        if (getRows() == 0 && quickView != null)
            quickView.clear();
    }

    public void showQuickView(boolean b) {}

    public int[] getSelectedRows() {
        return list.getSelectedIndices();
    }

    public void updateRow(int index, DcObject dco, boolean overwrite, boolean allowDeletes, boolean mark) {

        if (treePanel != null)
            treePanel.update(dco);

        list.updateElement(index, dco, overwrite, allowDeletes, mark);
        
        if (quickView != null && index > -1)
            quickView.setObject(getObjectForIndex(index));
        
    }

    public void updateRow(String ID, DcObject dco, boolean overwrite, boolean allowDeletes, boolean mark) {
        list.updateElement(ID, dco, overwrite, allowDeletes, mark);

        if (treePanel != null)
            treePanel.update(dco);
        
        if (quickView != null && getObjectForID(ID) != null)
            quickView.setObject(getObjectForID(ID));
    }

    public void removeFromCache(String id) {
        DcObject dco = list.getObjectForID(id);
        if (dco != null) 
            dco.markAsUnchanged();
    }

    protected JPanel getActionPanel(boolean insert, JComponent[] extraComponents) {
        panelActionsLeft.setLayout(new FlowLayout(FlowLayout.LEFT));
        panelActionsRight.setLayout(new FlowLayout(FlowLayout.LEFT));

        // Create the components
        buttonClear = ComponentFactory.getButton(DcResources.getText("lblClear"));
        buttonRemove = ComponentFactory.getButton(DcResources.getText("lblRemove"));
        buttonCancel = ComponentFactory.getButton(DcResources.getText("lblCancel"));

        buttonRemove.setToolTipText(DcResources.getText("tpRemoveRow"));
        buttonClear.addActionListener(new ClearButtonAction());
        buttonRemove.addActionListener(new RemoveRowAction());

        buttonSave = ComponentFactory.getButton(DcResources.getText("lblSave"));
        buttonClear = ComponentFactory.getButton(DcResources.getText("lblClear"));
        buttonRemove = ComponentFactory.getButton(DcResources.getText("lblRemove"));

        buttonCancel.setMnemonic(KeyEvent.VK_X);
        buttonClear.setMnemonic(KeyEvent.VK_L);
        buttonCancel.setMnemonic(KeyEvent.VK_C);
        buttonSave.setMnemonic(KeyEvent.VK_S);
        buttonRemove.setMnemonic(KeyEvent.VK_R);

        buttonSave.setToolTipText(DcResources.getText("tpSaveChanges"));
        buttonClear.setToolTipText(DcResources.getText("tpClear"));
        buttonRemove.setToolTipText(DcResources.getText("tpRemoveRow"));
        buttonCancel.setToolTipText(DcResources.getText("tpCancel"));

        buttonCancel.addActionListener(new CancelAction());
        buttonSave.addActionListener(new SaveAllAction());
        buttonClear.addActionListener(new ClearButtonAction());
        buttonRemove.addActionListener(new RemoveRowAction());

        // Build the panel
        panelActionsRight.add(buttonSave);
        if (insert)
            panelActionsLeft.add(buttonRemove);

        int counter = 2;
        if (extraComponents != null) {
            for (int i = 0; i < extraComponents.length; i++) {
                add(extraComponents[i]);
                counter++;
            }
        }

        if (insert)
            panelActionsRight.add(buttonClear);

        panelActionsRight.add(buttonCancel);
        
        panelActions = new JPanel();
        panelActions.setLayout(Layout.getGBL());
        panelActions.add(panelActionsLeft, Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.WEST, GridBagConstraints.NONE,
                        new Insets(0, 5, 0, 5), 0, 0));
        panelActions.add(panelActionsRight, Layout.getGBC( 1, 0, 1, 1, 1.0, 1.0
                ,GridBagConstraints.EAST, GridBagConstraints.NONE,
                 new Insets(0, 5, 0, 5), 0, 0));
        

        return panelActions;
    }

    public DcObject getDcObject(String ID) {
        return list.getObjectForID(ID);
    }
    
    protected class SavingThread extends DataTask {

        private final boolean update;

        public SavingThread(Collection objects, boolean update) {
            super(objects);
            this.update = update;
        }

        public void run() {
            try {
                updateProgressBar(0);
                initProgressBar(objects.size());
                setStatus(DcResources.getText("msgSavingXItems", "" + objects.size()));
                list.setCursor(new Cursor(Cursor.WAIT_CURSOR));

                startRunning();
                denyActions();

                boolean ignoreErrors = false;
                int counter = 1;
                
                for (Iterator iter = objects.iterator(); iter.hasNext(); counter++) {
                    try {
                        DcObject dco = (DcObject) iter.next();

                        dco.setEndOfBatch(counter == objects.size());
                        dco.setSilence(true);

                        if (dco.isEndOfBatch()) {
                            dco.setSilence(false);
                            dco.addRequest(new StatusUpdateRequest(dco.getModule().getIndex(), MainFrame._INSERTTAB, "msgDataSaved"));
                        }

                        if (update) {
                            updateProgressBar(counter);
                            dco.addRequest(new RemoveFromCacheRequest(getModule().getIndex(), dco.getID()));
                            dco.saveUpdate(true);
                        } else {
                            dco.addRequest(new UpdateStatusProgressBarRequest(dco.getModule().getIndex(), MainFrame._INSERTTAB, counter));
                            dco.saveNew(true);
                        }
                    } catch (ValidationException vExp) {
                        if (!ignoreErrors) {
                            SaveQuestionBox question = new SaveQuestionBox(vExp);
                            if (question.getResult() == SaveQuestionBox._CANCEL)
                                stopRunning();
                            else if (question.getResult() == SaveQuestionBox._IGNORE)
                                ignoreErrors = true;
                        }
                    }

                    try {
                        while (DatabaseManager.getQueueSize() > 0)
                            sleep(300);
                    } catch (Exception ignore) {}
                    }
            } finally {
                list.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
                allowActions();
                stopRunning();
            }
        }
    }

    
    
    public class FillerThread extends DataTask {
        
        private final DcObject[] objects;
        private final boolean clearViewFirst;
        
        public FillerThread(DcObject[] objects, boolean clearViewFirst) {
            super();
            setName("Filler-Thread");
            this.objects = objects;
            this.clearViewFirst = clearViewFirst;
        }
        
        public void run() {
            try {
                try {
                    sleep(500);
                } catch (Exception exp) {}
                
                denyActions();
                list.listenersEnabled(false);
                DcObjectListModel model = list.getDcModel();
                model.setListenersEnabled(false);
                
                if (clearViewFirst) 
                    clear();
    
                list.clearSelection();
                
                updateQuickView = false;
                
                updateProgressBar(0);
                initProgressBar(objects.length);
                int i = 0;
                
                for (; i < objects.length && keepOnRunning(); i++) {
                    DcObject dco = objects[i];
                    
                    if (keepOnRunning())
                        model.addElement(list.getDisplayElement(dco));
                    
                    setStatus(DcResources.getText("msgAddingXToView", dco.toString()));
                    list.clearSelection();
                    updateProgressBar(i + 1);
                    list.revalidate();
                    list.repaint();
                    
                    try {
                        sleep(2);
                    } catch (Exception e) {}
                }
                
                if (clearViewFirst)
                    setStatus(DcResources.getText("msgSearchResult", "" + i));
                else 
                    setStatus(DcResources.getText("msgAddedXItemsToView", "" + i));
                
                try {
                    list.listenersEnabled(true);
                    list.getDcModel().fireIntervalAdded(model, 0, objects.length);
                } catch (Exception ignore) {}
                
            } finally {
                updateQuickView = true;
                list.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
                allowActions();
                list.listenersEnabled(true);
                stopRunning();
            }  
            
            revalidate();
            setListRowCount(objects.length);
            setSelectedIndex();
        }
    }    
    
    public class CancelAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            allowActions();
            if (task != null)
                task.cancel();
            updateProgressBar(0);
        }
    }

    public class ClearButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            clear();
        }
    }

    public class RemoveRowAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            delete();
        }
    }

    private class ListMouseListener implements MouseListener {
        public void mouseReleased(MouseEvent e) {
            if (SwingUtilities.isRightMouseButton(e)) {
                if (list.getSelectedIndex() == -1) {
                    int index = list.locationToIndex(e.getPoint());
                    list.setSelectedIndex(index);
                }

                if (list.getSelectedIndex() > -1) {
                    DcPopupMenu menu = getPopupMenu();
                    menu.setInvoker(list);
                    menu.show(list, e.getX(), e.getY());
                }
            }

            if (e.getClickCount() == 2 && list.getSelectedIndex() > -1) {
                open();
                e.consume();
            }
        }
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {}
    }

    public void componentHidden(ComponentEvent e) {
    }

    public void componentMoved(ComponentEvent e) {
    }

    public void componentResized(ComponentEvent e) {
        setListRowCount(getRows());
    }

    public void componentShown(ComponentEvent e) {
    }
    
    
}
