/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.views.list;

import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.util.ArrayList;
import java.util.Collection;

import javax.swing.ImageIcon;
import javax.swing.JScrollPane;
import javax.swing.JViewport;
import javax.swing.ToolTipManager;

import net.sf.dc.console.Layout;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.console.dialogs.QuestionBox;
import net.sf.dc.console.menu.DcPopupMenu;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.resources.DcResources;

public class ListInsertView extends ListBaseView {

    public ListInsertView(DcModule module, String title, ImageIcon icon) {
        super(module, title, icon);  
        setHelpIndex("dc.Inserting");
        buildPanel();
    }    
    
    public void open() {
        DcObject dco = getSelectedObject();
        dco.markAsUnchanged();
        open(dco, false, false);
    }
    
    public void reload(String ID) {
        
    }
    
    public boolean isChangesSaved() {
        boolean isChangesSaved = true;
        boolean saved = list.getDcModel().getSize() == 0;
        if (!saved) {
            new QuestionBox(DcResources.getText("msgNotSaved"));
            if (QuestionBox.result.equals(Boolean.TRUE)) {
                save();
                isChangesSaved = false;
            } else {
                list.markAsUnchanged();
            }
        }
        return isChangesSaved;
    }    
    
    public void applyGrouping(int[] fields) {}
    public void undoGrouping() {}
    
    public void save() {
        int rowCount = list.getDcModel().getSize();
        int[] rows = new int[rowCount];
        for (int i = 0; i < rowCount; i++) {
            rows[i] = i;
        }
        save(rows);
    }
    
    public void saveSelected() {
        save(list.getSelectedIndices());
    }   
    
    public void save(int[] rows) {
        if (rows.length > 0) {
            if (!isTaskRunning()) {
                Collection objects = new ArrayList();
                for (int i = 0; i < rows.length; i++)
                    objects.add(getObject(rows[i]));
                
                task = new SavingThread(objects, false);
                task.start();
            }
        } else {
            new MessageBox(DcResources.getText("msgNoItemsToSave"), MessageBox._WARNING);    
        }
    }    
    
    public void delete() {
        list.remove(list.getSelectedIndices());
    }    
    
    protected DcPopupMenu getPopupMenu() {
        return new DcPopupMenu(getSelectedObject(), DcPopupMenu._INSERT);
    }
    
    private void buildPanel() {

        //**********************************************************
        //Search result panel
        //**********************************************************
        panelResult.setLayout(Layout.getGBL());

        scrollResults = new JScrollPane(list);
        scrollResults.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        scrollResults.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        scrollResults.getViewport().setScrollMode(JViewport.SIMPLE_SCROLL_MODE);
        
        panelResult.add(scrollResults,  Layout.getGBC( 0, 0, 1, 1, 80.0, 80.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                        new Insets(5, 5, 5, 5), 0, 0));

        list.addMouseListener(mouseListener);
        
        //**********************************************************
        //Main panel
        //**********************************************************
        setLayout(Layout.getGBL());
        
        panelActions = getActionPanel(true, null);
        add(    panelResult,      Layout.getGBC( 0, 1, 3, 1, 100.0, 100.0
               ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                new Insets(5, 5, 5, 5), 0, 0)); 
        add(    panelActions,     Layout.getGBC( 0, 2, 3, 1, 1.0, 1.0
                ,GridBagConstraints.SOUTH, GridBagConstraints.HORIZONTAL,
                new Insets(0, 5, 0, 5), 0, 0));
        add(    panelStatus,      Layout.getGBC( 0, 3, 3, 1, 1.0, 1.0
                ,GridBagConstraints.SOUTH, GridBagConstraints.HORIZONTAL,
                new Insets(0, 5, 0, 5), 0, 0));

        ToolTipManager.sharedInstance().registerComponent(list);
    } 
}
