/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.views.table;

import java.awt.Cursor;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.JComponent;
import javax.swing.JScrollPane;
import javax.swing.JViewport;
import javax.swing.ToolTipManager;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.MainFrame;
import net.sf.dc.console.components.DcViewDivider;
import net.sf.dc.console.components.panels.TreePanel;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.console.dialogs.QuestionBox;
import net.sf.dc.console.dialogs.SaveQuestionBox;
import net.sf.dc.console.menu.DcPopupMenu;
import net.sf.dc.console.reporting.ReportingDialog;
import net.sf.dc.console.views.MasterView;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.exceptions.ValidationException;
import net.sf.dc.processes.DataTask;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.wf.requests.RemoveFromCacheRequest;
import net.sf.dc.wf.requests.StatusUpdateRequest;
import net.sf.dc.wf.requests.UpdateStatusProgressBarRequest;

public class TableSearchView extends TableBaseView {

    protected RowSelectionListener rowSelectionListener = new RowSelectionListener();
    protected ChangeRowByKey changeRowByKey = new ChangeRowByKey();

    private String parentID = "";

    public TableSearchView(DcModule module, String title) {
        super(module, title, module.getIconSmall());
        setHelpIndex("dc.Searching.SearchResult");

        JComponent[] extraComponents = getAllExtraComponents();

        panelTableActions = new TableActionsPanel(false, extraComponents);
        panelTableActions.addSaveActionListener(new SaveUpdatedValuesAction());

        buildPanel();
    }

    public void showQuickView(boolean b) {
        quickView.setVisible(b);
    }

    public void setVisible(boolean b) {
        super.setVisible(b);
        table.setVisible(b);
    }
    
    public void reload(String ID) {
        super.reload(ID);
        if (quickView != null)
            quickView.setImages();
    }    
    
    /**
     * Retrieves the module specific components
     */
    private JComponent[] getAllExtraComponents() {
        JComponent[] components = new JComponent[0];

        JComponent[] childComponents = new JComponent[0];
        if (hasChild())
        	childComponents = getChildModule().getSearchView().get(MasterView._TABLE_VIEW).getExtraComponents();

        JComponent[] parentComponents = getExtraComponents();
        components = new JComponent[parentComponents.length + childComponents.length];
        int counter = 0;

        if (childComponents != null) {
            for (int i = 0; i < childComponents.length; i++) {
                components[counter] = childComponents[i];
                counter++;
            }
        }

        if (parentComponents != null) {
            for (int i = 0; i < parentComponents.length; i++) {
                components[counter] = parentComponents[i];
                counter++;
            }
        }
        return components;
    }

    public JComponent[] getExtraComponents() {
        return new JComponent[0];
    }

    public void setParentID(String sID) {
        parentID = sID;
    }

    public String getParentID() {
    	return parentID;
    }

    public boolean isChangesSaved() {
        cancelEdit();

        boolean saved = changesSaved();
        saved &= hasChild() ? ((TableSearchView) childView).changesSaved() : saved;
        
        return saved;
    }

    public void removeFromCache(String sID) {
    	table.removeFromCache(sID);
        removeNewChild(sID);
    }

    public Collection getSelectedObjects() {
        int[] rows = table.getSelectedRows();

        Collection objects = new ArrayList();
        if (rows.length > 0) {
            for (int i = 0; i < rows.length; i++) {
                int row = rows[i];
                objects.add(getObjectForRow(row, false));
            }
        } else {
            new MessageBox(DcResources.getText("msgNoRowsSelected"), MessageBox._WARNING);
        }

        return objects;
    }

    public DcObject getSelectedObject() {
    	int row = table.getSelectedRow();
    	DcObject dco = null;
    	if (row != -1) {
    		dco = getObjectForRow(row, true);
    	} else {
    		new MessageBox(DcResources.getText("msgNoRowsSelected"), MessageBox._WARNING);
    	}
    	return dco;
    }

    public DcModule getModuleForRow(int row) {
        return table.getModuleForRow(row);
    }
    
    public void open() {
        cancelEdit();
        int row = table.getSelectedRow();
        if (row == -1) {
            new MessageBox(DcResources.getText("msgSelectRowToOpen"), MessageBox._WARNING);
        } else {
            DcObject dco = getObjectForRow(row, true);
            if (DcModules.getCurrentModule().getIndex() == DcModules._GENERIC) {
                dco.reload();
                dco.loadChildren();
            }
            
            open(dco, true, false);
        }
    }

    public void updateRow(String id, DcObject dco, boolean overwrite, boolean allowDeletes, boolean mark) {
        super.updateRow(id, dco, overwrite, allowDeletes, mark);
        int row = table.getRowNumberWithID(id);
        
        if (quickView != null && row > -1 && dispatchEvents)
            quickView.setObject(getObjectForRow(row, true));

        if (treePanel != null) {
            DcObject o = getModule().getDcObject();
            o.setValue(DcObject._ID, id);
            treePanel.update(o);
        }
    }

    public void createReport() {
        Collection objects = new ArrayList();
        
        if (treePanel != null && treePanel.isActive()) {
            objects.addAll(treePanel.getObjects());
            
        } else {
            for (int i = 0; i < table.getRowCount(); i++)
                objects.add(getObjectForRow(i, false));
        }
    
        if (objects.size() > 0) {
            ReportingDialog dialog = new ReportingDialog(objects);
            dialog.setVisible(true);
        } else {
            new MessageBox(DcResources.getText("msgReportNoItems"), MessageBox._WARNING);
        }
    }

    public void removeNewChild (String sID) {}

    public void delete() {
        if (!isTaskRunning()) {
            Collection dcos = table.getSelectedObjects();
            if (dcos.size() > 0) {
                task = new DeletingThread(table.getSelectedObjects());
                task.start();
            } else {
                new MessageBox(DcResources.getText("msgSelectItemToDel"), MessageBox._WARNING);
            }
        }
    }

    public void saveSelected() {
        if (!isTaskRunning()) {
            task = new SavingThread(table.getSelectedObjects());
            task.start();
        }
    }

    public void save() {
        save(true);
    }

    public void save(boolean threaded) {
        if (!isTaskRunning()) {
            Collection objects = table.getChangedObjects();
    
            boolean childrenSaved = true;
            if (hasChild()) {
                childView.cancelEdit();
            	childrenSaved = childView.changesSaved();
            }
    
            if (objects.size() > 0 || !childrenSaved) {
                task = new SavingThread(objects);
                task.startRunning();
                if (threaded)
                    task.start();
                else
                	task.run();
            } else {
                new MessageBox(DcResources.getText("msgNoChangesToSave"), MessageBox._WARNING);
            }
        }
    }

    public DcPopupMenu getPopupMenu() {
        DcObject dco = getSelectedObject();
        DcPopupMenu popupMenu = new DcPopupMenu(dco, DcPopupMenu._SEARCH);
        addCopyPaste(popupMenu);
        return popupMenu;
    }

    public DcObject getObjectForRow(int row, boolean allChildren) {
        DcObject dco = table.getDCObject(row);
        dco.removeChildren();
        if (hasChild()) {
            Collection children = new ArrayList();
            if (allChildren)
                children.addAll(childView.getObjects());
            else
                children.addAll(((TableSearchView) childView).getChangedObjects());

            for (Iterator iter = children.iterator(); iter.hasNext(); ) {
                DcObject child = (DcObject) iter.next();
                child.addRequest(new RemoveFromCacheRequest(childView.getModule().getIndex(), child.getID()));
                dco.addChild(child);
            }
        }

        dco.setSilence(true);
        dco.setBatch(true);
        return dco;
    }

    public void removeRows(String[] ids) {
        super.removeRows(ids);

        if (hasChild()) {
            childView.clear();
            childView.setParentID(null);
        }
    }

    public void applyGrouping(int[] fields) {
        treePanel.groupBy(fields);
    }
    
    public void undoGrouping() {
        
    }
    
    public Collection getChangedObjects() {
        return table.getChangedObjects();
    }

    public boolean changesSaved() {
        return table.noChanges();
    }

    public void saveChildItems() {
        if (hasChild() && table.getSelectedRow() > -1) {
            childView.cancelEdit();
            if (!childView.changesSaved()) {
            	new QuestionBox(DcResources.getText("msgNotSaved"));
                if (QuestionBox.result.equals(Boolean.TRUE)) {
                    save(false);
                }
            }
        }
    }

    public void loadChildValues() {}

    private void buildPanel() {
        //**********************************************************
        //Search result panel
        //**********************************************************
        panelInput.setLayout(Layout.getGBL());
        
        table = ComponentFactory.getDCTable(getModule(), false, true);
        table.addMouseListener(tableMouseListener);

        scrollResults1 = new JScrollPane(table);
        scrollResults1.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        scrollResults1.getViewport().setScrollMode(JViewport.SIMPLE_SCROLL_MODE);

        panelInput.add(scrollResults1,  Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                      ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                       new Insets(5, 5, 5, 5), 0, 0));

        table.getSelectionModel().addListSelectionListener(rowSelectionListener);
        table.addKeyListener(changeRowByKey);

        if (hasChild()) {
        	DcModule module  = getChildModule();
            childView = (TableSearchView) module.getSearchView().get(MasterView._TABLE_VIEW);

            scrollResults2 = new JScrollPane(childView.table);
            scrollResults2.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
            scrollResults2.getViewport().setScrollMode(JViewport.SIMPLE_SCROLL_MODE);

            panelInput.add(    scrollResults2,  Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                              ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                               new Insets(5, 5, 5, 5), 0, 0));
        }
        
        treePanel = new TreePanel(getModule().getIndex(), this);
        vdGroupingPane  = new DcViewDivider(treePanel, panelInput, Repository.settings.stTreeDividerLocation);
        quickView = getModule().getQuickView();
        
        vdQuickPane = new DcViewDivider(vdGroupingPane, quickView, Repository.settings.stQuickViewDividerLocation);
        if (quickView != null)
            quickView.setVisible(DcSettings.getValueAsBoolean(Repository.settings.stShowQuickView));

        setBorder(null);
        
        //**********************************************************
        //Main panel
        //**********************************************************
        setLayout(Layout.getGBL());

        add(    vdQuickPane,               Layout.getGBC( 0, 0, 1, 1, 100.0, 100.0
               ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                new Insets(5, 5, 0, 5), 0, 0));
        add(    panelTableActions,Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
               ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                new Insets(0, 5, 0, 5), 0, 0));
        add(    panelStatus,      Layout.getGBC( 0, 2, 1, 1, 1.0, 1.0
                ,GridBagConstraints.SOUTH, GridBagConstraints.HORIZONTAL,
                new Insets(0, 5, 0, 5), 0, 0));

        ToolTipManager.sharedInstance().registerComponent(table);
    }

    protected void addAfterDeleteRequest(DcObject dco) {}

    // ****************************************************************************
    // ACTIONS
    // ****************************************************************************
    private class DeletingThread extends DataTask {

        public DeletingThread(Collection objects) {
        	super(objects);
        }

        public void run() {
            try {
                updateProgressBar(0);
                initProgressBar(objects.size());

                setStatus(DcResources.getText("msgDeletingXItems", "" + objects.size()));

                startRunning();
                cancelEdit();
                denyActions();

                new QuestionBox(DcResources.getText("msgDeleteQuestion"));
                if (!QuestionBox.result.booleanValue()) {
                    stopRunning();
                }

                int counter = 1;
                for (Iterator iter = objects.iterator(); iter.hasNext() && keepOnRunning(); counter++) {
                    DcObject dco = (DcObject) iter.next();

                    dco.markAsUnchanged();
                    dco.setSilence(true);
                    dco.setEndOfBatch(counter == objects.size());

                    dco.addRequest(new UpdateStatusProgressBarRequest(
                            dco.getModule().getIndex(), MainFrame._SEARCHTAB, counter));                        
                    
                    if (counter == objects.size()) {
                        dco.addRequest(new StatusUpdateRequest(dco.getModule().getIndex(),
                                MainFrame._SEARCHTAB, "msgDeleteSuccessfull"));
                        
                        addAfterDeleteRequest(dco);
                        dco.setSilence(false);
                    }

                    dco.delete();

                    try { sleep(300); } catch (Exception ignore) {}
                }

                table.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
            } finally {
                if (getChildModule() != null)
                    getChildModule().getCurrentSearchView().clear();
                
            	stopRunning();
            	allowActions();
            }
            
            setSelectedRow();
        }
    }
    
    private class SavingThread extends DataTask {
        public SavingThread(Collection objects) {
            super(objects);
        }

        public void run() {
            try {
                updateProgressBar(0);
                initProgressBar(objects.size());
                setStatus(DcResources.getText("msgSavingXItems", "" + objects.size()));
                table.setCursor(new Cursor(Cursor.WAIT_CURSOR));

                startRunning();
                cancelEdit();
                denyActions();

                if (hasChild() && !childView.changesSaved() && (objects.size() == 0))
                    objects.addAll(((TableSearchView) childView).getChangedObjects());

                boolean ignoreErrors = false;
                int counter = 1;
                for (Iterator iter = objects.iterator(); iter.hasNext(); counter++) {
                	try {
	                    updateProgressBar(counter);
	                    DcObject dco = (DcObject) iter.next();
	                    dco.setEndOfBatch(counter == objects.size());
	                    dco.addRequest(new RemoveFromCacheRequest(getModule().getIndex(), dco.getID()));
                        dco.setSilence(true);

                        if (dco.isEndOfBatch()) {
                            dco.setSilence(false);
                            dco.addRequest(
                              new StatusUpdateRequest(dco.getModule().getIndex(), MainFrame._SEARCHTAB, "msgDataSaved"));
                        }

                        dco.saveUpdate(true);
                    } catch (ValidationException vExp) {
                        if (!ignoreErrors) {
                            SaveQuestionBox question = new SaveQuestionBox(vExp);
                            if (question.getResult() == SaveQuestionBox._CANCEL)
                                stopRunning();
                            else if (question.getResult() == SaveQuestionBox._IGNORE)
                                ignoreErrors = true;
                        }
                    }

                    try { sleep(300); } catch (Exception ignore) {}
                }
            } finally {
                table.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
                allowActions();
            	stopRunning();
            }
        }
    }
    
    public void setDataForSelectedRow() {
        if (quickView != null && quickView.isVisible() && dispatchEvents) {
            int row = table.getSelectedRow() != -1 ? table.getSelectedRow() : 0;
            
            try {
                quickView.setObject(getObjectForRow(row, true));
            } catch (Exception ignore) {}
        }

        if (hasChild() && dispatchEvents) {
           String ID = table.getObjectID(table.getSelectedRow());
           childView.setParentID(ID);
           loadChildValues();
        }
    }

    protected class RowSelectionListener implements ListSelectionListener {
    	public void valueChanged(ListSelectionEvent e) {
            if (e.getValueIsAdjusting()) {
                if (e.getFirstIndex() != e.getLastIndex()) {
                    setDataForSelectedRow();
                }
            }
    	}
    }

    protected class ChangeRowByKey implements KeyListener {
        private int rowFrom = -1;
        private int rowTo = -1;

        public void keyPressed(KeyEvent e) {
           if (     e.getKeyCode() == KeyEvent.VK_UP || 
                    e.getKeyCode() == KeyEvent.VK_DOWN)
               rowFrom = table.getSelectedRow();
           else 
               rowFrom = -1;
        }

        public void keyReleased(KeyEvent e) {
            if ((e.getKeyCode() == KeyEvent.VK_ENTER || e.getKeyCode() == KeyEvent.VK_UP || e.getKeyCode() == KeyEvent.VK_DOWN)) {
                rowTo = table.getSelectedRow();
                if (rowFrom != -1 && (rowFrom != rowTo)) {

                    if (hasChild() && dispatchEvents) {
                        saveChildItems();
                        loadChildValues();
                    }
                    
                    DcObject dco = getObjectForRow(rowTo, true);

                    if (quickView != null && quickView.isVisible() && dispatchEvents)
                        quickView.setObject(dco);
                }
            }
        }

        public void keyTyped(KeyEvent arg0) {}
    }

    private class SaveUpdatedValuesAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            save();
        }
    }
}
