/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.wizards;

import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.ListSelectionModel;

import net.sf.dc.console.Layout;
import net.sf.dc.console.audiocd.OnlineAudioCdSearchForm;
import net.sf.dc.console.book.AmazonBookSearchForm;
import net.sf.dc.console.forms.OnlineSearchForm;
import net.sf.dc.console.movie.OnlineMovieSearchForm;
import net.sf.dc.console.software.OnlineSoftwareSearchForm;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.resources.DcResources;

public class InternetWizardPanel extends WizardPanel {

    private OnlineSearchForm internetSearchForm = null;
    private NewItemWizard wizard;

    public InternetWizardPanel(NewItemWizard wizard, DcModule module, DcObject dco) {
        build(module, dco);
        this.wizard = wizard;
    }

    public DcObject apply() throws WizardException {
        DcObject result = internetSearchForm.getSelectedObject();
        if (result == null) 
            throw new WizardException(DcResources.getText("msgWizardSelectItem"));

        return result;
    }

    public String getHelpText() {
        return DcResources.getText("msgInternetSearch");
    }

    public void setObject(DcObject dco) {}

    public void setFocus() {
        internetSearchForm.setFocus();
    }
    
    public void setVisible(boolean b) {
        if (b && internetSearchForm != null)  
            internetSearchForm.setFocus();
        
        super.setVisible(b);
    }     
    
    private void build(DcModule module, DcObject dco) {
        if (module.isAmazonUpdateAllowed()) {
            if (module.getIndex() == DcModules._SOFTWARE) {
                internetSearchForm = new OnlineSoftwareSearchForm(null, DataCrow.mainFrame.getSelectedTab(), false);
            } else {
                internetSearchForm = new AmazonBookSearchForm(null, DataCrow.mainFrame.getSelectedTab(), false);
            }
        } else if (module.isImdbUpdateAllowed()) {
            internetSearchForm = new OnlineMovieSearchForm(null, 0, false);
        } else if (module.isFreedBUpdateAllowed()) {
            internetSearchForm = new OnlineAudioCdSearchForm(false);
        }

        if (internetSearchForm != null) {
            internetSearchForm.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
            internetSearchForm.addDoubleClickListener(new DblClickSelectionListener());
            setLayout(Layout.getGBL());
            add(internetSearchForm.getContentPane(), Layout.getGBC(0, 0, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                            new Insets(5, 5, 5, 5), 0, 0));
        }
    }
    
    public class DblClickSelectionListener implements MouseListener {
        public void mouseReleased(MouseEvent e) {
            if (e.getClickCount() == 2)
                wizard.next();
        }
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {}
    }     
}