/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.wizards;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeListener;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;

import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.KeyStroke;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcDialog;
import net.sf.dc.console.components.DcFrame;
import net.sf.dc.console.components.DcLongTextField;
import net.sf.dc.console.dialogs.ItemTypeDialog;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.wf.requests.CloseWindowRequest;

public class NewItemWizard extends DcDialog {

    private final LinkedList panels = new LinkedList();
    private int current = 0;

    private DcLongTextField textHelp = ComponentFactory.getLongTextField(8000);
    private JButton buttonNext = ComponentFactory.getButton(DcResources.getText("lblNext"));
    private JButton buttonBack = ComponentFactory.getButton(DcResources.getText("lblBack"));
    private JButton buttonRestart = ComponentFactory.getButton(DcResources.getText("lblFinishRestart"));
    private JButton buttonFinish = ComponentFactory.getButton(DcResources.getText("lblFinish"));

    private DcObject dco;
    private boolean closed = false;

    public NewItemWizard(DcFrame parent) {
        super(parent);
        setModal(false);
        setTitle(DcResources.getText("lblNewWizard"));
        setHelpIndex("dc.Wizard");

        DcModule module = DcModules.get(DcSettings.getValueAsInt(Repository.settings.stModule));
        if (module.getIndex() == DcModules._GENERIC) {
        	ItemTypeDialog dialog = new ItemTypeDialog();
            dialog.setVisible(true);

            if (dialog.getSelectedModule() < 0) {
                closed = true;
                close();
            } else {
                module = DcModules.get(dialog.getSelectedModule());
            }
        }

        if (!closed) {
            dco = module.getDcObject();

            addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    saveSettings();
                }
            });

            if (    module.isAmazonUpdateAllowed() ||
                    module.isFreedBUpdateAllowed() ||
                    module.isImdbUpdateAllowed()) {

                JPanel panelInternetSearch = new InternetWizardPanel(this, module, dco);
                panels.add(panelInternetSearch);
            }

            panels.add(new ItemDetailsWizardPanel(dco));
            build(panels);

            setSize(DcSettings.getValueAsDimension(Repository.settings.stWizardFormSize));

            setCenteredLocation();
        }
    }

    private void applyPanel() {
        buttonBack.setEnabled(current != 0);
        buttonNext.setVisible(current != panels.size() - 1);
        buttonFinish.setVisible(current == panels.size() - 1);
        buttonRestart.setVisible(current == panels.size() - 1);

        textHelp.setText(getCurrent().getHelpText());

        String title = DcResources.getText("msgNewItemWizard",
                       new String[] {dco.getModule().getObjectName(),
                                     String.valueOf(current + 1),
                                     String.valueOf(panels.size())});

        if (buttonFinish.isVisible())
            buttonFinish.requestFocus();
        else
            buttonNext.requestFocus();

        setTitle(title);
    }

    public void setVisible(boolean b) {
        super.setVisible(b && !closed);
        
        if (b) {
            if (getCurrent() instanceof InternetWizardPanel)
                ((InternetWizardPanel) getCurrent()).setFocus();
            
        }
            
    }

    private WizardPanel getCurrent() {
        return (WizardPanel) panels.get(current);
    }

    public void dispose() {
        if (!closed)
            saveSettings();

        super.dispose();
    }

    private void saveSettings() {
        DcSettings.setValue(Repository.settings.stWizardFormSize, getSize());
    }

    private void finish() throws Exception {
            dco = getCurrent().apply();
            dco.addRequest(new CloseWindowRequest(this));
            dco.saveNew(true);
    }

    public void next() {
        try {
            dco = getCurrent().apply();

            current += 1;
            if (current <= panels.size()) {
                int counter = 0;
                for (Iterator iter = panels.iterator(); iter.hasNext(); counter++) {
                    WizardPanel panel = (WizardPanel) iter.next();
                    panel.setObject(dco);
                    panel.setVisible(counter == current);
                }
            } else {
                current -= 1;
            }

            applyPanel();
        } catch (WizardException wzexp) {
            if (wzexp.getMessage().length() > 1)
                new MessageBox(wzexp.getMessage(), MessageBox._WARNING);
        }
    }

    private void back() {
        current -= 1;
        int counter = 0;

        if (current >= 0) {
            for (Iterator iter = panels.iterator(); iter.hasNext(); counter++) {
                WizardPanel panel = (WizardPanel) iter.next();
                panel.setVisible(counter == current);
            }
        } else {
            current += 1;
        }

        applyPanel();
    }

    private void close() {
        dco = null;
        dispose();
        setVisible(false);
    }

    private void build(Collection panels) {
        JPanel panelActions = new JPanel();
        JPanel panelInformation = new JPanel();

        panelActions.setLayout(Layout.getGBL());
        panelInformation.setLayout(Layout.getGBL());

        // Help panel
        panelInformation.setLayout(Layout.getGBL());

        textHelp.setBorder(null);
        JScrollPane scroller = new JScrollPane(textHelp);
        scroller.setBorder(null);
        scroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        scroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_NEVER);
        textHelp.setEditable(false);

        panelInformation.add(textHelp, Layout.getGBC(0, 0, 1, 1, 1.0, 1.0
                        ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                         new Insets(5, 5, 5, 5), 0, 0));

        // Action panel
        JButton buttonCancel = ComponentFactory.getButton(DcResources.getText("lblCancel"));
        buttonNext.addActionListener(new NextAction());
        buttonBack.addActionListener(new BackAction());
        buttonFinish.addActionListener(new FinishAction());
        buttonRestart.addActionListener(new RestartAction());
        buttonCancel.addActionListener(new CancelAction());

        buttonNext.setMnemonic('N');
        buttonBack.setMnemonic('B');
        buttonFinish.setMnemonic('F');
        buttonRestart.setMnemonic('R');
        buttonCancel.setMnemonic('C');
        
        buttonBack.setEnabled(false);
        buttonFinish.setVisible(false);
        buttonRestart.setVisible(false);

        panelActions.add(buttonBack,    Layout.getGBC(0, 0, 1, 1, 1.0, 1.0
                                ,GridBagConstraints.NORTHEAST, GridBagConstraints.NONE,
                                 new Insets(5, 5, 5, 5), 0, 0));
        panelActions.add(buttonNext,    Layout.getGBC(1, 0, 1, 1, 1.0, 1.0
                                ,GridBagConstraints.NORTHEAST, GridBagConstraints.NONE,
                                 new Insets(5, 5, 5, 5), 0, 0));
        panelActions.add(buttonRestart, Layout.getGBC(2, 0, 1, 1, 1.0, 1.0
                                ,GridBagConstraints.NORTHEAST, GridBagConstraints.NONE,
                                 new Insets(5, 5, 5, 5), 0, 0));
        panelActions.add(buttonFinish,  Layout.getGBC(3, 0, 1, 1, 1.0, 1.0
                                ,GridBagConstraints.NORTHEAST, GridBagConstraints.NONE,
                                 new Insets(5, 5, 5, 5), 0, 0));
        panelActions.add(buttonCancel,  Layout.getGBC(4, 0, 1, 1, 1.0, 1.0
                                ,GridBagConstraints.NORTHEAST, GridBagConstraints.NONE,
                                 new Insets(5, 5, 5, 5), 0, 0));

        // Main
        getContentPane().setLayout(Layout.getGBL());

        getContentPane().add(panelInformation,Layout.getGBC(1, 0, 1, 1, 1.0, 1.0
                                      ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                       new Insets(5, 5, 5, 5), 0, 0));
        getContentPane().add(panelActions,Layout.getGBC(1, 3, 1, 2, 1.0, 1.0
                                      ,GridBagConstraints.SOUTHEAST, GridBagConstraints.NONE,
                                       new Insets(5, 5, 5, 15), 0, 0));
        // the actual content panels
        int counter = 0;
        for (Iterator iter = panels.iterator(); iter.hasNext(); counter++) {
            JPanel panel = (JPanel) iter.next();
            panel.setMinimumSize(new Dimension(350,350));
            panel.setPreferredSize(new Dimension(350,350));

            panel.setVisible(counter == 0);
            getContentPane().add(panel,   Layout.getGBC(1, 1, 1, 1, 20.0, 20.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                             new Insets(5, 5, 5, 5), 0, 0));
        }

        setResizable(true);
        pack();
        addKeyListener(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0, false), new NextOnEnterAction(), "next");
        applyPanel();
    }

    private class NextOnEnterAction implements Action {
        public boolean isEnabled() {
            return true;
        }

        public void setEnabled(boolean arg0) {}
        public void actionPerformed(ActionEvent arg0) {
            if (current < panels.size() - 1) {
                next();
            } else {
                try {
                    finish();
                } catch (Exception exp) {
                    new MessageBox(exp.getMessage(), MessageBox._WARNING);
                }
            }
        }
        public void putValue(String arg0, Object arg1) {}
        public void addPropertyChangeListener(PropertyChangeListener arg0) {}

        public Object getValue(String arg0) {
            return null;
        }
        public void removePropertyChangeListener(PropertyChangeListener arg0) {}
    }


    public class NextAction implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            next();
        }
    }

    public class RestartAction implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            try {
                finish();
                saveSettings();
                NewItemWizard wizard = new NewItemWizard(DataCrow.mainFrame);
                wizard.setVisible(true);
            } catch (Exception exp) {
                new MessageBox(exp.getMessage(), MessageBox._WARNING);
            }
        }
    }

    private class CancelAction implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            saveSettings();
            close();
        }
    }

    private class BackAction implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            back();
        }
    }

    private class FinishAction implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            try {
                finish();
            } catch (Exception exp) {
                new MessageBox(exp.getMessage(), MessageBox._WARNING);
            }
        }
    }
}
