/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.data;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;

import net.sf.dc.core.DataCrow;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.messages.Messages;
import net.sf.dc.settings.DcSettings;

public class DataFilters {
    
    private static final Map filters = new HashMap(); 
    
    private static final Map activeFilters = new HashMap();
    
    private static final String filename = DataCrow.baseDir + "data" + File.separator + "filters.properties";
    
    public DataFilters() {}
    
    /**
     * Loads the settings file
     */
    public void load() {
        Properties properties = new Properties();
        
        try {
            FileInputStream fis = new FileInputStream(filename);
            properties.load(fis);
            fis.close();
        } catch (Exception exp) {} // use default settings
        
        for (Iterator iter = properties.keySet().iterator(); iter.hasNext(); ) {
            String key = (String) iter.next();
            String value = properties.getProperty(key);
            
            try {
                int modIdx = new Integer(key.substring(0, key.indexOf("/&/"))).intValue();
                
                Collection c = (Collection) filters.get(new Integer(modIdx));
                c = c == null ? new ArrayList() : c;
                
                c.add(new DataFilter(value));
                filters.put(new Integer(modIdx), c);
                
            } catch (Exception exp) {
                Messages.add(exp);
            }
        }
    }
    
    public static void setCurrent(int module, DataFilter df) {
    	activeFilters.put(new Integer(module), df);
    }
    
    public static DataFilter getDefaultDataFilter(int module) {
        DcModule m = DcModules.get(module);
        
        String[] fields = DcSettings.getValueAsStringArray(m.getSearchOrderSettingsKey());
        DcField[] f = new DcField[fields.length];
        int idx = 0;

        for (int i = 0; i < fields.length; i++) {
            for (Iterator iter = m.getFields().iterator(); iter.hasNext(); ) {
                DcField field = (DcField) iter.next();
                if (field.getDatabaseFieldName().equals(fields[i])) {
                    f[idx++] = field;
                }
            }
        }
        
        DataFilterOptions dfo = new DataFilterOptions(f, true, false);
        return new DataFilter(module, m.getDcObject(), dfo);        
    }
    
    public static DataFilter getCurrent(int module) {
    	DataFilter df = (DataFilter) activeFilters.get(new Integer(module));
    	if (df == null)
    	    df = getDefaultDataFilter(module);
    		
    	return df;
    }
    
    public void save() {
        Properties properties = new Properties();
        for (Iterator iter = filters.keySet().iterator(); iter.hasNext(); ) {
            Object key = iter.next();
            Collection c = (Collection) filters.get(key);
            
            if (c != null) {
                for (Iterator iterFilters = c.iterator(); iterFilters.hasNext(); ) {
                    DataFilter df = (DataFilter) iterFilters.next();
                    String value = df.toStorageString();
                    properties.setProperty(key + "/&/" + df.getName(), value);
                }
            }
        }
        
        try {
            FileOutputStream fos = new FileOutputStream(filename);
            properties.store(fos, "");
            fos.close();
        } catch (Exception exp) {
            Messages.add(exp);  
        }         
        
    }
    
    public static void delete(DataFilter df) {
        Collection c = get(df.getModule());
        c.remove(df);
    }
    
    public static Collection get(int module) {
        Collection c = (Collection) filters.get(new Integer(module));
        return c != null ? c : new ArrayList();
    }
    
    public static void add(DataFilter df) {
        Collection c = (Collection) filters.get(new Integer(df.getModule()));
        c = c == null ? new ArrayList() : c;
        
        c.add(df);
        filters.put(new Integer(df.getModule()), c);
    }
}
