/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcMediaObject;
import net.sf.dc.core.objects.audiocd.AudioCD;

public abstract class DcMediaModule extends DcModule {
    
    public DcMediaModule(int index, 
                         boolean topModule,
                         String name,
                         String tableName,
                         String tableShortName,
                         String tableJoin,
                         boolean canBeLended,
                         boolean allowImdbUpdate,
                         boolean allowAmazonUpdate,
                         boolean allowFreeDbUpdate) {
        
        super(index, topModule, name, tableName, tableShortName,
              tableJoin, allowImdbUpdate, allowAmazonUpdate, allowFreeDbUpdate);
        
        setCanBeLended(canBeLended);
    }

    public String getOnlineUpdateFieldsSettingsKey() {
        return Repository.settings.stOnlineUpdateFieldSettings + getName().replace(" ", "");
    }
    
    public String getOnlineServiceSettingsKey() {
        return Repository.settings.stOnlineServiceSettings + getName().replace(" ", "");
    }
    
    public String getGroupedBySettingsKey() {
        return Repository.settings.stGroupedBy + getName().replace(" ", "");
    }
    
    protected void initializeFields() {
    	super.initializeFields();

    	fields.put( new Integer(DcMediaObject._A_TITLE),
                    new DcField(DcMediaObject._A_TITLE, getIndex(), "Title",
                                false, true, false, true, false,
                                255, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "Title"));
        fields.put( new Integer(DcMediaObject._B_DESCRIPTION),
                    new DcField(DcMediaObject._B_DESCRIPTION, getIndex(), "Description",  
                                false, true, false, true, false, 
                                8000, ComponentFactory._LONGTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "Description"));
        fields.put( new Integer(DcMediaObject._C_YEAR),
                    new DcField(AudioCD._C_YEAR, getIndex(), "Year",  
                                false, true, false, true, false, 
                                4, ComponentFactory._NUMBERFIELD, getIndex(), Repository.datatypes._INTEGER,
                                "Year"));
        fields.put( new Integer(DcMediaObject._D_LOCATION),
                    new DcField(DcMediaObject._D_LOCATION, getIndex(), "Location",  
                                false, true, false, true, false, 
                                255, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "Location"));
        fields.put( new Integer(DcMediaObject._E_RATING),
                    new DcField(DcMediaObject._E_RATING, getIndex(), "Rating",  
                                false, true, false, true, false, 
                                255, ComponentFactory._RATINGCOMBOBOX, getIndex(), Repository.datatypes._INTEGER,
                                "Rating"));    
        fields.put( new Integer(DcMediaObject._U1_USER_LONGTEXT),
                    new DcField(DcMediaObject._U1_USER_LONGTEXT, getIndex(), "User Long Text Field",  
                                false, false, false, true, false, 
                                4000, ComponentFactory._LONGTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "UserLongText1"));         
        fields.put( new Integer(DcMediaObject._U2_USER_SHORTTEXT1),
                    new DcField(DcMediaObject._U2_USER_SHORTTEXT1, getIndex(), "User Short Text Field 1",  
                                false, false, false, true, false, 
                                255, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "UserShortText1")); 
        fields.put( new Integer(DcMediaObject._U3_USER_SHORTTEXT2),
                    new DcField(DcMediaObject._U3_USER_SHORTTEXT2, getIndex(), "User Short Text Field 2",  
                                false, false, false, true, false, 
                                255, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "UserShortText2")); 
        fields.put( new Integer(DcMediaObject._U4_USER_NUMERIC1),
                    new DcField(DcMediaObject._U4_USER_NUMERIC1, getIndex(), "User Numeric Field 1",  
                                false, false, false, true, false, 
                                255, ComponentFactory._NUMBERFIELD, getIndex(), Repository.datatypes._INTEGER,
                                "UserInteger1"));
        fields.put( new Integer(DcMediaObject._U5_USER_NUMERIC2),
                    new DcField(DcMediaObject._U5_USER_NUMERIC2, getIndex(), "User Numeric Field 2",  
                                false, false, false, true, false, 
                                255, ComponentFactory._NUMBERFIELD, getIndex(), Repository.datatypes._INTEGER,
                                "UserInteger2"));
    }    
}
