/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.KeyStroke;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.components.panels.QuickViewPanel;
import net.sf.dc.console.forms.FilterForm;
import net.sf.dc.console.importcd.ImportCDDialog;
import net.sf.dc.console.menu.DcMenuBar;
import net.sf.dc.console.views.MasterView;
import net.sf.dc.console.views.View;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.template.ITemplateModule;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.DcStorageMedium;
import net.sf.dc.core.objects.enhancement.IValueEnhancer;
import net.sf.dc.db.DatabaseManager;
import net.sf.dc.processes.cdimport.ImportCDProcess;
import net.sf.dc.processes.synchronizer.Synchronizer;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.settings.definitions.DcFieldDefinitions;

public abstract class DcModule {

    protected MasterView insertView;
    protected MasterView searchView;
    protected FilterForm searchForm;

    protected DcModule childModule;
    protected Map fields = new HashMap();
    private Map systemFields = new HashMap();

    private final int index;
    private final String name;

    private boolean canBeLended = false;
    private final boolean topModule;
    private final boolean allowImdbUpdate;
    private final boolean allowAmazonUpdate;
    private final boolean allowFreeDbUpdate;

    protected String tableName = "";
    protected String tableShortName = "";
    protected String tableJoin = "";

    public DcModule(        int index,
                            boolean topModule,
                            String name,
                            String tableName,
                            String tableShortName,
                            String tableJoin,
                            boolean allowImdbUpdate,
                            boolean allowAmazonUpdate,
                            boolean allowFreeDbUpdate) {

        this.index = index;
        this.tableName = tableName;
        this.tableShortName  = tableShortName;
        this.tableJoin = tableJoin;
        this.name = name;
        this.topModule = topModule;
        this.allowAmazonUpdate = allowAmazonUpdate;
        this.allowImdbUpdate = allowImdbUpdate;
        this.allowFreeDbUpdate = allowFreeDbUpdate;

        initializeSystemFields();
        initializeFields();
    }
    
    public Synchronizer getSynchronizer() {
        return null;
    }
    
    public abstract DcCategoryModule getCategoryModule();
    public abstract DcStateModule getStateModule();
    public abstract DcStorageMediaModule getStorageMediaModule();
    public abstract ITemplateModule getTemplateModule();
    public abstract DcGenreModule getGenreModule();
    public abstract String getDescription();
    
    public Collection getOnlineServices() {
        return new ArrayList();
    }
    
    public String getGroupedBySettingsKey() {
        return null;
    }
    
    public String getOnlineUpdateFieldsSettingsKey() {
        return null;
    }

    public String getOnlineServiceSettingsKey() {
        return null;
    }
    
    public Collection loadData() {
        return DatabaseManager.runQueryDirect(getDcObject(), true);
    }    

    private void initializeSystemFields() {
        systemFields.put(new Integer(DcObject._SYS_MODULE),
                         new DcField(DcObject._SYS_MODULE, getIndex(), "Item",
                                     true, true, true, false, false,
                                     255, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                     ""));
        systemFields.put(new Integer(DcObject._SYS_AVAILABLE),
                         new DcField(DcObject._SYS_AVAILABLE, getIndex(), "Available",
                                     true, true, true, true, false,
                                     4, ComponentFactory._AVAILABILITYCOMBO, getIndex(), Repository.datatypes._BOOLEAN,
                                     ""));
        systemFields.put(new Integer(DcObject._SYS_LOANEDBY),
                         new DcField(DcObject._SYS_LOANEDBY, getIndex(), "Loaned By",
                                     true, true, true, true, false,
                                     255, ComponentFactory._CONTACTPERSONSCOMBO, DcModules._CONTACTPERSON, Repository.datatypes._DCOBJECTREFERENCE,
                                     ""));
        systemFields.put(new Integer(DcObject._SYS_LOANDURATION),
                         new DcField(DcObject._SYS_LOANDURATION, getIndex(), "Days Loaned",
                                     true, true, true, true, false,
                                     10, ComponentFactory._NUMBERFIELD, getIndex(), Repository.datatypes._INTEGER,
                                     ""));
    }

    public MasterView[] getViews() {
        if (getSearchView() != null && getInsertView() != null) {
            return new MasterView[] {getSearchView(), getInsertView()};
        } else if (getSearchView() != null) {
             return new MasterView[] {getSearchView()};
        } else if (getInsertView() != null) {
            return new MasterView[] {getInsertView()};
        } else {
        	return new MasterView[0];
        }
    }

    public DcField getField(int index) {
        DcField field = (DcField) fields.get(new Integer(index));
        return field == null ? getSystemField(index) : field;
    }

    public void addValueEnhancer(IValueEnhancer enhancer, int field) {
        getField(field).addValueEnhancer(enhancer);
    }
    
    public DcField getSystemField(int index) {
        return (DcField) systemFields.get(new Integer(index));
    }

    public void setChildModule(DcModule childModule) {
    	this.childModule = childModule;
    }
    
    public abstract boolean isEnabled();
    public abstract void isEnabled(boolean b);

    public DcMenuBar getMenuBar() {
        return new DcMenuBar(this);
    }

    public DcModule getChildModule() {
    	return childModule;
    }

    public FilterForm getSearchForm() {
        searchForm = searchForm == null ? new FilterForm(this, getSearchView()) : searchForm;
        return searchForm;
    }

    public Collection getFields() {
        return fields.values();
    }

    public int[] getFieldIndices() {
        Set keys = fields.keySet();
        int[] indices = new int[keys.size()];
        int counter = 0;
        for (Iterator iter = keys.iterator(); iter.hasNext(); counter++) {
            indices[counter] = ((Integer) iter.next()).intValue();
        }
        return indices;
    }

    protected void initializeFields() {
        fields.put(new Integer(DcObject._ID),
                   new DcField(DcObject._ID, getIndex(), "ID",
                               false, true, true, false, false,
                               15, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._BIGINTEGER,
                               "ID"));
        fields.put(new Integer(DcObject._SYS_CREATED),
        		   new DcField(DcObject._SYS_CREATED, getIndex(), "Created",
	                           false, true, true, false, true,
	                           10, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                               "Created"));
        fields.put(new Integer(DcObject._SYS_MODIFIED),
                   new DcField(DcObject._SYS_MODIFIED, getIndex(), "Modified",
                               false, true, true, false, true,
                               10, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                 			   "Modified"));
    }

    public boolean isTopModule() {
        return topModule;
    }

    public int getFieldCount() {
        return fields.size();
    }

    public boolean canBeLended() {
        return canBeLended && DcModules.get(DcModules._CONTACTPERSON).isEnabled();
    }

    public void setCanBeLended(boolean b) {
        this.canBeLended = b;
    }
    
    public void removeEnhancers() {
        for (Iterator iter = fields.values().iterator(); iter.hasNext(); ) {
            DcField field = (DcField) iter.next();
            field.removeEnhancers();
        }
    }

    public abstract ImageIcon getIconSmall();
    
    public abstract ImageIcon getIcon();
    
    public DcFieldDefinitions getFieldDefinitions() {
        if (getFieldDefinitionsSettingsKey() != null)
            return (DcFieldDefinitions) DcSettings.getValueAsDefinitionGroup(getFieldDefinitionsSettingsKey());
        else 
            return new DcFieldDefinitions();
    }
    
    public abstract String getFieldDefinitionsSettingsKey();
    public abstract String getSearchOrderSettingsKey();
    public abstract String getObjectName();
    public abstract String getObjectNamePlural();
    public abstract KeyStroke getKeyStroke();
    public abstract QuickViewPanel getQuickView();

    public abstract DcObject[] getDefaultData() throws Exception;
    public abstract void startImport();

    public abstract DcObject getDcObject();

    public abstract JMenu getToolsMenu();

    protected abstract void initializeUI();

    public abstract ImportCDProcess getCDImportProcess(ImportCDDialog ui,
                                                       DcStorageMedium storageMedium,
                                                       String sourceDir,
													   boolean confirmDirs,
                                                       boolean recursiveDirs,
													   String nameOfCD);

    public MasterView getInsertView() {
        if (searchView == null && insertView == null)
            initializeUI();

        return insertView;
    }

    public MasterView getSearchView() {
        if (searchView == null && insertView == null)
            initializeUI();
        return searchView;
    }

    public View getCurrentSearchView() {
        return getSearchView().getCurrent();
    }

    public View getCurrentInsertView() {
        return getInsertView().getCurrent();
    }

    public boolean isFreedBUpdateAllowed() {
        return allowFreeDbUpdate;
    }

    public boolean isAmazonUpdateAllowed() {
        return allowAmazonUpdate;
    }

    public boolean isImdbUpdateAllowed() {
        return allowImdbUpdate;
    }

    public String getName() {
        return name;
    }

    public String getTableName() {
    	return tableName;
    }

    public String getTableShortName() {
        return tableShortName;
    }

    public String getTableJoin() {
    	return tableJoin;
    }

    public int getIndex() {
        return index;
    }

    /**
     * Creates a string representation for this module
     */
    public String toString() {
        return name;
    }
}
