/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedHashMap;

import net.sf.dc.core.Repository;
import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.settings.DcSettings;

public class DcModules {

    public static final int _SOFTWARE = 1;
    public static final int _MOVIE = 2;
    public static final int _MUSICALBUM = 3;
    public static final int _AUDIOCD = 4;
    public static final int _BOOK = 5;
    public static final int _IMAGE = 8;
    public static final int _CONTACTPERSON = 6;
    public static final int _GENERIC = 7;

    public static final int _AUDIOTRACK = 100;
    public static final int _SOFTWARECAT = 101;
    public static final int _BOOKCAT = 103;
    public static final int _LOAN = 104;
    public static final int _CONTACTPERSONCATEGORY = 105;

    public static final int _SOFTWARESTORAGE = 106;
    public static final int _MOVIESTORAGE = 107;
    public static final int _MUSICTRACK = 109;
    
    public static final int _BOOKSTATE = 150;
    public static final int _AUDIOCDSTATE = 151;
    public static final int _SOFTWARESTATE = 152;
    public static final int _MOVIESTATE = 153;
    
    public static final int _SOFTWAREPLATFORM = 155;
    
    public static final int _MUSICALBUMSTORAGE = 156;
    public static final int _MUSICALBUMSTATE = 157;
    public static final int _MUSICTRACKSTATE = 158;
    public static final int _IMAGESTORAGE = 159;
    public static final int _IMAGESTATE = 160;
    public static final int _IMAGECAT = 161;
    
    public static final int _MUSICALBUMGENREMAPPING = 162;
    public static final int _MUSICTRACKGENREMAPPING = 163;
    public static final int _AUDIOCDGENREMAPPING = 164;
    public static final int _AUDIOTRACKGENREMAPPING = 165;
    public static final int _MOVIEGENREMAPPING = 166;
    
    public static final int _MOVIEGENRE = 170;
    public static final int _MUSICGENRE = 171;
    
    public static final int _PICTURE = 200;
    
    public static final int _TEMPLATESOFTWARE = 500;
    public static final int _TEMPLATEAUDIOCD = 501;
    public static final int _TEMPLATEBOOK = 502;
    public static final int _TEMPLATEMOVIE = 503;
    public static final int _TEMPLATEMUSICALBUM = 505;
    public static final int _TEMPLATEMUSICTRACK = 506;
    public static final int _TEMPLATEIMAGE = 507;

    private static final LinkedHashMap modules = new LinkedHashMap();
    
    public static void register(DcModule module) {
        modules.put(new Integer(module.getIndex()), module);
    }

    public static DcModule get(int index) {
        return (DcModule) modules.get(new Integer(index));
    }

    public static DcObject getObjectForTable(String tableName) {
        for (Iterator iter = modules.values().iterator(); iter.hasNext(); ) {
            DcModule module = (DcModule) iter.next();
            if (module.getTableName().equalsIgnoreCase(tableName)) {
                return module.getDcObject();
            }
        }
        return null;
    }

    public static void loadData() {
        new DataManager();
    }
    
    public static DcModule getCurrentModule() {
        return DcModules.get(DcSettings.getValueAsInt(Repository.settings.stModule));
    }

    public static Iterator getAllModules() {
        return modules.values().iterator();
    }
    
    public static Iterator getModules() {
        Collection c = new ArrayList();
        for (Iterator iter = getAllModules(); iter.hasNext(); ) {
            DcModule module = (DcModule) iter.next();
            if (module.isEnabled())
                c.add(module);
        }
        return c.iterator();
    }
}
