/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules.audiocd;

import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.Collection;

import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.audiocd.AudioCDQuickViewPane;
import net.sf.dc.console.components.panels.ParentInsertTablePanel;
import net.sf.dc.console.components.panels.ParentSearchTablePanel;
import net.sf.dc.console.components.panels.QuickViewPanel;
import net.sf.dc.console.importcd.ImportCDDialog;
import net.sf.dc.console.views.MasterView;
import net.sf.dc.console.views.list.ListInsertView;
import net.sf.dc.console.views.list.ListSearchView;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.actions.ManageGenresAction;
import net.sf.dc.core.actions.ManageStatesAction;
import net.sf.dc.core.actions.ManageTemplatesAction;
import net.sf.dc.core.modules.DcCategoryModule;
import net.sf.dc.core.modules.DcGenreModule;
import net.sf.dc.core.modules.DcMediaModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.modules.DcStateModule;
import net.sf.dc.core.modules.DcStorageMediaModule;
import net.sf.dc.core.modules.template.ITemplateModule;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.DcStorageMedium;
import net.sf.dc.core.objects.audiocd.AudioCD;
import net.sf.dc.core.objects.audiocd.AudioTrack;
import net.sf.dc.core.services.AsinSearchMode;
import net.sf.dc.core.services.BarcodeSearchMode;
import net.sf.dc.core.services.DcAmazonServer;
import net.sf.dc.core.services.DcFreeDbServer;
import net.sf.dc.core.services.DcMusicBrainzServer;
import net.sf.dc.core.services.DcRegion;
import net.sf.dc.core.services.DcServer;
import net.sf.dc.core.services.KeywordSearchMode;
import net.sf.dc.core.services.MbAlbumTitleSearch;
import net.sf.dc.core.services.MbArtistSearch;
import net.sf.dc.freedb.Freedb;
import net.sf.dc.processes.cdimport.ImportCDProcess;
import net.sf.dc.processes.synchronizer.AudioCdSynchronizer;
import net.sf.dc.processes.synchronizer.Synchronizer;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;

public class AudioCDModule extends DcMediaModule {

    public AudioCDModule() {
        super(DcModules._AUDIOCD,
              true,
              DcResources.getText(Repository.systemlabels.sysAudioCD),
              "audiocds",
              "aucd",
              " audiocds aucd LEFT JOIN audiotracks autr ON (aucd.ID = autr.Album)",
              true,
              false,
              false,
              true);
    }
    
    public AudioCDModule( int index, boolean topModule, String name,
                          String tableName, String tableShortName, String tableJoin,
                          boolean canBeLended, boolean allowImdbUpdate, boolean allowAmazonUpdate, 
                          boolean allowFreeDbUpdate) {

        super(index, topModule, name, tableName, tableShortName,
             tableJoin, canBeLended, allowImdbUpdate, allowAmazonUpdate, allowFreeDbUpdate);
    }  
    
    public Synchronizer getSynchronizer() {
        return new AudioCdSynchronizer();
    }

    public Collection getOnlineServices() {
        Collection servers = new ArrayList();
        DcMusicBrainzServer musicBrainzServer = new DcMusicBrainzServer("MusicBrainz", "http://musicbrainz.org/ws/1/", true);
        musicBrainzServer.addRegion(new DcRegion("uk", "The main server", "http://musicbrainz.org/ws/1"));
        musicBrainzServer.addMode(new MbAlbumTitleSearch());
        musicBrainzServer.addMode(new MbArtistSearch());
        
        DcFreeDbServer freeDbServer = new DcFreeDbServer("FreeDB Servers", "http://freedb.freedb.org", false);
        boolean freedbReachable = false;
        try {
            String[] c = new Freedb("freedb.org:80", null).getAvailableServers();
            for (int i = 0; i < c.length; i++) {
                freeDbServer.addRegion(new DcRegion(c[i], c[i], c[i]));
            }
            freedbReachable = true;
        } catch (Exception exp) {}
        
        
        DcServer amazonServer = new DcAmazonServer();
        amazonServer.addMode(new KeywordSearchMode("Music", "Keyword"));
        amazonServer.addMode(new BarcodeSearchMode("Music", "Barcode"));
        amazonServer.addMode(new AsinSearchMode("Asin"));        
        
        servers.add(musicBrainzServer);
        servers.add(amazonServer);
        if (freedbReachable)
            servers.add(freeDbServer);
        
        return servers;        
    }    
    
    public String getDescription() {
        return DcResources.getText("msgAudioCdModuleDesc");
    }    
    
    public ImageIcon getIcon() {
        return IconLibrary._icoAudioCDModule;
    }
    
    public ImageIcon getIconSmall() {
        return IconLibrary._icoAudioCDModule16;
    }

    public String getObjectName() {
    	return DcResources.getText(Repository.systemlabels.sysAudioCD);
    }

    public String getObjectNamePlural() {
        return DcResources.getText(Repository.systemlabels.sysAudioCDPlural);
    }

    public KeyStroke getKeyStroke() {
        return KeyStroke.getKeyStroke(KeyEvent.VK_4, InputEvent.CTRL_MASK);
    }
    
    public JMenu getToolsMenu() {
        JMenu menu = ComponentFactory.getMenu(DcResources.getText("lblTools"));

        JMenuItem menuState = ComponentFactory.getMenuItem(IconLibrary._icoAnchor, DcResources.getText("lblManageStates"));  
        menuState.addActionListener(new ManageStatesAction(DcModules._AUDIOCD));
        
        JMenuItem menuTemplate = ComponentFactory.getMenuItem(IconLibrary._icoTemplate, DcResources.getText("lblManageTemplates"));  
        menuTemplate.addActionListener(new ManageTemplatesAction(DcModules._AUDIOCD));

        JMenuItem menuGenre = ComponentFactory.getMenuItem(IconLibrary._icoAnchor, DcResources.getText("lblManageGenres"));  
        menuGenre.addActionListener(new ManageGenresAction());

        menu.add(menuGenre);
        menu.add(menuState);
        menu.addSeparator();
        menu.add(menuTemplate);
        menu.addSeparator();

    	return menu;
    }
    
    public boolean isEnabled() {
        return DcSettings.getValueAsBoolean(Repository.settings.stAudioCDModuleEnabled);
    }
    
    public void isEnabled(boolean b) {
        DcSettings.setValue(Repository.settings.stAudioCDModuleEnabled, new Boolean(b));
    }

    public DcObject getDcObject() {
        return new AudioCD();
    }

    public DcCategoryModule getCategoryModule() {
        return null;
    }

    public DcGenreModule getGenreModule() {
        return (DcGenreModule) DcModules.get(DcModules._MUSICGENRE);
    } 
    
    public DcStorageMediaModule getStorageMediaModule() {
        return null;
    }
    
    public DcStateModule getStateModule() {
        return (DcStateModule) DcModules.get(DcModules._AUDIOCDSTATE);
    }

    public ITemplateModule getTemplateModule() {
        return (ITemplateModule) DcModules.get(DcModules._TEMPLATEAUDIOCD);
    }
    
    protected void initializeUI() {
        insertView = new MasterView();
        insertView.addView(MasterView._TABLE_VIEW, new ParentInsertTablePanel(DcModules._AUDIOCD));
        insertView.addView(MasterView._LIST_VIEW, new ListInsertView(this,
                DcResources.getText("lblNewItem", getObjectNamePlural()), IconLibrary._icoItemsNew));

        searchView = new MasterView();
        searchView.addView(MasterView._TABLE_VIEW, new ParentSearchTablePanel(DcModules._AUDIOCD, AudioTrack._F_TRACKNUMBER));
        searchView.addView(MasterView._LIST_VIEW, new ListSearchView(this, getObjectNamePlural(), getIconSmall()));
    }

    public String getFieldDefinitionsSettingsKey() {
        return Repository.settings.stFieldDefinitionsAudioCD;
    }

    public String getSearchOrderSettingsKey() {
        return Repository.settings.stAudioCDSearchOrder;
    }

    public void startImport() {}

    public QuickViewPanel getQuickView() {
        return new AudioCDQuickViewPane();
    }

    public ImportCDProcess getCDImportProcess(ImportCDDialog ui, DcStorageMedium storageMedium, String sourceDir, boolean confirmDirs, boolean recursiveDirs, String nameOfCD) {
        return null;
    } 

    protected void initializeFields() {
        super.initializeFields();

        fields.put(new Integer(DcObject._SYS_AVAILABLE), getField(DcObject._SYS_AVAILABLE));
        fields.put(new Integer(DcObject._SYS_LOANEDBY), getField(DcObject._SYS_LOANEDBY));
        fields.put(new Integer(DcObject._SYS_LOANDURATION), getField(DcObject._SYS_LOANDURATION));

        fields.put( new Integer(AudioCD._F_ARTIST),
                    new DcField(AudioCD._F_ARTIST, getIndex(), "Artist",
                                false, true, false, true, false,
                                4000, ComponentFactory._LONGTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "Artist"));
        fields.put( new Integer(AudioCD._G_GENRES),
                    new DcField(AudioCD._G_GENRES, getIndex(), "Genres",
                                true, true, true, true, false,
                                255, ComponentFactory._REFERENCESFIELD, DcModules._AUDIOCDGENREMAPPING, Repository.datatypes._DCOBJECTCOLLECTION,
                                "Genres"));  
        fields.put( new Integer(AudioCD._H_PICTUREFRONT),
                    new DcField(AudioCD._H_PICTUREFRONT, getIndex(), "Picture Front",
                                true, true, true, false, false,
                                4000, ComponentFactory._PICTUREFIELD, getIndex(), Repository.datatypes._PICTURE,
                                "PictureFront"));
        fields.put( new Integer(AudioCD._I_PICTUREBACK),
                    new DcField(AudioCD._I_PICTUREBACK, getIndex(), "Picture Back",
                                true, true, true, false, false,
                                4000, ComponentFactory._PICTUREFIELD, getIndex(), Repository.datatypes._PICTURE,
                                "PictureBack"));
        fields.put( new Integer(AudioCD._J_PICTURECD),
                    new DcField(AudioCD._J_PICTURECD, getIndex(), "Picture CD",
                                true, true, true, false, false,
                                4000, ComponentFactory._PICTUREFIELD, getIndex(), Repository.datatypes._PICTURE,
                                "PictureCD"));
        fields.put( new Integer(AudioCD._K_DISCID),
                    new DcField(AudioCD._K_DISCID, getIndex(), "Disc ID",
                                false, true, false, true, true,
                                255, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "DiscID"));
        fields.put( new Integer(AudioCD._L_STATE),
                    new DcField(AudioCD._L_STATE, getIndex(), "State",  
                                false, true, false, true, false, 
                                50, ComponentFactory._STATECOMBO, DcModules._AUDIOCDSTATE, Repository.datatypes._DCOBJECTREFERENCE,
                                "State"));
        fields.put( new Integer(AudioCD._M_WEBPAGE),
                    new DcField(AudioCD._M_WEBPAGE, getIndex(), "Webpage",
                                false, true, false, false, false,
                                255, ComponentFactory._URLFIELD, getIndex(), Repository.datatypes._STRING,
                                "Webpage"));   
        
    }

    public DcObject[] getDefaultData() {
        return new DcObject[0];
    }
}
