/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules.audiocd;

import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.KeyStroke;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.audiocd.AudioTrackForm;
import net.sf.dc.console.components.panels.ChildInsertTablePanel;
import net.sf.dc.console.components.panels.ChildSearchTablePanel;
import net.sf.dc.console.components.panels.QuickViewPanel;
import net.sf.dc.console.forms.DcSimpleItemView;
import net.sf.dc.console.forms.ItemForm;
import net.sf.dc.console.importcd.ImportCDDialog;
import net.sf.dc.console.views.MasterView;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DcCategoryModule;
import net.sf.dc.core.modules.DcGenreModule;
import net.sf.dc.core.modules.DcMediaModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.modules.DcStateModule;
import net.sf.dc.core.modules.DcStorageMediaModule;
import net.sf.dc.core.modules.IChildModule;
import net.sf.dc.core.modules.template.ITemplateModule;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.DcStorageMedium;
import net.sf.dc.core.objects.audiocd.AudioCD;
import net.sf.dc.core.objects.audiocd.AudioTrack;
import net.sf.dc.processes.cdimport.ImportCDProcess;
import net.sf.dc.resources.DcResources;

public class AudioTrackModule extends DcMediaModule implements IChildModule {

    public AudioTrackModule() {
        super(DcModules._AUDIOTRACK,
              false,
              DcResources.getText(Repository.systemlabels.sysAudioTrack),
              "audiotracks",
              "autr",
              "audiotracks",
              false,
              false,
              false,
              false);
    }

    public String getDescription() {
        return "";
    }
    
    public DcStateModule getStateModule() {
        return null;
    } 
    
    public KeyStroke getKeyStroke() {
        return null;
    }
    
    public String getObjectName() {
        return DcResources.getText(Repository.systemlabels.sysAudioTrack);
    }

    public String getObjectNamePlural() {
        return DcResources.getText(Repository.systemlabels.sysAudioTrackPlural);
    }

    public DcSimpleItemView getItemView(ItemForm form, DcObject parent, int module, boolean readonly) {
        return new AudioTrackForm(form, (AudioCD) parent, module, readonly);
    }
    
    public int getSortIdx() {
        return AudioTrack._F_TRACKNUMBER;
    }
    
    public ImageIcon getIcon() {
        return IconLibrary._icoAudioCDModule;
    }

    public DcGenreModule getGenreModule() {
        return (DcGenreModule) DcModules.get(DcModules._MUSICGENRE);
    } 
    
    public ImageIcon getIconSmall() {
        return IconLibrary._icoAudioCDModule16;
    }

    public JMenu getToolsMenu() {
        return null;
    }

    public boolean isEnabled() {
        return DcModules.get(DcModules._AUDIOCD).isEnabled();
    }
    
    public void isEnabled(boolean b){}    
    
    public DcObject getDcObject() {
        return new AudioTrack();
    }

    public void startImport() {
    }

    public QuickViewPanel getQuickView() {
        return null;
    }

    public DcCategoryModule getCategoryModule() {
        return null;
    }
    
    public ITemplateModule getTemplateModule() {
        return null;
    }       

    public DcStorageMediaModule getStorageMediaModule() {
        return null;
    }
    
    protected void initializeUI() {
        insertView = new MasterView();
        insertView.addView(MasterView._TABLE_VIEW, new ChildInsertTablePanel(DcModules._AUDIOTRACK));

        searchView = new MasterView();
        searchView.addView(MasterView._TABLE_VIEW, 
                           new ChildSearchTablePanel(DcModules._AUDIOTRACK, DcModules._AUDIOCD));
    }

    public String getFieldDefinitionsSettingsKey() {
        return Repository.settings.stFieldDefinitionsAudioTrack;
    }

    public String getSearchOrderSettingsKey() {
        return null;
    }

    public ImportCDProcess getCDImportProcess(ImportCDDialog ui, DcStorageMedium storageMedium, String sourceDir, boolean confirmDirs, boolean recursiveDirs, String nameOfCD) {
        return null;
    }     

    protected void initializeFields() {
        super.initializeFields();

        fields.put( new Integer(AudioTrack._F_TRACKNUMBER),
                    new DcField(AudioTrack._F_TRACKNUMBER, getIndex(), "Track",
                                false, true, false, false, false,
                                255, ComponentFactory._NUMBERFIELD, getIndex(), Repository.datatypes._INTEGER,
                                "Track"));
        fields.put( new Integer(AudioTrack._G_LYRICS),
                    new DcField(AudioTrack._G_LYRICS, getIndex(), "Lyric",
                                false, true, false, true, false,
                                255, ComponentFactory._LONGTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "Lyric"));
        fields.put( new Integer(AudioTrack._H_PLAYLENGTH),
                    new DcField(AudioTrack._H_PLAYLENGTH, getIndex(), "Playlength",
                                false, true, false, false, false,
                                255, ComponentFactory._TIMEFIELD, getIndex(), Repository.datatypes._INTEGER,
                                "PlayLength"));
        fields.put( new Integer(AudioTrack._I_ALBUM),
                    new DcField(AudioTrack._I_ALBUM, getIndex(), "Album",
                                false, true, true, false, false,
                                255, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._DCPARENTREFERENCE,
                                "Album"));
        fields.put( new Integer(AudioTrack._K_ARTIST),
                    new DcField(AudioTrack._K_ARTIST, getIndex(), "Artist",
                                false, true, false, true, false,
                                255, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "Artist"));    
        fields.put( new Integer(AudioTrack._J_GENRES),
                    new DcField(AudioTrack._J_GENRES, getIndex(), "Genres",
                                true, true, true, true, false,
                                255, ComponentFactory._REFERENCESFIELD, DcModules._AUDIOTRACKGENREMAPPING, Repository.datatypes._DCOBJECTCOLLECTION,
                                "Genres"));          
    }

    public DcObject[] getDefaultData() {
        return new DcObject[0];
    }
}
