/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules.book;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.Collection;

import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.book.BookQuickViewPane;
import net.sf.dc.console.components.panels.QuickViewPanel;
import net.sf.dc.console.forms.DcSimpleItemView;
import net.sf.dc.console.importcd.ImportCDDialog;
import net.sf.dc.console.views.MasterView;
import net.sf.dc.console.views.list.ListInsertView;
import net.sf.dc.console.views.list.ListSearchView;
import net.sf.dc.console.views.table.TableInsertView;
import net.sf.dc.console.views.table.TableSearchView;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.actions.ManageStatesAction;
import net.sf.dc.core.actions.ManageTemplatesAction;
import net.sf.dc.core.modules.DcCategoryModule;
import net.sf.dc.core.modules.DcGenreModule;
import net.sf.dc.core.modules.DcMediaModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.modules.DcStateModule;
import net.sf.dc.core.modules.DcStorageMediaModule;
import net.sf.dc.core.modules.template.ITemplateModule;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.DcStorageMedium;
import net.sf.dc.core.objects.book.Book;
import net.sf.dc.core.services.AsinSearchMode;
import net.sf.dc.core.services.DcAmazonServer;
import net.sf.dc.core.services.DcServer;
import net.sf.dc.core.services.KeywordSearchMode;
import net.sf.dc.processes.cdimport.ImportCDProcess;
import net.sf.dc.processes.synchronizer.BookSynchronizer;
import net.sf.dc.processes.synchronizer.Synchronizer;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;

public class BookModule extends DcMediaModule {

    public BookModule() {
        super(DcModules._BOOK,
              true,
              DcResources.getText(Repository.systemlabels.sysBook),
              "books",
              "boo",
              "books",
              true,
              false,
              true,
              false);
    }
    
    public BookModule(  int index, 
                        boolean topModule,
                        String name,
                        String tableName,
                        String tableShortName,
                        String tableJoin,
                        boolean canBeLended,
                        boolean allowImdbUpdate,
                        boolean allowAmazonUpdate,
                        boolean allowFreeDbUpdate) {
        
        super(index, topModule, name, tableName, tableShortName,
        tableJoin, canBeLended, allowImdbUpdate, allowAmazonUpdate, allowFreeDbUpdate);
    }

    public String getDescription() {
        return DcResources.getText("msgBookModuleDesc");
    }
    
    public ImageIcon getIcon() {
        return IconLibrary._icoBookModule;
    }
    
    public ImageIcon getIconSmall() {
        return IconLibrary._icoBookModule16;
    }

    public boolean isEnabled() {
        return DcSettings.getValueAsBoolean(Repository.settings.stBookModuleEnabled);
    }
    
    public Synchronizer getSynchronizer() {
        return new BookSynchronizer();
    }     
    
    public Collection getOnlineServices() {
        DcServer server = new DcAmazonServer();
        server.addMode(new KeywordSearchMode("Books", "Keywords"));
        server.addMode(new AsinSearchMode("Isbn / Asin"));
        
        Collection servers = new ArrayList();
        servers.add(server);
        return servers;
    }    
    
    public KeyStroke getKeyStroke() {
        return KeyStroke.getKeyStroke(KeyEvent.VK_5, InputEvent.CTRL_MASK);
    }    
    
    public void isEnabled(boolean b) {
        DcSettings.setValue(Repository.settings.stBookModuleEnabled, new Boolean(b));
    }    
    
    public String getObjectName() {
        return DcResources.getText(Repository.systemlabels.sysBook);
    }

    public String getObjectNamePlural() {
        return DcResources.getText(Repository.systemlabels.sysBookPlural);
    }

    public QuickViewPanel getQuickView() {
        return new BookQuickViewPane();
    }

    public DcCategoryModule getCategoryModule() {
        return (DcCategoryModule) DcModules.get(DcModules._BOOKCAT);
    }

    public DcGenreModule getGenreModule() {
        return null;
    } 
    
    public DcStorageMediaModule getStorageMediaModule() {
        return null;
    }    
    
    public ITemplateModule getTemplateModule() {
        return (ITemplateModule) DcModules.get(DcModules._TEMPLATEBOOK);
    }        
    
    public DcStateModule getStateModule() {
        return (DcStateModule) DcModules.get(DcModules._BOOKSTATE);
    }     

    public JMenu getToolsMenu() {
        JMenu menu = ComponentFactory.getMenu(DcResources.getText("lblTools"));

        JMenuItem menuCat = ComponentFactory.getMenuItem(IconLibrary._icoAnchor, DcResources.getText("lblManageCategories"));
        menuCat.addActionListener(new CategoryFormMenuAction());

        JMenuItem menuState = ComponentFactory.getMenuItem(IconLibrary._icoAnchor, DcResources.getText("lblManageStates"));  
        menuState.addActionListener(new ManageStatesAction(DcModules._BOOK));
        
        JMenuItem menuTemplate = ComponentFactory.getMenuItem(IconLibrary._icoTemplate, DcResources.getText("lblManageTemplates"));  
        menuTemplate.addActionListener(new ManageTemplatesAction(DcModules._BOOK));
        
        menu.add(menuState);
        menu.add(menuCat);
        menu.addSeparator();
        menu.add(menuTemplate);
        menu.addSeparator();
        
        return menu;
    }

    public DcObject getDcObject() {
        return new Book();
    }

    protected void initializeUI() {
        insertView = new MasterView();
        insertView.addView(MasterView._TABLE_VIEW, new TableInsertView(this, DcResources.getText("lblNewItem", getObjectNamePlural()), IconLibrary._icoItemsNew));
        insertView.addView(MasterView._LIST_VIEW, new ListInsertView(this, DcResources.getText("lblNewItem", getObjectNamePlural()), IconLibrary._icoItemsNew));

        searchView = new MasterView();
        searchView.addView(MasterView._TABLE_VIEW, new TableSearchView(this, getObjectNamePlural()));
        searchView.addView(MasterView._LIST_VIEW, new ListSearchView(this, getObjectNamePlural(), getIconSmall()));
    }

    public String getFieldDefinitionsSettingsKey() {
        return Repository.settings.stFieldDefinitionsBook;
    }

    public String getSearchOrderSettingsKey() {
        return Repository.settings.stBookSearchOrder;
    }

    public void startImport() {}

    public ImportCDProcess getCDImportProcess(ImportCDDialog ui, DcStorageMedium storageMedium, String sourceDir, boolean confirmDirs, boolean recursiveDirs, String nameOfCD) {
        return null;
    }  

    protected void initializeFields() {
        super.initializeFields();

        fields.put(new Integer(DcObject._SYS_AVAILABLE), getField(DcObject._SYS_AVAILABLE));
        fields.put(new Integer(DcObject._SYS_LOANEDBY), getField(DcObject._SYS_LOANEDBY));
        fields.put(new Integer(DcObject._SYS_LOANDURATION), getField(DcObject._SYS_LOANDURATION));

        fields.put( new Integer(Book._F_PUBLISHER),
                    new DcField(Book._F_PUBLISHER, getIndex(), "Publisher",
                                false, true, false, true, false,
                                255, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "Publisher"));
        fields.put( new Integer(Book._G_AUTHOR),
                    new DcField(Book._G_AUTHOR, getIndex(), "Author",
                                false, true, false, true, false,
                                255, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "Author"));
        fields.put( new Integer(Book._H_WEBPAGE),
                    new DcField(Book._H_WEBPAGE, getIndex(), "Webpage",
                                false, true, false, false, false,
                                255, ComponentFactory._URLFIELD, getIndex(), Repository.datatypes._STRING,
                                "Webpage"));
        fields.put( new Integer(Book._I_CATEGORY),
                    new DcField(Book._I_CATEGORY, getIndex(), "Category",
                                false, true, false, true, false,
                                255, ComponentFactory._BOOKCATEGORYCOMBO, DcModules._BOOKCAT, Repository.datatypes._DCOBJECTREFERENCE,
                                "Type"));
        fields.put( new Integer(Book._J_ISBN10),
                    new DcField(Book._J_ISBN10, getIndex(), "Isbn 10",
                                false, true, false, true, false,
                                20, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "ISBN"));
        fields.put( new Integer(Book._K_PICTUREFRONT),
                    new DcField(Book._K_PICTUREFRONT, getIndex(), "Picture Front",
                                true, true, false, false, false,
                                4000, ComponentFactory._PICTUREFIELD, getIndex(), Repository.datatypes._PICTURE,
                                "PictureFront"));
        fields.put( new Integer(Book._L_STATE),
                    new DcField(Book._L_STATE, getIndex(), "State",  
                                false, true, false, true, false, 
                                50, ComponentFactory._STATECOMBO, DcModules._BOOKSTATE, Repository.datatypes._DCOBJECTREFERENCE,
                                "State"));
        fields.put( new Integer(Book._M_FILENAME),
                    new DcField(Book._M_FILENAME, getIndex(), "Filename",
                                false, true, false, true, false,
                                255, ComponentFactory._FILELAUNCHFIELD, getIndex(), Repository.datatypes._STRING,
                                "Filename"));
        fields.put( new Integer(Book._N_ISBN13),
                    new DcField(Book._N_ISBN13, getIndex(), "Isbn 13",
                                false, true, false, true, false,
                                20, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "ISBN13"));
        fields.put( new Integer(Book._O_SERIES),
                    new DcField(Book._O_SERIES, getIndex(), "Series",
                                false, true, false, true, false,
                                255, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "Series"));
        fields.put( new Integer(Book._P_VOLUME_NR),
                    new DcField(Book._P_VOLUME_NR, getIndex(), "Volume (nr)",
                                false, true, false, true, false,
                                4, ComponentFactory._NUMBERFIELD, getIndex(), Repository.datatypes._INTEGER,
                                "VolumeNumber"));        
        fields.put( new Integer(Book._Q_VOLUME_TITLE),
                    new DcField(Book._Q_VOLUME_TITLE, getIndex(), "Volume name",
                                false, true, false, true, false,
                                255, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "VolumeName"));          
    }

    public DcObject[] getDefaultData() {
        return new DcObject[0];
    }

    private class CategoryFormMenuAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            BookCategoryModule bookCatMod =
                (BookCategoryModule) DcModules.get(DcModules._BOOKCAT);
            DcSimpleItemView view = bookCatMod.getForm();
            view.setVisible(true);
        }
    }
}
