/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules.contactperson;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;

import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.components.panels.QuickViewPanel;
import net.sf.dc.console.contactperson.ContactPersonQuickViewPane;
import net.sf.dc.console.forms.DcSimpleItemView;
import net.sf.dc.console.importcd.ImportCDDialog;
import net.sf.dc.console.views.MasterView;
import net.sf.dc.console.views.list.ListInsertView;
import net.sf.dc.console.views.list.ListSearchView;
import net.sf.dc.console.views.table.TableInsertView;
import net.sf.dc.console.views.table.TableSearchView;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DcCategoryModule;
import net.sf.dc.core.modules.DcGenreModule;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.modules.DcStateModule;
import net.sf.dc.core.modules.DcStorageMediaModule;
import net.sf.dc.core.modules.template.ITemplateModule;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.DcStorageMedium;
import net.sf.dc.core.objects.contactperson.ContactPerson;
import net.sf.dc.processes.cdimport.ImportCDProcess;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;

public class ContactPersonModule extends DcModule {

    public ContactPersonModule() {
        super(DcModules._CONTACTPERSON,
              true,
              DcResources.getText(Repository.systemlabels.sysContactPerson),
              "persons",
              "per",
              "persons",
              false,
              false,
              false);
    }

    public String getDescription() {
        return DcResources.getText("msgContactPersonModuleDesc");
    }
    
    public ImageIcon getIcon() {
        return IconLibrary._icoPersonModule;
    }
    
    public ImageIcon getIconSmall() {
        return IconLibrary._icoPersonModule16;
    }

    public boolean isEnabled() {
        return DcSettings.getValueAsBoolean(Repository.settings.stContactPersonModuleEnabled);
    }
    
    public void isEnabled(boolean b) {
        DcSettings.setValue(Repository.settings.stContactPersonModuleEnabled, new Boolean(b));
    }     
    
    public KeyStroke getKeyStroke() {
        return KeyStroke.getKeyStroke(KeyEvent.VK_7, InputEvent.CTRL_MASK);
    }    
    
    public String getObjectName() {
        return DcResources.getText(Repository.systemlabels.sysContactPerson);
    }

    public String getObjectNamePlural() {
        return DcResources.getText(Repository.systemlabels.sysContactPersonPlural);
    }

    public DcCategoryModule getCategoryModule() {
        return (DcCategoryModule) DcModules.get(DcModules._CONTACTPERSONCATEGORY);
    }

    public String getGroupedBySettingsKey() {
        return Repository.settings.stGroupedBy + getName().replace(" ", "");
    }    
    
    public DcStateModule getStateModule() {
        return null;
    }     
    
    public DcStorageMediaModule getStorageMediaModule() {
        return null;
    }    

    public ITemplateModule getTemplateModule() {
        return null;
    }   

    public DcGenreModule getGenreModule() {
        return null;
    } 
        
    public JMenu getToolsMenu() {
        JMenu menu = ComponentFactory.getMenu(DcResources.getText("lblTools"));
        JMenuItem menuItem = ComponentFactory.getMenuItem(IconLibrary._icoAnchor, DcResources.getText("lblManageCategories"));
        menuItem.addActionListener(new PersonTypeFormMenuAction());
        menu.add(menuItem);
        return menu;
    }

    public DcObject getDcObject() {
        return new ContactPerson();
    }

    public QuickViewPanel getQuickView() {
        return new ContactPersonQuickViewPane();
    }

    protected void initializeUI() {
        insertView = new MasterView();
        insertView.addView(MasterView._TABLE_VIEW, new TableInsertView(this, DcResources.getText("lblNewItem", getObjectNamePlural()), IconLibrary._icoItemsNew));
        insertView.addView(MasterView._LIST_VIEW, new ListInsertView(this, DcResources.getText("lblNewItem", getObjectNamePlural()), IconLibrary._icoItemsNew));

        searchView = new MasterView();
        searchView.addView(MasterView._TABLE_VIEW, new TableSearchView(this, getObjectNamePlural()));
        searchView.addView(MasterView._LIST_VIEW, new ListSearchView(this, getObjectNamePlural(), getIconSmall()));
    }

    public String getFieldDefinitionsSettingsKey() {
        return Repository.settings.stFieldDefinitionsPerson;
    }

    public String getSearchOrderSettingsKey() {
        return Repository.settings.stPersonSearchOrder;
    }

    public void startImport() {}
    
    public ImportCDProcess getCDImportProcess(ImportCDDialog ui, DcStorageMedium storageMedium, String sourceDir, boolean confirmDirs, boolean recursiveDirs, String nameOfCD) {
        return null;
    }  

    protected void initializeFields() {
        super.initializeFields();

        fields.put( new Integer(ContactPerson._A_NAME),
                    new DcField(ContactPerson._A_NAME, getIndex(), "Name",
                            false, true, false, true, false,
                            255, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                            "Name"));
        fields.put( new Integer(ContactPerson._B_DESCRIPTION),
                    new DcField(ContactPerson._B_DESCRIPTION, getIndex(), "Description",
                            false, true, false, true, false,
                            4000, ComponentFactory._LONGTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                            "Description"));
        fields.put( new Integer(ContactPerson._C_PHOTO),
                    new DcField(ContactPerson._C_PHOTO, getIndex(), "Photo",
                            false, true, true, false, false,
                            4000, ComponentFactory._PICTUREFIELD, getIndex(), Repository.datatypes._PICTURE,
                            "Photo"));
        fields.put( new Integer(ContactPerson._D_CATEGORY),
                    new DcField(ContactPerson._D_CATEGORY, getIndex(), "Category",
                            false, true, false, true, false,
                            50, ComponentFactory._CONTACTPERSONCATEGORYCOMBO, DcModules._CONTACTPERSONCATEGORY, Repository.datatypes._DCOBJECTREFERENCE,
                            "PersonType"));
        fields.put( new Integer(ContactPerson._E_EMAIL),
                    new DcField(ContactPerson._E_EMAIL, getIndex(), "Email",
                            false, true, false, true, false,
                            255, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                            "Email"));
        fields.put( new Integer(ContactPerson._F_ADDRESS),
                    new DcField(ContactPerson._F_ADDRESS, getIndex(), "Address",
                            false, true, false, true, false,
                            4000, ComponentFactory._LONGTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                            "Address"));
        fields.put( new Integer(ContactPerson._G_PHONE_HOME),
                    new DcField(ContactPerson._G_PHONE_HOME, getIndex(), "Phone (Home)",
                            false, true, false, true, false,
                            50, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                            "PhoneHome"));
        fields.put( new Integer(ContactPerson._H_PHONE_WORK),
                    new DcField(ContactPerson._H_PHONE_WORK, getIndex(), "Phone (Work)",
                            false, true, false, true, false,
                            50, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                            "PhoneWork"));
    }

    public DcObject[] getDefaultData() {
        return new DcObject[0];
    }

    private class PersonTypeFormMenuAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            ContactPersonCategoryModule personTypeMod = (ContactPersonCategoryModule) DcModules.get(DcModules._CONTACTPERSONCATEGORY);
            DcSimpleItemView view = personTypeMod.getForm();
            view.setVisible(true);
        }
    }
}
