/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules.image;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;

import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.components.panels.QuickViewPanel;
import net.sf.dc.console.forms.DcSimpleItemView;
import net.sf.dc.console.image.ImageQuickViewPane;
import net.sf.dc.console.importcd.ImportCDDialog;
import net.sf.dc.console.views.MasterView;
import net.sf.dc.console.views.list.ListInsertView;
import net.sf.dc.console.views.list.ListSearchView;
import net.sf.dc.console.views.table.TableInsertView;
import net.sf.dc.console.views.table.TableSearchView;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.actions.ManageStatesAction;
import net.sf.dc.core.actions.ManageStorageMediaAction;
import net.sf.dc.core.actions.ManageTemplatesAction;
import net.sf.dc.core.modules.DcCategoryModule;
import net.sf.dc.core.modules.DcGenreModule;
import net.sf.dc.core.modules.DcMediaModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.modules.DcStateModule;
import net.sf.dc.core.modules.DcStorageMediaModule;
import net.sf.dc.core.modules.template.ITemplateModule;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.DcStorageMedium;
import net.sf.dc.core.objects.image.Image;
import net.sf.dc.processes.cdimport.ImportCDProcess;
import net.sf.dc.processes.cdimport.ImportImageCD;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;

public class ImageModule extends DcMediaModule {

    public ImageModule() {
        super(DcModules._IMAGE,
              true,
              DcResources.getText(Repository.systemlabels.sysImage),
              "images",
              "img",
              "images",
              false,
              false,
              false,
              false);
    }

    public ImageModule(     int index, 
                            boolean topModule,
                            String name,
                            String tableName,
                            String tableShortName,
                            String tableJoin,
                            boolean canBeLended,
                            boolean allowImdbUpdate,
                            boolean allowAmazonUpdate,
                            boolean allowFreeDbUpdate) {

        super(index, topModule, name, tableName, tableShortName,
                tableJoin, canBeLended, allowImdbUpdate, allowAmazonUpdate, allowFreeDbUpdate);
    }    
    
    public String getDescription() {
        return DcResources.getText("msgImageModuleDesc");
    }    
    
    public ImageIcon getIcon() {
        return IconLibrary._icoImageModule;
    }

    public ImageIcon getIconSmall() {
        return IconLibrary._icoImageModule16;
    }        
    
    public KeyStroke getKeyStroke() {
        return KeyStroke.getKeyStroke(KeyEvent.VK_6, InputEvent.CTRL_MASK);
    }
    
    public String getObjectName() {
        return DcResources.getText(Repository.systemlabels.sysImage);
    }

    public String getObjectNamePlural() {
        return DcResources.getText(Repository.systemlabels.sysImagePlural);
    }

    public boolean isEnabled() {
        return DcSettings.getValueAsBoolean(Repository.settings.stImageModuleEnabled);
    }
    
    public void isEnabled(boolean b) {
        DcSettings.setValue(Repository.settings.stImageModuleEnabled, new Boolean(b));
    }      
    
    public DcCategoryModule getCategoryModule() {
        return (DcCategoryModule) DcModules.get(DcModules._IMAGECAT);
    }
    
    public ITemplateModule getTemplateModule() {
        return (ITemplateModule) DcModules.get(DcModules._TEMPLATEIMAGE);
    }       

    public DcGenreModule getGenreModule() {
        return null;
    } 
    
    public DcStorageMediaModule getStorageMediaModule() {
        return (DcStorageMediaModule) DcModules.get(DcModules._IMAGESTORAGE);
    }    
    
    public DcStateModule getStateModule() {
        return (DcStateModule) DcModules.get(DcModules._IMAGESTATE);
    } 

    public QuickViewPanel getQuickView() {
        return new ImageQuickViewPane();
    }

    public void startImport() {
        ImportCDDialog importCDDialog = new ImportCDDialog(getIndex(), DataCrow.mainFrame, DcResources.getText("lblImageImport"));
        importCDDialog.setVisible(true);
    }

    public JMenu getToolsMenu() {
        JMenu menu = ComponentFactory.getMenu(DcResources.getText("lblTools"));

        JMenuItem menuCat = ComponentFactory.getMenuItem(IconLibrary._icoAnchor, DcResources.getText("lblManageCategories"));
        menuCat.addActionListener(new CatagoryFormMenuAction());

        JMenuItem menuState = ComponentFactory.getMenuItem(IconLibrary._icoAnchor, DcResources.getText("lblManageStates"));  
        menuState.addActionListener(new ManageStatesAction(DcModules._IMAGE));

        JMenuItem menuStorageMedia = ComponentFactory.getMenuItem(IconLibrary._icoSave, DcResources.getText("lblManageStorageMedia"));  
        menuStorageMedia.addActionListener(new ManageStorageMediaAction());

        JMenuItem menuTemplate = ComponentFactory.getMenuItem(IconLibrary._icoTemplate, DcResources.getText("lblManageTemplates"));  
        menuTemplate.addActionListener(new ManageTemplatesAction(DcModules._IMAGE));
        
        JMenuItem menuImport = ComponentFactory.getMenuItem(IconLibrary._icoImport, DcResources.getText("lblImageImport"));
        menuImport.addActionListener(new ImportCDAction());
        menuImport.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_I, InputEvent.CTRL_MASK));

        menu.add(menuCat);
        menu.add(menuState);
        menu.add(menuStorageMedia);
        menu.addSeparator();
        menu.add(menuTemplate);
        menu.addSeparator();
        menu.add(menuImport);

        return menu;
    }

    protected void initializeUI() {
        insertView = new MasterView();
        insertView.addView(MasterView._TABLE_VIEW, new TableInsertView(this, DcResources.getText("lblNewItem", getObjectNamePlural()), IconLibrary._icoItemsNew));
        insertView.addView(MasterView._LIST_VIEW, new ListInsertView(this, DcResources.getText("lblNewItem", getObjectNamePlural()), IconLibrary._icoItemsNew));

        searchView = new MasterView();
        searchView.addView(MasterView._TABLE_VIEW, new TableSearchView(this, getObjectNamePlural()));
        searchView.addView(MasterView._LIST_VIEW, new ListSearchView(this, getObjectNamePlural(), getIconSmall()));
    }

    public String getFieldDefinitionsSettingsKey() {
        return Repository.settings.stFieldDefinitionsImage;
    }

    public String getSearchOrderSettingsKey() {
        return Repository.settings.stImageSearchOrder;
    }

    public DcObject getDcObject() {
        return new Image();
    }

    public ImportCDProcess getCDImportProcess(ImportCDDialog ui,
                                              DcStorageMedium storageMedium,
                                              String sourceDir,
                                              boolean confirmDirs,
                                              boolean recursiveDirs,
                                              String nameOfCD) {
        return new ImportImageCD(ui, storageMedium, sourceDir, confirmDirs, recursiveDirs, nameOfCD);
    }

    protected void initializeFields() {
        super.initializeFields();

        fields.put( new Integer(Image._F_WIDTH),
                    new DcField(Image._F_WIDTH, getIndex(), "Width",
                                false, true, false, true, true,
                                10, ComponentFactory._NUMBERFIELD, getIndex(), Repository.datatypes._INTEGER,
                                "Width"));
        fields.put( new Integer(Image._G_HEIGHT),
                    new DcField(Image._G_HEIGHT, getIndex(), "Height",
                                false, true, false, true, true,
                                255, ComponentFactory._NUMBERFIELD, getIndex(), Repository.datatypes._INTEGER,
                                "Height"));
        fields.put( new Integer(Image._H_FILENAME),
                    new DcField(Image._H_FILENAME, getIndex(), "Filename",
                                false, true, false, true, false,
                                255, ComponentFactory._FILELAUNCHFIELD, getIndex(), Repository.datatypes._STRING,
                                "Filename"));
        fields.put( new Integer(Image._I_IMAGE),
                    new DcField(Image._I_IMAGE, getIndex(), "Image",
                                true, true, false, false, false,
                                4000, ComponentFactory._PICTUREFIELD, getIndex(), Repository.datatypes._PICTURE,
                                "Image"));
        fields.put( new Integer(Image._J_STATE),
                    new DcField(Image._J_STATE, getIndex(), "State",  
                                false, true, false, true, false, 
                                50, ComponentFactory._STATECOMBO, DcModules._IMAGESTATE, Repository.datatypes._DCOBJECTREFERENCE,
                                "State"));
        fields.put( new Integer(Image._L_STORAGEMEDIUM),
                    new DcField(Image._L_STORAGEMEDIUM, getIndex(), "Storage Medium",  
                                false, true, false, true, false, 
                                50, ComponentFactory._MEDIACOMBO, DcModules._IMAGESTORAGE, Repository.datatypes._DCOBJECTREFERENCE,
                                "StorageMedium"));       
        fields.put( new Integer(Image._K_CATEGORY),
                    new DcField(Image._K_CATEGORY, getIndex(), "Category",
                                false, true, false, true, false,
                                255, ComponentFactory._IMAGECATEGORYCOMBO, DcModules._IMAGECAT, Repository.datatypes._DCOBJECTREFERENCE,
                                "Category"));    
        fields.put( new Integer(Image._M_FILESIZE),
                    new DcField(Image._M_FILESIZE, getIndex(), "Filesize",
                                false, true, false, true, true,
                                255, ComponentFactory._NUMBERFIELD, getIndex(), Repository.datatypes._INTEGER,
                                "Filesize"));    
        fields.put( new Integer(Image._N_DATE),
                    new DcField(Image._N_DATE, getIndex(), "Date",
                                false, true, false, false, false,
                                255, ComponentFactory._DATEFIELD, getIndex(), Repository.datatypes._DATE,
                                "ImageDate"));         
        fields.put( new Integer(Image._O_COMPRESSION),
                    new DcField(Image._O_COMPRESSION, getIndex(), "Compression (Bits)",
                                false, true, false, false, true,
                                5, ComponentFactory._NUMBERFIELD, getIndex(), Repository.datatypes._INTEGER,
                                "Compression"));         
        fields.put( new Integer(Image._P_PLACE),
                    new DcField(Image._P_PLACE, getIndex(), "Place",
                                false, true, false, false, false,
                                500, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "Place"));         
        fields.put( new Integer(Image._Q_CAMERA),
                    new DcField(Image._Q_CAMERA, getIndex(), "Camera",
                                false, true, false, false, true,
                                255, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "Camera"));       
    }

    public DcObject[] getDefaultData() throws Exception {
        return new DcObject[0];
    }

    private class CatagoryFormMenuAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            ImageCategoryModule mod =
                (ImageCategoryModule) DcModules.get(DcModules._IMAGECAT);
            DcSimpleItemView view = mod.getForm();
            view.setVisible(true);
        }
    }

    private class ImportCDAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startImport();
        }
    }
}
