/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules.movie;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.Collection;

import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.components.panels.QuickViewPanel;
import net.sf.dc.console.importcd.ImportCDDialog;
import net.sf.dc.console.movie.MovieQuickViewPane;
import net.sf.dc.console.views.MasterView;
import net.sf.dc.console.views.list.ListInsertView;
import net.sf.dc.console.views.list.ListSearchView;
import net.sf.dc.console.views.table.TableInsertView;
import net.sf.dc.console.views.table.TableSearchView;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.actions.ManageGenresAction;
import net.sf.dc.core.actions.ManageStatesAction;
import net.sf.dc.core.actions.ManageStorageMediaAction;
import net.sf.dc.core.actions.ManageTemplatesAction;
import net.sf.dc.core.modules.DcCategoryModule;
import net.sf.dc.core.modules.DcGenreModule;
import net.sf.dc.core.modules.DcMediaModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.modules.DcStateModule;
import net.sf.dc.core.modules.DcStorageMediaModule;
import net.sf.dc.core.modules.template.ITemplateModule;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.DcStorageMedium;
import net.sf.dc.core.objects.movie.Movie;
import net.sf.dc.core.services.AsinSearchMode;
import net.sf.dc.core.services.BarcodeSearchMode;
import net.sf.dc.core.services.DcAmazonServer;
import net.sf.dc.core.services.DcCombinedAmazonImdbServer;
import net.sf.dc.core.services.DcImdbServer;
import net.sf.dc.core.services.DcServer;
import net.sf.dc.core.services.DcWikipediaServer;
import net.sf.dc.core.services.KeywordSearchMode;
import net.sf.dc.processes.cdimport.ImportCDProcess;
import net.sf.dc.processes.cdimport.ImportMovieCD;
import net.sf.dc.processes.synchronizer.MovieSynchronizer;
import net.sf.dc.processes.synchronizer.Synchronizer;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;

public class MovieModule extends DcMediaModule {

    public MovieModule() {
        super(DcModules._MOVIE,
              true,
              DcResources.getText(Repository.systemlabels.sysMovie),
              "movies",
              "mov",
              "movies",
              true,
              true,
              false,
              false);
    }

    public MovieModule(     int index, 
                            boolean topModule,
                            String name,
                            String tableName,
                            String tableShortName,
                            String tableJoin,
                            boolean canBeLended,
                            boolean allowImdbUpdate,
                            boolean allowAmazonUpdate,
                            boolean allowFreeDbUpdate) {

        super(index, topModule, name, tableName, tableShortName,
        tableJoin, canBeLended, allowImdbUpdate, allowAmazonUpdate, allowFreeDbUpdate);
    }     
    
    public Synchronizer getSynchronizer() {
        return new MovieSynchronizer();
    }     
    
    public String getDescription() {
        return DcResources.getText("msgMovieModuleDesc");
    }
    
    public ImageIcon getIcon() {
        return IconLibrary._icoMovieModule;
    }
    
    public ImageIcon getIconSmall() {
        return IconLibrary._icoMovieModule16;
    }    

    public String getObjectName() {
        return DcResources.getText(Repository.systemlabels.sysMovie);
    }

    public String getObjectNamePlural() {
        return DcResources.getText(Repository.systemlabels.sysMoviePlural);
    }

    public Collection getOnlineServices() {
        DcServer amazonServer = new DcAmazonServer();
        amazonServer.addMode(new KeywordSearchMode("DVD", "DVD"));
        amazonServer.addMode(new KeywordSearchMode("VHS", "VHS"));
        amazonServer.addMode(new BarcodeSearchMode("Video", "Barcode"));
        amazonServer.addMode(new AsinSearchMode("Asin"));
        
        DcCombinedAmazonImdbServer combinedServer = new DcCombinedAmazonImdbServer();
        combinedServer.addMode(new KeywordSearchMode("DVD", "DVD"));
        combinedServer.addMode(new KeywordSearchMode("VHS", "VHS"));
        combinedServer.addMode(new BarcodeSearchMode("Video", "Barcode"));
        combinedServer.addMode(new AsinSearchMode("Asin"));
        
        DcServer imdbServer = new DcImdbServer();
        
        DcServer wikiServer = new DcWikipediaServer();
        
        Collection servers = new ArrayList();
        servers.add(combinedServer);
        servers.add(amazonServer);
        servers.add(imdbServer);
        servers.add(wikiServer);
        
        return servers;
    }    
    
    public boolean isEnabled() {
        return DcSettings.getValueAsBoolean(Repository.settings.stMovieModuleEnabled);
    }
    
    public KeyStroke getKeyStroke() {
        return KeyStroke.getKeyStroke(KeyEvent.VK_3, InputEvent.CTRL_MASK);
    }    
    
    public void isEnabled(boolean b) {
        DcSettings.setValue(Repository.settings.stMovieModuleEnabled, new Boolean(b));
    }       
    
    public DcCategoryModule getCategoryModule() {
        return null;
    }
    
    public DcStorageMediaModule getStorageMediaModule() {
        return (DcStorageMediaModule) DcModules.get(DcModules._MOVIESTORAGE);
    }    
    
    public DcStateModule getStateModule() {
        return (DcStateModule) DcModules.get(DcModules._MOVIESTATE);
    }     

    public DcGenreModule getGenreModule() {
        return (DcGenreModule) DcModules.get(DcModules._MOVIEGENRE);
    } 
    
    public ITemplateModule getTemplateModule() {
        return (ITemplateModule) DcModules.get(DcModules._TEMPLATEMOVIE);
    }     

    public JMenu getToolsMenu() {
        JMenu menu = ComponentFactory.getMenu(DcResources.getText("lblTools"));
        JMenuItem menuImport = ComponentFactory.getMenuItem(IconLibrary._icoImport, DcResources.getText("lblMovieFileImport"));
        menuImport.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_I, InputEvent.CTRL_MASK));
        menuImport.addActionListener(new ImportCDAction());

        JMenuItem menuState = ComponentFactory.getMenuItem(IconLibrary._icoAnchor, DcResources.getText("lblManageStates"));  
        menuState.addActionListener(new ManageStatesAction(DcModules._MOVIE));
        
        JMenuItem menuStorageMedia = ComponentFactory.getMenuItem(IconLibrary._icoSave, DcResources.getText("lblManageStorageMedia"));  
        menuStorageMedia.addActionListener(new ManageStorageMediaAction());
        
        JMenuItem menuTemplate = ComponentFactory.getMenuItem(IconLibrary._icoTemplate, DcResources.getText("lblManageTemplates"));  
        menuTemplate.addActionListener(new ManageTemplatesAction(DcModules._MOVIE));
        
        JMenuItem menuGenre = ComponentFactory.getMenuItem(IconLibrary._icoAnchor, DcResources.getText("lblManageGenres"));  
        menuGenre.addActionListener(new ManageGenresAction());
        
        menu.add(menuGenre);
        menu.add(menuState);
        menu.add(menuStorageMedia);
        menu.addSeparator();
        menu.add(menuTemplate);
        menu.addSeparator();
        menu.add(menuImport);

        return menu;
    }

    public DcObject getDcObject() {
        return new Movie();
    }

    public QuickViewPanel getQuickView() {
        return new MovieQuickViewPane();
    }

    protected void initializeUI() {
        insertView = new MasterView();
        insertView.addView(MasterView._TABLE_VIEW, new TableInsertView(this, DcResources.getText("lblNewItem", getObjectNamePlural()), IconLibrary._icoItemsNew));
        insertView.addView(MasterView._LIST_VIEW, new ListInsertView(this, DcResources.getText("lblNewItem", getObjectNamePlural()), IconLibrary._icoItemsNew));

        searchView = new MasterView();
        searchView.addView(MasterView._TABLE_VIEW, new TableSearchView(this, getObjectNamePlural()));
        searchView.addView(MasterView._LIST_VIEW, new ListSearchView(this, getObjectNamePlural(), getIconSmall()));
    }

    public String getFieldDefinitionsSettingsKey() {
        return Repository.settings.stFieldDefinitionsMovie;
    }

    public String getSearchOrderSettingsKey() {
        return Repository.settings.stMovieSearchOrder;
    }

    public void startImport() {
        ImportCDDialog importCDDialog= new ImportCDDialog(
                                            getIndex(),
                                            DataCrow.mainFrame,
                                            DcResources.getText("lblMovieFileImport"));
        importCDDialog.setVisible(true);
    }

    public ImportCDProcess getCDImportProcess(ImportCDDialog ui,
                                              DcStorageMedium storageMedium,
                                              String sourceDir,
                                              boolean confirmDirs,
                                              boolean recursiveDirs,
							     			  String nameOfCD) {

        return new ImportMovieCD(ui, storageMedium, sourceDir, confirmDirs, recursiveDirs, nameOfCD);
    }

    protected void initializeFields() {
        super.initializeFields();

        fields.put(new Integer(DcObject._SYS_AVAILABLE), getField(DcObject._SYS_AVAILABLE));
        fields.put(new Integer(DcObject._SYS_LOANEDBY), getField(DcObject._SYS_LOANEDBY));
        fields.put(new Integer(DcObject._SYS_LOANDURATION), getField(DcObject._SYS_LOANDURATION));

        fields.put( new Integer(Movie._F_TITLE_LOCAL),
                    new DcField(Movie._F_TITLE_LOCAL, getIndex(), "Title (local)",
                                false, true, false, true, false,
                                255, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "Title2"));
        fields.put( new Integer(Movie._G_WEBPAGE),
                    new DcField(Movie._G_WEBPAGE, getIndex(), "Webpage",
                                false, true, false, false, false,
                                255, ComponentFactory._URLFIELD, getIndex(), Repository.datatypes._STRING,
                                "Webpage"));
        fields.put( new Integer(Movie._H_GENRES),
                    new DcField(Movie._H_GENRES, getIndex(), "Genres",
                                true, true, true, true, false,
                                255, ComponentFactory._REFERENCESFIELD, DcModules._MOVIEGENREMAPPING, Repository.datatypes._DCOBJECTCOLLECTION,
                                "Genres"));  
        fields.put( new Integer(Movie._I_ACTORS),
                    new DcField(Movie._I_ACTORS, getIndex(), "Actors",
                                false, true, false, true, false,
                                255, ComponentFactory._LONGTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "Actors"));
        fields.put( new Integer(Movie._J_DIRECTOR),
                    new DcField(Movie._J_DIRECTOR, getIndex(), "Director",
                                false, true, false, true, false,
                                255, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "Director"));
        fields.put( new Integer(Movie._K_COUNTRY),
                    new DcField(Movie._K_COUNTRY, getIndex(), "Country",
                                false, true, false, true, false,
                                255, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "Country"));

        fields.put( new Integer(Movie._L_PLAYLENGTH),
                    new DcField(Movie._L_PLAYLENGTH, getIndex(), "Playlength",
                                false, true, false, false, false,
                                10, ComponentFactory._TIMEFIELD, getIndex(), Repository.datatypes._INTEGER,
                                "PlayLength"));
        fields.put( new Integer(Movie._M_FILENAME),
                    new DcField(Movie._M_FILENAME, getIndex(), "Filename",
                                false, true, false, true, false,
                                255, ComponentFactory._FILELAUNCHFIELD, getIndex(), Repository.datatypes._STRING,
                                "Filename"));
        fields.put( new Integer(Movie._N_VIDEOCODEC),
                    new DcField(Movie._N_VIDEOCODEC, getIndex(), "Video Codec",
                                false, true, false, true, true,
                                255, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "VideoCodec"));
        fields.put( new Integer(Movie._O_AUDIOCODEC),
                    new DcField(Movie._O_AUDIOCODEC, getIndex(), "Audio Codec",
                                false, true, false, true, true,
                                255, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "AudioCodec"));
        fields.put( new Integer(Movie._P_WIDTH),
                    new DcField(Movie._P_WIDTH, getIndex(), "Width",
                                false, true, false, true, true,
                                10, ComponentFactory._NUMBERFIELD, getIndex(), Repository.datatypes._INTEGER,
                                "Width"));
        fields.put( new Integer(Movie._Q_HEIGHT),
                    new DcField(Movie._Q_HEIGHT, getIndex(), "Height",
                                false, true, false, true, true,
                                10, ComponentFactory._NUMBERFIELD, getIndex(), Repository.datatypes._INTEGER,
                                "Height"));
        fields.put( new Integer(Movie._R_FPS),
                    new DcField(Movie._R_FPS, getIndex(), "Fps",
                                false, true, false, true, true,
                                10, ComponentFactory._NUMBERFIELD, getIndex(), Repository.datatypes._INTEGER,
                                "Fps"));
        fields.put( new Integer(Movie._S_FRAMES),
                    new DcField(Movie._S_FRAMES, getIndex(), "Frames",
                                false, true, false, true, true,
                                10, ComponentFactory._NUMBERFIELD, getIndex(), Repository.datatypes._INTEGER,
                                "Frames"));
        fields.put( new Integer(Movie._T_AUDIOBITRATE),
                    new DcField(Movie._T_AUDIOBITRATE, getIndex(), "Audio Bitrate",
                                false, true, false, true, true,
                                10, ComponentFactory._NUMBERFIELD, getIndex(), Repository.datatypes._INTEGER,
                                "AudioBitRate"));
        fields.put( new Integer(Movie._U_AUDIOSAMPLINGRATE),
                    new DcField(Movie._U_AUDIOSAMPLINGRATE, getIndex(), "Audio Sample Rate",
                                false, true, false, true, true,
                                10, ComponentFactory._NUMBERFIELD, getIndex(), Repository.datatypes._INTEGER,
                                "AudioSampleRate"));
        fields.put( new Integer(Movie._V_AUDIOCHANNEL),
                    new DcField(Movie._V_AUDIOCHANNEL, getIndex(), "Audio Channels",
                                false, true, false, true, true,
                                10, ComponentFactory._NUMBERFIELD, getIndex(), Repository.datatypes._INTEGER,
                                "AudioChannels"));
        fields.put( new Integer(Movie._W_VIDEOBITRATE),
                    new DcField(Movie._W_VIDEOBITRATE, getIndex(), "Video Bitrate",
                                false, true, false, true, true,
                                10, ComponentFactory._NUMBERFIELD, getIndex(), Repository.datatypes._INTEGER,
                                "VideoBitRate"));
        fields.put( new Integer(Movie._X_PICTUREFRONT),
                    new DcField(Movie._X_PICTUREFRONT, getIndex(), "Picture Front",
                                true, true, true, false, false,
                                4000, ComponentFactory._PICTUREFIELD, getIndex(), Repository.datatypes._PICTURE,
                                "PictureFront"));
        fields.put( new Integer(Movie._Y_PICTUREBACK),
                    new DcField(Movie._Y_PICTUREBACK, getIndex(), "Picture Back",
                                true, true, true, false, false,
                                4000, ComponentFactory._PICTUREFIELD, getIndex(), Repository.datatypes._PICTURE,
                                "PictureBack"));
        fields.put( new Integer(Movie._Z_PICTURECD),
                    new DcField(Movie._Z_PICTURECD, getIndex(), "Picture CD",
                                true, true, true, false, false,
                                4000, ComponentFactory._PICTUREFIELD, getIndex(), Repository.datatypes._PICTURE,
                                "PictureCD"));
        fields.put( new Integer(Movie._Z_PICTURECD),
                    new DcField(Movie._Z_PICTURECD, getIndex(), "Picture CD",
                                true, true, true, false, false,
                                4000, ComponentFactory._PICTUREFIELD, getIndex(), Repository.datatypes._PICTURE,
                                "PictureCD"));
        fields.put( new Integer(Movie._0_LANGUAGE),
                    new DcField(Movie._0_LANGUAGE, getIndex(), "Language",
                                false, true, false, true, false,
                                30, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "Language"));
        fields.put( new Integer(Movie._1_AUDIOLANGUAGE),
                    new DcField(Movie._1_AUDIOLANGUAGE, getIndex(), "Audio Language",
                                false, true, false, true, false,
                                50, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "AudioLanguage"));
        fields.put( new Integer(Movie._2_SUBTITLELANGUAGE),
                    new DcField(Movie._2_SUBTITLELANGUAGE, getIndex(), "Subtitle Language",
                                false, true, false, true, false,
                                255, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "SubtitleLanguage"));
        fields.put( new Integer(Movie._3_CERTIFICATION),
                    new DcField(Movie._3_CERTIFICATION, getIndex(), "Certification",
                                false, true, false, true, true,
                                255, ComponentFactory._LONGTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "Certification"));
        fields.put( new Integer(Movie._4_FILESIZE),
                    new DcField(Movie._4_FILESIZE, getIndex(), "File Size",
                                false, true, false, true, true,
                                15, ComponentFactory._NUMBERFIELD, getIndex(), Repository.datatypes._INTEGER,
                                "Filesize"));
        fields.put( new Integer(Movie._5_ASIN),
                    new DcField(Movie._5_ASIN, getIndex(), "ASIN",
                                false, true, false, true, true,
                                255, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "ASIN"));
        fields.put( new Integer(Movie._6_STATE),
                    new DcField(Movie._6_STATE, getIndex(), "State",  
                                false, true, false, true, false, 
                                50, ComponentFactory._STATECOMBO, DcModules._MOVIESTATE, Repository.datatypes._DCOBJECTREFERENCE,
                                "State"));   
        fields.put( new Integer(Movie._7_STORAGEMEDIUM),
                    new DcField(Movie._7_STORAGEMEDIUM, getIndex(), "Storage Medium",  
                                false, true, false, true, false, 
                                50, ComponentFactory._MEDIACOMBO, DcModules._MOVIESTORAGE, Repository.datatypes._DCOBJECTREFERENCE,
                                "StorageMedium"));     
        fields.put( new Integer(Movie._8_SERIES),
                    new DcField(Movie._8_SERIES, getIndex(), "Series",
                                false, true, false, true, false,
                                255, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "Program"));
        fields.put( new Integer(Movie._9_SEASON),
                    new DcField(Movie._9_SEASON, getIndex(), "Season",
                                false, true, false, true, false,
                                4, ComponentFactory._NUMBERFIELD, getIndex(), Repository.datatypes._INTEGER,
                                "Season"));        
        fields.put( new Integer(Movie._10_EPISODE_NR),
                    new DcField(Movie._10_EPISODE_NR, getIndex(), "Episode (nr)",
                                false, true, false, true, false,
                                4, ComponentFactory._NUMBERFIELD, getIndex(), Repository.datatypes._INTEGER,
                                "EpisodeNumber"));        
        fields.put( new Integer(Movie._11_EPISODE_TITLE),
                    new DcField(Movie._11_EPISODE_TITLE, getIndex(), "Episode name",
                                false, true, false, true, false,
                                255, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "EpisodeName"));        
        
    }

    public DcObject[] getDefaultData() {
        return new DcObject[0];
    }

    private class ImportCDAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startImport();
        }
    }
}
