/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules.musicalbum;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.Collection;

import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.components.panels.ParentInsertTablePanel;
import net.sf.dc.console.components.panels.ParentSearchTablePanel;
import net.sf.dc.console.components.panels.QuickViewPanel;
import net.sf.dc.console.importcd.ImportCDDialog;
import net.sf.dc.console.importcd.ImportMusicFilesDialog;
import net.sf.dc.console.musicfile.MusicAlbumQuickViewPane;
import net.sf.dc.console.views.MasterView;
import net.sf.dc.console.views.list.ListInsertView;
import net.sf.dc.console.views.list.ListSearchView;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.actions.ManageGenresAction;
import net.sf.dc.core.actions.ManageStatesAction;
import net.sf.dc.core.actions.ManageStorageMediaAction;
import net.sf.dc.core.actions.ManageTemplatesAction;
import net.sf.dc.core.modules.DcCategoryModule;
import net.sf.dc.core.modules.DcGenreModule;
import net.sf.dc.core.modules.DcMediaModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.modules.DcStateModule;
import net.sf.dc.core.modules.DcStorageMediaModule;
import net.sf.dc.core.modules.template.ITemplateModule;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.DcStorageMedium;
import net.sf.dc.core.objects.musicalbum.MusicAlbum;
import net.sf.dc.core.objects.musicalbum.MusicTrack;
import net.sf.dc.core.services.DcAmazonServer;
import net.sf.dc.core.services.DcMusicBrainzServer;
import net.sf.dc.core.services.DcRegion;
import net.sf.dc.core.services.DcServer;
import net.sf.dc.core.services.KeywordSearchMode;
import net.sf.dc.core.services.MbAlbumTitleSearch;
import net.sf.dc.processes.cdimport.ImportCDProcess;
import net.sf.dc.processes.cdimport.ImportMusicFilesCD;
import net.sf.dc.processes.synchronizer.MusicAlbumSynchronizer;
import net.sf.dc.processes.synchronizer.Synchronizer;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;

public class MusicAlbumModule extends DcMediaModule {

    public MusicAlbumModule() {
        super(DcModules._MUSICALBUM,
              true,
              DcResources.getText(Repository.systemlabels.sysMusicAlbum),
              "musicalbums",
              "musa",
              "musicalbums",
              false,
              false,
              false,
              false);
    }
    
    public MusicAlbumModule(int index, 
                            boolean topModule,
                            String name,
                            String tableName,
                            String tableShortName,
                            String tableJoin,
                            boolean canBeLended,
                            boolean allowImdbUpdate,
                            boolean allowAmazonUpdate,
                            boolean allowFreeDbUpdate) {

        super(  index, topModule, name, tableName, tableShortName,
                tableJoin, canBeLended, allowImdbUpdate, allowAmazonUpdate, allowFreeDbUpdate);
    }     
 
    public Synchronizer getSynchronizer() {
        return new MusicAlbumSynchronizer();
    }    
    
    public String getDescription() {
        return DcResources.getText("msgMusicAlbumModuleDesc");
    }      
    
    public ImageIcon getIcon() {
        return IconLibrary._icoMusicModule;
    }
    
    public ImageIcon getIconSmall() {
        return IconLibrary._icoMusicModule16;
    }    

    public String getObjectName() {
        return DcResources.getText(Repository.systemlabels.sysMusicAlbum);
    }

    public String getObjectNamePlural() {
        return DcResources.getText(Repository.systemlabels.sysMusicAlbumPlural);
    }

    public DcGenreModule getGenreModule() {
        return (DcGenreModule) DcModules.get(DcModules._MUSICGENRE);
    } 
    
    public KeyStroke getKeyStroke() {
        return KeyStroke.getKeyStroke(KeyEvent.VK_2, InputEvent.CTRL_MASK);
    }    
    
    public boolean isEnabled() {
        return DcSettings.getValueAsBoolean(Repository.settings.stMusicAlbumModuleEnabled);
    }
    
    public void isEnabled(boolean b) {
        DcSettings.setValue(Repository.settings.stMusicAlbumModuleEnabled, new Boolean(b));
    }  
    
    public DcCategoryModule getCategoryModule() {
        return null;
    }
    
    public DcStateModule getStateModule() {
        return (DcStateModule) DcModules.get(DcModules._MUSICALBUMSTATE);
    }     
    
    public ITemplateModule getTemplateModule() {
        return (ITemplateModule) DcModules.get(DcModules._TEMPLATEMUSICALBUM);
    }     
    
    public DcStorageMediaModule getStorageMediaModule() {
        return (DcStorageMediaModule) DcModules.get(DcModules._MUSICALBUMSTORAGE);
    }    

    public JMenu getToolsMenu() {
        JMenu menu = ComponentFactory.getMenu(DcResources.getText("lblTools"));

        JMenuItem menuImport = ComponentFactory.getMenuItem(IconLibrary._icoImport, DcResources.getText("lblMusicFileImport"));
        menuImport.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_I, InputEvent.CTRL_MASK));
        menuImport.addActionListener(new ImportCDAction());

        JMenuItem menuAlbumState = ComponentFactory.getMenuItem(IconLibrary._icoAnchor, 
        							DcResources.getText("lblManageStatesX", getObjectName()));  
        menuAlbumState.addActionListener(new ManageStatesAction(DcModules._MUSICALBUM));

        JMenuItem menuTrackState = ComponentFactory.getMenuItem(IconLibrary._icoAnchor, 
        							DcResources.getText("lblManageStatesX", DcModules.get(DcModules._MUSICTRACK).getObjectName()));  
        menuTrackState.addActionListener(new ManageStatesAction(DcModules._MUSICTRACK));

        JMenuItem menuGenre = ComponentFactory.getMenuItem(IconLibrary._icoAnchor, DcResources.getText("lblManageGenres"));  
        menuGenre.addActionListener(new ManageGenresAction());
        
        JMenuItem menuStorageMedia = ComponentFactory.getMenuItem(IconLibrary._icoSave, 
        			DcResources.getText("lblManageStorageMedia"));  
        menuStorageMedia.addActionListener(new ManageStorageMediaAction());
        
        JMenuItem menuAlbumTemplate = ComponentFactory.getMenuItem(IconLibrary._icoTemplate, 
        			DcResources.getText("lblManageTemplatesX", getObjectName()));  
        menuAlbumTemplate.addActionListener(new ManageTemplatesAction(DcModules._MUSICALBUM));

        JMenuItem menuTrackTemplate = ComponentFactory.getMenuItem(IconLibrary._icoTemplate,
        			DcResources.getText("lblManageTemplatesX", DcModules.get(DcModules._MUSICTRACK).getObjectName()));  
        menuTrackTemplate.addActionListener(new ManageTemplatesAction(DcModules._MUSICTRACK));

        menu.add(menuGenre);
        menu.add(menuAlbumState);
        menu.add(menuTrackState);
        menu.add(menuStorageMedia);
        menu.addSeparator();
        menu.add(menuAlbumTemplate);
        menu.add(menuTrackTemplate);
        menu.addSeparator();
        menu.add(menuImport);

        return menu;
    }

    public QuickViewPanel getQuickView() {
        return new MusicAlbumQuickViewPane();
    }

    public DcObject getDcObject() {
        return new MusicAlbum();
    }

    protected void initializeUI() {
        insertView = new MasterView();
        insertView.addView(MasterView._TABLE_VIEW, new ParentInsertTablePanel(DcModules._MUSICALBUM));
        insertView.addView(MasterView._LIST_VIEW, new ListInsertView(this, DcResources.getText("lblNewItem", getObjectNamePlural()), IconLibrary._icoItemsNew));

        searchView = new MasterView();
        searchView.addView(MasterView._TABLE_VIEW, new ParentSearchTablePanel(DcModules._MUSICALBUM, MusicTrack._F_TRACKNUMBER));
        searchView.addView(MasterView._LIST_VIEW, new ListSearchView(this, getObjectNamePlural(), getIconSmall()));
    }

    public String getFieldDefinitionsSettingsKey() {
        return Repository.settings.stFieldDefinitionsMusicAlbum;
    }
    
    public String getSearchOrderSettingsKey() {
        return Repository.settings.stMusicAlbumSearchOrder;
    }

    public void startImport() {
        ImportMusicFilesDialog dlg = new ImportMusicFilesDialog(getIndex(), DataCrow.mainFrame, DcResources.getText("lblMusicFileImport"));
        dlg.setVisible(true);
    }
    
    public Collection getOnlineServices() {
        Collection servers = new ArrayList();
        
        DcMusicBrainzServer musicBrainzServer = new DcMusicBrainzServer("MusicBrainz", "http://musicbrainz.org/ws/1/", true);
        musicBrainzServer.addRegion(new DcRegion("uk", "The main server", "http://musicbrainz.org/ws/1"));
        musicBrainzServer.addMode(new MbAlbumTitleSearch());
        servers.add(musicBrainzServer);
        
        DcServer amazonServer = new DcAmazonServer();
        amazonServer.addMode(new KeywordSearchMode("Music", "Keyword"));
        servers.add(amazonServer);
        
        return servers;
    }

    public ImportCDProcess getCDImportProcess(ImportCDDialog ui,
                                              DcStorageMedium storageMedium,
                                              String sourceDir,
                                              boolean confirmDirs,
                                              boolean recursiveDirs,
                                              String nameOfCD) {
        return new ImportMusicFilesCD(ui, storageMedium, sourceDir, confirmDirs, recursiveDirs, nameOfCD, 0);
    }

    protected void initializeFields() {
        super.initializeFields();

        fields.put( new Integer(MusicAlbum._F_ARTIST),
                    new DcField(MusicAlbum._F_ARTIST, getIndex(), "Artist",
                                false, true, false, true, false,
                                255, ComponentFactory._LONGTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "Artist"));
        fields.put( new Integer(MusicAlbum._G_GENRES),
                    new DcField(MusicAlbum._G_GENRES, getIndex(), "Genres",
                                true, true, true, true, false,
                                255, ComponentFactory._REFERENCESFIELD, DcModules._MUSICALBUMGENREMAPPING, Repository.datatypes._DCOBJECTCOLLECTION,
                                "Genres"));  
        fields.put( new Integer(MusicAlbum._H_STATE),
                    new DcField(MusicAlbum._H_STATE, getIndex(), "State",
                                false, true, false, true, false, 
                                50, ComponentFactory._STATECOMBO, DcModules._MUSICALBUMSTATE, Repository.datatypes._DCOBJECTREFERENCE,
                                "State"));
        fields.put( new Integer(MusicAlbum._I_STORAGEMEDIUM),
                    new DcField(MusicAlbum._I_STORAGEMEDIUM, getIndex(), "Storage Medium",  
                                false, true, false, true, false, 
                                50, ComponentFactory._MEDIACOMBO, DcModules._MUSICALBUMSTORAGE, Repository.datatypes._DCOBJECTREFERENCE,
                                "StorageMedium"));
        fields.put( new Integer(MusicAlbum._J_PICTUREFRONT),
                    new DcField(MusicAlbum._J_PICTUREFRONT, getIndex(), "Picture Front",
                                true, true, true, false, false,
                                4000, ComponentFactory._PICTUREFIELD, getIndex(), Repository.datatypes._PICTURE,
                                "PictureFront"));
        fields.put( new Integer(MusicAlbum._K_PICTUREBACK),
                    new DcField(MusicAlbum._K_PICTUREBACK, getIndex(), "Picture Back",
                                true, true, true, false, false,
                                4000, ComponentFactory._PICTUREFIELD, getIndex(), Repository.datatypes._PICTURE,
                                "PictureBack"));
        fields.put( new Integer(MusicAlbum._L_PICTURECD),
                    new DcField(MusicAlbum._L_PICTURECD, getIndex(), "Picture CD",
                                true, true, true, false, false,
                                4000, ComponentFactory._PICTUREFIELD, getIndex(), Repository.datatypes._PICTURE,
                                "PictureCD"));
        fields.put( new Integer(MusicAlbum._M_DISCID),
                    new DcField(MusicAlbum._M_DISCID, getIndex(), "Disc ID",
                                false, true, false, true, true,
                                255, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "DiscID"));
        fields.put( new Integer(MusicAlbum._N_WEBPAGE),
                    new DcField(MusicAlbum._N_WEBPAGE, getIndex(), "Webpage",
                                false, true, false, false, false,
                                255, ComponentFactory._URLFIELD, getIndex(), Repository.datatypes._STRING,
                                "Webpage"));   
    }

    public DcObject[] getDefaultData() {
        return new DcObject[0];
    }

    private class ImportCDAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startImport();
        }
    }
}