/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules.software;

import java.io.File;
import java.net.URL;

import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DcCategoryModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.software.SoftwareCategory;
import net.sf.dc.resources.DcResources;
import net.sf.ext.util.Utilities;

public class SoftwareCategoryModule extends DcCategoryModule {
    
    public SoftwareCategoryModule() {
        super(DcModules._SOFTWARECAT,
              false,
              DcResources.getText(Repository.systemlabels.sysCategory),
              "softwarecategories",
              "soca",
              "softwarecategories",
              false,
              false,
              false);
    }

    public String getObjectName() {
        return DcResources.getText(Repository.systemlabels.sysCategory);
    }
    
    public String getObjectNamePlural() {
        return DcResources.getText(Repository.systemlabels.sysCategoryPlural);
    }

    public DcObject getDcObject() {
        return new SoftwareCategory();
    }
    
    public String getFieldDefinitionsSettingsKey() {
        return Repository.settings.stFieldDefinitionsSoftwareCategory;
    }
    
    public DcObject[] getDefaultData() throws Exception {
        String link = "file://" + Utilities.getCurrentDirectory() + File.separator + 
                      "icons_user" + File.separator;
            
        String[][] values = {{"3D Shooter",  link + "BlackOps.gif"},
                             {"Action Game", link + "BangBoy.gif"},
                             {"Adventure Game", link + "PlumpNdapper.gif"},
                             {"System Tool", link + "ZerToolz.gif"},
                             {"Emulator", link + "Zipdisk.gif"},
                             {"Internet", link + "NetObserver.gif"},
                             {"Racing game", link + "ToasterBot.gif"},
                             {"Strategy game", link + "TheCrownSitsHeavilyToday.gif"},
                             {"Converter", link + "GrrZap.gif"},
                             {"Role Playing Game", link + "He-MansAxe.gif"},
                             {"Family Game", link + "LeSmileyDeClassique.gif"},
                             {"Graphics Editor", link + "PhotoNetMagicHooya!.gif"},
                             {"Operating System", link + "BetterVirtualPC.gif"},
                             {"Editor", link + "Milton.gif"},
                             {"Driver", link + "UmaxAstra1200sscanner.gif"},
                             {"e-Books", link + "HitachiRasterOpsmonitor.gif"}};

        DcObject[] objects = new DcObject[values.length];
        for (int i = 0; i < values.length; i++) {
            DcObject dco = new SoftwareCategory();

            URL url = new URL(values[i][1]);
            String s = Utilities.fileToBase64String(url);

            dco.setValue(SoftwareCategory._A_NAME, values[i][0]);
            dco.setValue(SoftwareCategory._B_ICON, s);                            
            
            objects[i] = dco;
        }
        return objects;
    }
}
