/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules.software;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.modules.DcSimpleModule;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.software.SoftwarePlatform;
import net.sf.dc.resources.DcResources;

public class SoftwarePlatformModule extends DcSimpleModule {
    
    public SoftwarePlatformModule() {
        super(DcModules._SOFTWAREPLATFORM,
              false,
              DcResources.getText(Repository.systemlabels.sysPlatform),
              "softwareplatforms",
              "sopl",
              "softwareplatforms",
              false,
              false,
              false);
    }

    public String getObjectName() {
        return DcResources.getText(Repository.systemlabels.sysPlatform);
    }
    
    public String getObjectNamePlural() {
        return DcResources.getText(Repository.systemlabels.sysPlatformPlural);
    }

    public DcObject getDcObject() {
        return new SoftwarePlatform();
    }
    
    public String getFieldDefinitionsSettingsKey() {
        return Repository.settings.stFieldDefinitionsSoftwarePlatform;
    }
    
    protected void initializeFields() {
        super.initializeFields();
        fields.put( new Integer(SoftwarePlatform._A_NAME), 
                    new DcField(SoftwarePlatform._A_NAME, getIndex(), "Name", 
                                false, true, false, true, false,
                                255, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                                "Name"));
        fields.put( new Integer(SoftwarePlatform._B_ICON), 
                    new DcField(SoftwarePlatform._B_ICON, getIndex(), "Icon", 
                                false, true, true, false, false,
                                255, ComponentFactory._PICTUREFIELD, getIndex(), Repository.datatypes._ICON,
                                "Icon"));
    }     
    
    public DcObject[] getDefaultData() throws Exception {
        String[] values = {"Windows XP",
                           "Windows 2000",
                           "Windows 98",
                           "Windows 95",
                           "Windows (32-bit)",
                           "Windows (16-bit)",
                           "Windows",
                           "RedHat Linux",
                           "SuSE Linux", 
                           "Mandrake", 
                           "Linux",
                           "Mac OS X",
                           "Nintendo",
                           "Nintendo Wii",
                           "PlayStation 3",
                           "PlayStation 2",
                           "PlayStation",
                           "XBOX 360",
                           "XBOX",
                           "Amiga",
                           "Atari",
                           "SEGA",
                           "PSP",
                           "OS Independent"};
        DcObject[] objects = new DcObject[values.length];
        for (int i = 0; i < values.length; i++) {
            DcObject dco = new SoftwarePlatform();
            dco.setValue(SoftwarePlatform._A_NAME, values[i]);
            objects[i] = dco;
        }
        return objects;
    }    
}

