/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules.template;

import java.util.Iterator;

import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.KeyStroke;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.components.panels.QuickViewPanel;
import net.sf.dc.console.forms.DcSimpleItemView;
import net.sf.dc.console.forms.TemplateForm;
import net.sf.dc.console.importcd.ImportCDDialog;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.modules.musicalbum.MusicTrackModule;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.DcStorageMedium;
import net.sf.dc.core.objects.template.TemplateMusicTrack;
import net.sf.dc.processes.cdimport.ImportCDProcess;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.settings.definitions.DcFieldDefinition;
import net.sf.dc.settings.definitions.DcFieldDefinitions;

public class TemplateMusicTrackModule extends MusicTrackModule implements ITemplateModule {
    
    private TemplateForm form;
    
    public TemplateMusicTrackModule() {
        super(DcModules._TEMPLATEMUSICTRACK, false, 
              DcResources.getText(Repository.systemlabels.sysTemplate),
              "musictracktemplate", "mustratem", "musictracktemplate",
              false, false, false, false);
    }
    
    public ImageIcon getIcon() {
        return IconLibrary._icoTemplate;
    }

    public ImageIcon getIconSmall() {
        return IconLibrary._icoTemplate;
    }        
    
    public DcSimpleItemView getForm() {
        initializeUI();
        return form;    
    }   

    public DcObject getDcObject() {
        return new TemplateMusicTrack();
    }  
    
    public ITemplateModule getTemplateModule() {
        return null;
    }    
    
    public String getObjectName() {
        return DcResources.getText(Repository.systemlabels.sysTemplate);
    }

    public String getObjectNamePlural() {
        return DcResources.getText(Repository.systemlabels.sysTemplatePlural);
    }
    
    public String getFieldDefinitionsSettingsKey() {
        return Repository.settings.stFieldDefinitionsTemplateMusicTrack;
    }       
    
    public DcFieldDefinitions getFieldDefinitions() {
        DcFieldDefinitions fds = (DcFieldDefinitions) DcSettings.getValueAsDefinitionGroup(super.getFieldDefinitionsSettingsKey());
        DcFieldDefinitions definitions = new DcFieldDefinitions();
        
        definitions.addDefinition(new DcFieldDefinition(51, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(50, null, true, true, true));
        
        for (Iterator iter = fds.iterator(); iter.hasNext(); ) {
            DcFieldDefinition fd = (DcFieldDefinition) iter.next();
            definitions.addDefinition(
                new DcFieldDefinition(fd.getIndex(), fd.getLabel(), fd.isEnabled(), false, fd.isVisible()));
        }
        
        return definitions;
    }      

    protected void initializeFields() {
        super.initializeFields();
        fields.put( new Integer(TemplateMusicTrack._SYS_TEMPLATENAME),
                    new DcField(TemplateMusicTrack._SYS_TEMPLATENAME, getIndex(), "Template Name",
                        false, true, false, true, false,
                        255, ComponentFactory._SHORTTEXTFIELD, getIndex(), Repository.datatypes._STRING,
                        "TemplateName"));
        fields.put( new Integer(TemplateMusicTrack._SYS_DEFAULT),
                    new DcField(TemplateMusicTrack._SYS_DEFAULT, getIndex(), "Default",
                        false, true, false, true, false,
                        255, ComponentFactory._CHECKBOX, getIndex(), Repository.datatypes._BOOLEAN,
                        "DefaultTemplate"));         
    }

    protected void initializeUI() {
        if (form == null)
            form = new TemplateForm(getIndex(), false);
    }

    
    // **********************
    // Not implemented
    // **********************
    public void startImport() {}
    
    public QuickViewPanel getQuickView() {
        return null;
    }

    public DcObject[] getDefaultData() {
        return new DcObject[0];
    }
    
    public KeyStroke getKeyStroke() {
        return null;
    }    
    
    public ImportCDProcess getCDImportProcess(ImportCDDialog ui, DcStorageMedium storageMedium, String sourceDir,
            boolean confirmDirs, boolean recursiveDirs, String nameOfCD) {
        return null;
    }
    
    public JMenu getToolsMenu() {
        return new JMenu();
    }
}
