/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.objects;

import java.util.ArrayList;
import java.util.Collection;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.enhancement.IValueEnhancer;

public class DcField {

    private String systemName;
    private String label;
    private boolean enabled;
    private boolean required;
    private boolean hidden;
    private boolean searchable;
    private boolean readonly;
    private boolean techinfo;
    private boolean uiOnly;
    private int maximumLength;
    private int fieldType;
    private int valueType;
    private int index;
    private int module;
    private int modRef;
    private String databaseFieldName;
    
    private Collection enhancers = new ArrayList();

    public DcField( int index,
                    int module,
                    String label,
                    boolean uiOnly,
                    boolean enabled,
                    boolean readonly,
                    boolean searchable,
                    boolean techinfo,
                    int maximumLength,
                    int fieldType,
                    int modRef,
                    int valueType,
                    String databaseFieldName) {

        setEnabled(enabled);
        setIndex(index);
        setModule(module);
        setLabel(label);
        setSystemName(label);
        setReadOnly(readonly);
        setSearchable(searchable);
        setTechinfo(techinfo);
        setMaximumLength(maximumLength);
        setFieldType(fieldType);
        setValueType(valueType);
        setDatabaseFieldName(databaseFieldName);
        setUiOnly(uiOnly);
        setModRef(modRef);
    }
    
    public void setModRef(int modRef) {
        this.modRef = modRef;
    }

    public int getModRef() {
        return modRef;
    }
    
    public void setIndex(int index) {
        this.index = index;
    }

    public int getIndex() {
        return index;
    }

    public void setModule(int module) {
        this.module = module;
    }

    public int getModule() {
        return module;
    }

    public void setTechinfo(boolean techinfo) {
        this.techinfo = techinfo;
    }

    public void setUiOnly(boolean uiOnly) {
        this.uiOnly = uiOnly;
    }

    public boolean isUiOnly() {
        return uiOnly;
    }

    public void setEnabled(boolean enabled) {
        this.enabled = enabled;
    }

    public boolean isEnabled() {
        return enabled;
    }

    public void setSearchable(boolean searchable) {
        this.searchable = searchable;
    }

    public String getDatabaseFieldName() {
        return databaseFieldName;
    }

    public int getFieldType() {
        return fieldType;
    }

    public int getValueType() {
        return valueType;
    }

    public String getLabel() {
        return label;
    }

    public String getSystemName() {
        return systemName;
    }

    public boolean isHidden() {
        return hidden;
    }

    public boolean isTechnicalInfo() {
        return techinfo;
    }

    public boolean isRequired() {
        return required;
    }

    public boolean isReadOnly() {
        return readonly;
    }

    public boolean isSearchable() {
        return searchable;
    }

    public int getMaximumLength() {
        return maximumLength;
    }

    public void setLabel(String s) {
        label = s;
    }

    private void setSystemName(String s) {
        systemName = s;
    }

    public void setReadOnly(boolean b) {
        readonly = b;
    }

    public void setRequired(boolean b) {
        required = b;
    }

    public void setHidden(boolean b) {
        hidden = b;
    }

    public void setMaximumLength(int i) {
        maximumLength = i;
    }

    public void setFieldType(int index) {
        fieldType = index;
    }

    public void setValueType(int index) {
        valueType = index;
    }

    public void setDatabaseFieldName(String s) {
        databaseFieldName = s;
    }

    public String toString() {
        return label;
    }
    
    public void removeEnhancers() {
        enhancers.clear();
    }
    
    public void addValueEnhancer(IValueEnhancer enhancer) {
        enhancers.add(enhancer);
    }
    
    public IValueEnhancer[] getValueEnhancers() {
        IValueEnhancer[] e = new IValueEnhancer[enhancers.size()];
        return (IValueEnhancer[]) enhancers.toArray(e);
    }

    public String getDataBaseFieldType() {
        String s = "";
        if (getValueType() == Repository.datatypes._STRING) {
            if (getFieldType() == ComponentFactory._LONGTEXTFIELD) {
                s = Repository.database._FIELDLONGSTRING +
                    "(" + getMaximumLength() + ")";
            } else {
                s = Repository.database._FIELDSTRING +
                    "(" + getMaximumLength() + ")";
            }
        } else if (getValueType() == Repository.datatypes._BIGINTEGER ||
                   getValueType() == Repository.datatypes._DCOBJECTREFERENCE ||
                   getValueType() == Repository.datatypes._DCOBJECTCOLLECTION ||
                   getValueType() == Repository.datatypes._DCPARENTREFERENCE) {
            s = Repository.database._FIELDBIGINT;
        } else if (getValueType() == Repository.datatypes._INTEGER) {
            s = Repository.database._FIELDINTEGER;
        } else if (getValueType() == Repository.datatypes._ICON ||
                   getValueType() == Repository.datatypes._PICTURE ||
                   getValueType() == Repository.datatypes._BLOB) {
            s = Repository.database._FIELDOBJECT;
        } else if (getValueType() == Repository.datatypes._BOOLEAN) {
            s = Repository.database._FIELDBOOLEAN;
        } else if (getValueType() == Repository.datatypes._DATE) {
            s = Repository.database._FIELDDATE;
        }

        return s;
    }
}
