/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.objects;

import java.util.Calendar;
import java.util.Collection;
import java.util.Iterator;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.core.Repository;
import net.sf.dc.core.data.DataManager;
import net.sf.dc.resources.DcResources;
import net.sf.ext.util.Base64;
import net.sf.ext.util.Converter;
import net.sf.ext.util.Utilities;

public class DcValue {

	private static final Calendar cal;

    static {
    	cal = Calendar.getInstance();
    }

    private boolean changed = false;
    private Object value = null;

    public boolean isChanged() {
        return changed;
    }

    public void setChanged(boolean b) {
        changed = b;
        
        if (!b && value instanceof Picture && value != null)
            ((Picture) value).markAsUnchanged();
    }

    public void setValueLowLevel(Object newValue, DcField field) {
        if (!field.isUiOnly())
            setChanged(true);
        
        this.value = newValue;
    }
    
    public void setValue(Object newValue, DcField field) {
        if (!field.isUiOnly())
            setChanged(true);
        else if (field.getValueType() == Repository.datatypes._DCOBJECTCOLLECTION)
            setChanged(true);

        if (field.getValueType() == Repository.datatypes._PICTURE) {
            int length = newValue != null && newValue instanceof byte[] ? ((byte[]) newValue).length : 0;
            if (newValue != null && newValue instanceof Picture) {
                Picture newPicture = (Picture) newValue;
                value = newPicture;
                setChanged(true);
            } else if (value instanceof Picture) {
                Picture currentPic = (Picture) value;
                if (length > 0) {
                    byte[] bCur = (byte[]) currentPic.getValue(Picture._D_IMAGE);
                    bCur = bCur == null ? new byte[0] : bCur;
                    byte[] bNew = (byte[]) newValue;
                    
                    if (!Utilities.sameImage(bCur, bNew)) {
                        currentPic.setValue(Picture._D_IMAGE, newValue);
                        currentPic.isUpdated(true);
                        setChanged(true);
                    }
                } else {
                    currentPic.isDeleted(true);
                    currentPic.setValue(Picture._D_IMAGE, null);
                    setChanged(true);
                }
            } else if (length > 0 && !(newValue instanceof Picture)) {
                Picture picture = new Picture();
                picture.isNew(true);
                picture.setValue(Picture._D_IMAGE, newValue);
                picture.setValue(Picture._B_FIELD, field.getDatabaseFieldName());
                value = picture;
                setChanged(true);
            }
        } else if (field.getValueType() == Repository.datatypes._ICON) {
            value = newValue;
            if (newValue != null && newValue instanceof byte[] && ((byte[]) newValue).length > 0)
                value = String.valueOf(Base64.encode((byte[]) newValue));
        } else {
            value = newValue;
            if (value != null) {
                if (field.getValueType() == Repository.datatypes._ICON) {
                    value = newValue.toString();
                } else if (field.getValueType() == Repository.datatypes._DCOBJECTCOLLECTION) {
                    
                    if (newValue == null || newValue instanceof Collection)
                        value = newValue;
                } else if (field.getValueType() == Repository.datatypes._DCOBJECTREFERENCE) {
                    if (newValue instanceof DcObject)
                        value = newValue;
                    else
                        value = DataManager.getObject(field.getModRef(), newValue.toString());

                    if (value == null)
                        value = DataManager.getObjectForDisplayValue(field.getModRef(), newValue.toString());

                } else if (field.getValueType() == Repository.datatypes._INTEGER) {
                    try {
                        if (newValue instanceof Integer) {
                            value = newValue;
                        } else if (newValue.toString().trim().length() > 0){
                            value = new Integer(Integer.parseInt(newValue.toString()));
                        }
                    } catch (Exception exp) {
                        value = null;
                    }
                } else if (field.getValueType() == Repository.datatypes._BOOLEAN) {
                    value = newValue;
                } else if (field.getValueType() != Repository.datatypes._DATE &&
                           field.getValueType() != Repository.datatypes._BYTEARRAY) {
                    try {
                        String s = Converter.databaseValueConverter(newValue.toString());
                        if (s.length() > field.getMaximumLength() && field.getMaximumLength() > 0)
                            value = s.substring(0, field.getMaximumLength());
                        else
                            value = s;
                    } catch (Exception exp) {
                        value = null;
                    }
                }
            }
        }
    }
    
    public void clear() {
        if (value != null) {
            if (value instanceof Picture)
                ((Picture) value).unload();
        }
        value = null;
    }
    
    public Object getValue() {
        return value;
    }

    public String getValueAsString() {
        return value != null ? value.toString() : "";
    }
    
    public String getDisplayString(int fieldType) {
        Object o = getValue();
        String text = "";

        try {
            if (fieldType == ComponentFactory._REFERENCESFIELD) {
                Collection mappings = (Collection) o;
                if (mappings != null) {
                    boolean first = true;
                    for (Iterator iter = mappings.iterator(); iter.hasNext(); ) {
                        DcObject dco = (DcObject) iter.next();
                        
                        if (!first) text += ", ";
                        
                        text += dco;
                        first = false;
                    }
                }
            } else if (fieldType == ComponentFactory._RATINGCOMBOBOX) {
                int value = -1;
                
                if (o != null)
                    value = Integer.parseInt(o.toString());

                text = value == -1 ? DcResources.getText("lblRatingNotRated") : 
                                     value + " " + DcResources.getText("lblRatingOutOf");

            } else if (fieldType == ComponentFactory._TIMEFIELD) {
                cal.clear();

                int value = 0;
                try {
                    value = Integer.parseInt(o.toString());
                } catch (Exception ignore) {}

                int minutes = 0;
                int seconds = 0;
                int hours = 0;

                if (value != 0) {
                    cal.set(Calendar.SECOND, value);
                    minutes = cal.get(Calendar.MINUTE);
                    seconds = cal.get(Calendar.SECOND);
                    hours = cal.get(Calendar.HOUR);
                }

                String sSeconds = getDoubleDigitString(seconds);
                String sMinutes = getDoubleDigitString(minutes);
                text = "" + hours + ":" + sMinutes + ":" + sSeconds;
            } else {
            	text = o == null ? "" : o.toString();
            }
        } catch (Exception ignore) {}

        return text;
    }

    private String getDoubleDigitString(int value) {
        StringBuffer sb = new StringBuffer();
        if (value == 0) {
            sb.append("00");
        } else if (value < 10) {
            sb.append("0");
            sb.append(value);
        } else {
            sb.append(value);
        }
        return sb.toString();
    }

    public void finalize() throws Throwable {
        try {
            value = null;
        } finally {
            super.finalize();
        }
    }
}
