/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.objects.enhancement;

import java.util.StringTokenizer;

import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcMediaObject;

public class TitleRewriter implements IValueEnhancer {

    private boolean enabled = false;
    private String workList;
    private final int field = DcMediaObject._A_TITLE;

    
    public TitleRewriter() {
    }    
    
    public TitleRewriter(boolean enabled, String list) {
        this.enabled = enabled;
        this.workList = list;
    }
    
    public int getField() {
        return field;
    }    

    public String getWordList() {
        return workList;
    } 
    
    public String toSaveString() {
        return enabled + "/&/" + workList;
    }

    public int getIndex() {
        return ValueEnhancers._TITLEREWRITERS;
    }
    
    public void parse(String s) {
        enabled = Boolean.valueOf(s.substring(0, s.indexOf("/&/"))).booleanValue();
        workList = s.substring(s.indexOf("/&/") + 3, s.length());
    }

    public Object apply(DcField field, Object value) {
        Object result = value;

        if (value != null && value instanceof String) {
            String s = (String) result;
            StringTokenizer st = new StringTokenizer(workList, ",");
            while (st.hasMoreElements()) {
                String keyword = ((String) st.nextElement()) + " ";
                
                if (s.toLowerCase().startsWith(keyword.toLowerCase())) {
                    int start = keyword.length();
                    int end = s.length();
                    
                    if (start < end) 
                        result = s.substring(start, end) + ", " + keyword.trim();
                }
            }
        }
        return result;
    }
    
    public boolean isRunOnUpdating() {
        return true;
    }
    
    public boolean isRunOnInsert() {
        return true;
    }
    

    public boolean isEnabled() {
        return enabled;
    }
}
