/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.objects.enhancement;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;

import net.sf.dc.core.DataCrow;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.messages.Messages;

public class ValueEnhancers {

    public static final int _AUTOINCREMENT = 0;
    public static final int _TITLEREWRITERS = 1;
    
    private static final Map enhancers = new HashMap();
    
    public static void registerEnhancer(DcField field, IValueEnhancer valueEnhancer) {
        Collection c = (Collection) enhancers.get(field);
        c = c == null ? new ArrayList() : c;
        
        Collection newEnhancers = new ArrayList();
        for (Iterator iter = c.iterator(); iter.hasNext(); ) {
            IValueEnhancer cur = (IValueEnhancer) iter.next();
            valueEnhancer.getIndex();
            if (cur.getIndex() != valueEnhancer.getIndex())
                newEnhancers.add(cur);
        }
        
        newEnhancers.add(valueEnhancer);
        enhancers.put(field, newEnhancers);
        
        field.addValueEnhancer(valueEnhancer);
    }
    
    public static void initialize() {
        load(_AUTOINCREMENT);
        load(_TITLEREWRITERS);
    }
    
    public static Collection getEnhancers(DcField field) {
        Collection c = (Collection) enhancers.get(field);
        return c == null ? new ArrayList() : c;
    }
    
    public static Collection getEnhancers(int module, int idx) {
        Collection c = new ArrayList();
        for (Iterator iter = enhancers.keySet().iterator(); iter.hasNext(); ) {
            DcField field = (DcField) iter.next();

            if (field.getModule() == module) {
                Collection e = (Collection) enhancers.get(field);    
                for (Iterator iterEnhancers = e.iterator(); iterEnhancers.hasNext(); ) {
                    IValueEnhancer enhancer = (IValueEnhancer) iterEnhancers.next();
                    if (idx == enhancer.getIndex()) 
                        c.add(enhancer);
                }
            }
        }
        return c;
    }
    
    private static DcField getField(String s) {
        try {
            int mod = Integer.parseInt(s.substring(0, s.indexOf("/&/")));
            String columnName = s.substring(s.indexOf("/&/") + 3, s.length());
            
            DcModule module = DcModules.get(mod);
            for (Iterator iter = module.getFields().iterator(); iter.hasNext(); ) {
                DcField field = (DcField) iter.next();
                if (field.getDatabaseFieldName().equalsIgnoreCase(columnName)) {
                    return field;
                }
            }
        } catch (Exception exp) {
            Messages.add(exp);
        }
        return null;
    }
    
    public static void save() {
        save(_AUTOINCREMENT);
        save(_TITLEREWRITERS);
    }
    
    private static void save(int idx) {
        Properties incrementers = new Properties();
        Properties titlerewriters = new Properties();
        
        for (Iterator iter = enhancers.keySet().iterator(); iter.hasNext(); ) {
            DcField field = (DcField) iter.next();
            
            Collection e = (Collection) enhancers.get(field);    
            for (Iterator iterEnhancers = e.iterator(); iterEnhancers.hasNext(); ) {
                IValueEnhancer enhancer = (IValueEnhancer) iterEnhancers.next();
                String key = field.getModule() + "/&/" + field.getDatabaseFieldName();
                if (enhancer.getIndex() == _AUTOINCREMENT) 
                    incrementers.put(key, enhancer.toSaveString());
                else
                    titlerewriters.put(key, enhancer.toSaveString());
            }
        }
        
        try {
            store(incrementers, DataCrow.baseDir + "/data/enhancers_autoincrement.properties");
            store(titlerewriters, DataCrow.baseDir + "/data/enhancers_titlerewriters.properties");
        } catch (Exception exp) {
            Messages.add(exp);    
        }         
    }
    
    private static void store(Properties properties, String filename) throws Exception{
        FileOutputStream fos = new FileOutputStream(filename);
        properties.store(fos, "");
        fos.close();
    }
    
    private static void load(int idx) {
        Properties properties = new Properties();
        
        String filename = idx == _AUTOINCREMENT ? "enhancers_autoincrement.properties" :
                                                  "enhancers_titlerewriters.properties";
        filename = DataCrow.baseDir + "/data/" + filename;
        
        if (new File(filename).exists()) {
            try {
                FileInputStream fis = new FileInputStream(filename);
                properties.load(fis);
                fis.close();
                
                for (Iterator iter = properties.keySet().iterator(); iter.hasNext(); ) {
                    String key = (String) iter.next();
                    DcField field = getField(key);
                    
                    String value = properties.getProperty(key);
                    
                    IValueEnhancer enhancer;
                    if (idx == _AUTOINCREMENT)
                        enhancer = new AutoIncrementer(field.getIndex()); 
                    else 
                        enhancer = new TitleRewriter();
                    
                    enhancer.parse(value);
                    
                    registerEnhancer(field, enhancer);
                }
            } catch (Exception exp) {
                Messages.add(exp);
            }
        }
    }    
}
