/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.objects.musicalbum;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.audiocd.AudioCD;
import net.sf.dc.core.objects.audiocd.AudioTrack;
import net.sf.dc.core.services.DcRegion;
import net.sf.dc.core.services.DcSearchMode;
import net.sf.dc.core.services.DcServer;
import net.sf.dc.messages.Messages;
import net.sf.dc.processes.http.IOnlineUpdateListener;
import net.sf.dc.processes.http.InternetUpdateProcess;
import net.sf.ext.util.StringUtils;
import entagged.audioformats.AudioFile;
import entagged.audioformats.AudioFileIO;
import entagged.audioformats.Tag;
import entagged.audioformats.exceptions.CannotReadException;
import entagged.audioformats.generic.TagTextField;

public class MusicFile implements IOnlineUpdateListener {
    
    private String album;
    private String artist;
    private String genre;
    private String year;
    private String title;
    private String encodingType;
    
    private int track;
    private int bitrate;
    private int length;
    
    private Collection albums = new ArrayList();
    
    public MusicFile() {
        
    }
    
    public MusicFile(String filename) throws CannotReadException {
        AudioFile audioFile = AudioFileIO.read(new File(filename));
        
        Tag tag = audioFile.getTag();
        
        if (tag != null) {
            album = getBest(tag.getAlbum());
            artist = getBest(tag.getArtist());
            genre = getGenre(getBest(tag.getGenre()));
            year = getBest(tag.getYear());
            title = getBest(tag.getTitle());
            
            bitrate = audioFile.getBitrate();
            length = audioFile.getLength();
            encodingType = audioFile.getEncodingType();
            
            try {
                String s = getBest(tag.getTrack());
                track = Integer.parseInt(s);
            } catch (Exception exp) {}
            
            try {
                String s = getBest(tag.getTrack());
                track = Integer.parseInt(s);
            } catch (Exception exp) {}

        }
    }
    
    public boolean onlineUpdate(MusicAlbum ma, DcServer server, DcRegion region) {
        Collection modes = server.getModes();

        boolean updated = false;
        
        String title = normalize((String) ma.getValue(MusicAlbum._A_TITLE));
        String artist = normalize((String) ma.getValue(MusicAlbum._F_ARTIST));
        if ((title == null || title.length() == 0) ||
            (artist == null || artist.length() == 0)) 
            return updated;
        
        DcSearchMode mode = null;
        for (Iterator iterMode = modes.iterator(); iterMode.hasNext(); ) {
            mode = (DcSearchMode) iterMode.next();
        
            InternetUpdateProcess process = 
                server.getProcess(this, (String) ma.getValue(MusicAlbum._A_TITLE), mode, region);
            process.start();
            
            try {
                process.join();
            } catch (Exception exp) {}
            
            for (Iterator iter = albums.iterator(); iter.hasNext(); ) {
                AudioCD ac = (AudioCD) iter.next();

                String titleNew = normalize((String) ac.getValue(AudioCD._A_TITLE));
                String artistNew = normalize((String) ac.getValue(AudioCD._F_ARTIST));
                
                
                if (titleNew != null && artistNew != null) {
                    if ((titleNew.toLowerCase().equals(title.toLowerCase())) && 
                        (artistNew.toLowerCase().equals(artist.toLowerCase()))) {
                        
                        boolean match = updateTracks(ma.getChildren(), ac.getChildren());
                        if (match) {
                            updated = true;
                            ma.setValue(MusicAlbum._J_PICTUREFRONT, ac.getValue(AudioCD._H_PICTUREFRONT));
                            ma.setValue(MusicAlbum._K_PICTUREBACK, ac.getValue(AudioCD._I_PICTUREBACK));
                            ma.setValue(MusicAlbum._L_PICTURECD, ac.getValue(AudioCD._J_PICTURECD));
                        }
                    }
                }
            }
            albums.clear();
        }
        return updated;
    }
    
    private String normalize(String s) {
        if (s != null)
            return s.replaceAll("[,.!@#$%^&{}()'~`]", "");
        
        return s;
    }
    
    private boolean updateTracks(Collection oldTracks, Collection newTracks) {
        boolean match = false;
        for (Iterator iterOld = oldTracks.iterator(); iterOld.hasNext(); ) {
            MusicTrack tr = (MusicTrack) iterOld.next();
            
            String titleOld = (String) tr.getValue(MusicTrack._A_TITLE);
            Integer lengthOld = (Integer) tr.getValue(MusicTrack._J_PLAYLENGTH);
            Integer trackOld = (Integer) tr.getValue(MusicTrack._F_TRACKNUMBER);

            for (Iterator iterNew = newTracks.iterator(); iterNew.hasNext(); ) {
                AudioTrack at = (AudioTrack) iterNew.next();
                
                String titleNew = (String) at.getValue(AudioTrack._A_TITLE);
                Integer lengthNew = (Integer) at.getValue(AudioTrack._H_PLAYLENGTH);
                Integer trackNew = (Integer) at.getValue(AudioTrack._F_TRACKNUMBER);

                if ((titleOld != null && titleNew != null) && titleNew.toLowerCase().equals(titleOld.toLowerCase())) {
                    
                    match = true;
                    merge(at, tr);
                
                } else if (newTracks.size() == oldTracks.size() && 
                           ((lengthOld != null && lengthNew != null) && lengthNew.equals(lengthOld)) ||  
                           ((trackOld != null && trackNew != null) && trackNew.equals(trackOld))) {
                
                    match = true;
                    merge(at, tr);
                }
            }
        }
        return match;
    }
    
    private void merge(AudioTrack at, MusicTrack mt) {
        setValue(mt, MusicTrack._A_TITLE, at.getValue(AudioTrack._A_TITLE));
        setValue(mt, MusicTrack._B_DESCRIPTION, at.getValue(AudioTrack._B_DESCRIPTION));
        setValue(mt, MusicTrack._C_YEAR, at.getValue(AudioTrack._C_YEAR));
        setValue(mt, MusicTrack._E_RATING, at.getValue(AudioTrack._E_RATING));
        setValue(mt, MusicTrack._F_TRACKNUMBER, at.getValue(AudioTrack._F_TRACKNUMBER));
        setValue(mt, MusicTrack._G_ARTIST, at.getValue(AudioTrack._G_LYRICS));
        setValue(mt, MusicTrack._J_PLAYLENGTH, at.getValue(AudioTrack._H_PLAYLENGTH));
    }
    
    private void setValue(DcObject dco, int field, Object newValue) {
        if (newValue != null && newValue.toString().length() > 0)
            dco.setValue(field, newValue);
    }
    
    private String getGenre(String s) {
        String genre = s != null && s.length() > 0 ? s : null;
        
        if (s != null) {
            try {
                String genreIdx = StringUtils.getValueBetweenStrings("(", ")", genre);
                int index = Integer.parseInt(genreIdx);
                genre = Repository.collections.colMusicGenres[index];
            } catch (Exception exp) {}
        }
        
        return genre;
    }    
        
    private String getBest(List list) {
        String s = "";

        Collection all = new ArrayList(list);
        for (Iterator iter = all.iterator(); iter.hasNext(); ) {
            try {
                TagTextField tagField = (TagTextField) iter.next();
                s = tagField.getContent();
            } catch (Exception exp) {
                Messages.add(exp);
            }
        }

        return s;
    }

    public String getAlbum() {
        return album;
    }

    public String getArtist() {
        return artist;
    }

    public String getGenre() {
        return genre;
    }

    public String getTitle() {
        return title;
    }

    public int getTrack() {
        return track;
    }

    public String getYear() {
        return year;
    }

    public int getBitrate() {
        return bitrate;
    }

    public String getEncodingType() {
        return encodingType;
    }

    public int getLength() {
        return length;
    }

    public void addError(Throwable t) {
        Messages.add(t);
    }

    public void addError(String message) {}

    public void addMessage(String message) {}

    public void addObject(DcObject dco) {
        albums.add(dco);
    }

    public void addWarning(String warning) {}

    public DcModule getModule() {
        return DcModules.get(DcModules._MUSICALBUM);
    }

    public void processed(int i) {}

    public void processing() {}

    public void processingTotal(int i) {}

    public int resultCount() {
        return albums.size();
    }

    public void stopped() {}      
}


