/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.objects.template;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import net.sf.dc.core.data.DataFilter;
import net.sf.dc.core.data.DataFilterOptions;
import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;

public class Templates {
    
    private static final Collection templates = new ArrayList();
    
    public static void refresh() {
        templates.clear();
        
        for (Iterator iter = DcModules.getAllModules(); iter.hasNext(); ) {
            DcModule module = (DcModule) iter.next();
            
            if (module.getTemplateModule() != null) {
                DcModule templateModule = (DcModule) module.getTemplateModule();
                DcObject dco = templateModule.getDcObject();
                
                DataFilterOptions dfo = new DataFilterOptions(new DcField[] {dco.getField(50)}, false, false);
                DataFilter df = new DataFilter(dco.getModule().getIndex(), dco, dfo);
                DcObject[] objects = DataManager.get(dco.getModule().getIndex(), df);
                
                if (objects != null) {
                    for (int i = 0; i < objects.length; i++) {
                        templates.add(objects[i]);
                    }
                }
            }
        }
    }
    
    public static ITemplate getDefault(int module) {
        for (Iterator iter = templates.iterator(); iter.hasNext(); ) {
            ITemplate template = (ITemplate) iter.next();
            
            if (template.getModule().getIndex() == module && template.isDefault())
                return template;
        }
        
        return null;
    }

    public static Collection getTemplates(int idx) {
        Collection result = new ArrayList();
        for (Iterator iter = templates.iterator(); iter.hasNext(); ) {
            ITemplate template = (ITemplate) iter.next();
            
            if (template.getModule().getIndex() == idx)
                result.add(template.clone());
        }
        return result;
    }
}
