/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.services;

import net.sf.dc.core.modules.DcModules;
import net.sf.dc.processes.http.AmazonAudioCdUpdate;
import net.sf.dc.processes.http.AmazonBookUpdate;
import net.sf.dc.processes.http.AmazonMovieUpdate;
import net.sf.dc.processes.http.AmazonSoftwareUpdate;
import net.sf.dc.processes.http.IOnlineUpdateListener;
import net.sf.dc.processes.http.InternetUpdateProcess;

public class DcAmazonServer extends DcServer {

    public DcAmazonServer() {
        this("Amazon");
    }
    
    public DcAmazonServer(String name) {
        super(name, "http://webservices.amazon.com", true);
        addRegion(new DcRegion("", "United States (english)", "http://webservices.amazon.com"));
        addRegion(new DcRegion("uk", "United Kingdom (english)", "http://webservices.amazon.co.uk"));
        addRegion(new DcRegion("de", "Germany (german)", "http://webservices.amazon.de"));
        addRegion(new DcRegion("fr", "France (french)", "http://webservices.amazon.fr"));
        addRegion(new DcRegion("ca", "Canada (english)", "http://webservices.amazon.ca"));
    }
    
    public InternetUpdateProcess getProcess(IOnlineUpdateListener listener, 
                                            String searchString, 
                                            DcSearchMode mode, 
                                            DcRegion region) {
        
        int currentModule = DcModules.getCurrentModule().getIndex();
        
        if (currentModule == DcModules._SOFTWARE) {
            return new AmazonSoftwareUpdate(listener, searchString, mode, region);
        } else if (currentModule == DcModules._MOVIE) {
            return new AmazonMovieUpdate(listener, searchString, mode, region);
        } else if (currentModule == DcModules._BOOK) {
            return new AmazonBookUpdate(listener, searchString, mode, region);
        } else if (currentModule == DcModules._AUDIOCD) {
            return new AmazonAudioCdUpdate(listener, searchString, mode, region);
        }

        return null;
    }
}
