/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.services;

import java.util.ArrayList;
import java.util.Collection;

import net.sf.dc.processes.http.IOnlineUpdateListener;
import net.sf.dc.processes.http.InternetUpdateProcess;

public abstract class DcServer {
    
    private final String displayName;
    private final String url;
    private final boolean isKeywordSearchAllowed;
    
    private final Collection modes = new ArrayList();
    private final Collection regions = new ArrayList();
    
    public DcServer(String displayName, String url, boolean isKeywordSearchAllowed) {
        this.displayName = displayName;
        this.url = url;
        this.isKeywordSearchAllowed = isKeywordSearchAllowed;
    }
    
    public abstract InternetUpdateProcess getProcess
        (IOnlineUpdateListener listener, String searchString, DcSearchMode mode, DcRegion region);
    
    public void addMode(DcSearchMode mode) {
        modes.add(mode);
    }
    
    public void addRegion(DcRegion region) {
        regions.add(region);
    }
    
    public String getUrl() {
        return url;
    }

    public String getName() {
        return displayName;
    }
    
    public Collection getModes() {
        return modes;
    }
    
    public Collection getRegions() {
        return regions;
    }
    
    public boolean hasModes() {
        return modes != null && modes.size() > 0;
    }
    
    public boolean isKeywordSearchAllowed() {
        return isKeywordSearchAllowed;
    }
    
    public String toString() {
        return displayName;
    }
    
    public boolean equals(Object o) {
        boolean equals = false;
        if (o instanceof DcServer) {
            equals = ((DcServer) o).getName().equals(getName());
        }
        return equals;
    }
}
