/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2007 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.services;

import net.sf.dc.processes.http.IOnlineUpdateListener;
import net.sf.dc.processes.http.InternetUpdateProcess;
import net.sf.dc.processes.http.wikipedia.WikipediaDutchMovieUpdate;
import net.sf.dc.processes.http.wikipedia.WikipediaEnglishMovieUpdate;
import net.sf.dc.processes.http.wikipedia.WikipediaGermanMovieUpdate;
import net.sf.dc.processes.http.wikipedia.WikipediaItalionMovieUpdate;
import net.sf.dc.processes.http.wikipedia.WikipediaSpanishMovieUpdate;

public class DcWikipediaServer extends DcServer {

    public DcWikipediaServer() {
        super("Wikipedia", "http://www.google.nl/search?q=site%3Awikipedia.org+", true);
        
        addRegion(new DcRegion("en", "Default (english)", "http://en.wikipedia.org/"));
        addRegion(new DcRegion("nl", "Dutch", "http://nl.wikipedia.org/"));
        addRegion(new DcRegion("de", "German", "http://de.wikipedia.org/"));
        addRegion(new DcRegion("it", "Italien", "http://it.wikipedia.org/"));
        addRegion(new DcRegion("es", "Spanish", "http://es.wikipedia.org/"));
    }
    
    public InternetUpdateProcess getProcess(IOnlineUpdateListener listener, 
            String searchString, 
            DcSearchMode mode, 
            DcRegion region) {

        if (region.getCode().equals("en"))
            return new WikipediaEnglishMovieUpdate(listener, this, region, searchString);
        else if (region.getCode().equals("nl"))
            return new WikipediaDutchMovieUpdate(listener, this, region, searchString);
        else if (region.getCode().equals("de"))
            return new WikipediaGermanMovieUpdate(listener, this, region, searchString);
        else if (region.getCode().equals("it"))
            return new WikipediaItalionMovieUpdate(listener, this, region, searchString);        
        else if (region.getCode().equals("es"))
            return new WikipediaSpanishMovieUpdate(listener, this, region, searchString);        
        
        return null;
    }     
}
