/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.db;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.StringTokenizer;

import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.console.dialogs.QuestionBox;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.modules.DcGenreModule;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcGenre;
import net.sf.dc.core.objects.DcMapping;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.exceptions.ValidationException;
import net.sf.dc.messages.Messages;
import net.sf.dc.messages.MessagesWriter;


public class DatabaseConversion {
    
    public void start() {
        convertGenres();
    }
    
    public void convertGenres() {

        Connection connection = DatabaseManager.getConnection();
      
        try {
            Statement stmt = connection.createStatement();
            
            try {
                ResultSet rs = stmt.executeQuery("SELECT TOP 1 Genre from AudioCDs;");
                rs.close();
            } catch (Exception exp) {
                return;
            }
          
            new QuestionBox("The Genres for movies, audio cd's and music albums needs to be converted. Do you want to continue ?");

            if (Boolean.FALSE.equals(QuestionBox.result))
                System.exit(0);
      
            DataCrow.closeSplashScreen();
          
            MessagesWriter._windowShow = true;
            MessagesWriter._windowToFront = true;
          
            convertGenres(DcModules.get(DcModules._AUDIOCD), DcModules.get(DcModules._AUDIOCDGENREMAPPING), stmt);
            convertGenres(DcModules.get(DcModules._AUDIOTRACK), DcModules.get(DcModules._AUDIOTRACKGENREMAPPING), stmt);
            convertGenres(DcModules.get(DcModules._MUSICALBUM), DcModules.get(DcModules._MUSICALBUMGENREMAPPING), stmt);
            convertGenres(DcModules.get(DcModules._MUSICTRACK), DcModules.get(DcModules._MUSICTRACKGENREMAPPING), stmt);
            convertGenres(DcModules.get(DcModules._MOVIE), DcModules.get(DcModules._MOVIEGENREMAPPING), stmt);
          
            stmt.close();

            MessagesWriter._windowToFront = false;
            MessagesWriter._windowShow = false;
            new MessageBox("Conversion was successfully completed. Data Crow will now continue.", MessageBox._INFORMATION);

        } catch (Exception exp) {
            Messages.add(exp);
            new MessageBox("Conversion was unsuccessful! " + exp.getMessage(), MessageBox._ERROR);
        }
      
        try {
            connection.close();
        } catch (Exception exp) {}
        
    }
    
    private void convertGenres(DcModule module, DcModule mappingModule, Statement stmt) throws SQLException, ValidationException {
        DcGenreModule genreMod = module.getGenreModule();

        Messages.add("Processing Genres for " + module, Messages._SIMPLE);
        
        ResultSet rs = stmt.executeQuery("SELECT ID, Genre from " + module.getTableName() + " where Genre IS NOT NULL;");
        while (rs.next()) {
            String mediaID = rs.getString("ID");

            String genre = rs.getString("Genre").toUpperCase();
            
            Collection genreNames = new ArrayList();
            if (genre.indexOf(",") > -1) {
                StringTokenizer st = new StringTokenizer(genre, ",");
                while (st.hasMoreTokens()) {
                    String genreName = st.nextToken();
                    genreNames.add(genreName.trim());
                }
            } else {
                genreNames.add(genre.trim());
            }
            
            
            for (Iterator iter = genreNames.iterator(); iter.hasNext(); ) {
                String genreName = (String) iter.next();
                
                ResultSet rsGenre =
                    stmt.executeQuery("SELECT TOP 1 * FROM " + genreMod.getTableName() + 
                                      " WHERE UPPER (" + genreMod.getField(DcGenre._A_NAME).getDatabaseFieldName() + ")" +
                                      " = '" + genreName + "'");
                String genreID = null;
                while (rsGenre.next()) {
                    genreID = rsGenre.getString("ID");
                }
                
                try { rsGenre.close(); } catch (Exception e) {}
                
                if (genreID == null) {
                    DcObject dco = genreMod.getDcObject();
                    dco.setValue(DcGenre._A_NAME, genreName);
                    dco.setIDs();
                    dco.keepInSync(false);
                    dco.saveNew(false);
                    genreID = dco.getID();
                }
                
                DcMapping mapping = (DcMapping) mappingModule.getDcObject();
                mapping.setValue(DcMapping._A_MEDIA_OBJECT_ID, mediaID);
                mapping.setValue(DcMapping._B_SIMPLE_OBJECT_ID, genreID);
                
                mapping.keepInSync(false);
                mapping.saveNew(false);
            }
        }
        
        stmt.execute("ALTER TABLE " + module.getTableName() + " DROP COLUMN Genre");
        
        try {
             rs.close();
        } catch (Exception exp) {}
        
        Messages.add("Successfully converted the genres for " + module, Messages._SIMPLE);
    }
}
