/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.processes.cdimport;

import java.awt.Toolkit;
import java.io.File;
import java.lang.ref.SoftReference;
import java.util.Date;

import net.sf.dc.console.importcd.ImportCDDialog;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.DcStorageMedium;
import net.sf.dc.core.objects.Picture;
import net.sf.dc.core.objects.image.Image;
import net.sf.dc.messages.Messages;
import net.sf.ext.util.Utilities;

import com.drew.imaging.jpeg.JpegMetadataReader;
import com.drew.imaging.jpeg.JpegProcessingException;
import com.drew.metadata.Directory;
import com.drew.metadata.Metadata;
import com.drew.metadata.exif.ExifDirectory;
import com.drew.metadata.iptc.IptcDirectory;

public class ImportImageCD extends ImportCDProcess {

    private static final Toolkit tk = Toolkit.getDefaultToolkit();
    
    public ImportImageCD(  ImportCDDialog ui,
                           DcStorageMedium storageMedium,
                           String sourceDir,
                           boolean confirmDirs,
                           boolean recursiveDirs,
                           String cdName) {

        super(ui, storageMedium, sourceDir, confirmDirs, recursiveDirs, cdName);
    }

    public String[] getExtensions() {
        return new String[] {"jpg", "gif", "jpeg", "png"};
    }

    protected int getModule() {
        return DcModules._IMAGE;
    }

    protected DcObject parseInformation(String filename) {
        Image image = new Image();
        try {
            // load image without adding it to the cache
            
            java.awt.Image awtImg = tk.getImage(filename);
            new SoftReference(awtImg);
            
            image.setIDs();
            
            String title = filename.substring(filename.lastIndexOf(File.separator) + 1, filename.lastIndexOf("."));
            image.setValue(Image._A_TITLE, title);
            
            int width = awtImg.getWidth(null);
            int height = awtImg.getHeight(null);
            
            image.setValue(Image._F_WIDTH, width != -1 ? new Integer(width) : null);
            image.setValue(Image._G_HEIGHT, height != -1 ? new Integer(height) : null);
            image.setValue(Image._H_FILENAME, filename);
            image.setValue(Image._M_FILESIZE, new Integer(getFileSize(filename)));
            
            Picture pic = new Picture(); 
            byte[] img = Utilities.readFile(new File(filename));
            pic.setValue(Picture._A_OBJECTID, image.getID());
            pic.setValue(Picture._B_FIELD, image.getField(Image._I_IMAGE).getDatabaseFieldName());
            pic.setValue(Picture._C_FILENAME, filename);
            pic.setValue(Picture._D_IMAGE, img);
            pic.isNew(true);
            
            image.setValue(Image._I_IMAGE, pic);
            
            File jpegFile = new File(filename); 
            
            try {
                Metadata metadata = JpegMetadataReader.readMetadata(jpegFile);
                if (metadata.containsDirectory(ExifDirectory.class)) {
                    Directory exifDirectory = metadata.getDirectory(ExifDirectory.class);
                    
                    String camera = exifDirectory.getString(ExifDirectory.TAG_MODEL);
                    int compression = exifDirectory.getInt(ExifDirectory.TAG_COMPRESSION);
                    Date date = exifDirectory.getDate(ExifDirectory.TAG_DATETIME);
                    String description = exifDirectory.getString(ExifDirectory.TAG_IMAGE_DESCRIPTION);
    
                    image.setValue(Image._N_DATE, date);
                    image.setValue(Image._B_DESCRIPTION, description);
                    image.setValue(Image._Q_CAMERA, camera);
                    image.setValue(Image._O_COMPRESSION, new Integer(compression));
                }
                
                if (metadata.containsDirectory(IptcDirectory.class)) {
                    Directory iptcDirectory = metadata.getDirectory(IptcDirectory.class);
                    
                    String city = iptcDirectory.getString(IptcDirectory.TAG_CITY);
                    String country = iptcDirectory.getString(IptcDirectory.TAG_COUNTRY_OR_PRIMARY_LOCATION);
                    String state = iptcDirectory.getString(IptcDirectory.TAG_PROVINCE_OR_STATE);
                    
                    String location = "";
                    location += country != null ? country + ", " : "";
                    location += state != null ? state  + ", " : "";
                    location += city != null ? city : "";
                    
                    image.setValue(Image._P_PLACE, location);
                }
                
                awtImg.flush();
                
            } catch (JpegProcessingException jpe) {}
            
        } catch (Exception exp) {
            Messages.add(exp);
        }
        
        return image;
    }
}
