/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.processes.cdimport;

import net.sf.dc.console.importcd.ImportCDDialog;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.DcStorageMedium;
import net.sf.dc.core.objects.movie.Movie;
import net.sf.dc.core.objects.movie.MovieFile;
import net.sf.dc.messages.Messages;
import net.sf.ext.mediamanager.FilePropertiesMovie;
import net.sf.ext.util.Utilities;

public class ImportMovieCD extends ImportCDProcess {

    public ImportMovieCD(  ImportCDDialog ui,
                           DcStorageMedium storageMedium,
                           String sourceDir,
                           boolean confirmDirs,
                           boolean recursiveDirs,
                           String cdName) {

        super(ui, storageMedium, sourceDir, confirmDirs, recursiveDirs, cdName);
    }

    public String[] getExtensions() {
        return new String[] {"mpg",
                "mpeg",
                "riff",
                "avi",
                "vob",
                "ogm",
                "mov",
                "ifo",
                "mkv",
                "asf"};
    }
    
    protected int getModule() {
        return DcModules._MOVIE;
    }

    protected DcObject parseInformation(String filename) {
        Movie movie = new Movie();
        try {
            movie.setValue(Movie._M_FILENAME, filename);
            movie.setValue(Movie._D_LOCATION, cdName);
            movie.setValue(Movie._4_FILESIZE, new Integer(getFileSize(filename)));
            movie.setValue(Movie._A_TITLE, getName(filename));
            movie.setValue(Movie._2_SUBTITLELANGUAGE, Utilities.fillSubtitles(filename, ""));

            FilePropertiesMovie movieFile1 = new FilePropertiesMovie(filename);
            Integer playlength = new Integer(movieFile1.getDuration());

            if (playlength.intValue() <= 0 && movieFile1.getVideoWidth() <= 0) {
                return movie;
            }

            movie.setValue(Movie._L_PLAYLENGTH, playlength);
            movie.setValue(Movie._N_VIDEOCODEC, movieFile1.getVideoCodec());
            movie.setValue(Movie._O_AUDIOCODEC, movieFile1.getAudioCodec());
            movie.setValue(Movie._P_WIDTH, new Integer(movieFile1.getVideoWidth()));
            movie.setValue(Movie._Q_HEIGHT, new Integer(movieFile1.getVideoHeight()));
            movie.setValue(Movie._R_FPS, new Integer(movieFile1.getVideoRate()));
            movie.setValue(Movie._S_FRAMES, new Integer(movieFile1.getDuration() * movieFile1.getVideoRate()));
            movie.setValue(Movie._T_AUDIOBITRATE, new Integer(movieFile1.getAudioBitRate()));
            movie.setValue(Movie._U_AUDIOSAMPLINGRATE, new Integer(movieFile1.getAudioRate()));
            movie.setValue(Movie._V_AUDIOCHANNEL, new Integer(movieFile1.getAudioChannels()));
            movie.setValue(Movie._2_SUBTITLELANGUAGE, movieFile1.getSubtitles());
            movie.setValue(Movie._7_STORAGEMEDIUM, storageMedium);
            
            int bitrate = movieFile1.getVideoBitRate();
            if (bitrate <= 0) {
                try {
                    MovieFile movieFile2 = new MovieFile(filename);
                    bitrate = new Double(movieFile2.getVideoBitrate()).intValue();
                } catch (Exception ignore) {}
            }

            movie.setValue(Movie._W_VIDEOBITRATE, new Integer(bitrate));
        } catch (Exception exp) {
            Messages.add(exp);
        }
        return movie;
    }    
}
