/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.processes.cdimport;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.StringTokenizer;

import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.console.importcd.ImportCDDialog;
import net.sf.dc.console.importcd.ImportMusicFilesDialog;
import net.sf.dc.console.views.View;
import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcGenre;
import net.sf.dc.core.objects.DcMapping;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.DcStorageMedium;
import net.sf.dc.core.objects.audiocd.AudioCD;
import net.sf.dc.core.objects.musicalbum.MusicAlbum;
import net.sf.dc.core.objects.musicalbum.MusicFile;
import net.sf.dc.core.objects.musicalbum.MusicTrack;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DcResources;
import net.sf.ext.util.Converter;

public class ImportMusicFilesCD extends ImportCDProcess {

    public static final int _DONOTUSE = 0;
    public static final int _DIRISALBUM = 1;
    public static final int _DIRISARTISTS = 2;
    public static final int _DIRISARTIST_SUBDIRISALBUM = 3;
    
    private int dirUsage;
    
    public ImportMusicFilesCD( ImportCDDialog ui,
                               DcStorageMedium storageMedium,
                               String sourceDir,
                               boolean confirmDirs,
                               boolean recursiveDirs,
							   String cdName,
                               int dirUsage) {
        
    	super(ui, storageMedium, sourceDir, confirmDirs, recursiveDirs, cdName);
        
        this.dirUsage = dirUsage;
    }

    protected int getModule() {
        return DcModules._MUSICALBUM;
    }


    public String[] getExtensions() {
        return new String[] {"mp3", "ogg", "mp4", "flac", "flc", "ape", "asf", "wav", "mpc"};
    }
    
    public void writeOutput(Collection filenames) {
        try {
            ui.addMessage(DcResources.getText("msgImportFoundXResults", "" + filenames.size()));
            ui.initProgressBar(filenames.size());
            int counter = 1;
            
            Collection albums = new ArrayList();
            for (   Iterator iter = filenames.iterator(); iter.hasNext()
                    && keepOnRunning; counter++) {

                String sFilename = iter.next().toString();
                ui.addMessage(DcResources.getText("msgProcessingFileX", sFilename));
                DcObject dco = parseInformation(sFilename, albums);

                if (dco != null) {
                    dco.applyTemplate();
                    dco.setIDs();

                    String title = (String) dco.getValue(MusicAlbum._A_TITLE);
                    if (getAlbum(title, albums) == null)
                        albums.add(dco);
                }

                try {
                    sleep(100);
                } catch (Exception ignore) {}

                ui.updateProgressBar(counter);
            }
            
            // use online service (etc) to get missing information
            if (keepOnRunning)
                onlineUpdate(albums);
            
            ui.initProgressBar(albums.size());
            counter = 1;
            
            DcModule module = DcModules.get(getModule());
            View view = module.getCurrentInsertView();
            view.add(albums);
            
        } catch (Exception exp) {
            new MessageBox(ui, DcResources.getText("msgProcessCrash"), MessageBox._ERROR);
            Messages.add(exp);
        } finally {
            filenames.clear();
            filenames = null;
            ui.addMessage(DcResources.getText("msgImportStops"));
            ui.allowActions();
        }
    }     

    private void onlineUpdate(Collection albums) {
        
        ImportMusicFilesDialog dlg =((ImportMusicFilesDialog) ui);
        
        if (dlg.useOnlineService()) {
            int counter = 1;
            ui.initProgressBar(albums.size());
            
            for (Iterator iter = albums.iterator(); iter.hasNext() && keepOnRunning; counter++) {
                MusicAlbum ma = (MusicAlbum) iter.next();
                
                ui.addMessage(DcResources.getText("msgSearchingOnlineFor", "" + ma));
                
                MusicFile musicFile = new MusicFile();
                musicFile.onlineUpdate(ma, dlg.getServer(), dlg.getRegion());
                
                ui.updateProgressBar(counter);
            }
        }
    }
    
    protected DcObject parseInformation(String filename) {
        return null;
    }
    
    protected DcObject parseInformation(String filename, Collection albums) {
        String[] extensions = getExtensions();
        boolean equals = false;
        for (int i = 0; i < extensions.length; i++) {
            if (filename.toLowerCase().endsWith(extensions[i])) {
            	equals = true;
                break;
            }
        }

        if (!equals)
    		return null;

        return read(filename, albums);
    }

    private MusicAlbum read(String sFile, Collection albums) {
        File file = new File(sFile);
        MusicAlbum ma = parseWithEntagged(file, albums);
        
        if (ma != null) {
            ma.setValue(MusicAlbum._I_STORAGEMEDIUM, storageMedium);
            ma.setValue(MusicAlbum._D_LOCATION, cdName);
        }
        return ma;
    }
    
    public MusicAlbum getAlbum(String title, Collection albums) {
    	if (title == null)
    		return null;
    	
        for (Iterator iter = albums.iterator(); iter.hasNext(); ) {
            MusicAlbum ma = (MusicAlbum) iter.next();
            
            String albumName = (String) ma.getValue(MusicAlbum._A_TITLE);
            albumName = albumName == null ? "" : albumName;
            if (albumName.toLowerCase().equals(title.toLowerCase()))
                return ma;
        }
        return null;
    }
    
    private String getDirectoryName(String filename, int pos) {
        
        String dir = filename.indexOf(sourceDir) > -1 ?
                     filename.substring(filename.indexOf(sourceDir) + sourceDir.length(), filename.length()) : filename;
        
        StringTokenizer st = new StringTokenizer(dir, File.separator);
        
        LinkedList ll = new LinkedList();
        while (st.hasMoreElements()) {
        	ll.add(st.nextToken());
        }
        
        String s = null;
        if (pos + 1 < ll.size()) {
            try {
            	s = (String) ll.get(pos);	
            } catch (Exception ignore) {}
        }
        
        return s;
    }
    
    private MusicAlbum parseWithEntagged(File file, Collection albums) {
        MusicAlbum ma = new MusicAlbum();

        try {
            String filename = file.toString();
            MusicFile musicFile = new MusicFile(filename);
            
            String artist = musicFile.getArtist();
            String album = musicFile.getAlbum();
            
            String s = null; 
            switch (dirUsage) {
            case _DONOTUSE:
                break;
            case _DIRISALBUM:
            	s = getDirectoryName(filename, 0);
            	album = s != null ? s : album;
                break;
            case _DIRISARTISTS:
            	s = getDirectoryName(filename, 0);
            	artist = s != null ? s : artist;
            	break;
            case _DIRISARTIST_SUBDIRISALBUM:
            	s = getDirectoryName(filename, 0);
            	artist = s != null ? s : artist;

            	s = getDirectoryName(filename, 1);
            	album = s != null ? s : album;

            	break;
            }

            if (album != null) {
                album = Converter.databaseValueConverter(album);
            	ma = getAlbum(album, albums);
            }
            
            if (ma == null) {
                ma = new MusicAlbum();
                ma.setValue(MusicAlbum._A_TITLE, album);
                ma.setValue(MusicAlbum._F_ARTIST, artist);
            }
            
            String sGenre = musicFile.getGenre();
            Collection genres = DataManager.createMapping(ma, sGenre, 
                                                          DcModules._MUSICGENRE, 
                                                          DcModules._MUSICALBUMGENREMAPPING);
            if (genres.size() > 0)
                ma.setValue(AudioCD._G_GENRES, genres);
            
            DcGenre genre = (DcGenre) DataManager.getObjectForDisplayValue(DcModules._MUSICGENRE, sGenre);            
            
            if (ma != null) {
                MusicTrack mt = new MusicTrack();
                
                mt.setValue(MusicTrack._K_QUALITY, new Integer(musicFile.getBitrate()));
                mt.setValue(MusicTrack._J_PLAYLENGTH, new Integer(musicFile.getLength()));
                mt.setValue(MusicTrack._L_ENCODING, musicFile.getEncodingType());
                mt.setValue(MusicTrack._A_TITLE, musicFile.getTitle());
                mt.setValue(MusicTrack._G_ARTIST, artist);
                mt.setValue(MusicTrack._C_YEAR, musicFile.getYear());
                
                if (genre != null) {
                    Collection mappings = new ArrayList();
                    DcMapping mapping = (DcMapping) DcModules.get(DcModules._MUSICTRACKGENREMAPPING).getDcObject();
                    mapping.setValue(DcMapping._B_SIMPLE_OBJECT_ID, genre.getID());
                    mappings.add(mapping);
                    mt.setValue(MusicTrack._H_GENRES, mappings);                   
                }
                
                mt.setValue(MusicTrack._F_TRACKNUMBER, new Integer(musicFile.getTrack()));
                mt.setValue(MusicTrack._I_FILENAME, filename);
                mt.setValue(MusicTrack._N_FILESIZE, new Integer(getFileSize(file.toString())));
                
                ma.addChild(mt);
            }
        } catch (Exception exp) {
            Messages.add(exp);
        }

        return ma;
    }
}
