/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.processes.http;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcGenre;
import net.sf.dc.core.objects.DcMapping;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.audiocd.AudioCD;
import net.sf.dc.core.objects.audiocd.AudioTrack;
import net.sf.dc.core.services.DcRegion;
import net.sf.dc.core.services.DcSearchMode;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DcResources;
import net.sf.ext.util.StringUtils;

public class AmazonAudioCdUpdate extends AmazonUpdate {

    public AmazonAudioCdUpdate(IOnlineUpdateListener listener, String searchString, DcSearchMode mode, DcRegion region) {
        super(listener, searchString, mode, region);
    }

    protected Collection convertXmlToObjects(String xmlString, String locale) {
        Collection objects = new ArrayList();
        
        AudioCD base = new AudioCD();

        base.setValue(AudioCD._A_TITLE, getTitle(xmlString));
        base.setValue(AudioCD._B_DESCRIPTION, getDescription(xmlString));
        base.setValue(AudioCD._C_YEAR, getReleaseDate(xmlString));
        base.setValue(AudioCD._E_RATING, getRating(xmlString));
        base.setValue(AudioCD._F_ARTIST, getArtists(xmlString));
        base.setValue(AudioCD._E_RATING, getRating(xmlString));

        String sGenre = getStyle(xmlString);
        Collection genres = DataManager.createMapping(base, sGenre, 
                                                      DcModules._MUSICGENRE, 
                                                      DcModules._AUDIOCDGENREMAPPING);
        if (genres.size() > 0)
            base.setValue(AudioCD._G_GENRES, genres);
        
        DcGenre genre = (DcGenre) DataManager.getObjectForDisplayValue(DcModules._MUSICGENRE, sGenre);
        
        base.setValue(AudioCD._M_WEBPAGE, getAmazonURL(xmlString));
        setImages(base, xmlString, AudioCD._H_PICTUREFRONT, null);
        
        Collection trackSets = getAllTagValues("<Disc Number=", "</Disc>", xmlString);
        try {
            int disc = 1;
            for (Iterator iter = trackSets.iterator(); iter.hasNext(); disc++) {
                String trackSet = (String) iter.next();
                
                Collection tracks = getAllTagValues("<Track Number=", "</Track>", trackSet);
                DcObject audioCD = (DcObject) base.clone();
                audioCD.setChildren(new ArrayList());
                if (trackSets.size() > 1) {
                    String title = (String) audioCD.getValue(AudioCD._A_TITLE);
                    String addition = DcResources.getText("lblDiscXofX", new String[] {"" + disc, "" + trackSets.size()});
                    title += " " + addition;
                    audioCD.setValue(AudioCD._A_TITLE, title);
                }
                
                int counter = 1;
                for (Iterator iterTracks = tracks.iterator(); iterTracks.hasNext(); counter++) {
                    String track = (String) iterTracks.next();
                    String title = track.substring(track.indexOf(">") + 1, track.length());
                    AudioTrack audioTrack = new AudioTrack();
                    audioTrack.setValue(AudioTrack._A_TITLE, title);
                    audioTrack.setValue(AudioTrack._F_TRACKNUMBER, new Integer(counter));
                    
                    if (genre != null) {
                        Collection mappings = new ArrayList();
                        DcMapping mapping = (DcMapping) DcModules.get(DcModules._AUDIOTRACKGENREMAPPING).getDcObject();
                        mapping.setValue(DcMapping._B_SIMPLE_OBJECT_ID, genre.getID());
                        mappings.add(mapping);
                        audioTrack.setValue(AudioTrack._J_GENRES, mappings);
                    }
                    
                    audioCD.addChild(audioTrack);
                }

                audioCD.setIDs();
                objects.add(audioCD);
            }
        } catch (Exception exp) {
            Messages.add(exp);
        }
        
        return objects;
    }
    
    private String getArtists(String xmlString) {
         String artists = StringUtils.getValueBetweenStrings("<Artist>", "</Artist>", xmlString);
         if (artists == null || artists.trim().equals("")) {
             Collection c = getAllTagValues("<Creator Role=", "</Creator>", xmlString);
             artists = "";
             int counter = 0;
             for (Iterator iter = c.iterator(); iter.hasNext(); counter++) {
                 String artistRaw = (String) iter.next(); 
                 String artist = artistRaw.substring(artistRaw.indexOf(">") + 1, artistRaw.length());

                 if (counter > 0)
                     artists += ", ";
                 
                 artists += artist;
             }
         }
         
         return StringUtils.convertHttpString(artists.trim());
    }
}
