/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.processes.http;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import net.sf.dc.core.objects.book.Book;
import net.sf.dc.core.objects.book.InvalidBarCodeException;
import net.sf.dc.core.objects.book.Isbn;
import net.sf.dc.core.services.DcRegion;
import net.sf.dc.core.services.DcSearchMode;
import net.sf.dc.messages.Messages;
import net.sf.ext.util.StringUtils;

public class AmazonBookUpdate extends AmazonUpdate {

    public AmazonBookUpdate(IOnlineUpdateListener listener, String searchString, DcSearchMode mode, DcRegion region) {
        super(listener, searchString, mode, region);
    }

    protected Collection convertXmlToObjects(String xmlString, String locale) {
        Book book = new Book();

        book.setValue(Book._A_TITLE, getTitle(xmlString));
        book.setValue(Book._B_DESCRIPTION, getDescription(xmlString));
        book.setValue(Book._F_PUBLISHER, getPublisher(xmlString));
        book.setValue(Book._G_AUTHOR, getAuthors(xmlString));
        book.setValue(Book._C_YEAR, getPublicationDate(xmlString));
        book.setValue(Book._H_WEBPAGE, getAmazonURL(xmlString));
        book.setValue(Book._E_RATING, getRating(xmlString));
        
        String s = StringUtils.getValueBetweenStrings("<ISBN>", "</ISBN>", xmlString);
        Isbn isbn = new Isbn();
        
        try {
            if (isbn.isIsbn10(s)) {
                book.setValue(Book._J_ISBN10, s);
                book.setValue(Book._N_ISBN13, isbn.getIsbn13(s));
            } else if (isbn.isIsbn13(s)) {
                book.setValue(Book._N_ISBN13, s);
                book.setValue(Book._J_ISBN10, isbn.getIsbn10(s));
            }
        } catch (InvalidBarCodeException ibe) {
            Messages.add(ibe);
        }
        
        setImages(book, xmlString, Book._K_PICTUREFRONT, null);

        Collection objects = new ArrayList();
        objects.add(book);
        return objects;
    }
    
    private String getAuthors(String xmlString) {
        Collection c = getAllTagValues("<Author>", "</Author>", xmlString);
        
        String authors = "";
        int counter = 0;
        for (Iterator iter = c.iterator(); iter.hasNext(); counter++) {
            if (counter > 0)
                authors += ", ";
            authors += iter.next();
        }
        return StringUtils.convertHttpString(authors);
    }
}
