/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.processes.http;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.movie.Movie;
import net.sf.dc.core.services.DcImdbServer;
import net.sf.dc.core.services.DcRegion;
import net.sf.dc.core.services.DcSearchMode;
import net.sf.dc.messages.Messages;

public class AmazonImdbMovieUpdate extends AmazonMovieUpdate implements IOnlineUpdateListener {

    private Collection imdbMovies = new ArrayList();
    
    public AmazonImdbMovieUpdate(IOnlineUpdateListener listener, String searchString, DcSearchMode mode, DcRegion region) {
        super(listener, searchString, mode, region);
    }
    
    protected Collection convertXmlToObjects(String xmlString, String locale) {
        Collection objects = super.convertXmlToObjects(xmlString, locale);

        DcImdbServer imdbServer = new DcImdbServer();
        String code = region.getCode();
        code = code.equals("") ? "us" : code;
        
        String imdbUrl = imdbServer.getUrl();
        for (Iterator iter = imdbServer.getRegions().iterator(); iter.hasNext(); ) {
            DcRegion imdbRegion = (DcRegion) iter.next();
            if (imdbRegion.getCode().equals(code)) {
                imdbUrl = imdbRegion.getUrl();
                break;
            }
        }
        
        for (Iterator iter = objects.iterator(); iter.hasNext(); ) {
            Movie movie = (Movie) iter.next();
            
            String title = (String) movie.getValue(Movie._A_TITLE);
            if (title != null && title.trim().length() > 0) {
                title = getPlainTitle(title);

                IMDBMovieUpdate movieUpdate = new IMDBMovieUpdate(this, title, imdbUrl, 1);
                
                // not threaded
                movieUpdate.run();
                
                for (Iterator iterResult = imdbMovies.iterator(); iterResult.hasNext(); ) {
                    Movie imdbMovie = (Movie) iterResult.next();
                    String imdbTitle = getPlainTitle((String) imdbMovie.getValue(Movie._A_TITLE));
                    
                    if (imdbTitle.trim().toLowerCase().equals(title.trim().toLowerCase()))
                        movie.merge(imdbMovie);
                }
            }
        }
        imdbMovies.clear();
        return objects;
    }    
    
    private String getPlainTitle(String title) {
        String result = title;
       
        int end = title.indexOf("(") > -1 ? title.indexOf("(") : 
                    title.indexOf("[") > -1 ? title.indexOf("[") : -1;
       
        if (end > -1)
            result = title.substring(0, end);
        
        return result;
    }

    public void addError(Throwable t) {
        Messages.add(t);        
    }

    public void addError(String message) {}

    public void addMessage(String message) {}

    public void addObject(DcObject dco) {
        imdbMovies.add(dco);
    }

    public void addWarning(String warning) {}

    public DcModule getModule() {
        return getModule();
    }

    public void processed(int i) {}

    public void processing() {}

    public void processingTotal(int i) {}

    public int resultCount() {
        return imdbMovies.size();
    }

    public void stopped() {}
}
