/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.processes.http;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.software.Software;
import net.sf.dc.core.objects.software.SoftwarePlatform;
import net.sf.dc.core.services.DcRegion;
import net.sf.dc.core.services.DcSearchMode;

public class AmazonSoftwareUpdate extends AmazonUpdate {

    public AmazonSoftwareUpdate(IOnlineUpdateListener listener, String searchString, DcSearchMode mode, DcRegion region) {
        super(listener, searchString, mode, region);
    }

    protected Collection convertXmlToObjects(String xmlString, String locale) {
        Software software = new Software();

        DcObject storageMedium = getStorageMedium(xmlString, DataManager.get(DcModules._SOFTWARESTORAGE, null), "<Format>", "</Format>");

        software.setValue(Software._A_TITLE, getTitle(xmlString));
        software.setValue(Software._B_DESCRIPTION, getDescription(xmlString));
        
        software.setValue(Software._G_PUBLISHER, getPublisher(xmlString));
        software.setValue(Software._F_DEVELOPER, getManufacturer(xmlString));
        
        software.setValue(Software._C_YEAR, getReleaseDate(xmlString));
        software.setValue(Software._I_WEBPAGE, getAmazonURL(xmlString));
        software.setValue(Software._E_RATING, getRating(xmlString));
        software.setValue(Software._U_ASIN, getAsin(xmlString));
        
        software.setValue(Software._W_STORAGEMEDIUM, storageMedium);
        software.setValue(Software._H_PLATFORM, getPlatform(xmlString));

        setImages(software, xmlString,
                  Software._M_PICTUREFRONT, 
                  new int[] {Software._P_SCREENSHOTONE, Software._Q_SCREENSHOTTWO, Software._R_SCREENSHOTTHREE});
        
        Collection objects = new ArrayList();
        objects.add(software);
        return objects;
    }
    
    private DcObject getPlatform(String xmlString) {
        DcObject[] dcPlatforms = DataManager.get(DcModules._SOFTWAREPLATFORM, null); 
        Collection platforms = getAllTagValues("<Platform>", "</Platform>", xmlString);
        
        for (Iterator iter = platforms.iterator(); iter.hasNext(); ) {
            String platform = (String) iter.next();
            for (int i = 0; i < dcPlatforms.length; i++) {
                String dcPlatformName = dcPlatforms[i].getValue(SoftwarePlatform._A_NAME).toString().toLowerCase();
                if (platform.toLowerCase().equals(dcPlatformName)) 
                    return dcPlatforms[i];
            }        
        }
        
        for (Iterator iter = platforms.iterator(); iter.hasNext(); ) {
            String platform = (String) iter.next();
            for (int i = 0; i < dcPlatforms.length; i++) {
                String dcPlatformName = dcPlatforms[i].getValue(SoftwarePlatform._A_NAME).toString().toLowerCase();
                if (platform.toLowerCase().indexOf(dcPlatformName) > -1 || dcPlatformName.indexOf(platform) > -1)
                    return dcPlatforms[i];
            }        
        }  
        
        return null;
    }
}
