/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.processes.http;

import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;

import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.objects.DcMediaObject;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.DcStorageMedium;
import net.sf.dc.core.services.AsinSearchMode;
import net.sf.dc.core.services.BarcodeSearchMode;
import net.sf.dc.core.services.DcRegion;
import net.sf.dc.core.services.DcSearchMode;
import net.sf.dc.exceptions.ValidationException;
import net.sf.dc.http.HttpConnection;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.ext.util.StringUtils;

public abstract class AmazonUpdate extends InternetUpdateProcess {

    public static final int _KEYWORDSSEARCH = 0;
    public static final int _ASINSEARCH = 1;

    protected String searchString;

    private final String server;
    private final DcSearchMode mode;
    protected final DcRegion region;

    private final static Calendar cal = Calendar.getInstance();

    public AmazonUpdate(IOnlineUpdateListener listener, String searchString, DcSearchMode mode, DcRegion region) {
        super(listener);
        
        this.region = region;
        this.mode = mode;
        this.server = region.getUrl();
        this.searchString = searchString;
    }


    public void run() {
        try {
            search(getSearchURLs(searchString));
        } catch (Exception exp) {
            listener.addError(exp);
            listener.stopped();
        }
    }

    private String convertSearchString(String s) {
        String result = "";

		result = s.replaceAll("[]","e");
		result = s.replaceAll("[]","u");
		result = s.replaceAll("[]","i");
		result = s.replaceAll("[]","a");
		result = s.replaceAll("","o");

		result = s.replaceAll("[]","E");
		result = s.replaceAll("[]","U");
		result = s.replaceAll("[]","I");
		result = s.replaceAll("[]","A");
	    result = s.replaceAll("","O");
	    result = result.replaceAll(" ", "%20");
        
        if (mode instanceof AsinSearchMode) 
            result = s.replaceAll("-","");

        return result;
    }


    protected URL[] getSearchURLs(String keywords) throws Exception {
        Collection urls = new ArrayList();
        String s = convertSearchString(keywords);

        if (mode instanceof BarcodeSearchMode) {
            urls.add(new URL(server + "/onca/xml?Service=AWSECommerceService" +
                    "&AWSAccessKeyId=1N37KF9JXJXVF9JEFB02" +
                    "&" + mode.getSearchCommand(s) + "&IdType=UPC" +
                    "&ResponseGroup=Large"));
            urls.add(new URL(server + "/onca/xml?Service=AWSECommerceService" +
                    "&AWSAccessKeyId=1N37KF9JXJXVF9JEFB02" +
                    "&" + mode.getSearchCommand(s) + "&IdType=SKU" +
                    "&ResponseGroup=Large"));
            urls.add(new URL(server + "/onca/xml?Service=AWSECommerceService" +
                    "&AWSAccessKeyId=1N37KF9JXJXVF9JEFB02" +
                    "&" + mode.getSearchCommand(s) + "&IdType=EAN" +
                    "&ResponseGroup=Large"));
        } else {
            urls.add(new URL(server + "/onca/xml?Service=AWSECommerceService" +
                             "&AWSAccessKeyId=1N37KF9JXJXVF9JEFB02" +
                             "&" + mode.getSearchCommand(s) +
                             "&ResponseGroup=Large"));
        }

        URL[] u = new URL[urls.size()];
        return (URL[]) urls.toArray(u);
    }

    protected void search(URL[] urls) {
        listener.processing();
        isCancelled(false);

        try {
            listener.addMessage(DcResources.getText("msgConnectingToServer", server));
            
            String result = "";
            for (int i = 0 ; i < urls.length; i++) {
                HttpConnection connection = new HttpConnection(urls[i]);
                result += connection.getString();
                connection.close();
            }

            if (!isCancelled())
                createObjects(result);

            if (!isCancelled() && listener.resultCount() == 0)
                listener.addWarning(DcResources.getText("msgNoResultsForKeywords", searchString));

        } catch (Exception e) {
            listener.addError("Could not connect to Amazon! " + e.getMessage());
        } finally {
            listener.stopped();
        }
    }

    protected DcObject getStorageMedium(String xmlString, DcObject[] media, String tagStart, String tagEnd) {
        String medium = StringUtils.getValueBetweenStrings(tagStart, tagEnd, xmlString);
        if (medium != null && medium.trim().length() > 0) {
            for (int i = 0; i < media.length; i++) {
                if (    medium.toLowerCase().indexOf(media[i].toString().toLowerCase()) > -1 ||
                        media[i].toString().toLowerCase().indexOf(medium.toLowerCase()) > -1) {
                    return media[i];
                }
            }
        }

        // create the new medium
        if (    DcSettings.getValueAsBoolean(Repository.settings.stAutoCreateSubItems) &&
                medium != null && medium.trim().length() > 0) {
            
            DcModule module = listener.getModule().getStorageMediaModule();
            if (module != null) {
            	try {
	                DcObject dco = module.getDcObject();
	                dco.setIDs();
	                dco.setValue(DcStorageMedium._A_NAME, medium);
	                dco.saveNew(false);
	                return dco;
            	} catch (ValidationException exp) {}
            }
        }

        return null;
    }

    protected String getValues(String rawResult, String startTag, String endTag) {
        int indexBegin = 0;

        String values = "";
        indexBegin = rawResult.indexOf(startTag);
        String tmp = new String(rawResult);
        boolean first = true;
        while(indexBegin > -1) {
            indexBegin = indexBegin + (startTag.length());
            if (!first)
                values += ", ";

            String value = tmp.substring(indexBegin, tmp.length()).trim();
            value = value.substring(0, value.indexOf(endTag));

            if (value.indexOf(",") > -1) {
                String name = value.substring(0, value.indexOf(",")).trim();
                name = value.substring(value.indexOf(",") + 1, value.length()).trim() + " " + name;
                value = name;
            }

            values += value;

            tmp = tmp.substring(indexBegin + value.length() + endTag.length());
            indexBegin = tmp.indexOf(startTag);
            first = false;
        }

        return StringUtils.convertHttpString(values);
    }

    protected void createObjects(String rawResult) {
        Collection parts = new ArrayList();
        int index = rawResult.indexOf("<Item>");

        Collection asins = new ArrayList();
        int counter = 0;
        while (index > -1 && !isCancelled() && counter <  maximum) {
            int end = rawResult.indexOf("</Item>") + 7;
            if (end != -1) {
                String part = rawResult.substring(index, end);

                String asin = getAsin(part);
                if (!asins.contains(asin)) {
                    parts.add(part);
                    asins.add(asin);
                }

                rawResult = rawResult.substring(end, rawResult.length());
            }
            counter++;
            index = rawResult.indexOf("<Item>");
        }

        if (parts.size() == 0)
            return;

        listener.addMessage(DcResources.getText("msgFoundXResults", String.valueOf(parts.size())));
        listener.addMessage(DcResources.getText("msgStartParsingXResults", String.valueOf(parts.size())));
        listener.processingTotal(parts.size());
        counter = 1;
        for (Iterator iter = parts.iterator(); iter.hasNext() && !isCancelled(); ) {
            Collection objects = convertXmlToObjects(iter.next().toString(), region.getCode());
            for (Iterator iter2 = objects.iterator(); iter2.hasNext(); ) {
                DcObject dco = (DcObject) iter2.next();
                listener.addMessage(DcResources.getText("msgParsingSuccessfull", dco.getValue(DcMediaObject._A_TITLE).toString()));
                listener.addObject(dco);
            }
            listener.processed(counter++);
        }
        parts.clear();
        rawResult = null;
    }

    protected abstract Collection convertXmlToObjects(String s, String locale);

    protected String getGenre(String xmlString) {
        return getValueUpInTree("<Name>Genres</Name>", "<Name>", "</Name>", xmlString);
    }

    protected String getStyle(String xmlString) {
        return getValueUpInTree("<Name>Styles</Name>", "<Name>", "</Name>", xmlString);
    }

    private String getValueUpInTree(String base, String startTag, String endTag, String xmlString) {
        int end =  xmlString.indexOf(base);
        if (end > -1) {
            String part = xmlString.substring(0, end);
            int begin = part.lastIndexOf(startTag);
            end = part.lastIndexOf(endTag);

            if (begin > -1 && begin < end) {
                part = part.substring(begin, end + 10);
                return StringUtils.convertHttpString(StringUtils.getValueBetweenStrings(startTag, endTag, part));
            }
        }
        return null;
    }

    protected Integer getRating(String rawResult) {
        String rating = StringUtils.getValueBetweenStrings("<AverageRating>", "</AverageRating>", rawResult);
        Integer iRating = new Integer(0);
        try {
            if (rating.trim().length() > 0) {
                iRating = new Integer(Math.round(Float.valueOf(rating).floatValue()));
            }
        } catch (Exception ignore) {};

        return iRating;
    }

    protected String getAsin(String rawResult) {
        return StringUtils.convertHttpString(StringUtils.getValueBetweenStrings("<ASIN>", "</ASIN>", rawResult));
    }

    protected String getTitle(String rawResult) {
        return StringUtils.convertHttpString(StringUtils.getValueBetweenStrings("<Title>", "</Title>", rawResult));
    }

    protected String getAmazonURL(String rawResult) {
        return StringUtils.getValueBetweenStrings("<DetailPageURL>", "</DetailPageURL>", rawResult);
    }

    protected String getUPC(String rawResult) {
        return StringUtils.getValueBetweenStrings("<UPC>", "</UPC>", rawResult);
    }

    protected String getManufacturer(String rawResult) {
        return StringUtils.convertHttpString(StringUtils.getValueBetweenStrings("<Manufacturer>", "</Manufacturer>", rawResult));
    }

    protected String getPublisher(String rawResult) {
        return StringUtils.convertHttpString(StringUtils.getValueBetweenStrings("<Publisher>", "</Publisher>", rawResult));
    }

    protected Integer getTheatricalReleaseDate(String rawResult) {
        return getYear(rawResult, "<TheatricalReleaseDate>", "</TheatricalReleaseDate>");
    }

    protected Integer getReleaseDate(String rawResult) {
        return getYear(rawResult, "<ReleaseDate>", "</ReleaseDate>");
    }

    protected Integer getPublicationDate(String rawResult) {
        return getYear(rawResult, "<PublicationDate>", "</PublicationDate>");
    }

    protected Integer getYear(String rawResult, String startTag, String endTag) {
        Integer releaseYear = null;
        String releaseDate = StringUtils.getValueBetweenStrings(startTag, endTag, rawResult);

        try {
            Date date = new SimpleDateFormat("yyyy-MM-dd").parse(releaseDate);
            cal.setTime(date);
            releaseYear = new Integer(cal.get(Calendar.YEAR));
        } catch (Exception ignore) {
            try {
                releaseYear = new Integer(releaseDate.substring(0, 4));
            } catch (Exception ignore2) {}
        }

        return releaseYear;
    }

    protected void setImages(DcObject dco, String xmlString, int frontFieldIdx, int[] screenFieldIdx) {
        String mainImage = StringUtils.getValueBetweenStrings("<LargeImage>", "</LargeImage>", xmlString);
        String mainImageURL = StringUtils.getValueBetweenStrings("<URL>", "</URL>", mainImage);
        setImage(dco, frontFieldIdx, mainImageURL);

        if (screenFieldIdx != null) {
            Collection imageSets = getAllTagValues("<ImageSet Category=" + '"' + "variant", "</ImageSet>", xmlString);
            int counter = 0;
            for (Iterator iter = imageSets.iterator(); iter.hasNext() && counter < screenFieldIdx.length; counter++) {
                Collection screenshots = getAllTagValues("<LargeImage>", "</LargeImage>", (String) iter.next());
                for (Iterator iterScreens = screenshots.iterator(); iterScreens.hasNext();) {
                    String imageURL = StringUtils.getValueBetweenStrings("<URL>", "</URL>", (String) iterScreens.next());
                    setImage(dco, screenFieldIdx[counter], imageURL);
                }
            }
        }
    }

    protected Collection getAllTagValues(String startTag, String endTag, String rawResult) {
        Collection result = new ArrayList();

        if (rawResult.indexOf(startTag) > -1) {
            String platforms = rawResult.substring(rawResult.indexOf(startTag),
                                                   rawResult.lastIndexOf(endTag) + endTag.length());

            int indexBegin = platforms.indexOf(startTag);
            int indexEnd = 0;
            while(indexBegin > -1) {
                indexBegin = indexBegin + startTag.length();
                indexEnd = platforms.indexOf(endTag);

                String platform = platforms.substring(indexBegin, indexEnd) + " ";
                result.add(StringUtils.convertHttpString(platform));

                indexEnd = indexEnd + endTag.length();
                platforms = platforms.substring(indexEnd, platforms.length());
                indexBegin = platforms.indexOf(startTag);
            }
        }
        return result;
    }

    private void setImage(DcObject dco, int fieldIndex, String URL) {
        try {
            URL url = new URL(URL);
            HttpConnection connection = new HttpConnection(url);
            dco.setValue(fieldIndex, connection.getBytes());
            connection.close();
        } catch (Exception ignore) {
            //Messages.add(ignore);
        }
    }


    protected String getDescription(String xmlString) {
        String description = getEditoralReview(xmlString);
        String userReviews = getUserReview(xmlString);

        if (userReviews.trim().length() > 0) {
            if (description.length() > 0)
                description += "\n\n";

            description += userReviews;
        }

        return StringUtils.convertHttpString(description);
    }

    protected String getEditoralReview(String xmlString) {
        String xmlReviews = StringUtils.getValueBetweenStrings("<EditorialReviews>", "</EditorialReviews>", xmlString);
        int counter = 1;
        String reviews = "";
        while (xmlReviews.indexOf("<Content>") > -1) {
            if (counter > 1)
                reviews += "\n\n";

            int begin = xmlReviews.indexOf("<Content>");
            int end = xmlReviews.indexOf("</Content>");
            String review = xmlReviews.substring(begin + 9, end);

            if (review.indexOf("&lt;p clear=" + '"' + "all" + '"') > -1)
                review = review.substring(0, review.indexOf("&lt;p clear=" + '"' + "all" + '"'));

            reviews += review;

            xmlReviews = xmlReviews.substring(end + 10, xmlReviews.length());
            counter++;
        }
        return reviews;
    }


    protected String getUserReview(String xmlString) {
        String xmlReviews = StringUtils.getValueBetweenStrings("<CustomerReviews>", "</CustomerReviews>", xmlString);
        int counter = 1;
        String reviews = "";
        while (xmlReviews.indexOf("<Content>") > -1) {
            if (counter > 1) {
                reviews += "\n\n";
            }
            int begin = xmlReviews.indexOf("<Content>");
            int end = xmlReviews.indexOf("</Content>");
            reviews += "-- review " + (counter++) + " --\n" +
                       xmlReviews.substring(begin + 9, end);
            xmlReviews = xmlReviews.substring(end + 10, xmlReviews.length());
        }
        return reviews;
    }
}
