/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.processes.http;

import java.util.Vector;

import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.objects.audiocd.AudioCD;
import net.sf.dc.core.services.DcRegion;
import net.sf.dc.freedb.Freedb;
import net.sf.dc.http.covers.AmazonCoverSource;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DcResources;
import net.sf.ext.util.DiscIDReader;

public class FreeDBAudioCDUpdate extends InternetUpdateProcess {

    private final String server;

    public FreeDBAudioCDUpdate(IOnlineUpdateListener listener, DcRegion region) {
        super(listener);
        this.server = region.getUrl();
    }

    public Freedb getFreedbInstance() {
        Freedb freeDb = new Freedb(server, listener);
        return freeDb;
    }

    public String[] getCategories() {
        try {
        	Freedb freeDb = new Freedb(server, listener);
        	return freeDb.getCategories();
        } catch (Exception exp) {
        	Messages.add(exp);
        }
        return new String[0];
    }

    public void run() {
    	discIdSearch();
    }

    private void discIdSearch() {
        isCancelled(false);

        DiscIDReader discIdReader = new DiscIDReader();
        String discId = "";
        try {
        	discId = discIdReader.readDiscID();
            listener.addMessage(DcResources.getText("msgFoundDiscID", discId));
        } catch (Exception exp) {
            listener.addError(DcResources.getText("msgErrorDiscID", exp.getMessage()));
            listener.addMessage(DcResources.getText("msgQueryAborted"));
            listener.stopped();
            return;
        }

        if (!discIdReader.isSuccess()) {
            listener.addError(DcResources.getText("msgCouldNotReadDiscID"));
            listener.addMessage(DcResources.getText("msgQueryAborted"));
            listener.stopped();
            return;
        }

        listener.addMessage(DcResources.getText("msgConnectingToServer", server));
        Freedb freeDb = new Freedb(server, listener);

        discId = discId.replaceAll(" ", "+");
        try {
            AudioCD[] audioCDs = freeDb.queryDiscId(discId);
            audioCDs = discardEquals(audioCDs);

            listener.addMessage(DcResources.getText("msgFoundXResults", String.valueOf(audioCDs.length)));

            if (audioCDs.length == 0) {
                new MessageBox(DcResources.getText("msgNoResultsForDiscID", discId),
                               MessageBox._WARNING);
            } else {
                for (int i = 0; i < audioCDs.length; i++) {
                    if (!isCancelled()) {
                        AudioCD audioCD = audioCDs[i];
                    	AmazonCoverSource amazon = new AmazonCoverSource(listener, audioCD);
                    	amazon.search();
                        listener.addObject(audioCD);
                        listener.addMessage(DcResources.getText("msgParsingSuccessfull",
                                      audioCD.getValue(AudioCD._A_TITLE).toString()));
                    } else {
                    	break;
                    }
                }
            }
        } catch (Exception exp) {
            new MessageBox(DcResources.getText("msgFreedbError", exp.getMessage()),
                           MessageBox._ERROR);
            Messages.add(exp);
        } finally {
            listener.stopped();    
        }
    }

    private AudioCD[] discardEquals(AudioCD[] audioCDs) {
        Vector v = new Vector();
    	for (int i = 0; i < audioCDs.length; i++) {
            AudioCD audioCD = audioCDs[i];
            boolean unique = true;
            for (int j = 0; j < audioCDs.length && unique; j++) {
                if (i != j && audioCD.equals(audioCDs[j])) {
                	unique = false;
                }
            }
            if (unique) {
            	v.addElement(audioCD);
            }
        }

        if (v.size() == 0 && audioCDs.length > 0) {
        	v.addElement(audioCDs[0]);
        }

        audioCDs = null;
        AudioCD[] uniqueCDs = new AudioCD[v.size()];
        uniqueCDs = (AudioCD[]) v.toArray(uniqueCDs);
        return uniqueCDs;
    }
}
