/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.processes.http;

import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import net.sf.dc.core.Repository;
import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.software.Software;
import net.sf.dc.core.objects.software.SoftwareCategory;
import net.sf.dc.core.objects.software.SoftwarePlatform;
import net.sf.dc.core.services.DcServer;
import net.sf.dc.exceptions.ValidationException;
import net.sf.dc.http.HttpConnection;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.ext.util.StringUtils;

public class MobyGamesUpdate extends InternetUpdateProcess {

    private String searchString;
    private DcServer server;
    
    public MobyGamesUpdate(IOnlineUpdateListener listener, DcServer server, String query) {
        super(listener);
        
        this.server = server;
        searchString = query.replaceAll(" ", "+");

        int idx = searchString.indexOf('&');
        while (idx > -1) {
            searchString = searchString.substring(0, searchString.indexOf('&')) + "%26" +
                           searchString.substring(searchString.indexOf('&') + 1, searchString.length());
            idx = searchString.indexOf('&');
        }
    }

    public void run() {
        Collection IDs = getIDs();
        listener.processingTotal(IDs.size());

        if (IDs.size() == 0) {
            listener.addError(DcResources.getText("msgNoResultsForKeywords", searchString));
            listener.stopped();
            return;
        }

        listener.addMessage(DcResources.getText("msgFoundXResults", String.valueOf(IDs.size())));
        listener.addMessage(DcResources.getText("msgStartParsingXResults", String.valueOf(IDs.size())));
        int counter = 1;
        for (Iterator iter = IDs.iterator(); iter.hasNext() && !isCancelled(); counter++) {
            String ID = (String) iter.next(); 
            
            try {
                DcObject dco = getSoftwareItem(ID);
                
                listener.addMessage(DcResources.getText("msgParsingSuccessfull", dco.getDisplayString(Software._A_TITLE)));
                listener.addObject(dco);
                listener.processed(counter);

            } catch (Exception exp) {
                listener.addMessage(DcResources.getText(exp.getMessage()));
                listener.processed(counter);
                Messages.add(exp);
            }
        }
        listener.stopped();
    }   
    
   private Collection getIDs() {
        
        Collection ids = new ArrayList();
        
        String webpage = "";
        
        try {
            listener.addMessage(DcResources.getText("msgConnectingToServer", server.getUrl()));
    
            URL url = new URL(server.getUrl() + "search/quick?q=" + searchString);
            
            HttpConnection connection = new HttpConnection(url);
            webpage = connection.getString();
            connection.close();
        } catch (Exception e) {
            listener.addError(DcResources.getText("msgCouldNotConnectTo", server.getName()));
        }
            
        if (webpage.length() > 0) {    
            String temp = webpage;
            int idx = temp.indexOf("SUMMARY=\"Game Match\"");
            idx = idx == -1 ? temp.indexOf("results:") : idx;

            if (idx > -1) {
                temp = temp.substring(idx, temp.length());
                idx = temp.indexOf("</blockquote>");
                idx = idx == -1 ? temp.indexOf("\n") : idx; 
                
                if (idx > -1) {
                    temp = temp.substring(0, idx);
                    idx = temp.indexOf("<a href=\"");
                    
                    int counter = 0;
                    while (idx > -1 && !isCancelled() && counter < maximum) {
                        temp = temp.substring(idx + 9, temp.length());
                        String part = temp.substring(0, temp.indexOf("\""));
                        
                        if (!ids.contains(part)) {
                            ids.add(part);
                            counter++;
                        }
                        
                        idx = temp.indexOf("<a href=\"");
                    }
                }
            }
        }
        
        return ids;
    }

    private Software getSoftwareItem(String ID) throws Exception {
        Software software = new Software();
        
        ID = ID.startsWith("/") ? ID.substring(1, ID.length()) : ID;
        String address = ID.toLowerCase().startsWith("http") ? ID : server.getUrl() + ID;

        HttpConnection connection = new HttpConnection(new URL(address));
        String page = connection.getString();
        connection.close();

        String title = StringUtils.getValueBetweenStrings("<title>", "</title>", page);
        title = title.indexOf("- MobyGames") > -1 ? title.substring(0, title.indexOf("- MobyGames")) : title;
        software.setValue(Software._A_TITLE, title.trim());
        
        // try to shorten the result string
        int idx = page.toUpperCase().indexOf("<DIV CLASS=\"RIGHTPANEL\">");
        page = idx > -1 ? page.substring(idx, page.length()) : page;
        
        software.setValue(Software._G_PUBLISHER, getInfo(page, "PUBLISHED BY</DIV>", "PUBLISHED"));
        software.setValue(Software._F_DEVELOPER, getInfo(page, "DEVELOPED BY</DIV>", "DEVELOPED"));
        software.setValue(Software._H_PLATFORM, getPlatform(getInfo(page, "PLATFORM</DIV>", "PLATFORM")));
        software.setValue(Software._B_DESCRIPTION, getInformation(page, "DESCRIPTION</H2>", "RIGHTPANELMAIN", "<DIV"));
        software.setValue(Software._I_WEBPAGE, address);
        
        String category = getInfo(page, "GENRE</DIV>", "COREGAMEGENRE");
        software.setValue(Software._K_TYPE, getCategory(category));
        
        try {
            String released = getInfo(page, "RELEASED</DIV>", "RELEASED");
            if (released.indexOf(".") > -1)
                released = released.substring(released.lastIndexOf(".") + 1, released.length());
            else if (released.indexOf(",") > -1)
                released = released.substring(released.lastIndexOf(",") + 1, released.length()).trim();
            
            software.setValue(Software._C_YEAR, new Integer(released));
        } catch (Exception ignore) {}
        
        setCovers(software, address);
        setScreenshots(software, ID);
        
        return software;
    }
    
    private void setCovers(Software software, String address) throws Exception {
        
        String page = "";
        
        try {
            HttpConnection connection = new HttpConnection(new URL(address += "/cover-art"));
            page = connection.getString();
            connection.close();
        } catch (Exception exp) {
            return; // not available
        }
        
        String frontCoverTag = "FRONT COVER\"";
        String backCoverTag = "BACK COVER\"";
        String mediaCoverTag = "MEDIA\"";
        
        setMediaImage(software, frontCoverTag, Software._M_PICTUREFRONT, page);
        setMediaImage(software, backCoverTag, Software._N_PICTUREBACK, page);
        setMediaImage(software, mediaCoverTag, Software._O_PICTURECD, page);
    }
    
    private void setScreenshots(Software software, String ID) throws Exception {
        ID += "/screenshots/";
     
        String page = "";
        try {
            HttpConnection connection = new HttpConnection(new URL(server.getUrl() + ID));
            page = connection.getString();
            connection.close();
        } catch (Exception exp) {
            return; // not available
        }
        
        int[] fields = {Software._P_SCREENSHOTONE, Software._Q_SCREENSHOTTWO, Software._R_SCREENSHOTTHREE};
        
        try {
            int idx = page.toUpperCase().indexOf(ID.toUpperCase());
            int counter = 0;
            while (idx > -1 && counter < fields.length) {
                String part = page.substring(idx);
                String imagePageUrl = part.substring(0,  part.indexOf("\">"));
                
                HttpConnection connection = new HttpConnection(new URL(server.getUrl() + imagePageUrl));
                part = connection.getString();
                connection.close();
                
                int screenIdx = part.toLowerCase().indexOf("screenshot:");
                if (screenIdx > -1) {
                    try {
                        part = part.substring(screenIdx);
                        part = part.substring(part.toLowerCase().indexOf("src=\"") + 5);
                        String imageUrl = part.substring(0, part.indexOf("\""));
                        
                        imageUrl = !imageUrl.toLowerCase().startsWith("http") ? 
                                   server.getUrl() + (imageUrl.startsWith("/") ? imageUrl.substring(1) : imageUrl) :
                                   imageUrl;
                        
                        connection = new HttpConnection(new URL(imageUrl));
                        byte[] image = connection.getBytes();
                        software.setValue(fields[counter], image);
                    } catch (Exception exp) {
                        Messages.add(exp);
                    }
                }
                counter++;
                page = page.substring(idx + ID.length());
                idx = page.toUpperCase().indexOf(ID.toUpperCase());
            }
        } catch (Exception exp) {
            Messages.add(exp);
        }
    }
    
    private void setMediaImage(Software software, String tag, int field, String page) {
        try {
            int idx = page.toUpperCase().indexOf(tag);
            if (idx > -1) {
                String part = page.substring(0,  idx);
                part = part.substring(part.lastIndexOf("\n"));
                
                idx = part.indexOf("<a href=\"") > -1 ? part.indexOf("<a href=\"") + 9 : part.indexOf("src=\"") + 5;
                part = part.substring(idx);
                part = part.substring(0, part.indexOf("\""));
                part = part.startsWith("/") ? part.substring(1, part.length()) : part;
                
                String imagePageUrl = part.startsWith("http") ? part : server.getUrl() + part;
                
                HttpConnection connection = new HttpConnection(new URL(imagePageUrl));
                page = connection.getString();
                connection.close();
                
                idx = page.toLowerCase().indexOf("http://www.mobygames.com/images/covers/");
                if (idx > -1) {
                    part = page.substring(idx);
                    part = part.substring(0, part.indexOf("\""));
                    
                    String imageUrl = part.startsWith("http") ? part : server.getUrl() + part;
                    connection = new HttpConnection(new URL(imageUrl));
                    byte[] image = connection.getBytes();
                    connection.close();
                    software.setValue(field, image);
                }
            }
        } catch (Exception exp) {
            Messages.add(exp);
        }
    }
    
    private String getInformation(String page, String s1, String s2, String end) {
        String s = page.toUpperCase();
        int idx = s.indexOf(s1) > -1 ? s.indexOf(s1) : s.indexOf(s2);
        String part = null;
        if (idx > -1) {
            part = page.substring(idx, page.length());
            idx = part.toUpperCase().indexOf(end);
            if (idx > -1) {
                idx = part.toUpperCase().indexOf("</H2>");
                part = idx > -1 ? part.substring(idx + 5, part.length()) : part;
                part = part.substring(0, part.toUpperCase().indexOf(end));
                
                int index = part.indexOf("<a");
                while (index > -1) {
                    try {
                        String first = part.substring(0, index);
                        
                        String temp = part.substring(index, part.length());
                        String middle = temp.substring(temp.indexOf(">") + 1, temp.indexOf("</a>"));
                        String last = temp.substring(temp.indexOf("</a>") + 4);
                        part = first + middle + last;
                        
                        index = part.indexOf("<a");
                    } catch (Exception exp) {
                        Messages.add(exp);
                    }
                }
                part = StringUtils.convertHttpString(part);
            }
        }
        return part;
    }
    
    private String getInfo(String page, String s1, String s2) {
        String s = page.toUpperCase();
        int idx = s.indexOf(s1) > -1 ? s.indexOf(s1) : s.indexOf(s2);
        String part = null;

        try {
            if (idx > -1) {
                part = page.substring(idx, s.length());
                part = part.substring(part.toUpperCase().indexOf("<A HREF=\""), part.length());
                part = part.substring(part.indexOf(">") + 1, part.toUpperCase().indexOf("</A>"));
                part = StringUtils.convertHttpString(part);
            }
        } catch (StringIndexOutOfBoundsException exp) {}
        
        return part;
    }

    private DcObject getCategory(String category) {
        if (category != null) {
            DcObject[] dcCats = DataManager.get(DcModules._SOFTWARECAT, null); 
            for (int i = 0; i < dcCats.length; i++) {
                String dcCatName = dcCats[i].getValue(SoftwarePlatform._A_NAME).toString().toLowerCase();
                if (category.toLowerCase().indexOf(dcCatName) > -1 || dcCatName.indexOf(category) > -1)
                    return dcCats[i];
            }
        }
        
        if (DcSettings.getValueAsBoolean(Repository.settings.stAutoCreateSubItems)) {
            try {
                SoftwareCategory sc = new SoftwareCategory();
                sc.setValue(SoftwareCategory._A_NAME, category);
                sc.setSilence(true);
                sc.setIDs();
                sc.saveNew(false);
                
                return sc;
                
            } catch (ValidationException ignore) {}
        }        
        
        return null;
    } 
    
    private DcObject getPlatform(String platform) {
        if (platform != null) {
            DcObject[] dcPlatforms = DataManager.get(DcModules._SOFTWAREPLATFORM, null); 
            for (int i = 0; i < dcPlatforms.length; i++) {
                String dcPlatformName = dcPlatforms[i].getValue(SoftwarePlatform._A_NAME).toString().toLowerCase();
                if (platform.toLowerCase().indexOf(dcPlatformName) > -1 || dcPlatformName.indexOf(platform) > -1)
                    return dcPlatforms[i];
            }
        }
        
        // create new platform
        if (DcSettings.getValueAsBoolean(Repository.settings.stAutoCreateSubItems)) {
            try {
                SoftwarePlatform sp = new SoftwarePlatform();
                sp.setValue(SoftwarePlatform._A_NAME, platform);
                sp.setSilence(true);
                sp.setIDs();
                sp.saveNew(false);
                
                return sp;
                
            } catch (ValidationException ignore) {}
        }
        
        return null;
    }    
}
