/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2007 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.processes.http.wikipedia;

import java.util.ArrayList;
import java.util.Collection;
import java.util.StringTokenizer;

import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.movie.Movie;
import net.sf.dc.core.services.DcRegion;
import net.sf.dc.core.services.DcServer;
import net.sf.dc.processes.http.IOnlineUpdateListener;
import net.sf.dc.processes.http.RemovalTag;

public class WikipediaDutchMovieUpdate extends WikipediaMovieUpdate {

    public WikipediaDutchMovieUpdate(IOnlineUpdateListener listener, DcServer server, DcRegion region, String query) {
        super(listener, server, region, query);
    }

    public String[] getDescriptionTags() {
        return new String[] {"[bewerk]", "categorien"};
    }

    public RemovalTag[] getRemovalTags() {
        return  new RemovalTag[] {new RemovalTag(RemovalTag._REMOVETAG, "[bewerk]"),
                                  new RemovalTag(RemovalTag._REMOVELINE, "verhaal"),
                                  new RemovalTag(RemovalTag._REMOVELINE, "leeswaarschuwing")};
    }

    public void setAdditionInfo(Movie movie, String[] headers, String text, String webpage) {
        setGenres(movie, headers);
    }     
    
    private void setGenres(Movie movie, String[] headers) {
        String genres = getValueForTag(headers, "genre");
        
        if (genres != null) {
            StringTokenizer st = new StringTokenizer(genres, genres.indexOf(">") > -1 ? ">" : ",");
            Collection mappings = new ArrayList();
            while (st.hasMoreElements()) {
                String genre = ((String) st.nextToken()).trim();
                mappings.addAll(DataManager.createMapping(movie, genre, 
                                DcModules._MOVIEGENRE, DcModules._MOVIEGENREMAPPING));
            }
            
            if (mappings.size() > 0)
                movie.setValue(Movie._H_GENRES, mappings);
        }
    }
    
    public String[] getTags() {
        return new String[] {"regie", "productie", "uitgebracht", "hoofdrollen", "lengte", "land", "taal"};
    }
}
