/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2007 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.processes.http.wikipedia;

import java.net.URL;

import net.sf.dc.core.objects.movie.Movie;
import net.sf.dc.core.services.DcRegion;
import net.sf.dc.core.services.DcServer;
import net.sf.dc.http.HttpConnection;
import net.sf.dc.messages.Messages;
import net.sf.dc.processes.http.IOnlineUpdateListener;
import net.sf.dc.processes.http.RemovalTag;

public class WikipediaEnglishMovieUpdate extends WikipediaMovieUpdate {

    public WikipediaEnglishMovieUpdate(IOnlineUpdateListener listener, DcServer server, DcRegion region, String query) {
        super(listener, server, region, query);
    }

    public String[] getDescriptionTags() {
        return new String[] {"[edit]", " >\n>"};
    }

    public RemovalTag[] getRemovalTags() {
        return new RemovalTag[] {new RemovalTag(RemovalTag._REMOVETAG, "[edit]"),
                                 new RemovalTag(RemovalTag._REMOVELINE, "spoiler"),
                                 new RemovalTag(RemovalTag._REMOVELINE, "plot consummation"),
                                 new RemovalTag(RemovalTag._REMOVELINE, "synopsis"),
                                 new RemovalTag(RemovalTag._REMOVELINE, "plot")};
    }

    public String[] getTags() {
        return new String[] {"directed by", "produced by", "release date", "starring", "running time", "country", "language"};
    }

    protected void setImage(Movie movie, String webpage) {
        int idx = webpage.indexOf("class=\"image\"");
        
        if (idx == -1) return;
        
        // backtrack
        String s = webpage.substring(idx);
        while (s.indexOf("<a href=\"/") > 4) {
            idx -= 4;
            s = webpage.substring(idx);
        }
        
        try {
            s = s.substring(s.indexOf("<a href=\"/") + 10);
            s = region.getUrl() + s.substring(0, s.indexOf("\""));
        
            HttpConnection connection = new HttpConnection(new URL(s));
            String page = connection.getString();
            connection.close();
            
            page = page.substring(page.indexOf("<a href=\"http://upload.wikimedia.org/"));
            String link = page.substring(page.indexOf("<a href=\"") + 9);
            link = link.substring(0, link.indexOf("\""));
            
            if (link.indexOf("logo") > -1)
                return;
            
            connection = new HttpConnection(new URL(link));
            byte[] image = connection.getBytes();
            connection.close();
            
            movie.setValue(Movie._X_PICTUREFRONT, image);
        } catch (Exception exp) {
            Messages.add(exp);
        }        
    }
}
