/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2007 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.processes.http.wikipedia;

import java.util.ArrayList;
import java.util.Collection;
import java.util.StringTokenizer;

import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.movie.Movie;
import net.sf.dc.core.services.DcRegion;
import net.sf.dc.core.services.DcServer;
import net.sf.dc.processes.http.IOnlineUpdateListener;
import net.sf.dc.processes.http.RemovalTag;

public class WikipediaItalionMovieUpdate extends WikipediaMovieUpdate {

    public WikipediaItalionMovieUpdate(IOnlineUpdateListener listener, DcServer server, DcRegion region, String query) {
        super(listener, server, region, query);
    }

    public int getTagConstraint() {
        return 3;
    }    
    
    public String[] getDescriptionTags() {
        return new String[] {"[modifica]", ">\n>"};
    }

    public RemovalTag[] getRemovalTags() {
        return new RemovalTag[] {new RemovalTag(RemovalTag._REMOVETAG, "[modifica]"),
                                 new RemovalTag(RemovalTag._REMOVELINE, "attenzione: di seguito viene rivelata")};
    }

    public String[] getTags() {
        return new String[] {"regia:", "soggetto:", "anno:", "attori:", "durata:", "paese:", "dasdadada"};
    }
    
    public void setAdditionInfo(Movie movie, String[] headers, String text, String webpage) {
        setActors(movie, webpage);
        setTitle(movie, headers);
        setGenres(movie, headers);
    }
    
    private void setActors(Movie movie, String webpage) {
        int idx = webpage.toLowerCase().indexOf("attori</a>");
        
        try {
            String actors = "";
            if (idx > -1) {
                String tmp = webpage.substring(idx);
                tmp = tmp.substring(0, webpage.indexOf("</div>"));
                
                while (tmp.indexOf("<li>") > -1) {
                    String part = tmp.substring(tmp.indexOf("<li>"), tmp.indexOf("</li>"));
                    part = part.substring(part.indexOf("\">") + 2, part.indexOf("</a>"));
                    
                    actors += actors.length() > 0 ? ", " : "";
                    actors += part;
                    
                    tmp = tmp.substring(tmp.indexOf("</li>") + 6);
                }
            }
            movie.setValue(Movie._I_ACTORS, actors);
        } catch (Exception ignore) {}
    }
    
    private void setTitle(Movie movie, String[] headers) {
        String localTitle = (String) movie.getValue(Movie._A_TITLE);
        String orgTitle =getValueForTag(headers, "titolo originale:"); 
        
        if (orgTitle != null) {
            movie.setValue(Movie._A_TITLE, orgTitle);
            movie.setValue(Movie._F_TITLE_LOCAL, localTitle);
        }
    }
    
    private void setGenres(Movie movie, String[] headers) {
        String genres = getValueForTag(headers, "genere:");
        if (genres != null) {
            StringTokenizer st = new StringTokenizer(genres, ",");
            Collection mappings = new ArrayList();
            while (st.hasMoreElements()) {
                String genre = ((String) st.nextToken()).trim();
                mappings.addAll(DataManager.createMapping(movie, genre, 
                                DcModules._MOVIEGENRE, DcModules._MOVIEGENREMAPPING));
            }
            
            if (mappings.size() > 0)
                movie.setValue(Movie._H_GENRES, mappings);
        }
    }
}