/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2007 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.processes.http.wikipedia;

import java.net.URL;
import java.util.StringTokenizer;

import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.movie.Movie;
import net.sf.dc.core.services.DcRegion;
import net.sf.dc.core.services.DcServer;
import net.sf.dc.http.HttpConnection;
import net.sf.dc.messages.Messages;
import net.sf.dc.processes.http.IOnlineUpdateListener;
import net.sf.dc.processes.http.RemovalTag;
import net.sf.ext.util.StringUtils;

public abstract class WikipediaMovieUpdate extends WikipediaUpdate {

    protected static final int _DIRECTOR = 0;
    protected static final int _PRODUCER = 1;
    protected static final int _RELEASEDATE = 2;
    protected static final int _STARRING = 3;
    protected static final int _PLAYLENTGH = 4;
    protected static final int _COUNTRY = 5;
    protected static final int _LANGUAGE = 6;
    
    protected static final int _DESC_START = 0;
    protected static final int _DESC_END = 1;
    
    public WikipediaMovieUpdate(IOnlineUpdateListener listener, DcServer server, DcRegion region, String query) {
        super(listener, server, region, query);
    }
    
    public abstract String[] getTags();
    public abstract String[] getDescriptionTags();
    public abstract RemovalTag[] getRemovalTags();
    
    private boolean isMovie(String[] tags, String webpage) {
        String page = StringUtils.getValueBetweenStrings("<body", "</body>", webpage);
        page = page.replaceAll("\r", "\n");
        page = page.replaceAll("\n\n", "\n");
        
        StringTokenizer st = new StringTokenizer(page, "\n");
        String header = "";
        int count = 0;
        while (st.hasMoreElements() && count < 80) {
            header += "\n" + st.nextElement();
            count++;
        }
        
        int matched = 0;
        for (int i = 0; i < tags.length; i++)
            matched += header.toLowerCase().indexOf(tags[i]) > -1 ? 1 : 0;
        
        return matched >= getTagConstraint(); 
    }

    public int getTagConstraint() {
        return 4;
    }
    
    public void setAdditionInfo(Movie movie, String[] headers, String text, String webpage) {}
    
    public DcObject getDcObject(String link) {
        
        Movie movie = null;
        
        try {
            HttpConnection connection = new HttpConnection(new URL(link));
            String webpage = connection.getString();
            connection.close();

            String[] tags = getTags();
            String[] descTags = getDescriptionTags();
            if (isMovie(tags, webpage)) {
                movie = new Movie();

                String text = getCleanText(webpage);
                
                int idx = text.indexOf(descTags[_DESC_START]);

                StringTokenizer st = new StringTokenizer(text.substring(0, idx > 0 ? idx : text.length()), "\n");
                String[] header = new String[st.countTokens()];
                int counter = 0;
                while (st.hasMoreElements()) 
                    header[counter++] = (String) st.nextElement();
                
                movie.setValue(Movie._A_TITLE, getTitle(webpage));
                movie.setValue(Movie._B_DESCRIPTION, getDescription(text));
                movie.setValue(Movie._0_LANGUAGE, getValueForTag(header, tags[_LANGUAGE]));
                movie.setValue(Movie._G_WEBPAGE, link);
                movie.setValue(Movie._J_DIRECTOR,getValueForTag(header, tags[_DIRECTOR]));
                movie.setValue(Movie._I_ACTORS, getActors(header, tags[_STARRING]));
                movie.setValue(Movie._K_COUNTRY, getValueForTag(header, tags[_COUNTRY]));
                movie.setValue(Movie._C_YEAR, getYear(header));
                movie.setValue(Movie._L_PLAYLENGTH, getPlaylength(header));
                
                
                setAdditionInfo(movie, header, text, webpage);
                setImage(movie, webpage);
            }
        } catch (Exception exp) {
            Messages.add(exp);
        }
        
        return movie;
    }
    
    private String getTitle(String webpage) {
        String title = StringUtils.getValueBetweenStrings("<title>", "</title>", webpage);
        if (title.indexOf(" - Wikipedia") > -1)
            title = title.substring(0, title.indexOf(" - Wikipedia"));
        
        return title;
    }
    
    private String getDescription(String s) {
        String[] tags = getDescriptionTags();
        String tag1 = tags[_DESC_START];
        String tag2 = tags[_DESC_END];
        
        int idx = s.toLowerCase().indexOf(tag1);
        
        if (idx == -1)
            return "";
            
        String description = idx == -1 ? s : s.substring(idx + tag1.length());
        
        idx = description.toLowerCase().indexOf(tag2) > -1 ? description.toLowerCase().indexOf(tag2) :
              description.toLowerCase().indexOf(tag1) > -1 ? description.toLowerCase().indexOf(tag1) : description.length();
        description = description.substring(0, idx);

        return cleanup(getRemovalTags(), description);
    }
    
    protected String getValueForTag(String[] header, String tag) {
        for (int i = 0; i < header.length; i++) {
            String line = header[i].toLowerCase().trim();
            if (line.startsWith(tag) || line.equals(tag))
                return header[i+1];
        }
        return null;
    }
    
    protected String getActors(String[] header, String tag) {
        String s = getValueForTag(header, tag);
        StringTokenizer st = new StringTokenizer(s,  s.indexOf(",") > -1 ? "," : s.indexOf("  ") > -1 ? "  " : ">");
        String result = "";
        int i = 0;
        while (st.hasMoreElements()) {
            String actor = (String) st.nextElement();
            result += i > 0 ? ", " : "";
            result += actor.trim();
            i++;
        }
        return result;
    }

    private Integer getYear(String[] headers) {
        String[] tags = getTags();
        String s = getValueForTag(headers, tags[_RELEASEDATE]);
        
        String nr = "";
        for (int i = 0; i < s.length(); i++) {
            nr = Character.isDigit(s.charAt(i)) ? nr + s.charAt(i) : "";
            if (nr.length() == 4)
                return new Integer(nr);
        }
        return null;
    }
    
    private Integer getPlaylength(String[] headers) {
        String[] tags = getTags();
        String s = getValueForTag(headers, tags[_PLAYLENTGH]);
        
        if (s == null)
            return null;
            
        String nr = "";
        for (int i = 0; i < s.length(); i++) {
            nr += Character.isDigit(s.charAt(i)) ? s.charAt(i) : "";
            if (!Character.isDigit(s.charAt(i)) && nr.length() > 1)
                break;
        }
        return new Integer(nr) * 60;
    }   
    
    protected void setImage(Movie movie, String webpage) {}
}
