/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2007 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.processes.http.wikipedia;

import java.util.ArrayList;
import java.util.Collection;
import java.util.StringTokenizer;

import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.movie.Movie;
import net.sf.dc.core.services.DcRegion;
import net.sf.dc.core.services.DcServer;
import net.sf.dc.messages.Messages;
import net.sf.dc.processes.http.IOnlineUpdateListener;
import net.sf.dc.processes.http.RemovalTag;

public class WikipediaSpanishMovieUpdate extends WikipediaMovieUpdate {

    public WikipediaSpanishMovieUpdate(IOnlineUpdateListener listener, DcServer server, DcRegion region, String query) {
        super(listener, server, region, query);
    }
    
    public int getTagConstraint() {
        return 3;
    } 

    public String[] getDescriptionTags() {
        return new String[] {"[editar]", "secuelas"};
    }

    public RemovalTag[] getRemovalTags() {
        return new RemovalTag[] {new RemovalTag(RemovalTag._REMOVETAG, "[editar]"),
                                 new RemovalTag(RemovalTag._REMOVETAG, "advertencia: esta seccin contiene detalles")};
    }

    public String[] getTags() {
        return new String[] {"direccin", "produccin", "ao", "reparto", "duracin", "pas", "adadadadada"};
    }
    
    public void setAdditionInfo(Movie movie, String[] headers, String text, String webpage) {
        setTitle(movie, headers);
        setGenres(movie, headers);
        setDescription(movie, webpage);
    }
    
    private void setGenres(Movie movie, String[] headers) {
        String genres = getValueForTag(headers, "gnero");
        if (genres != null) {
            StringTokenizer st = new StringTokenizer(genres, genres.indexOf(">") > -1 ? ">" : ",");
            Collection mappings = new ArrayList();
            while (st.hasMoreElements()) {
                String genre = ((String) st.nextToken()).trim();
                mappings.addAll(DataManager.createMapping(movie, genre, 
                                DcModules._MOVIEGENRE, DcModules._MOVIEGENREMAPPING));
            }
            
            if (mappings.size() > 0)
                movie.setValue(Movie._H_GENRES, mappings);
        }
    }
    
    private void setTitle(Movie movie, String[] headers) {
        movie.setValue(Movie._F_TITLE_LOCAL, getValueForTag(headers, "ttulo"));
    }
    
    private void setDescription(Movie movie, String webpage) {
        String description = (String) movie.getValue(Movie._B_DESCRIPTION);
        description = description == null ? "" : description.trim();
        
        if (description.length() == 0 && webpage.indexOf("</table>") > -1) {
            int idx = webpage.indexOf("</table>");
            description = webpage.substring(idx + 8);
            
            try {
                String s = getCleanText(description);
                idx = s.toLowerCase().indexOf("obtenido de \"http://es.wikipedia.org/");
                s = s.substring(0, idx > -1 ? idx : s.length());
                s = cleanup(getRemovalTags(), s);
                movie.setValue(Movie._B_DESCRIPTION, s);
            } catch (Exception exp) {
                Messages.add(exp);
            }
        }
    }
}
