/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2007 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.processes.http.wikipedia;

import java.io.IOException;
import java.io.StringReader;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.StringTokenizer;

import javax.swing.JTextPane;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLEditorKit;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.software.Software;
import net.sf.dc.core.services.DcRegion;
import net.sf.dc.core.services.DcServer;
import net.sf.dc.http.HttpConnection;
import net.sf.dc.messages.Messages;
import net.sf.dc.processes.http.IOnlineUpdateListener;
import net.sf.dc.processes.http.InternetUpdateProcess;
import net.sf.dc.processes.http.RemovalTag;
import net.sf.dc.resources.DcResources;

/**
 * Allows multiple types of items to be retrieved
 */
public abstract class WikipediaUpdate extends InternetUpdateProcess {

    private JTextPane descriptionPane;
    private HTMLEditorKit kit = new HTMLEditorKit();
    private HTMLDocument document = new HTMLDocument();
    
    protected String searchString;
    protected DcServer server;
    protected DcRegion region;
    
    public WikipediaUpdate(IOnlineUpdateListener listener, DcServer server, DcRegion region, String query) {
        super(listener);
        
        this.server = server;
        this.region = region;

        descriptionPane = ComponentFactory.getTextPane();
        descriptionPane.setContentType("text/html");
        descriptionPane.setEditorKit(kit);
        descriptionPane.setDocument(document);
        descriptionPane.setEditable(false);

        searchString = query.replaceAll(" ", "_");

        int idx = searchString.indexOf('&');
        while (idx > -1) {
            searchString = searchString.substring(0, searchString.indexOf('&')) + "%26" +
                           searchString.substring(searchString.indexOf('&') + 1, searchString.length());
            idx = searchString.indexOf('&');
        }
    }
    
    public abstract DcObject getDcObject(String link);

    public void run() {
        Collection IDs = getIDs();
        listener.processingTotal(IDs.size());

        if (IDs.size() == 0) {
            listener.addError(DcResources.getText("msgNoResultsForKeywords", searchString));
            listener.stopped();
            return;
        }

        listener.addMessage(DcResources.getText("msgFoundXResults", String.valueOf(IDs.size())));
        listener.addMessage(DcResources.getText("msgInvestigatingXResults", String.valueOf(IDs.size())));
        int counter = 1;
        for (Iterator iter = IDs.iterator(); iter.hasNext() && !isCancelled(); counter++) {
            String link = (String) iter.next(); 
            
            try {
                DcObject dco = getDcObject(link);
                
                if (dco != null) {
                    listener.addMessage(DcResources.getText("msgParsingSuccessfull", dco.getDisplayString(Software._A_TITLE)));
                    listener.addObject(dco);
                }
                
                listener.processed(counter);

            } catch (Exception exp) {
                listener.addMessage(DcResources.getText(exp.getMessage()));
                listener.processed(counter);
                Messages.add(exp);
            }
        }
        
        descriptionPane.setText("");
        descriptionPane = null;
        kit = null;
        document = null;
        listener.stopped();
    }   
    
    public String cleanup(RemovalTag[] tags, String s) {
        StringTokenizer lines = new StringTokenizer(s, s.indexOf('\n') > -1 ? "\n" : "\r");
        
        String result = "";
        while (lines.hasMoreElements()) {
            String line = ((String) lines.nextElement()).trim();

            for (int i = 0; i < tags.length; i++)
                line = tags[i].apply(line);
            
            if (line.length() > 0 && !line.trim().equals("\n")) {
                result += result.length() > 0 ? "\n" : "";
                result += line;
            }
        }
        return result;
    }
  
    protected String getCleanText(String html) throws IOException, BadLocationException {
        String webpage = html;
        
        if (webpage.indexOf("<body") > -1)
            webpage = "<html>" + html.substring(html.indexOf("<body"), html.indexOf("</body>")) + "</html>";
        
        descriptionPane.read(new StringReader(webpage), "Data Crow");     
        Document doc = descriptionPane.getDocument();
        String text = doc.getText(0, doc.getLength());
        text = text.replaceAll("\r", "\n");
        text = text.replaceAll("\n\n", "\n");
        text = text.replaceAll("" + (char) 160 , " ");
        
        return text;
    }
    
    private Collection getIDs() {
        
        Collection ids = new ArrayList();
        
        String webpage = "";
        
        try {
            listener.addMessage(DcResources.getText("msgConnectingToServer", region.getUrl()));
    
            URL url = new URL(region.getUrl() + "wiki/Special:Search?fulltext=Search&search=" + searchString);
            HttpConnection connection = new HttpConnection(url);
            webpage = connection.getString();

            connection.close();
        } catch (Exception e) {
            listener.addError(DcResources.getText("msgCouldNotConnectTo", server.getName()));
        }

        
        String ref = "<li style=\"padding-bottom: 1em;\"><a href=\"/";
        int refIdx = webpage.toLowerCase().indexOf(ref.toLowerCase());
        while (refIdx > -1) {
            webpage = webpage.substring(refIdx + ref.length(), webpage.length());
            String link = webpage.substring(0, webpage.indexOf('"'));
            ids.add(region.getUrl() + link);
            refIdx = webpage.toLowerCase().indexOf(ref.toLowerCase());
        }
        
        return ids;
    }
    

}
