/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.processes.synchronizer;

import java.util.Collection;
import java.util.Iterator;

import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.audiocd.AudioCD;
import net.sf.dc.core.objects.audiocd.AudioTrack;
import net.sf.dc.core.services.DcRegion;
import net.sf.dc.core.services.DcSearchMode;
import net.sf.dc.core.services.DcServer;
import net.sf.dc.processes.http.InternetUpdateProcess;
import net.sf.dc.resources.DcResources;
import net.sf.ext.util.StringUtils;

public class AudioCdSynchronizer extends DefaultSynchronizer {

    public AudioCdSynchronizer() {
        super(DcResources.getText("lblMassItemUpdate", DcModules.get(DcModules._AUDIOCD).getObjectName()),
              DcModules._AUDIOCD);
    }
    
    public String getHelpText() {
        return DcResources.getText("msgAudioCdMassUpdateHelp");
    }
    
    public boolean onlineUpdate(DcObject dco, DcServer server, DcRegion region) {
        
        boolean updated = false;

        Collection modes = server.getModes();
        AudioCD acCurrent = (AudioCD) dco;
        
        String title = StringUtils.normalize((String) dco.getValue(AudioCD._A_TITLE));
        String artist = StringUtils.normalize((String) dco.getValue(AudioCD._F_ARTIST));
        if ((title == null || title.length() == 0) ||
            (artist == null || artist.length() == 0)) 
            return updated;
        
        DcSearchMode mode = null;
        for (Iterator iterMode = modes.iterator(); iterMode.hasNext(); ) {
            mode = (DcSearchMode) iterMode.next();
        
            InternetUpdateProcess process = 
                server.getProcess(this, (String) acCurrent.getValue(AudioCD._A_TITLE), mode, region);
            process.setMaximum(4);
            process.start();
            
            try {
                process.join();
            } catch (Exception exp) {}
            
            for (Iterator iter = result.iterator(); iter.hasNext(); ) {
                AudioCD acNew = (AudioCD) iter.next();

                String titleNew = StringUtils.normalize((String) acNew.getValue(AudioCD._A_TITLE));
                String artistNew = StringUtils.normalize((String) acNew.getValue(AudioCD._F_ARTIST));
                
                if ((titleNew.toLowerCase().equals(title.toLowerCase())) && 
                    (artistNew.toLowerCase().equals(artist.toLowerCase()))) {
                    
                    boolean match = updateTracks(acCurrent.getChildren(), acNew.getChildren());
                    if (match) {
                        updated = true;
                        setValue(acCurrent, AudioCD._H_PICTUREFRONT, acNew.getValue(AudioCD._H_PICTUREFRONT));
                        setValue(acCurrent, AudioCD._I_PICTUREBACK, acNew.getValue(AudioCD._I_PICTUREBACK));
                        setValue(acCurrent, AudioCD._J_PICTURECD, acNew.getValue(AudioCD._J_PICTURECD));
                    }
                }
            }
            result.clear();
        }
        return updated;
    }
    
    private boolean updateTracks(Collection oldTracks, Collection newTracks) {
        boolean match = false;
        for (Iterator iterOld = oldTracks.iterator(); iterOld.hasNext(); ) {
            AudioTrack currentTrack = (AudioTrack) iterOld.next();
            
            String titleOld = (String) currentTrack.getValue(AudioTrack._A_TITLE);
            Integer lengthOld = (Integer) currentTrack.getValue(AudioTrack._H_PLAYLENGTH);
            Integer trackOld = (Integer) currentTrack.getValue(AudioTrack._F_TRACKNUMBER);

            for (Iterator iterNew = newTracks.iterator(); iterNew.hasNext(); ) {
                AudioTrack newTrack = (AudioTrack) iterNew.next();
                
                String titleNew = (String) newTrack.getValue(AudioTrack._A_TITLE);
                Integer lengthNew = (Integer) newTrack.getValue(AudioTrack._H_PLAYLENGTH);
                Integer trackNew = (Integer) newTrack.getValue(AudioTrack._F_TRACKNUMBER);

                if ((titleOld != null && titleNew != null) && titleNew.toLowerCase().equals(titleOld.toLowerCase())) {
                    
                    match = true;
                    merge(currentTrack, newTrack);
                
                } else if (newTracks.size() == oldTracks.size() && 
                           ((lengthOld != null && lengthNew != null) && lengthNew.equals(lengthOld)) ||  
                           ((trackOld != null && trackNew != null) && trackNew.equals(trackOld))) {
                
                    match = true;
                    merge(currentTrack, newTrack);
                }
            }
        }
        return match;
    } 
    
    private void merge(AudioTrack currentTrack, AudioTrack newTrack) {
        setValue(currentTrack, AudioTrack._A_TITLE, newTrack.getValue(AudioTrack._A_TITLE));
        setValue(currentTrack, AudioTrack._B_DESCRIPTION, newTrack.getValue(AudioTrack._B_DESCRIPTION));
        setValue(currentTrack, AudioTrack._C_YEAR, newTrack.getValue(AudioTrack._C_YEAR));
        setValue(currentTrack, AudioTrack._E_RATING, newTrack.getValue(AudioTrack._E_RATING));
        setValue(currentTrack, AudioTrack._F_TRACKNUMBER, newTrack.getValue(AudioTrack._F_TRACKNUMBER));
        setValue(currentTrack, AudioTrack._G_LYRICS, newTrack.getValue(AudioTrack._G_LYRICS));
        setValue(currentTrack, AudioTrack._H_PLAYLENGTH, newTrack.getValue(AudioTrack._H_PLAYLENGTH));
    }    
}
