/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.processes.synchronizer;

import java.util.Collection;
import java.util.Iterator;

import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.book.Book;
import net.sf.dc.core.services.DcRegion;
import net.sf.dc.core.services.DcSearchMode;
import net.sf.dc.core.services.DcServer;
import net.sf.dc.processes.http.InternetUpdateProcess;
import net.sf.dc.resources.DcResources;
import net.sf.ext.util.StringUtils;

public class BookSynchronizer extends DefaultSynchronizer {

    public BookSynchronizer() {
        super(DcResources.getText("lblMassItemUpdate", DcModules.get(DcModules._BOOK).getObjectName()),
              DcModules._BOOK);
    }
    
    public String getHelpText() {
        return DcResources.getText("msgBookMassUpdateHelp");
    }
    
    public boolean onlineUpdate(DcObject dco, DcServer server, DcRegion region) {

        boolean updated = false;
        
        String title = StringUtils.normalize((String) dco.getValue(Book._A_TITLE));
        String author = StringUtils.normalize((String) dco.getValue(Book._G_AUTHOR));
        String isbn = StringUtils.normalize((String) dco.getValue(Book._N_ISBN13));
        
        if ((title == null || title.length() == 0)) 
            return updated;
        
        Collection modes = server.getModes();
        DcSearchMode mode = null;
        for (Iterator iterMode = modes.iterator(); iterMode.hasNext(); ) {
            mode = (DcSearchMode) iterMode.next();
            break;
        }
        
        InternetUpdateProcess process = 
            server.getProcess(this, (String) dco.getValue(Book._A_TITLE), mode, region);
        process.setMaximum(4);
        process.start();
        
        try {
            process.join();
        } catch (Exception exp) {}
        
        for (Iterator iter = result.iterator(); iter.hasNext() && !updated; ) {
            Book software = (Book) iter.next();

            String titleNew = StringUtils.normalize((String) software.getValue(Book._A_TITLE));
            String authorNew = StringUtils.normalize((String) software.getValue(Book._G_AUTHOR));
            String isbnNew = StringUtils.normalize((String) software.getValue(Book._N_ISBN13));
            
            boolean match = (isbn.length() > 0) && (isbnNew.length() > 0) ? isbn.equals(isbnNew) :
                            titleNew.toLowerCase().equals(title.toLowerCase()) && authorNew.toLowerCase().equals(author.toLowerCase());
                                     
            if (match) {
                updated = true;
                setValue(dco, Book._A_TITLE, software.getValue(Book._A_TITLE));
                setValue(dco, Book._B_DESCRIPTION, software.getValue(Book._B_DESCRIPTION));
                setValue(dco, Book._C_YEAR, software.getValue(Book._C_YEAR));
                setValue(dco, Book._E_RATING, software.getValue(Book._E_RATING));
                setValue(dco, Book._F_PUBLISHER, software.getValue(Book._F_PUBLISHER));
                setValue(dco, Book._G_AUTHOR, software.getValue(Book._G_AUTHOR));
                setValue(dco, Book._H_WEBPAGE, software.getValue(Book._H_WEBPAGE));
                setValue(dco, Book._J_ISBN10, software.getValue(Book._J_ISBN10));
                setValue(dco, Book._K_PICTUREFRONT, software.getValue(Book._K_PICTUREFRONT));
                setValue(dco, Book._N_ISBN13, software.getValue(Book._N_ISBN13));
            }
        }

        result.clear();
        return updated;
    }
}
