/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.processes.synchronizer;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import net.sf.dc.console.views.View;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.services.DcRegion;
import net.sf.dc.core.services.DcServer;
import net.sf.dc.db.DatabaseManager;
import net.sf.dc.exceptions.ValidationException;
import net.sf.dc.resources.DcResources;

public abstract class DefaultSynchronizer extends Synchronizer {

    public DefaultSynchronizer(String title, int module) {
        super(title, module);
    }
    
    public boolean canParseFiles() {
        return false;
    }

    public boolean canUseOnlineServices() {
        return true;
    }
    
    public abstract boolean onlineUpdate(DcObject dco, DcServer server, DcRegion region);

    public Thread getTask() {
        return new Task();
    }    
    
    private class Task extends Thread {
        public void run() {
            try {
                
                initialize();
                
                View view = DcModules.getCurrentModule().getCurrentSearchView();
                
                int items = view.getRows();
                Collection objects = new ArrayList();
                for (int i = 0; i < items && !isCancelled(); i++)
                    objects.add(view.getObject(i));
                
                initProgressBar(items);
                for ( Iterator iter = objects.iterator(); iter.hasNext() && !isCancelled(); ) {
                    DcObject dco = (DcObject) iter.next();
                    boolean updated = false;
                    
                    addMessage(DcResources.getText("msgSearchingOnlineFor", "" + dco));
                    updated = onlineUpdate(dco, getServer(), getRegion());
                    updateProgressBar();
                    
                    try {
                        if (updated) {
                            dco.setSilence(true);
                            dco.saveUpdate(true);
                            
                            while (DatabaseManager.getQueueSize() > 0) {
                                try {
                                    sleep(100);
                                } catch (Exception exp) {}
                            }
                        }
                    } catch (ValidationException ve) {
                        addMessage(ve.getMessage());
                    }
                }
            } finally {
                addMessage(DcResources.getText("msgSynchronizerEnded"));
                enableAction(true);
            }  
        }
    }

}
