/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.processes.synchronizer;

import java.util.Collection;
import java.util.Iterator;

import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.movie.Movie;
import net.sf.dc.core.services.DcRegion;
import net.sf.dc.core.services.DcSearchMode;
import net.sf.dc.core.services.DcServer;
import net.sf.dc.processes.http.InternetUpdateProcess;
import net.sf.dc.resources.DcResources;
import net.sf.ext.util.StringUtils;

public class MovieSynchronizer extends DefaultSynchronizer {

    public MovieSynchronizer() {
        super(DcResources.getText("lblMassItemUpdate", DcModules.get(DcModules._MOVIE).getObjectName()),
              DcModules._MOVIE);
    }
    
    public String getHelpText() {
        return DcResources.getText("msgMovieMassUpdateHelp");
    }
    
    public boolean onlineUpdate(DcObject dco, DcServer server, DcRegion region) {

        boolean updated = false;
        
        String title = StringUtils.normalize((String) dco.getValue(Movie._A_TITLE));
        String director = StringUtils.normalize((String) dco.getValue(Movie._J_DIRECTOR));
        if ((title == null || title.length() == 0)) 
            return updated;
        
        Collection modes = server.getModes();
        DcSearchMode mode = null;
        for (Iterator iterMode = modes.iterator(); iterMode.hasNext(); ) {
            mode = (DcSearchMode) iterMode.next();
            break;
        }
        
        InternetUpdateProcess process = server.getProcess(this, (String) dco.getValue(Movie._A_TITLE), mode, region);
        process.setMaximum(2);
        process.start();
        
        try {
            process.join();
        } catch (Exception exp) {}
        
        for (Iterator iter = result.iterator(); iter.hasNext() && !updated; ) {
            Movie software = (Movie) iter.next();

            String titleNew = StringUtils.normalize((String) software.getValue(Movie._A_TITLE));
            String directorNew = StringUtils.normalize((String) software.getValue(Movie._J_DIRECTOR));

            boolean match = director.length() > 0 && directorNew.length() > 0 ? 
                                title.equals(titleNew) && director.equals(directorNew) : title.equals(titleNew);
            
            if (match) {
                updated = true;
                setValue(dco, Movie._A_TITLE, software.getValue(Movie._A_TITLE));
                setValue(dco, Movie._B_DESCRIPTION, software.getValue(Movie._B_DESCRIPTION));
                setValue(dco, Movie._C_YEAR, software.getValue(Movie._C_YEAR));
                setValue(dco, Movie._E_RATING, software.getValue(Movie._E_RATING));
                setValue(dco, Movie._G_WEBPAGE, software.getValue(Movie._G_WEBPAGE));
                setValue(dco, Movie._H_GENRES, software.getValue(Movie._H_GENRES));
                setValue(dco, Movie._I_ACTORS, software.getValue(Movie._I_ACTORS));
                setValue(dco, Movie._J_DIRECTOR, software.getValue(Movie._J_DIRECTOR));
                setValue(dco, Movie._K_COUNTRY, software.getValue(Movie._K_COUNTRY));
                setValue(dco, Movie._0_LANGUAGE, software.getValue(Movie._0_LANGUAGE));
                setValue(dco, Movie._L_PLAYLENGTH, software.getValue(Movie._L_PLAYLENGTH));
                setValue(dco, Movie._X_PICTUREFRONT, software.getValue(Movie._X_PICTUREFRONT));
                setValue(dco, Movie._Y_PICTUREBACK, software.getValue(Movie._Y_PICTUREBACK));
                setValue(dco, Movie._Z_PICTURECD, software.getValue(Movie._Z_PICTURECD));
                setValue(dco, Movie._3_CERTIFICATION, software.getValue(Movie._3_CERTIFICATION));
                setValue(dco, Movie._5_ASIN, software.getValue(Movie._5_ASIN));
                setValue(dco, Movie._7_STORAGEMEDIUM, software.getValue(Movie._7_STORAGEMEDIUM));
            }
        }

        result.clear();
        return updated;
    }
}
