/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.processes.synchronizer;

import java.util.Collection;
import java.util.Iterator;

import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.software.Software;
import net.sf.dc.core.objects.software.SoftwarePlatform;
import net.sf.dc.core.services.DcMobyGamesServer;
import net.sf.dc.core.services.DcRegion;
import net.sf.dc.core.services.DcSearchMode;
import net.sf.dc.core.services.DcServer;
import net.sf.dc.processes.http.InternetUpdateProcess;
import net.sf.dc.resources.DcResources;
import net.sf.ext.util.StringUtils;

public class SoftwareSynchronizer extends DefaultSynchronizer {

    public SoftwareSynchronizer() {
        super(DcResources.getText("lblMassItemUpdate", DcModules.get(DcModules._SOFTWARE).getObjectName()),
              DcModules._SOFTWARE);
    }
    
    public String getHelpText() {
        return DcResources.getText("msgSoftwareMassUpdateHelp");
    }
    
    public boolean onlineUpdate(DcObject dco, DcServer server, DcRegion region) {

        boolean updated = false;
        
        String title = StringUtils.normalize((String) dco.getValue(Software._A_TITLE));
        SoftwarePlatform platform = (SoftwarePlatform) dco.getValue(Software._H_PLATFORM);
        if ((title == null || title.length() == 0) || platform == null) 
            return updated;
        
        Collection modes = server.getModes();
        DcSearchMode mode = null;
        for (Iterator iterMode = modes.iterator(); iterMode.hasNext(); ) {
            mode = (DcSearchMode) iterMode.next();
            break;
        }
        
        String searchString = (String) dco.getValue(Software._A_TITLE);
        searchString += server instanceof DcMobyGamesServer && dco.getValue(Software._H_PLATFORM) != null ? 
                            " " + dco.getDisplayString(Software._H_PLATFORM) : "";
        
        InternetUpdateProcess process = server.getProcess(this, searchString, mode, region);
        process.setMaximum(4);
        process.start();
        
        try {
            process.join();
        } catch (Exception exp) {}
        
        for (Iterator iter = result.iterator(); iter.hasNext() && !updated; ) {
            Software software = (Software) iter.next();

            String titleNew = StringUtils.normalize((String) software.getValue(Software._A_TITLE));
            SoftwarePlatform platformNew = (SoftwarePlatform) software.getValue(Software._H_PLATFORM);
            
            if (titleNew.indexOf(" for ") > -1)
                titleNew = titleNew.substring(0, titleNew.indexOf(" for "));
            
            if ((titleNew.toLowerCase().equals(title.toLowerCase()))) {
                
                boolean match = true;
                if (platform != null && platformNew != null)
                    match = platform.toString().equals(platformNew.toString());
                
                if (match) {
                    updated = true;
                    setValue(dco, Software._A_TITLE, software.getValue(Software._A_TITLE));
                    setValue(dco, Software._B_DESCRIPTION, software.getValue(Software._B_DESCRIPTION));
                    setValue(dco, Software._C_YEAR, software.getValue(Software._C_YEAR));
                    setValue(dco, Software._E_RATING, software.getValue(Software._E_RATING));
                    setValue(dco, Software._F_DEVELOPER, software.getValue(Software._F_DEVELOPER));
                    setValue(dco, Software._G_PUBLISHER, software.getValue(Software._G_PUBLISHER));
                    setValue(dco, Software._H_PLATFORM, software.getValue(Software._H_PLATFORM));
                    setValue(dco, Software._I_WEBPAGE, software.getValue(Software._I_WEBPAGE));
                    setValue(dco, Software._M_PICTUREFRONT, software.getValue(Software._M_PICTUREFRONT));
                    setValue(dco, Software._N_PICTUREBACK, software.getValue(Software._N_PICTUREBACK));
                    setValue(dco, Software._O_PICTURECD, software.getValue(Software._O_PICTURECD));
                    setValue(dco, Software._P_SCREENSHOTONE, software.getValue(Software._P_SCREENSHOTONE));
                    setValue(dco, Software._Q_SCREENSHOTTWO, software.getValue(Software._Q_SCREENSHOTTWO));
                    setValue(dco, Software._R_SCREENSHOTTHREE, software.getValue(Software._R_SCREENSHOTTHREE));
                }
            }
        }

        result.clear();
        return updated;
    }
}
