/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.processes.synchronizer;

import java.util.ArrayList;
import java.util.Collection;

import net.sf.dc.console.forms.ItemSynchronizerDialog;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.services.DcRegion;
import net.sf.dc.core.services.DcServer;
import net.sf.dc.messages.Messages;
import net.sf.dc.processes.http.IOnlineUpdateListener;
import net.sf.dc.settings.DcSettings;

public abstract class Synchronizer implements IOnlineUpdateListener {
    
    protected ItemSynchronizerDialog dlg;
    private String title;
    private final int module;
    
    public Synchronizer(String title, int module) {
        this.title = title;
        this.module = module;
    }
    
    public abstract Thread getTask();
    public abstract String getHelpText();
    
    public abstract boolean canParseFiles();
    public abstract boolean canUseOnlineServices();
    
    protected Collection result = new ArrayList();

    public  String getTitle() {
        return title;
    }

    public String getHelpIndex() {
        return "dc.Tools.MassItemUpdate";
    }
    
    public void synchronize(ItemSynchronizerDialog dlg) {
        this.dlg = dlg;
        
        Thread thread = getTask();
        thread.start();
    }
    
    public void setListener(ItemSynchronizerDialog dlg) {
        this.dlg = dlg;
    }
    
    public void addMessage(String message) {
        dlg.addMessage(message);
    }
    
    protected boolean isReparseFiles() {
        return dlg.isReparseFiles();
    }

    protected boolean useOnlineService() {
        return dlg.useOnlineService();
    }
    
    
    protected boolean isCancelled() {
        return dlg.isCancelled();
    }
    
    protected void initProgressBar(int max) {
        dlg.initProgressBar(max);
    }

    protected void updateProgressBar() {
        dlg.updateProgressBar();
    }
    
    protected void enableAction(boolean b) {
        dlg.enableActions(b);
    }

    protected void initialize() {
        result.clear();
        dlg.initialize();
    }
    
    protected DcServer getServer() {
        return dlg.getServer();
    }

    protected DcRegion getRegion() {
        return dlg.getRegion();
    }
    
    protected void setValue(DcObject dco, int field, Object newValue) {
        
        int[] fields = 
            DcSettings.getValueAsBoolean(Repository.settings.stODBOverwriteValues) ?
            DcSettings.getValueAsIntegerArray(DcModules.get(module).getOnlineUpdateFieldsSettingsKey()) :
            dco.getModule().getFieldIndices();
            
        boolean allowed = false;
        for (int i = 0;i < fields.length; i++) {
            if (fields[i] == field) {
                allowed = true;
                break;
            }
        }
        
        if (allowed) {
            if (dco.isFilled(field)) {
                if (DcSettings.getValueAsBoolean(Repository.settings.stODBOverwriteValues) && 
                    (newValue != null && !newValue.equals("") && !newValue.equals("-1"))) {
                    dco.setValue(field, newValue);
                }
            } else {
                dco.setValue(field, newValue);
            }
        }
    }
    
    public void addError(Throwable t) {
        Messages.add(t);
    }

    public void addError(String message) {}

    public void addObject(DcObject dco) {
        result.add(dco);
    }

    public void addWarning(String warning) {}

    public DcModule getModule() {
        return DcModules.get(DcModules._MUSICALBUM);
    }

    public void processed(int i) {}

    public void processing() {}

    public void processingTotal(int i) {}

    public int resultCount() {
        return result.size();
    }

    public void stopped() {}     
}