/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.reporting;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Iterator;

import net.sf.dc.core.DataCrow;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.messages.Messages;
import net.sf.ext.util.Converter;

public class ReportDictionaryGenerator {

    public ReportDictionaryGenerator() {}
    
    public void generate() {
        
        String lf = "\r\n";
        
        StringBuffer sb = new StringBuffer();
        for (Iterator iter = DcModules.getAllModules(); iter.hasNext(); ) {
            DcModule module = (DcModule) iter.next();
            
            if (module.isTopModule()) {
                sb.append("Module              [" + module.getName() + "]" + lf);
                sb.append("xsl reference name  [" + Converter.getValidXmlTag(module.getName()) + "]" + lf);
                
                for (int i = 0; i < 80; i++)
                    sb.append("~");
                
                sb.append(lf);
                int maxLength = 0; 
                for (Iterator iterFields = module.getFields().iterator(); iterFields.hasNext(); ) {
                    DcField field = (DcField) iterFields.next();
                    maxLength = field.getLabel().length() > maxLength ? field.getLabel().length() : maxLength;
                }
                
                for (Iterator iterFields = module.getFields().iterator(); iterFields.hasNext(); ) {
                    DcField field = (DcField) iterFields.next();
                    int gap = maxLength - field.getLabel().length();
                    sb.append("  ");
                    sb.append(field.getLabel());
                    
                    for (int i = 0; i < gap; i++)
                        sb.append(" ");
                    
                    sb.append(" = " + Converter.getValidXmlTag(field.getSystemName()) + lf);    
                }
                
                sb.append(lf);
                sb.append(lf);
            }
        }
        
        try {
            File file = new File(DataCrow.reportDir + "dictionary.txt");
            FileOutputStream fos = new FileOutputStream(file);
            BufferedOutputStream bos = new BufferedOutputStream(fos);
    
            bos.write(sb.toString().getBytes());
            bos.flush();
            bos.close();
        } catch (IOException exp) {
            Messages.add(exp);
        }
    }
}
