/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.reporting.reports;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.util.Collection;

import net.sf.dc.console.reporting.ReportingDialog;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DcResources;

public abstract class Report {

    protected Collection objects;
    protected File target;
    protected ReportingDialog dialog;
    protected BufferedOutputStream bos;

    protected ReportThread rt;
    
    protected boolean keepOnRunning = true;
    
    protected boolean successfull = true;

    public Report() {}
    
    public void compile(ReportingDialog dialog, Collection objects, File target) {
        successfull = true;
        keepOnRunning = true;
        this.dialog = dialog;
        this.objects = objects;
        this.target = target;
        
        rt = new ReportThread();
        rt.start();
    }
    
    public boolean isSuccessfull() {
        return successfull;
    }
    
    public void join() {
        try {
            rt.join();
        } catch (InterruptedException ignore) {}
    }
    
    public void cancel() {
        keepOnRunning = false;
    }
    
    protected void initialize() throws Exception {
        bos = new BufferedOutputStream(new FileOutputStream(target));
        dialog.initProgressBar(objects.size());
        dialog.addMessage(DcResources.getText("msgCreatingReportForXObjects", String.valueOf(objects.size())));
    }    
    
    
    private class ReportThread extends Thread {
        public void run() {
            try {
                create();
            } catch (Exception exp) {
                successfull = false;
                Messages.add(exp);
                dialog.addMessage(DcResources.getText("msgErrorWhileCreatingReport", exp.toString()));
            } finally {
                objects = null;
                target = null;
                bos = null;
                dialog.allowActions(true);
                dialog = null;
            }
        }
    }

    public abstract void create() throws Exception;
    public abstract String getFileType();
}
