/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.reporting.reports;

import java.io.IOException;
import java.util.Iterator;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.resources.DcResources;

public class TextReport extends Report {
    
    public TextReport() {
        super();
    }    
    
    private void writeBytes(String value, boolean addTab) throws IOException {
        value = value.replaceAll("\t", "");
        
        if (value.length() > 4 && value.indexOf("\n") > 0) {
            value = value.replaceAll("\r", " ");
            value = value.replaceAll("\n", " ");
        }
        
        if (addTab)
            value = "\t" + value;
        
        bos.write(value.getBytes("UTF8"));
    }
    
    public void create() throws Exception {
        initialize();
        
        if (objects == null || objects.size() == 0) return;
        
        // create the table and the header
        DcObject dco = (DcObject) objects.toArray()[0];
        
        int counter = 0;
        for (Iterator iter = dco.getFields().iterator(); iter.hasNext() && keepOnRunning; ) {
            DcField field = (DcField) iter.next();
            if (field.isEnabled()) {
                writeBytes(field.getLabel(), counter != 0);
                counter++;
            }                
        }        
        
        writeBytes("\r\n", false);
        
        counter = 0;
        for (Iterator iterData = objects.iterator(); iterData.hasNext() && keepOnRunning; counter++) {
            DcObject o = (DcObject) iterData.next();
            
            dialog.addMessage(DcResources.getText("msgAddingToReport", o.toString()));
            counter = 0;
            for (Iterator iter = dco.getFields().iterator(); iter.hasNext() && keepOnRunning; ) {
                DcField field = (DcField) iter.next();

                if (field.isEnabled()) { 
                    String s = "";
                    if (field.getFieldType() == ComponentFactory._PICTUREFIELD) {
                        if (o.getValue(field.getIndex()) == null || o.getValue(field.getIndex()).toString().length() < 10)
                            s = DcResources.getText("lblNo");
                        else 
                            s = DcResources.getText("lblYes");
                    } else {
                        s = o.getDisplayString(field.getIndex());
                    }
                    
                    writeBytes(s, counter != 0);
                    counter++;
                }
            }
            
            writeBytes("\r\n", false);
            dialog.updateProgressBar(counter + 1);
            bos.flush();
        }
        
        bos.close();
        
        dialog.addMessage(DcResources.getText("lblExportHasFinished"));
    }

    public String getFileType() {
        return "txt";
    }
    
    public String toString() {
        return DcResources.getText("lblTextReport");
    }
}
