/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.reporting.reports;

import java.io.IOException;
import java.util.Iterator;

import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.reporting.templates.ReportTemplateProperties;
import net.sf.dc.reporting.writer.XmlSchemaWriter;
import net.sf.dc.reporting.writer.XmlWriter;
import net.sf.dc.resources.DcResources;

public class XmlReport extends Report {
    
    private ReportTemplateProperties properties;
    
    public XmlReport(ReportTemplateProperties properties) {
        super();
        this.properties = properties;
    }  

    public void create() throws Exception {
        initialize();
        
        String schemaFile = target.toString();
        schemaFile = schemaFile.substring(0, schemaFile.lastIndexOf(".xml")) + ".xsd";

        generateXsd(schemaFile);
        generateXml(schemaFile);
    }
    
    private void generateXsd(String schemaFile) throws IOException {
        XmlSchemaWriter schema = new XmlSchemaWriter(schemaFile);
        schema.create(DcModules.getCurrentModule().getDcObject());
    }
    
    private void generateXml(String schemaFile) throws IOException {
        if (objects == null || objects.size() == 0) return;
        
        XmlWriter xmlWriter = new XmlWriter(bos, target.toString(), schemaFile, properties);
        xmlWriter.startDocument();
        
        int counter = 0;
        for (Iterator iterData = objects.iterator(); iterData.hasNext() && keepOnRunning; counter++) {
            DcObject dco = (DcObject) iterData.next();
            
            if (DcModules.getCurrentModule().getIndex() == DcModules._GENERIC)
                dco.reload();
            
            xmlWriter.startEntity(dco);
            dialog.addMessage(DcResources.getText("msgAddingToReport", dco.toString()));

            xmlWriter.writeAttribute(dco, DcObject._SYS_MODULE);
            for (Iterator iter = dco.getFields().iterator(); iter.hasNext() && keepOnRunning; ) {
                xmlWriter.writeAttribute(dco, ((DcField) iter.next()).getIndex());
            }

            if (dco.getChildren() == null || dco.getChildren().size() == 0)
                dco.loadChildren();
            
            if (dco.getModule().getChildModule() != null) {
                
                xmlWriter.startRelations(dco.getModule().getChildModule());
                xmlWriter.setIdent(2);

                for (Iterator iterChild = dco.getChildren().iterator(); iterChild.hasNext(); ) {
                    DcObject child = (DcObject) iterChild.next();
                    
                    xmlWriter.startEntity(child);
                    xmlWriter.writeAttribute(child, DcObject._SYS_MODULE);
                    int[] fields = child.getFieldIndices();
                    for (int i = 0; i < fields.length; i++)
                        xmlWriter.writeAttribute(child, fields[i]);
                    
                    xmlWriter.endEntity(child);
                }
                
                xmlWriter.resetIdent();
                xmlWriter.endRelations(dco.getModule().getChildModule());
            } 
            
            xmlWriter.endEntity(dco);
            dialog.updateProgressBar(counter + 1);
            bos.flush();
        }
        
        xmlWriter.endDocument();
        dialog.addMessage(DcResources.getText("lblExportHasFinished"));
    }
    
    public String getFileType() {
        return "xml";
    }
    
    public String toString() {
        return DcResources.getText("lblXmlReport");
    }    
}
