/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.reporting.templates;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Properties;

import net.sf.dc.messages.Messages;

public class ReportTemplateProperties {

    public static final String _NAME = "report_name";
    public static final String _COPY_IMAGES = "copy_images";
    public static final String _SCALE_IMAGES = "scale_images";
    public static final String _IMAGE_WIDTH = "image_width";
    public static final String _IMAGE_HEIGHT = "image_height";
    public static final String _MAX_TEXT_LENGTH = "max_text_length";
    
    private final String filename;
    private final Properties properties;
    
    public ReportTemplateProperties() {
        filename = null;
        properties = new Properties();
    }    
    
    public ReportTemplateProperties(String reportFile) {
        filename = reportFile.substring(0, reportFile.toLowerCase().lastIndexOf(".xsl")) + ".properties";
        properties = new Properties();
        
        try {
            File file = new File(filename);
            if (file.exists())
                properties.load(new FileInputStream(file));
        } catch (Exception exp) {
            Messages.add(exp);
        }
    }
    
    public void set(String key, Object value) {
        properties.setProperty(key, "" + value);
    }
    
    public String get(String key) {
        return (String) properties.get(key);
    }
    
    public boolean getBoolean(String key) {
        String value = get(key);
        Boolean b = new Boolean(value);
        return b.booleanValue();
    }
    
    public int getInt(String key) {
        String value = get(key);
        int i = 0;
        try {
            i = Integer.parseInt(value);
        } catch (NumberFormatException nfe) {}
        return i;
    }
    
    public void save() {
        try {
            File file = new File(filename);
            properties.store(new FileOutputStream(file), "");
        } catch (Exception exp) {
            Messages.add(exp);
        }
    }
}
