/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.reporting.templates;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import net.sf.dc.core.DataCrow;
import net.sf.dc.core.modules.DcMediaModule;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.modules.template.ITemplateModule;
import net.sf.ext.util.DirectoryReader;

public class ReportTemplates {

    private Map folders = new HashMap();
    
    public ReportTemplates() {
        // on first call initialize directories
        for (Iterator iter = DcModules.getModules(); iter.hasNext(); ) {
            DcModule module = (DcModule) iter.next();
            
            if (   (module instanceof DcMediaModule || module.getIndex() == DcModules._CONTACTPERSON) && 
                    module.isTopModule() &&
//                    module.getIndex() != DcModules._GENERIC &&
                  !(module instanceof ITemplateModule)) {
                
                String path = DataCrow.reportDir + 
                              module.getName().toLowerCase().replaceAll("[/\\*%., ]", "");
                File file = new File(path);
                if (!file.exists())
                    file.mkdirs();
                
                folders.put(new Integer(module.getIndex()), path);
            }
        }
    }
    
    public Collection getFolders() {
        return folders.values();
    }
    
    public Collection getReportFiles(int transformer) {
        return getReportFiles(DcModules.getCurrentModule().getIndex(), transformer);
    }
    
    public Collection getReportFiles(int module, int transformer) {
        String folder = (String) folders.get(new Integer(module));
        
        Collection reports = new ArrayList();
        if (folder != null) {
            String[] extensions = {"xsl", "xslt"};
            Collection files = DirectoryReader.read(folder, false, true, false, extensions, false);
            for (Iterator iter = files.iterator(); iter.hasNext(); ) {
                ReportTemplate rf = new ReportTemplate((String) iter.next());
                if (rf.supports(transformer))
                    reports.add(rf);
            }
        }
        
        return reports;
    }
}
