/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.reporting.transformers;

import java.io.BufferedOutputStream;
import java.io.File;
import java.util.Collection;

import net.sf.dc.console.reporting.ReportingDialog;
import net.sf.dc.messages.Messages;
import net.sf.dc.reporting.reports.XmlReport;
import net.sf.dc.reporting.templates.ReportTemplate;
import net.sf.dc.reporting.templates.ReportTemplateProperties;
import net.sf.dc.resources.DcResources;

public abstract class XmlTransformer {
    
    protected File source;
    protected File target;
    protected File template;
    
    protected Collection objects;
    protected ReportingDialog dialog;
    protected BufferedOutputStream bos;
    
    private ReportTemplateProperties properties;
    private XmlReport xmlReport;

    protected boolean keepOnRunning = true;

    public XmlTransformer() {}
    
    public void transform(  ReportingDialog dialog, 
                            Collection objects, 
                            File target, 
                            ReportTemplate reportFile) {
        
        this.dialog = dialog;
        this.objects = objects;
        
        this.target = target;
        this.template = new File(reportFile.getFilename());
        this.properties = reportFile.getProperties();
        
        String s = target.toString();
        s = s.substring(0, s.lastIndexOf(".")) + ".xml";
        this.source = new File(s);
        
        Transformer rt = new Transformer();
        rt.start();
    }
    
    public void cancel() {
        keepOnRunning = false;
        if (xmlReport != null)
            xmlReport.cancel();
    }    
    
    public abstract int getType();
    public abstract void transform() throws Exception ;
    public abstract String getFileType();
    
    private class Transformer extends Thread {
        public void run() {

            keepOnRunning = true;
            
            xmlReport = new XmlReport(properties);
            xmlReport.compile(dialog, objects, source);
            xmlReport.join();

            try {

                if (xmlReport.isSuccessfull() && keepOnRunning) {
                    dialog.allowActions(false);
                    dialog.addMessage(DcResources.getText("msgTransformingOutput", getFileType()));
                    transform();
                    dialog.addMessage(DcResources.getText("msgTransformationSuccessful", target.toString()));
                }

            } catch (Exception exp) {
                Messages.add(exp);
                dialog.addMessage(DcResources.getText("msgErrorWhileCreatingReport", exp.toString()));
            } finally {
                source = null;
                template = null;
                objects = null;
                target = null;
                bos = null;
                dialog.allowActions(true);
                dialog = null;
            }
        }
    }
}
