/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.reporting.writer;

import java.io.BufferedOutputStream;
import java.io.FileOutputStream;
import java.io.IOException;

import net.sf.ext.util.Converter;

public abstract class XmlBaseWriter {

    protected final String uberTag = "data-crow-objects";
    protected final BufferedOutputStream bos;

    protected XmlBaseWriter(BufferedOutputStream bos) {
        this.bos = bos;
    }

    protected XmlBaseWriter(String filename) throws IOException {
        FileOutputStream fos = new FileOutputStream(filename);
        bos = new BufferedOutputStream(fos);
    }    
    
    protected String getValidTag(String s) {
        return Converter.getValidXmlTag(s);
    }
    
    protected void newLine() throws IOException {
        bos.write("\r\n".getBytes());
    }
    
    protected void writeLine(String s, int level) throws IOException {
        for (int i = 0; i < level; i++)
            bos.write("    ".getBytes());

        writeTag(s);
        newLine();
    }
    
    
    protected void writeTag(String tag) throws IOException {
        bos.write(tag.getBytes("UTF8"));
    }
}
