/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.settings;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.io.File;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JFrame;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.views.MasterView;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.modules.audiocd.AudioCDModule;
import net.sf.dc.core.modules.book.BookModule;
import net.sf.dc.core.modules.contactperson.ContactPersonModule;
import net.sf.dc.core.modules.image.ImageModule;
import net.sf.dc.core.modules.movie.MovieModule;
import net.sf.dc.core.modules.musicalbum.MusicAlbumModule;
import net.sf.dc.core.modules.software.SoftwareModule;
import net.sf.dc.core.objects.DcGenre;
import net.sf.dc.core.objects.DcLookAndFeel;
import net.sf.dc.core.objects.DcMapping;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.DcState;
import net.sf.dc.core.objects.Loan;
import net.sf.dc.core.objects.audiocd.AudioCD;
import net.sf.dc.core.objects.audiocd.AudioTrack;
import net.sf.dc.core.objects.book.Book;
import net.sf.dc.core.objects.book.BookCategory;
import net.sf.dc.core.objects.contactperson.ContactPerson;
import net.sf.dc.core.objects.contactperson.ContactPersonCategory;
import net.sf.dc.core.objects.generic.GenericObject;
import net.sf.dc.core.objects.image.Image;
import net.sf.dc.core.objects.image.ImageCategory;
import net.sf.dc.core.objects.movie.Movie;
import net.sf.dc.core.objects.musicalbum.MusicAlbum;
import net.sf.dc.core.objects.musicalbum.MusicTrack;
import net.sf.dc.core.objects.software.Software;
import net.sf.dc.core.objects.software.SoftwareCategory;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.definitions.DcFieldDefinition;
import net.sf.dc.settings.definitions.DcFieldDefinitions;
import net.sf.dc.settings.definitions.Definitions;
import net.sf.dc.settings.definitions.ProgramDefinitions;
import net.sf.ext.jflexsett.settings.Setting;
import net.sf.ext.jflexsett.settings.Settings;
import net.sf.ext.jflexsett.settings.SettingsFile;
import net.sf.ext.jflexsett.settings.SettingsGroup;
import net.sf.ext.jflexsett.settings.ui.SettingsView;

public class DcSettings {

    public static final String _General = DcResources.getText("lblGroupGeneral");
    public static final String _Module = DcResources.getText("lblModuleSettings");
    public static final String _FileHandlers = DcResources.getText("lblProgramDefinitions");
    public static final String _TableColors = DcResources.getText("lblTableColorSettings");
    public static final String _OddRowColor = DcResources.getText("lblOddRowColor");
    public static final String _EvenRowColor = DcResources.getText("lblEvenRowColor");
    public static final String _HeaderColor = DcResources.getText("lblTableHeaderColor");
    public static final String _RowSelectionColor = DcResources.getText("lblRowSelectionColor");
    public static final String _HTTP = DcResources.getText("lblHTTPSettings");
    public static final String _Font = DcResources.getText("lblFont");

    private static Settings settings = new Settings();

    public DcSettings() {
        createGroups();
        createSettings();
        settings.setSettingsFile(new File(DataCrow.baseDir + "data/data_crow.properties"));
        
        try {
            SettingsFile.load(settings);
        } catch (Exception exp) {
            Messages.add(DcResources.getText("msgFailedToLoadUserSettings", exp.getMessage()), Messages._ERROR);
        }
    }

    public static void save() {
        SettingsFile.save(settings);
    }

    public static SettingsView getView(JFrame parent) {
        SettingsView view = new SettingsView(parent, "Data Crow Settings Panel", settings);
        
        try {
            ImageIcon icon = new ImageIcon(new URL("file://" + DataCrow.baseDir + "icons_system/logo.jpg"));
            view.setDisclaimer(icon);
        } catch (Exception ignore) {}

        view.setSize(new Dimension(875, 450));
        return view;
    }

    public static Object getValue(String key) {
        return settings.getValue(key);
    }

    public static Definitions getValueAsDefinitionGroup(String key) {
        return settings.getValueAsDefinitionGroup(key);
    }

    public static String[] getValueAsStringArray(String key) {
        return settings.getValueAsStringArray(key);
    }

    public static int[] getValueAsIntegerArray(String key) {
        return settings.getValueAsIntegerArray(key);
    }

    public static Font getValueAsFont(String key) {
        return settings.getValueAsFont(key);
    }

    public static DcLookAndFeel getValueAsLookAndFeel(String key) {
        return settings.getValueAsLookAndFeel(key);
    }
    
    public static int getValueAsInt(String key) {
        return settings.getValueAsInt(key);
    }

    public static boolean getValueAsBoolean(String key) {
        return settings.getValueAsBoolean(key);
    }


    public static Color getValueAsColor(String key) {
        return settings.getValueAsColor(key);
    }

    public static Dimension getValueAsDimension(String key) {
        return settings.getValueAsDimension(key);
    }


    public static String getValueAsString(String key) {
        return settings.getValueAsString(key);
    }

    public static void setValue(String key, Object value) {
        settings.setValue(key, value);
    }

    public static void setStringAsValue(String key, String value) {
        settings.setStringAsValue(key, value);
    }

    private void createGroups() {
        SettingsGroup generalGroup = new SettingsGroup(_General, "dc.Settings.GeneralSettings");
        SettingsGroup tableColorGroup = new SettingsGroup(_TableColors, "dc.Settings.ViewColors");
        SettingsGroup http = new SettingsGroup(_HTTP, "dc.Settings.HttpSettings");
        SettingsGroup fileHandlers = new SettingsGroup(_FileHandlers, "dc.Settings.ProgramDefinitions");
        SettingsGroup moduleGroup = new SettingsGroup(_Module, "dc.Settings.ModuleSettings");
        
        SettingsGroup fontGroup = new SettingsGroup(_Font, "dc.Settings.Font");
        
        tableColorGroup.addChild(new SettingsGroup(_OddRowColor, "dc.Settings.TableColors"));
        tableColorGroup.addChild(new SettingsGroup(_EvenRowColor, "dc.Settings.TableColors"));
        tableColorGroup.addChild(new SettingsGroup(_HeaderColor, "dc.Settings.TableColors"));
        tableColorGroup.addChild(new SettingsGroup(_RowSelectionColor, "dc.Settings.TableColors"));
        
        settings.addGroup(_Module, moduleGroup);
        settings.addGroup(_General, generalGroup);
        settings.addGroup(_Font, fontGroup);
        settings.addGroup(_HTTP, http);
        settings.addGroup(_TableColors, tableColorGroup);
        settings.addGroup(_FileHandlers, fileHandlers);
    }

    private void createSettings() {
        settings.addSetting(_General,
                new Setting(Repository.datatypes._INTEGER,
                            Repository.settings.stVersionMajor,
                            new Integer(Repository.system.versionMajor),
                            -1,
                            "",
                            "",
                            false,
                            false));
        settings.addSetting(_General,
                new Setting(Repository.datatypes._INTEGER,
                            Repository.settings.stVersionMinor,
                            new Integer(Repository.system.versionMinor),
                            -1,
                            "",
                            "",
                            false,
                            false));
        settings.addSetting(_General,
                new Setting(Repository.datatypes._INTEGER,
                            Repository.settings.stVersionBuild,
                            new Integer(Repository.system.versionBuild),
                            -1,
                            "",
                            "",
                            false,
                            false));
        settings.addSetting(_General,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.settings.stShowTipsOnStartup,
                            Boolean.TRUE,
                            -1,
                            "",
                            "",
                            false,
                            false));        
        settings.addSetting(_General,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.settings.stShowGroupingPanel,
                            Boolean.TRUE,
                            -1,
                            "",
                            "",
                            false,
                            false));   
        settings.addSetting(_General,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.settings.stShowQuickFilterBar,
                            Boolean.TRUE,
                            -1,
                            "",
                            "",
                            false,
                            false));          
        settings.addSetting(_General,
                new Setting(Repository.datatypes._INTEGER,
                            Repository.settings.stHsqlCacheScale,
                            new Integer(18),
                            -1,
                            "",
                            "",
                            false,
                            false));
        settings.addSetting(_General,
                new Setting(Repository.datatypes._INTEGER,
                            Repository.settings.stHsqlCacheSizeScale,
                            new Integer(20),
                            -1,
                            "",
                            "",
                            false,
                            false));
        settings.addSetting(_General,
                new Setting(Repository.datatypes._INTEGER,
                            Repository.settings.stGarbageCollectionIntervalMs,
                            new Integer(20000),
                            -1,
                            "",
                            "",
                            false,
                            false));

    	settings.addSetting(_RowSelectionColor,
                new Setting(Repository.datatypes._COLOR,
                            Repository.settings.stRowSelectionColor,
                            new Color(255, 253, 210),
                            ComponentFactory._COLORSELECTOR,
                            "",
                            DcResources.getText("lblRowSelectionColor"),
                            false,
                            true));

        settings.addSetting(_OddRowColor,
                new Setting(Repository.datatypes._COLOR,
                		    Repository.settings.stOddRowColor,
							new Color(250, 243, 215),
                            ComponentFactory._COLORSELECTOR,
						    "",
                            DcResources.getText("lblOddRowColor"),
                            false,
                            true));

    	settings.addSetting(_EvenRowColor,
                new Setting(Repository.datatypes._COLOR,
                		    Repository.settings.stEvenRowColor,
                            new Color(250, 244, 222),
                            ComponentFactory._COLORSELECTOR,
                            "",
							DcResources.getText("lblEvenRowColor"),
                            false,
                            true));

    	settings.addSetting(_HeaderColor,
                new Setting(Repository.datatypes._COLOR,
                            Repository.settings.stTableHeaderColor,
                            new Color(244, 242, 231),
                            ComponentFactory._COLORSELECTOR,
                            "",
                            DcResources.getText("lblTableHeaderColor"),
                            false,
                            true));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.settings.stCheckUniqueness,
                            new Boolean(true),
                            ComponentFactory._CHECKBOX,
                            DcResources.getText("tpUniqueness"),
                            DcResources.getText("lblUniqueness"),
                            false,
                            true));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.settings.stCheckRequiredFields,
                            new Boolean(true),
                            ComponentFactory._CHECKBOX,
                            DcResources.getText("tpRequiredFields"),
                            DcResources.getText("lblCheckRequiredFields"),
                            false,
                            true));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._STRING,
                            Repository.settings.stConnectionString,
                            "dc",
                            -1,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._STRING,
                            Repository.settings.stDatabaseDriver,
                            "org.hsqldb.jdbcDriver",
                            -1,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.settings.stShowTableTooltip,
                            new Boolean(true),
                            ComponentFactory._CHECKBOX,
                            DcResources.getText("tpShowTableTooltips"),
                            DcResources.getText("lblShowTableTooltips"),
                            false,
                            true));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._INTEGER,
                            Repository.settings.stModule,
                            new Integer(1),
                            -1,
                            "",
                            "",
                            true,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.settings.stShowHiddenFieldsInForm,
                            Boolean.TRUE,
                            ComponentFactory._CHECKBOX,
                            DcResources.getText("tpShowHiddenFieldsInForms"),
                            DcResources.getText("lblShowHiddenFieldsInForms"),
                            false,
                            true));
        settings.addSetting(_Font,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.settings.stFontAntiAliasing,
                            Boolean.FALSE,
                            ComponentFactory._CHECKBOX,
                            "",
                            "Font Anti-aliasing",
                            false,
                            true));        
    	settings.addSetting(_Font,
                new Setting(Repository.datatypes._FONT,
                		    Repository.settings.stSystemFontNormal,
							new Font("Arial", Font.PLAIN, 11),
							ComponentFactory._FONTSELECTOR,
							DcResources.getText("tpFont"),
							DcResources.getText("lblFontNormal"),
							false,
							true));
    	settings.addSetting(_Font,
                new Setting(Repository.datatypes._FONT,
                		    Repository.settings.stSystemFontBold,
                            new Font("Arial", Font.PLAIN, 11),
							ComponentFactory._FONTSELECTOR,
							DcResources.getText("tpFont"),
							DcResources.getText("lblFontBold"),
							false,
							true));
        settings.addSetting(_General,
                new Setting(Repository.datatypes._LOOKANDFEEL,
                            Repository.settings.stLookAndFeel,
                            new DcLookAndFeel("beigeazulthemepack.zip", null, "beigeazulthemepack.zip", 0),
                            ComponentFactory._LOOKANDFEELSELECTOR,
                            "",
                            DcResources.getText("lblLookAndFeel"),
                            false,
                            false));        
    	settings.addSetting(_HTTP,
                new Setting(Repository.datatypes._STRING,
                            Repository.settings.stHTTPUserAgent,
                            "Mozilla/4.04 (Windows; I; Win95)",
                            -1,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_HTTP,
                new Setting(Repository.datatypes._STRING,
                            Repository.settings.stProxyServerName,
                            "",
                            ComponentFactory._SHORTTEXTFIELD,
                            DcResources.getText("tpProxyServerName"),
                            DcResources.getText("lblProxyServerName"),
                            true,
                            true));
    	settings.addSetting(_HTTP,
                new Setting(Repository.datatypes._INTEGER,
                            Repository.settings.stProxyServerPort,
                            new Integer(0),
                            ComponentFactory._NUMBERFIELD,
                            DcResources.getText("tpProxyServerPort"),
                            DcResources.getText("lblProxyServerPort"),
                            true,
                            true));
    	settings.addSetting(_HTTP,
                new Setting(Repository.datatypes._STRING,
                            Repository.settings.stProxyUserName,
                            "",
                            ComponentFactory._SHORTTEXTFIELD,
                            DcResources.getText("tpProxyUserName"),
                            DcResources.getText("lblProxyUserName"),
                            true,
                            true));
    	settings.addSetting(_HTTP,
                new Setting(Repository.datatypes._STRING,
                            Repository.settings.stProxyPassword,
                            "",
                            ComponentFactory._PASSWORDFIELD,
                            DcResources.getText("tpProxyPassword"),
                            DcResources.getText("lblProxyPassword"),
                            true,
                            true));
        settings.addSetting(_HTTP,
                new Setting(Repository.datatypes._INTEGER,
                            Repository.settings.stSelectedView,
                            new Integer(MasterView._LIST_VIEW),
                            -1,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_HTTP,
                new Setting(Repository.datatypes._STRING,
                            Repository.settings.stBrowserPath,
                            "",
                            ComponentFactory._FILEFIELD,
                            "",
                            DcResources.getText("lblBrowserPath"),
                            true,
                            true));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stExpertFormSize,
                            new Dimension(700, 600),
                            -1,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stHelpFormSize,
                            new Dimension(700, 600),
                            -1,
                            "",
                            "",
                            false,
                            false));
        settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stOnlineUpdateOverwriteSettingsDialogSize,
                            new Dimension(600, 450),
                            -1,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stTabbedPaneSize,
                            new Dimension(958, 767),
                            -1,
                            "",
                            "",
                            false,
                            false));
        settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stPersonItemFormViewSize,
                            new Dimension(450, 550),
                            -1,
                            "",
                            "",
                            false,
                            false));
        settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stReferencesDialogSize,
                            new Dimension(300, 600),
                            -1,
                            "",
                            "",
                            false,
                            false));
        settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stAudioCDItemFormViewSize,
                            new Dimension(450, 550),
                            -1,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stSoftwareItemFormViewSize,
                            new Dimension(564,686),
                            -1,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stImageItemFormViewSize,
                            new Dimension(450, 550),
                            -1,
                            "",
                            "",
                            false,
                            false));    	
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stBookItemFormViewSize,
                            new Dimension(450, 550),
                            -1,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stMusicAlbumFormViewSize,
                            new Dimension(450, 550),
                            -1,
                            "",
                            "",
                            false,
                            false));
        settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stSimpleItemViewSize,
                            new Dimension(450, 550),
                            -1,
                            "",
                            "",
                            false,
                            false));        
        settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stSimpleItemFormSize,
                            new Dimension(450, 550),
                            -1,
                            "",
                            "",
                            false,
                            false));          
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stMovieItemFormViewSize,
                            new Dimension(556, 715),
                            -1,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stWizardFormSize,
                            new Dimension(800, 550),
                            -1,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stItemFormViewSize,
                            new Dimension(450, 550),
                            -1,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stUpdateAllDialogSize,
                            new Dimension(600, 600),
                            -1,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stMainViewSize,
                            new Dimension(1095, 871),
                            -1,
                            "",
                            "",
                            false,
                            false));
        settings.addSetting(_General,
                new Setting(Repository.datatypes._INTEGER,
                            Repository.settings.stMainViewState,
                            new Integer(-1),
                            -1,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stFieldSettingsDialogSize,
                            new Dimension(500, 300),
                            -1,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stTextViewerSize,
                            new Dimension(500, 700),
                            -1,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stOnlineSearchFormSize,
                            new Dimension(700, 600),
                            -1,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                         Repository.settings.stReportingDialogSize,
                         new Dimension(700, 490),
                         -1,
                         "",
                         "",
                         false,
                         false));
        settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                         Repository.settings.stModuleSelectDialogSize,
                         new Dimension(300, 500),
                         -1,
                         "",
                         "",
                         false,
                         false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stImportAudioCDDialogSize,
                            new Dimension(700, 600),
                            -1,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
               new Setting(Repository.datatypes._DIMENSION,
                           Repository.settings.stImportCDDialogSize,
                           new Dimension(550, 400),
                           -1,
                           "",
                           "",
                           false,
                           false));
        settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stImportMusicFileDialogSize,
                            new Dimension(550, 800),
                            -1,
                            "",
                            "",
                            false,
                            false));        
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stItemSynchronizerDialogSize,
                            new Dimension(550, 400),
                            -1,
                            "",
                            "",
                            false,
                            false));    	
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.settings.stShowModuleList,
                            new Boolean(true),
                            -1,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.settings.stODBOverwriteValues,
                            new Boolean(false),
                            -1,
                            "",
                            "",
                            false,
                            false));
        settings.addSetting(_General,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.settings.stAutoCreateSubItems,
                            new Boolean(true),
                            ComponentFactory._CHECKBOX,
                            DcResources.getText("tpAutoCreateMissingItems"),
                            DcResources.getText("lblAutoCreateMissingItems"),
                            false,
                            false));     
        
        settings.addSetting(_General,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.settings.stAutoAddPerfectMatch,
                            new Boolean(false),
                            ComponentFactory._CHECKBOX,
                            DcResources.getText("tpAutoAddPerfectMatch"),
                            DcResources.getText("lblAutoAddPerfectMatch"),
                            false,
                            false));        
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.settings.stBroadband,
                            new Boolean(true),
                            -1,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._STRING,
                            Repository.settings.stLastDirectoryUsed,
                            "",
                            -1,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.settings.stShowMenuBarLabels,
                            Boolean.TRUE,
                            -1,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.settings.stUpdateAllSelectedItemsOnly,
                            Boolean.TRUE,
                            -1,
                            "",
                            "",
                            false,
                            false));
    	settings.addSetting(_General,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.settings.stShowQuickView,
                            Boolean.TRUE,
                            -1,
                            "",
                            "",
                            false,
                            false));
        settings.addSetting(_General,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.settings.stDeleteImageFileAfterImport,
                            Boolean.FALSE,
                            ComponentFactory._CHECKBOX,
                            DcResources.getText("tpDeleteImageFile"),
                            DcResources.getText("lblDeleteImageFile"),
                            false,
                            true));
        settings.addSetting(_General,
                new Setting(Repository.datatypes._INTEGER,
                            Repository.settings.stQuickViewDividerLocation,
                            new Integer(595),
                            -1,
                            "",
                            "",
                            false,
                            false));
        settings.addSetting(_General,
                new Setting(Repository.datatypes._INTEGER,
                            Repository.settings.stTreeDividerLocation,
                            new Integer(200),
                            -1,
                            "",
                            "",
                            false,
                            false));        
        settings.addSetting(_FileHandlers,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stProgramDefinitions,
                            new ProgramDefinitions(),
                            ComponentFactory._PROGRAMDEFINITIONFIELD,
                            DcResources.getText("tpProgramDefinitions"),
                            "",
                            false,
                            true));

        settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stResourcesEditorViewSize,
                            new Dimension(542, 680),
                            -1,
                            "",
                            "",
                            false,
                            false));
        settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stBackupDialogSize,
                            new Dimension(500, 500),
                            -1,
                            "",
                            "",
                            false,
                            false));
        settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stFileImportDialogSize,
                            new Dimension(500, 600),
                            -1,
                            "",
                            "",
                            false,
                            false));
        settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stLoanFormSize,
                            new Dimension(500, 500),
                            -1,
                            "",
                            "",
                            false,
                            false));
        settings.addSetting(_General,
                new Setting(Repository.datatypes._DIMENSION,
                            Repository.settings.stSortDialogSize,
                            new Dimension(300, 200),
                            -1,
                            "",
                            "",
                            false,
                            false));
        settings.addSetting(_General,
                new Setting(Repository.datatypes._STRINGARRAY,
                            Repository.settings.stImageSearchOrder,
                            new String[] {DcResources.getText("lblLocation"), "Title"},
                            -1,
                            "",
                            "",
                            false,
                            false));        
        settings.addSetting(_General,
                new Setting(Repository.datatypes._STRINGARRAY,
                            Repository.settings.stSoftwareSearchOrder,
                            new String[] {DcResources.getText("lblLocation"), "Title"},
                            -1,
                            "",
                            "",
                            false,
                            false));
        settings.addSetting(_General,
                new Setting(Repository.datatypes._STRINGARRAY,
                            Repository.settings.stAudioCDSearchOrder,
                            new String[] {DcResources.getText("lblLocation"), "Title"},
                            -1,
                            "",
                            "",
                            false,
                            false));
        settings.addSetting(_General,
                new Setting(Repository.datatypes._STRINGARRAY,
                            Repository.settings.stMovieSearchOrder,
                            new String[] {DcResources.getText("lblLocation"), "Title"},
                            -1,
                            "",
                            "",
                            false,
                            false));
        settings.addSetting(_General,
                new Setting(Repository.datatypes._STRINGARRAY,
                            Repository.settings.stMusicAlbumSearchOrder,
                            new String[] {"Title"},
                            -1,
                            "",
                            "",
                            false,
                            false));        
        settings.addSetting(_General,
                new Setting(Repository.datatypes._STRINGARRAY,
                            Repository.settings.stPersonSearchOrder,
                            new String[] {"Name"},
                            -1,
                            "",
                            "",
                            false,
                            false));
        settings.addSetting(_General,
                new Setting(Repository.datatypes._STRINGARRAY,
                            Repository.settings.stGenericSearchOrder,
                            new String[] {"Title"},
                            -1,
                            "",
                            "",
                            false,
                            false));
        settings.addSetting(_General,
                new Setting(Repository.datatypes._STRINGARRAY,
                            Repository.settings.stBookSearchOrder,
                            new String[] {DcResources.getText("lblLocation"), "Title"},
                            -1,
                            "",
                            "",
                            false,
                            false));
        settings.addSetting(_Module,
                new Setting(Repository.datatypes._STRING,
                            Repository.settings.stModuleSettings,
                            "",
                            ComponentFactory._MODULESELECTOR,
                            "",
                            "",
                            false,
                            true));        
        settings.addSetting(_Module,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.settings.stGenericModuleEnabled,
                            Boolean.TRUE,
                            -1,
                            "",
                            "",
                            false,
                            false));  
        settings.addSetting(_Module,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.settings.stSoftwareModuleEnabled,
                            Boolean.TRUE,
                            -1,
                            "",
                            "",
                            false,
                            false));
        settings.addSetting(_Module,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.settings.stImageModuleEnabled,
                            Boolean.TRUE,
                            -1,
                            "",
                            "",
                            false,
                            false));           
        settings.addSetting(_Module,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.settings.stAudioCDModuleEnabled,
                            Boolean.TRUE,
                            -1,
                            "",
                            "",
                            false,
                            false));        
        settings.addSetting(_Module,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.settings.stBookModuleEnabled,
                            Boolean.TRUE,
                            -1,
                            "",
                            "",
                            false,
                            false));        
        settings.addSetting(_Module,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.settings.stContactPersonModuleEnabled,
                            Boolean.TRUE,
                            -1,
                            "",
                            "",
                            false,
                            false));        
        settings.addSetting(_Module,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.settings.stMusicAlbumModuleEnabled,
                            Boolean.TRUE,
                            -1,
                            "",
                            "",
                            false,
                            false));         
        settings.addSetting(_Module,
                new Setting(Repository.datatypes._BOOLEAN,
                            Repository.settings.stMovieModuleEnabled,
                            Boolean.TRUE,
                            -1,
                            "",
                            "",
                            false,
                            false));     
        settings.addSetting(_General,
                new Setting(Repository.datatypes._STRING,
                            Repository.settings.stReportFile,
                            "",
                            -1,
                            "",
                            "",
                            false,
                            false));      
        
        setSoftwareDefaults();
        setSoftwareCategoryDefaults();
        setAudioCDDefaults();
        setAudioTrackDefaults();
        setBookDefaults();
        setBookCategoryDefaults();
        setMovieDefaults();
        
        setMusicAlbumDefaults();
        setMusicTrackDefaults();
        
        setPersonDefaults();
        setGenericObjectDefaults();
        setLoanDefaults();
        setPersonTypeDefaults();
        setStateDefaults();
        setSoftwarePlatformDefaults();
        setStorageMediaDefaults();
        setGenreDefaults();
        
        setImageDefaults();
        setImageCategoryDefaults();
        
        setGroupingSettings(new SoftwareModule().getGroupedBySettingsKey());
        setGroupingSettings(new AudioCDModule().getGroupedBySettingsKey());
        setGroupingSettings(new BookModule().getGroupedBySettingsKey());
        setGroupingSettings(new MovieModule().getGroupedBySettingsKey());
        setGroupingSettings(new MusicAlbumModule().getGroupedBySettingsKey());
        setGroupingSettings(new ImageModule().getGroupedBySettingsKey());
        setGroupingSettings(new ContactPersonModule().getGroupedBySettingsKey());
        
        setOnlineUpdateFieldsSettings(new SoftwareModule());
        setOnlineUpdateFieldsSettings(new BookModule());
        setOnlineUpdateFieldsSettings(new MovieModule());
        setOnlineUpdateFieldsSettings(new AudioCDModule());

        setOnlineServiceSettings(new SoftwareModule());
        setOnlineServiceSettings(new BookModule());
        setOnlineServiceSettings(new MovieModule());
        setOnlineServiceSettings(new AudioCDModule());        
    }
    
    private void setOnlineServiceSettings(DcModule module) {
        String[] values;
        if (module.getIndex() == DcModules._AUDIOCD)
            values = new String[] {"Amazon,http://webservices.amazon.com", "Keywords"};
        else 
            values = new String[] {"MusicBrainz,http://musicbrainz.org/ws/1/", "Keywords"};
        
        settings.addSetting(_General,
                new Setting(Repository.datatypes._STRINGARRAY,
                            module.getOnlineServiceSettingsKey(),
                            values,
                            -1,
                            "",
                            "",
                            false,
                            false));   
    }

    private void setOnlineUpdateFieldsSettings(DcModule module) {
        settings.addSetting(_General,
                new Setting(Repository.datatypes._INTEGERARRAY,
                            module.getOnlineUpdateFieldsSettingsKey(),
                            module.getFieldIndices(),
                            -1,
                            "",
                            "",
                            false,
                            false));   
    }    
    
    private void setGroupingSettings(String key) {
        settings.addSetting(_General,
                new Setting(Repository.datatypes._INTEGERARRAY,
                            key,
                            new int[] {-1, -1},
                            -1,
                            "",
                            "",
                            false,
                            false));   
    }
    
    private void setGenericObjectDefaults() {
        DcFieldDefinitions definitions = new DcFieldDefinitions();
        definitions.addDefinition(new DcFieldDefinition(GenericObject._ID, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_AVAILABLE, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_LOANEDBY, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_LOANDURATION, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(GenericObject._A_TITLE, null, true, true, true));
        definitions.addDefinition(new DcFieldDefinition(GenericObject._B_DESCRIPTION, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(GenericObject._C_YEAR, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(GenericObject._D_LOCATION, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(GenericObject._E_RATING, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(GenericObject._SYS_MODULE, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_CREATED, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_MODIFIED, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(GenericObject._U1_USER_LONGTEXT, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(GenericObject._U2_USER_SHORTTEXT1, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(GenericObject._U3_USER_SHORTTEXT2, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(GenericObject._U4_USER_NUMERIC1, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(GenericObject._U5_USER_NUMERIC2, null, false, false, false));

        settings.addSetting(_General,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stFieldDefinitionsGeneric,
                            definitions, -1, "", "", false, false));
    }

    private void setLoanDefaults() {
        DcFieldDefinitions definitions = new DcFieldDefinitions();
        definitions.addDefinition(new DcFieldDefinition(Loan._ID, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Loan._A_STARTDATE, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Loan._B_ENDDATE, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Loan._C_CONTACTPERSONID, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Loan._D_OBJECTID, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_CREATED, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_MODIFIED, null, true, false, false));
        
        settings.addSetting(_General,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stFieldDefinitionsLoan,
                            definitions, -1, "", "", false, false));
    }

    private void setPersonDefaults() {
        DcFieldDefinitions definitions = new DcFieldDefinitions();
        definitions.addDefinition(new DcFieldDefinition(ContactPerson._ID, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(ContactPerson._A_NAME, null, true, true, true));
        definitions.addDefinition(new DcFieldDefinition(ContactPerson._B_DESCRIPTION, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(ContactPerson._C_PHOTO, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(ContactPerson._D_CATEGORY, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(ContactPerson._E_EMAIL, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(ContactPerson._F_ADDRESS, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(ContactPerson._G_PHONE_HOME, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(ContactPerson._H_PHONE_WORK, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_CREATED, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_MODIFIED, null, true, false, false));

        settings.addSetting(_General,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stFieldDefinitionsPerson,
                            definitions, -1, "", "", false, false));
    }

    private void setPersonTypeDefaults() {
        DcFieldDefinitions definitions = new DcFieldDefinitions();
        definitions.addDefinition(new DcFieldDefinition(ContactPersonCategory._ID, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(ContactPersonCategory._A_NAME, null, true, true, true));
        definitions.addDefinition(new DcFieldDefinition(ContactPersonCategory._B_ICON, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_CREATED, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_MODIFIED, null, true, false, false));

        settings.addSetting(_General,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stFieldDefinitionsPersonType,
                            definitions, -1, "", "", false, false));
    }

    private void setSoftwareCategoryDefaults() {
        DcFieldDefinitions definitions = new DcFieldDefinitions();
        definitions.addDefinition(new DcFieldDefinition(SoftwareCategory._ID, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(SoftwareCategory._A_NAME, null, true, true, true));
        definitions.addDefinition(new DcFieldDefinition(SoftwareCategory._B_ICON, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_CREATED, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_MODIFIED, null, true, false, false));

        settings.addSetting(_General,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stFieldDefinitionsSoftwareCategory,
                            definitions, -1, "", "", false, false));
    }
    
    private void setGenreDefaults() {
        DcFieldDefinitions definitions = new DcFieldDefinitions();
        definitions.addDefinition(new DcFieldDefinition(DcGenre._ID, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcGenre._A_NAME, null, true, true, true));
        definitions.addDefinition(new DcFieldDefinition(DcGenre._B_ICON, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_CREATED, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_MODIFIED, null, true, false, false));

        settings.addSetting(_General,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stFieldDefinitionsGenre,
                            definitions, -1, "", "", false, false));
        
        
        DcFieldDefinitions definitions2 = new DcFieldDefinitions();
        definitions2.addDefinition(new DcFieldDefinition(DcMapping._ID, null, true, false, false));
        definitions2.addDefinition(new DcFieldDefinition(DcMapping._A_MEDIA_OBJECT_ID, null, true, true, true));
        definitions2.addDefinition(new DcFieldDefinition(DcMapping._B_SIMPLE_OBJECT_ID, null, true, false, true));
        definitions2.addDefinition(new DcFieldDefinition(DcMapping._SYS_CREATED, null, false, false, false));
        definitions2.addDefinition(new DcFieldDefinition(DcMapping._SYS_MODIFIED, null, false, false, false));
        
        settings.addSetting(_General,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stFieldDefinitionsMapping,
                            definitions2, -1, "", "", false, false));        
    }    

    private void setBookCategoryDefaults() {
        DcFieldDefinitions definitions = new DcFieldDefinitions();
        definitions.addDefinition(new DcFieldDefinition(BookCategory._ID, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(BookCategory._A_NAME, null, true, true, true));
        definitions.addDefinition(new DcFieldDefinition(BookCategory._B_ICON, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_CREATED, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_MODIFIED, null, true, false, false));

        settings.addSetting(_General,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stFieldDefinitionsBookCategory,
                            definitions, -1, "", "", false, false));
    }
    
    private void setImageCategoryDefaults() {
        DcFieldDefinitions definitions = new DcFieldDefinitions();
        definitions.addDefinition(new DcFieldDefinition(ImageCategory._ID, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(ImageCategory._A_NAME, null, true, true, true));
        definitions.addDefinition(new DcFieldDefinition(ImageCategory._B_ICON, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_CREATED, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_MODIFIED, null, true, false, false));

        settings.addSetting(_General,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stFieldDefinitionsImageCategory,
                            definitions, -1, "", "", false, false));
    }    
    
    private void setStorageMediaDefaults() {
        DcFieldDefinitions definitions = new DcFieldDefinitions();
        definitions.addDefinition(new DcFieldDefinition(DcState._ID, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcState._A_NAME, null, true, true, true));
        definitions.addDefinition(new DcFieldDefinition(DcState._B_ICON, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_CREATED, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_MODIFIED, null, true, false, false));

        settings.addSetting(_General,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stFieldDefinitionsSoftwareStorage,
                            definitions, -1, "", "", false, false));
        settings.addSetting(_General,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stFieldDefinitionsMovieStorage,
                            definitions, -1, "", "", false, false));        
        settings.addSetting(_General,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stFieldDefinitionsMusicAlbumStorage,
                            definitions, -1, "", "", false, false));    
        settings.addSetting(_General,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stFieldDefinitionsImageStorage,
                            definitions, -1, "", "", false, false));            
    }    
    
    private void setStateDefaults() {
        DcFieldDefinitions definitions = new DcFieldDefinitions();
        definitions.addDefinition(new DcFieldDefinition(DcState._ID, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcState._A_NAME, null, true, true, true));
        definitions.addDefinition(new DcFieldDefinition(DcState._B_ICON, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_CREATED, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_MODIFIED, null, true, false, false));

        settings.addSetting(_General,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stFieldDefinitionsAudioCDState,
                            definitions, -1, "", "", false, false));
        settings.addSetting(_General,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stFieldDefinitionsBookState,
                            definitions, -1, "", "", false, false));        
        settings.addSetting(_General,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stFieldDefinitionsMovieState,
                            definitions, -1, "", "", false, false));        
        settings.addSetting(_General,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stFieldDefinitionsMusicAlbumState,
                            definitions, -1, "", "", false, false));
        settings.addSetting(_General,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stFieldDefinitionsMusicTrackState,
                            definitions, -1, "", "", false, false));        
        settings.addSetting(_General,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stFieldDefinitionsSoftwareState,
                            definitions, -1, "", "", false, false));  
        settings.addSetting(_General,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stFieldDefinitionsImageState,
                            definitions, -1, "", "", false, false));         
    }    
    
    private void setSoftwarePlatformDefaults() {
        DcFieldDefinitions definitions = new DcFieldDefinitions();
        definitions.addDefinition(new DcFieldDefinition(DcState._ID, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcState._A_NAME, null, true, true, true));
        definitions.addDefinition(new DcFieldDefinition(DcState._B_ICON, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_CREATED, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_MODIFIED, null, true, false, false));
        
        settings.addSetting(_General,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stFieldDefinitionsSoftwarePlatform,
                            definitions, -1, "", "", false, false));
    }      
    
    private void setBookDefaults() {
        DcFieldDefinitions definitions = new DcFieldDefinitions();
        definitions.addDefinition(new DcFieldDefinition(Book._ID, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_AVAILABLE, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_LOANEDBY, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_LOANDURATION, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Book._A_TITLE, null, true, true, true));
        definitions.addDefinition(new DcFieldDefinition(Book._O_SERIES, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(Book._P_VOLUME_NR, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(Book._Q_VOLUME_TITLE, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(Book._B_DESCRIPTION, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Book._F_PUBLISHER, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Book._G_AUTHOR, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Book._C_YEAR, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Book._H_WEBPAGE, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Book._M_FILENAME, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Book._I_CATEGORY, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Book._J_ISBN10, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Book._N_ISBN13, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Book._K_PICTUREFRONT, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Book._D_LOCATION, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Book._L_STATE, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Book._E_RATING, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Book._U1_USER_LONGTEXT, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(Book._U2_USER_SHORTTEXT1, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(Book._U3_USER_SHORTTEXT2, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(Book._U4_USER_NUMERIC1, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(Book._U5_USER_NUMERIC2, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_CREATED, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_MODIFIED, null, true, false, false));

        settings.addSetting(_General,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stFieldDefinitionsBook,
                            definitions, -1, "", "", false, false));
    }

    private void setAudioTrackDefaults() {
        DcFieldDefinitions definitions = new DcFieldDefinitions();
        definitions.addDefinition(new DcFieldDefinition(AudioTrack._ID, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioTrack._F_TRACKNUMBER, null, true, true, true));
        definitions.addDefinition(new DcFieldDefinition(AudioTrack._A_TITLE, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(AudioTrack._B_DESCRIPTION, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioTrack._C_YEAR, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioTrack._D_LOCATION, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioTrack._E_RATING, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioTrack._G_LYRICS, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioTrack._H_PLAYLENGTH, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(AudioTrack._I_ALBUM, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioTrack._J_GENRES, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioTrack._K_ARTIST, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioTrack._U1_USER_LONGTEXT, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioTrack._U2_USER_SHORTTEXT1, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioTrack._U3_USER_SHORTTEXT2, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioTrack._U4_USER_NUMERIC1, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioTrack._U5_USER_NUMERIC2, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_CREATED, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_MODIFIED, null, true, false, false));

        settings.addSetting(_General,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stFieldDefinitionsAudioTrack,
                            definitions, -1, "", "", false, false));
    }

    private void setAudioCDDefaults() {
        DcFieldDefinitions definitions = new DcFieldDefinitions();
        definitions.addDefinition(new DcFieldDefinition(AudioCD._ID, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_AVAILABLE, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_LOANEDBY, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_LOANDURATION, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioCD._A_TITLE, null, true, true, true));
        definitions.addDefinition(new DcFieldDefinition(AudioCD._F_ARTIST, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(AudioCD._B_DESCRIPTION, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioCD._C_YEAR, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(AudioCD._G_GENRES, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(AudioCD._H_PICTUREFRONT, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(AudioCD._I_PICTUREBACK, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioCD._J_PICTURECD, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioCD._K_DISCID, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioCD._D_LOCATION, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(AudioCD._L_STATE, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioCD._M_WEBPAGE, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioCD._E_RATING, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(AudioCD._U1_USER_LONGTEXT, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioCD._U2_USER_SHORTTEXT1, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioCD._U3_USER_SHORTTEXT2, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioCD._U4_USER_NUMERIC1, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(AudioCD._U5_USER_NUMERIC2, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_CREATED, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_MODIFIED, null, true, false, false));

        settings.addSetting(_General,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stFieldDefinitionsAudioCD,
                            definitions, -1, "", "", false, false));
    }

    private void setMovieDefaults() {
        DcFieldDefinitions definitions = new DcFieldDefinitions();
        definitions.addDefinition(new DcFieldDefinition(Movie._ID, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_AVAILABLE, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_LOANEDBY, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_LOANDURATION, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._A_TITLE, null, true, true, true));
        definitions.addDefinition(new DcFieldDefinition(Movie._8_SERIES, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._9_SEASON, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._10_EPISODE_NR, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._11_EPISODE_TITLE, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._F_TITLE_LOCAL, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._B_DESCRIPTION, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._G_WEBPAGE, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._H_GENRES, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Movie._I_ACTORS, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._J_DIRECTOR, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._K_COUNTRY, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._C_YEAR, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Movie._L_PLAYLENGTH, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Movie._M_FILENAME, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._N_VIDEOCODEC, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._O_AUDIOCODEC, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._P_WIDTH, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._Q_HEIGHT, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._R_FPS, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._S_FRAMES, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._T_AUDIOBITRATE, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._U_AUDIOSAMPLINGRATE, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._V_AUDIOCHANNEL, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._W_VIDEOBITRATE, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._X_PICTUREFRONT, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Movie._Y_PICTUREBACK, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._Z_PICTURECD, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._0_LANGUAGE, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Movie._1_AUDIOLANGUAGE, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._2_SUBTITLELANGUAGE, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._3_CERTIFICATION, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._4_FILESIZE, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._7_STORAGEMEDIUM, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._D_LOCATION, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Movie._6_STATE, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._E_RATING, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Movie._5_ASIN, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._U1_USER_LONGTEXT, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._U2_USER_SHORTTEXT1, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._U3_USER_SHORTTEXT2, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._U4_USER_NUMERIC1, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(Movie._U5_USER_NUMERIC2, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_CREATED, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_MODIFIED, null, true, false, false));

        settings.addSetting(_General,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stFieldDefinitionsMovie,
                            definitions, -1, "", "", false, false));
    }

    private void setMusicAlbumDefaults() {
        DcFieldDefinitions definitions = new DcFieldDefinitions();
        definitions.addDefinition(new DcFieldDefinition(MusicAlbum._ID, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(MusicAlbum._A_TITLE, null, true, true, true));
        definitions.addDefinition(new DcFieldDefinition(MusicAlbum._B_DESCRIPTION, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(MusicAlbum._F_ARTIST, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(MusicAlbum._G_GENRES, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(MusicAlbum._C_YEAR, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(MusicAlbum._H_STATE, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(MusicAlbum._I_STORAGEMEDIUM, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(MusicAlbum._D_LOCATION, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(MusicAlbum._E_RATING, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(MusicAlbum._J_PICTUREFRONT, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(MusicAlbum._K_PICTUREBACK, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(MusicAlbum._L_PICTURECD, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(MusicAlbum._M_DISCID, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(MusicAlbum._N_WEBPAGE, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(MusicAlbum._U1_USER_LONGTEXT, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(MusicAlbum._U2_USER_SHORTTEXT1, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(MusicAlbum._U3_USER_SHORTTEXT2, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(MusicAlbum._U4_USER_NUMERIC1, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(MusicAlbum._U5_USER_NUMERIC2, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_CREATED, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_MODIFIED, null, true, false, false));

        settings.addSetting(_General,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stFieldDefinitionsMusicAlbum,
                            definitions, -1, "", "", false, false));
    }    
    
    private void setMusicTrackDefaults() {
        DcFieldDefinitions definitions = new DcFieldDefinitions();
        definitions.addDefinition(new DcFieldDefinition(MusicTrack._ID, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(MusicTrack._F_TRACKNUMBER, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(MusicTrack._A_TITLE, null, true, true, true));
        definitions.addDefinition(new DcFieldDefinition(MusicTrack._G_ARTIST, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(MusicTrack._P_ALBUM, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(MusicTrack._C_YEAR, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(MusicTrack._H_GENRES, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(MusicTrack._I_FILENAME, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(MusicTrack._J_PLAYLENGTH, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(MusicTrack._K_QUALITY, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(MusicTrack._L_ENCODING, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(MusicTrack._M_LYRIC, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(MusicTrack._N_FILESIZE, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(MusicTrack._D_LOCATION, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(MusicTrack._O_STATE, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(MusicTrack._E_RATING, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(MusicTrack._U1_USER_LONGTEXT, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(MusicTrack._U2_USER_SHORTTEXT1, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(MusicTrack._U3_USER_SHORTTEXT2, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(MusicTrack._U4_USER_NUMERIC1, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(MusicTrack._U5_USER_NUMERIC2, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_CREATED, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_MODIFIED, null, true, false, false));

        settings.addSetting(_General,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stFieldDefinitionsMusicTrack,
                            definitions, -1, "", "", false, false));
    }     
    
    private void setImageDefaults() {
        DcFieldDefinitions definitions = new DcFieldDefinitions();
        definitions.addDefinition(new DcFieldDefinition(Image._ID, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Image._A_TITLE, null, true, true, true));
        definitions.addDefinition(new DcFieldDefinition(Image._B_DESCRIPTION, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Image._F_WIDTH, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Image._G_HEIGHT, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Image._H_FILENAME, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Image._C_YEAR, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Image._D_LOCATION, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Image._E_RATING, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Image._I_IMAGE, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Image._J_STATE, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Image._K_CATEGORY, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Image._L_STORAGEMEDIUM, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Image._M_FILESIZE, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Image._N_DATE, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Image._O_COMPRESSION, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Image._P_PLACE, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Image._Q_CAMERA, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Image._U1_USER_LONGTEXT, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(Image._U2_USER_SHORTTEXT1, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(Image._U3_USER_SHORTTEXT2, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(Image._U4_USER_NUMERIC1, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(Image._U5_USER_NUMERIC2, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_CREATED, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_MODIFIED, null, true, false, false));

        settings.addSetting(_General,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stFieldDefinitionsImage,
                            definitions, -1, "", "", false, false));
    }    
    
    private void setSoftwareDefaults() {
        DcFieldDefinitions definitions = new DcFieldDefinitions();
        definitions.addDefinition(new DcFieldDefinition(Software._ID, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_AVAILABLE, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_LOANEDBY, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_LOANDURATION, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Software._A_TITLE, null, true, true, true));
        definitions.addDefinition(new DcFieldDefinition(Software._B_DESCRIPTION, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Software._F_DEVELOPER, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Software._G_PUBLISHER, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Software._H_PLATFORM, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Software._C_YEAR, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Software._I_WEBPAGE, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Software._J_FILENAME, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Software._K_TYPE, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Software._L_MULTI, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Software._M_PICTUREFRONT, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Software._N_PICTUREBACK, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Software._O_PICTURECD, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Software._P_SCREENSHOTONE, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Software._Q_SCREENSHOTTWO, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Software._R_SCREENSHOTTHREE, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Software._S_SERIALKEY, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Software._T_FILESIZE, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Software._W_STORAGEMEDIUM, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Software._D_LOCATION, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Software._V_STATE, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Software._E_RATING, null, true, false, true));
        definitions.addDefinition(new DcFieldDefinition(Software._U_ASIN, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(Software._U1_USER_LONGTEXT, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(Software._U2_USER_SHORTTEXT1, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(Software._U3_USER_SHORTTEXT2, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(Software._U4_USER_NUMERIC1, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(Software._U5_USER_NUMERIC2, null, false, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_CREATED, null, true, false, false));
        definitions.addDefinition(new DcFieldDefinition(DcObject._SYS_MODIFIED, null, true, false, false));

        settings.addSetting(_General,
                new Setting(Repository.datatypes._DEFINITIONGROUP,
                            Repository.settings.stFieldDefinitionsSoftware,
                            definitions, -1, "", "", false, false));
    }
}
