/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.wf;

import java.sql.ResultSet;
import java.sql.Types;
import java.util.ArrayList;
import java.util.Collection;

import javax.swing.SwingUtilities;

import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.Loan;
import net.sf.dc.core.objects.Picture;
import net.sf.dc.core.objects.generic.GenericObject;
import net.sf.dc.db.DatabaseManager;
import net.sf.dc.db.Query;
import net.sf.dc.messages.Messages;
import net.sf.dc.wf.requests.RequestCollection;

public class WorkFlow {

    public static void insertValues(DcObject o) {
        DatabaseManager.insertValues(o);
    }

    public static void updateValues(DcObject o) {
        DatabaseManager.updateValues(o);
    }

    public static void deleteValues(DcObject o) {
        DatabaseManager.deleteValues(o);
    }

    public static Collection runQueryDirect(Query query, boolean logQuery) {
        return DatabaseManager.runQueryDirect(query, logQuery);
    }

    public static ResultSet runQueryDirectUnclosed(String query) throws Exception {
        return DatabaseManager.runQueryDirectUnclosed(query, false);
    }

    public static Collection runQueryDirect(String query, boolean logQuery) {
        return DatabaseManager.runQueryDirect(query, logQuery);
    }

    public static void handleRequests(Collection objects, RequestCollection requests, boolean qryWasSuccess) {
        Runnable runMe = new UIUpdater(objects, requests, qryWasSuccess);
        SwingUtilities.invokeLater(runMe);
    }

    public static boolean checkUniqueness(DcObject o, boolean bUpdate) {
        return DatabaseManager.uniqueValues(o, bUpdate);
    }

    public Collection convertToDCObjects(ResultSet result) {
        ArrayList objects = new ArrayList();

        try {
        	result.isLast();
        } catch (Exception exp) {
            return objects;
        }

        try {
            int columnCount = result.getMetaData().getColumnCount();
            String[] columns = new String[columnCount];

            int k = 0;
            for (int i = 1; i < columnCount + 1; i++) {
                columns[k] = result.getMetaData().getColumnName(i);
                k++;
            }

            String sTableName = result.getMetaData().getTableName(1);

            while (result.next()) {
                DcObject dco = DcModules.getObjectForTable(sTableName);

                for (int i = 0; i < columns.length; i++) {
                    int type = result.getMetaData().getColumnType(i + 1);
                    String column = columns[i];
                    Object value = null;

                    if (type == Types.BOOLEAN) {
                        value = new Boolean(result.getBoolean(column));
                    } else if (type == Types.DATE) {
                        value = result.getDate(column);
                    } else {
                        value = result.getString(column);
                    }
                    dco.setValueForColumn(column, value);
                }

                if (dco.getModule().canBeLended()) {
                    Loan loan = DataManager.getCurrentLoan(dco.getID());
                    dco.setValue(DcObject._SYS_AVAILABLE, new Boolean(loan.isAvailable(dco.getID())));
                    dco.setValue(DcObject._SYS_LOANEDBY, loan.getPersonDescription());
                    dco.setValue(DcObject._SYS_LOANDURATION, loan.getDaysLoaned());
                }

                if (!(dco instanceof Picture))
                    dco.initializeImages();

                dco.initializeReferences();

                
                dco.setValue(GenericObject._SYS_MODULE, dco.getModule());
                objects.add(dco);
            }
        } catch (Exception exp) {
            Messages.add(exp);
        }
        return objects;
    }
}
