/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.wf.requests;

import java.util.Collection;

import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.generic.GenericObject;

public class SynchronizeWithManagerRequest implements IRequest {
    
    public static final int _UPDATE = 0;
    public static final int _DELETE = 1;
    public static final int _ADD = 2;
    
    private int type; 
    private int module;
    private DcObject dco;
    
    public SynchronizeWithManagerRequest(int type, DcObject dco) {
        this.type = type;
        this.dco = dco;
        
        if (dco instanceof GenericObject)
            this.module = DcModules.getCurrentModule().getIndex();
        else         
            this.module = dco.getModule().getIndex();
    }

    public void execute(Collection objects) {
        switch (type) {
        case _UPDATE:
            DataManager.update(dco, module);
            break;
        case _ADD:
            DataManager.add(dco, module);
            break;
        case _DELETE:
            DataManager.remove(dco, module);
            break;
        }
        
        dco = null;
    }

    public boolean getExecuteOnFail() {
        return false;
    }

    public void setExecuteOnFail(boolean b) {}
}
