package net.sf.ext.util;

import java.io.File;

import edu.stanford.ejalbert.BrowserLauncher;

import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.Repository;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.settings.definitions.ProgramDefinitions;

public class FileLauncher {

    private final String filename;
    
    public FileLauncher(String filename) {
        this.filename = filename;
    }
    
    public void launchFile() {
        if (filename == null || filename.trim().length() == 0) {
            new MessageBox(DcResources.getText("msgNoFilename"), MessageBox._WARNING);
            return;
        }
        
        File file = new File(filename);
        if (!file.exists()) {
            new MessageBox(DcResources.getText("msgFileDoesNotExist", filename), MessageBox._WARNING);
            return;
        }
            
        String extension = Utilities.getExtension(file);
        if (extension == null || extension.length() == 0) {
            new MessageBox(DcResources.getText("msgInvalidExtension", filename), MessageBox._WARNING);
            return;
        }
            
        ProgramDefinitions definitions = (ProgramDefinitions) DcSettings.getValue(Repository.settings.stProgramDefinitions);
        String program = null;
        if (definitions != null) 
            program = definitions.getProgramForExtension(extension);

        if (program == null || program.trim().length() == 0) {
            try {
                try {
                    // use the browser launcer (v2)
                    BrowserLauncher.openURL("file://" + filename);
                } catch (Exception exp) {
                    // last change: just start it on the command line
                    runCmd(filename);
                }
            } catch (Exception exp) {
                new MessageBox(DcResources.getText("msgNoProgramDefinedForExtension", extension), MessageBox._WARNING);  
            }
        } else {
            String cmd = program + " " + '"' + filename + '"';
            try {
                runCmd(cmd);
            } catch (Exception exp) {
                new MessageBox(DcResources.getText("msgErrorWhileExecuting", cmd), MessageBox._WARNING);
            } 
        }
    }   
    
    private void runCmd(String cmd) throws Exception {
        Process p = Runtime.getRuntime().exec(cmd);
        try { 
            p.waitFor();
        } catch (Exception exp) {}        
    }
}
