/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.ext.util;

import java.util.ArrayList;
import java.util.Collection;

import net.sf.dc.messages.Messages;

public class StringUtils {
	
    public static String getValueBetweenStrings(String start, String end, String sSource) {
        String s = "";
        try {
            int indexBegin = sSource.indexOf(start);
            if (indexBegin != -1) {
                s = sSource.substring(indexBegin + start.length());
            }
            
            int indexEnd = s.indexOf(end);
            if (indexEnd != -1) {
                s = s.substring(0, indexEnd);
            }
        } catch (Exception exp) {
        	//Messages.add(exp);
        }
        return s;
    }
    
    public static String getValueBetweenStrings(char start, char end, String sSource) {
        String s = "";
        try {
            int indexBegin = sSource.indexOf(start);
            if (indexBegin != -1) {
                s = sSource.substring(indexBegin + 1);
            }
            
            int indexEnd = s.indexOf(end);
            if (indexEnd != -1) {
                s = s.substring(0, indexEnd);
            }
        } catch (Exception exp) {
            Messages.add(exp);
        }
        return s;
    }    
    
    public static String[] getListElements(String s, String sep) {
        Collection values = new ArrayList();
        
        String value = s;
        int index = value.indexOf(sep);
        while (index > -1) {
            values.add(value.substring(0, index));
            value = value.substring(index + sep.length(), value.length());
            index = value.indexOf(sep);
        }

        values.add(value);
        String[] sValues = new String[values.size()];
        return (String[]) values.toArray(sValues);
    }    
    
    public static String normalize(String s) {
        s = s != null ? s.replaceAll("[,.!@#$%^&{}'~`:]", "") : "";
        
        int start = s.indexOf("(");
        int end = s.indexOf(")");
        
        while ((start > -1 && end > -1) && start < end) {
            s = s.substring(0, start) + " " + s.substring(end + 1, s.length());
            start = s.indexOf("(");
            end = s.indexOf(")");
        }
        
        return s.trim();
    }
    
    
    /**
     * Clean the string of any unwanted characters
     * @param s string to clean
     */
    public static String convertHttpString(String s) {
        if ((s.startsWith("\"")) && (s.endsWith("\""))) {
            s = s.substring(1, s.length() - 1);
        }
        
        s = replaceAll(s, "&amp;", "&");
        s = replaceAll(s, "&lt;", "<");
        s = replaceAll(s, "&gt;", ">");
        
        
        int indexBegin = s.toLowerCase().indexOf("<a href");
        int indexEnd = s.toLowerCase().indexOf("</a>");
        while (indexBegin > -1 && indexEnd > -1 && indexBegin < indexEnd) {
            s = s.substring(0, indexBegin) + " " + s.substring(indexEnd + 4, s.length());

            indexBegin = s.toLowerCase().indexOf("<a href");
            indexEnd = s.toLowerCase().indexOf("</a>");
        }
        
        s = replaceAll(s, "<hr>", "");
        s = replaceAll(s, "</hr>", "\n");
        
        s = replaceAll(s, "<h1>", "");
        s = replaceAll(s, "</h1>", "\n");

        s = replaceAll(s, "<h2>", "");
        s = replaceAll(s, "</h2>", "\n");

        s = replaceAll(s, "<h3>", "");
        s = replaceAll(s, "</h3>", "\n");

        s = replaceAll(s, "&nbsp;", " ");

        s = replaceAll(s, "<br />", "\n");
        s = replaceAll(s, "<br/>", "\n");
        s = replaceAll(s, "<br >", "\n");
        s = replaceAll(s, "<br>", "\n");
        
        s = replaceAll(s, "<b>", "");
        s = replaceAll(s, "</b>", "");

        s = replaceAll(s, "<i>", "");
        s = replaceAll(s, "</i>", "");
        
        s = replaceAll(s, "<li>", "");
        s = replaceAll(s, "</li>", "");

        s = replaceAll(s, "<ul>", "");
        s = replaceAll(s, "</ul>", "");
        
        s = replaceAll(s, "<p>", "");
        s = replaceAll(s, "</p>", "");
        
        s = replaceAll(s, "&#x27;", "`");
        s = replaceAll(s, "&#09;", "\t");
        s = replaceAll(s, "&#10;", "\n");
        s = replaceAll(s, "&#13;", "\r");
        s = replaceAll(s, "&#32;", "");
        s = replaceAll(s, "&#33;", "!");
        s = replaceAll(s, "&quot;", "" + '"');
        s = replaceAll(s, "&#34;", "" + '"');
        s = replaceAll(s, "&#35;", "#");
        s = replaceAll(s, "&#38;", "&");
        s = replaceAll(s, "&#39;", "`");
        s = replaceAll(s, "&#40;", "(");
        s = replaceAll(s, "&#41;", ")");
        s = replaceAll(s, "&#42;", "*");
        s = replaceAll(s, "&#43;", "+");
        s = replaceAll(s, "&#44;", ",");
        s = replaceAll(s, "&#45;", "-");
        s = replaceAll(s, "&#46;", ".");
        s = replaceAll(s, "&#47;", "/");
        s = replaceAll(s, "&#58;", ":");
        s = replaceAll(s, "&#59;", ";");
        s = replaceAll(s, "&#60;", "<");
        s = replaceAll(s, "&#61;", "=");
        s = replaceAll(s, "&#62;", ">");
        s = replaceAll(s, "&#63;", "?");        
        s = replaceAll(s, "&#64;", "@");
        s = replaceAll(s, "&#91;", "[");
        s = replaceAll(s, "&#92;", "\\");
        s = replaceAll(s, "&#93;", "]");
        s = replaceAll(s, "&#94;", "^");
        s = replaceAll(s, "&#95;", "_");
        s = replaceAll(s, "&#96;", "`");
        s = replaceAll(s, "&#123;", "{");
        s = replaceAll(s, "&#125;", "}");
        s = replaceAll(s, "&#126;", "~");
        s = replaceAll(s, "&#133;", "");
        s = replaceAll(s, "&#146;", "`");
        s = replaceAll(s, "&#150;", " ");
        s = replaceAll(s, "&#151;", " ");
        s = replaceAll(s, "&#160;", " ");
        s = replaceAll(s, "&#192;", "");
        s = replaceAll(s, "&#192;", "");
        s = replaceAll(s, "&#193;", "");
        s = replaceAll(s, "&#194;", "");
        s = replaceAll(s, "&#195;", "");
        s = replaceAll(s, "&#196;", "");
        s = replaceAll(s, "&#197;", "");
        s = replaceAll(s, "&#198;", "");
        s = replaceAll(s, "&#199;", "");
        s = replaceAll(s, "&#200;", "");
        s = replaceAll(s, "&#201;", ""); 
        s = replaceAll(s, "&#202;", ""); 
        s = replaceAll(s, "&#203;", "");
        s = replaceAll(s, "&#204;", "");
        s = replaceAll(s, "&#205;", ""); 
        s = replaceAll(s, "&#206;", ""); 
        s = replaceAll(s, "&#207;", ""); 
        s = replaceAll(s, "&#208;", ""); 
        s = replaceAll(s, "&#209;", ""); 
        s = replaceAll(s, "&#210;", ""); 
        s = replaceAll(s, "&#211;", ""); 
        s = replaceAll(s, "&#212;", ""); 
        s = replaceAll(s, "&#213;", ""); 
        s = replaceAll(s, "&#214;", ""); 
        s = replaceAll(s, "&#215;", ""); 
        s = replaceAll(s, "&#216;", ""); 
        s = replaceAll(s, "&#217;", ""); 
        s = replaceAll(s, "&#218;", ""); 
        s = replaceAll(s, "&#219;", ""); 
        s = replaceAll(s, "&#220;", ""); 
        s = replaceAll(s, "&#221;", ""); 
        s = replaceAll(s, "&#222;", ""); 
        s = replaceAll(s, "&#223;", "");
        s = replaceAll(s, "&#224;", ""); 
        s = replaceAll(s, "&#225;", ""); 
        s = replaceAll(s, "&#226;", ""); 
        s = replaceAll(s, "&#227;", ""); 
        s = replaceAll(s, "&#228;", ""); 
        s = replaceAll(s, "&#229;", ""); 
        s = replaceAll(s, "&#230;", ""); 
        s = replaceAll(s, "&#231;", ""); 
        s = replaceAll(s, "&#232;", ""); 
        s = replaceAll(s, "&#233;", ""); 
        s = replaceAll(s, "&#234;", ""); 
        s = replaceAll(s, "&#235;", ""); 
        s = replaceAll(s, "&#236;", ""); 
        s = replaceAll(s, "&#237;", ""); 
        s = replaceAll(s, "&#238;", ""); 
        s = replaceAll(s, "&#239;", ""); 
        s = replaceAll(s, "&#240;", ""); 
        s = replaceAll(s, "&#241;", ""); 
        s = replaceAll(s, "&#242;", ""); 
        s = replaceAll(s, "&#243;", ""); 
        s = replaceAll(s, "&#244;", ""); 
        s = replaceAll(s, "&#245;", ""); 
        s = replaceAll(s, "&#246;", ""); 
        s = replaceAll(s, "&#247;", ""); 
        s = replaceAll(s, "&#248;", ""); 
        s = replaceAll(s, "&#249;", ""); 
        s = replaceAll(s, "&#250;", ""); 
        s = replaceAll(s, "&#251;", ""); 
        s = replaceAll(s, "&#252;", ""); 
        s = replaceAll(s, "&#253;", ""); 
        s = replaceAll(s, "&#254;", ""); 
        s = replaceAll(s, "&#255;", "");
        s = replaceAll(s, "&#338;", ""); 
        s = replaceAll(s, "&#339;", ""); 
        s = replaceAll(s, "&#352;", ""); 
        s = replaceAll(s, "&#353;", ""); 
        s = replaceAll(s, "&#376;", ""); 
        s = replaceAll(s, "&#402;", ""); 
        
        s = replaceAll(s, "&#710;", "^");
        s = replaceAll(s, "&#732;", "");
        s = replaceAll(s, "&#8194;", "");
        s = replaceAll(s, "&#8195;", "");
        s = replaceAll(s, "&#8201;", "");
        s = replaceAll(s, "&#8204;", "");
        s = replaceAll(s, "&#8205;", "");
        s = replaceAll(s, "&#8206;", "");
        s = replaceAll(s, "&#8207;", "");
        
        s = replaceAll(s, "&#8211;", "-");
        s = replaceAll(s, "&#8212;", "--");
        s = replaceAll(s, "&#8216;", "`");
        s = replaceAll(s, "&#8217;", "`");
        s = replaceAll(s, "&#8218;", ",");
        s = replaceAll(s, "&#8220;", " " + '"');
        s = replaceAll(s, "&#8221;", "" + '"' + " ");
        s = replaceAll(s, "&#8222;", " " + '"');
        s = replaceAll(s, "&#8224;", "");
        s = replaceAll(s, "&#8225;", "");
        s = replaceAll(s, "&#8225;", "");
        s = replaceAll(s, "&#8225;", "");
        s = replaceAll(s, "&#8240;", "");
        s = replaceAll(s, "&#8249;", "{");
        s = replaceAll(s, "&#8250;", "}");
        s = replaceAll(s, "&#8364;", ""); 
        s = replaceAll(s, "&#8482;", "");   
        s = replaceAll(s, "  ", " ");


        
        try {
            if (s.indexOf("<table") > -1 && s.indexOf("</table>") > -1)
                s = s.substring(0, s.indexOf("<table")) + s.substring(s.indexOf("</table>") +  8, s.length());
        } catch (Exception ignore) {}
        
        return s.trim();         
    }   
    
    private static String replaceAll(String s, String c, String with) {
        String result = s.replaceAll(c, with);
        result = result.replaceAll(c.toUpperCase(), with);
        result = result.replaceAll(c.toLowerCase(), with);
        return result;
    }
}
